@@ -3,10 +3,10 @@
  *  Module    : extern.h
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2001-11-10
+ *  Updated   : 2003-01-22
  *  Notes     :
  *
- * Copyright (c) 1997-2002 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1997-2003 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Iain Lea.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -55,234 +52,243 @@
  * warnings are enabled. (Not all of the functions are ANSI or POSIX).
  */
 #ifdef DECL__FLSBUF
-	extern int _flsbuf (int, FILE *);
+	extern int _flsbuf(int, FILE *);
 #endif /* DECL__FLSBUF */
 #ifdef DECL_ATOI
-	extern int atoi (char *);
+	extern int atoi(char *);
 #endif /* DECL_ATOI */
 #ifdef DECL_ATOL
-	extern long atol (char *);
+	extern long atol(char *);
 #endif /* DECL_ATOL */
 #ifndef HAVE_MEMCPY
 #	ifdef DECL_BCOPY
-		extern int bcopy (char *, char *, int);
+		extern int bcopy(char *, char *, int);
 #	endif /* DECL_BCOPY */
 #endif /* !HAVE_MEMCPY */
-#ifdef DECL_BZERO /* FD_ZERO() might use this */
-	extern void bzero (char *, int);
-#endif /* DECL_BZERO */
+
+#if 0 /* trouble on Linux/gcc 3.1 */
+#	ifdef DECL_BZERO /* FD_ZERO() might use this */
+		extern void bzero(char *, int);
+#	endif /* DECL_BZERO */
+#endif /* 0 */
+
 #ifdef DECL_CALLOC
-	extern void *calloc (size_t, size_t);
+	extern void *calloc(size_t, size_t);
 #endif /* DECL_CALLOC */
 #ifdef DECL_FCLOSE
-	extern int fclose (FILE *);
+	extern int fclose(FILE *);
 #endif /* DECL_FCLOSE */
 #ifdef DECL_FDOPEN
-	extern FILE *fdopen (int, const char *);
+	extern FILE *fdopen(int, const char *);
 #endif /* DECL_FDOPEN */
 #ifdef DECL_FFLUSH
-	extern int fflush (FILE *);
+	extern int fflush(FILE *);
 #endif /* DECL_FFLUSH */
 #ifdef DECL_FGETC
-	extern int fgetc (FILE *);
+	extern int fgetc(FILE *);
 #endif /* DECL_FGETC */
 #if defined(DECL_FILENO) && !defined(fileno)
-	extern int fileno (FILE *);
+	extern int fileno(FILE *);
 #endif /* DECL_FILENO && !fileno */
 #ifdef DECL_FPRINTF
-	extern int fprintf (FILE *, const char *, ...);
+	extern int fprintf(FILE *, const char *, ...);
 #endif /* DECL_FPRINTF */
 #ifdef DECL_FPUTC
-	extern int fputc (int, FILE *);
+	extern int fputc(int, FILE *);
 #endif /* DECL_FPUTC */
 #ifdef DECL_FPUTS
-	extern int fputs (const char *, FILE *);
+	extern int fputs(const char *, FILE *);
 #endif /* DECL_FPUTS */
 #ifdef DECL_FREAD
-	extern size_t fread (void *, size_t, size_t, FILE *);
+	extern size_t fread(void *, size_t, size_t, FILE *);
 #endif /* DECL_FREAD */
 #ifdef DECL_FREE
-	extern void free (void *);
+	extern void free(void *);
 #endif /* DECL_FREE */
 #ifdef DECL_FSEEK
-	extern int fseek (FILE *, long, int);
+	extern int fseek(FILE *, long, int);
 #endif /* DECL_FSEEK */
 #ifdef DECL_FWRITE
-	extern size_t fwrite (void *, size_t, size_t, FILE *);
+	extern size_t fwrite(void *, size_t, size_t, FILE *);
 #endif /* DECL_FWRITE */
 #ifdef DECL_GETCWD
-	extern char *getcwd (char *, size_t);
+	extern char *getcwd(char *, size_t);
 #endif /* DECL_GETCWD */
 #ifdef DECL_GETENV
-	extern char *getenv (const char *);
+	extern char *getenv(const char *);
 #endif /* DECL_GETENV */
 #ifdef DECL_GETHOSTBYNAME
-	extern struct hostent *gethostbyname (const char *);
+	extern struct hostent *gethostbyname(const char *);
 #endif /* DECL_GETHOSTBYNAME */
 #ifdef DECL_GETHOSTNAME
-	extern int gethostname (char *, int);
+	extern int gethostname(char *, int);
 #endif /* DECL_GETHOSTNAME */
 #ifdef DECL_GETLOGIN
-	extern char *getlogin (void);
+	extern char *getlogin(void);
 #endif /* DECL_GETLOGIN */
 #ifdef DECL_GETOPT
-	extern int getopt (int, char * const*, const char *);
+	extern int getopt(int, char * const*, const char *);
 #endif /* DECL_GETOPT */
 #ifdef DECL_GETPASS
 	extern char *getpass(const char *);
 #endif /* DECL_GETPASS */
 #ifdef DECL_GETPWNAM
-	extern struct passwd *getpwnam (const char *);
+	extern struct passwd *getpwnam(const char *);
 #endif /* DECL_GETPWNAM */
 #ifdef DECL_GETSERVBYNAME
-	extern struct servent *getservbyname (const char *, const char *);
+	extern struct servent *getservbyname(const char *, const char *);
 #endif /* DECL_GETSERVBYNAME */
+#ifdef DECL_GETWD
+	extern char *getwd(char *);
+#endif /* DECL_GETWD */
 #ifdef DECL_INET_ADDR
-	extern unsigned long inet_addr (const char *);
+	extern unsigned long inet_addr(const char *);
 #endif /* DECL_INET_ADDR */
+
 #if 0 /* breaks gcc 3.0 -std=c89 on SuSE 7.1 */
 #	ifdef DECL_INET_ATON
 		extern int inet_aton(const char *, struct in_addr *);
 #	endif /* DECL_INET_ATON */
 #endif /* 0 */
+
 #ifdef DECL_IOCTL
-	extern int ioctl (int, int, void *);
+	extern int ioctl(int, int, void *);
 #endif /* DECL_IOCTL */
 #if defined(DECL_ISASCII) && !defined(isascii)
-	extern int isascii (int);
+	extern int isascii(int);
 #endif /* DECL_ISASCII && !isascii */
 #ifdef DECL_KILL
-	extern int kill (pid_t, int);
+	extern int kill(pid_t, int);
 #endif /* DECL_KILL */
 #ifdef DECL_MALLOC
-	extern void *malloc (size_t);
+	extern void *malloc(size_t);
 #endif /* DECL_MALLOC */
-
-#if 0 /* this breaks egcs 2.91.66/gcc 2.95.1 on SuSE 6.2 */
-#	ifdef DECL_MEMSET
-		extern void *memset (void *, int, size_t);
-#	endif /* DECL_MEMSET */
-#endif /* 0 */
-
+#ifdef DECL_MEMSET
+	extern void *memset(void *, int, size_t);
+#endif /* DECL_MEMSET */
 #ifdef DECL_MKSTEMP
-	extern int mkstemp (char *);
+	extern int mkstemp(char *);
 #endif /* DECL_MKSTEMP */
 #ifdef DECL_MKTEMP
-	extern char *mktemp (char *);
+	extern char *mktemp(char *);
 #endif /* DECL_MKTEMP */
 #ifdef DECL_PCLOSE
-	extern int pclose (FILE *);
+	extern int pclose(FILE *);
 #endif /* DECL_PCLOSE */
 #ifdef DECL_PERROR
-	extern void perror (const char *);
+	extern void perror(const char *);
 #endif /* DECL_PERROR */
 #ifdef DECL_POPEN
-	extern FILE *popen (const char *, const char *);
+	extern FILE *popen(const char *, const char *);
 #endif /* DECL_POPEN */
 #ifdef DECL_PRINTF
-	extern int printf (const char *, ...);
+	extern int printf(const char *, ...);
 #endif /* DECL_PRINTF */
 #ifdef DECL_PUTENV
-	extern int putenv (const char *);
+	extern int putenv(const char *);
 #endif /* DECL_PUTENV */
 #ifdef DECL_QSORT
-	extern void qsort (void *, size_t, size_t, int (*)(t_comptype*, t_comptype*));
+	extern void qsort(void *, size_t, size_t, int (*)(t_comptype*, t_comptype*));
 #endif /* DECL_QSORT */
 #ifdef DECL_REALLOC
-	extern void *realloc (void *, size_t);
+	extern void *realloc(void *, size_t);
 #endif /* DECL_REALLOC */
 #ifdef DECL_RENAME
-	extern int rename (const char *, const char *);
+	extern int rename(const char *, const char *);
 #endif /* DECL_RENAME */
 #ifdef DECL_REWIND
-	extern void rewind (FILE *);
+	extern void rewind(FILE *);
 #endif /* DECL_REWIND */
 #ifdef DECL_SELECT
-	extern int select (int, fd_set *, fd_set *, fd_set *, struct timeval *);
+	extern int select(int, fd_set *, fd_set *, fd_set *, struct timeval *);
 #endif /* DECL_SELECT */
 #ifndef HAVE_PUTENV
 #	ifdef DECL_SETENV
-		extern int setenv (const char *, const char *, int);
+		extern int setenv(const char *, const char *, int);
 #	endif /* DECL_SETENV */
 #endif /* !HAVE_PUTENV */
 #ifdef DECL_SOCKET
-	extern int socket (int, int, int);
+	extern int socket(int, int, int);
 #endif /* DECL_SOCKET */
 #ifdef DECL_SNPRINTF
-	extern int snprintf (char *, size_t, const char *, ...);
+	extern int snprintf(char *, size_t, const char *, ...);
 #endif /* DECL_SNPRINTF */
 #ifdef DECL_SSCANF
-	extern int sscanf (const char *, const char *, ...);
+	extern int sscanf(const char *, const char *, ...);
 #endif /* DECL_SSCANF */
 #ifdef DECL_STRCASECMP
-	extern int strcasecmp (const char *, const char *);
+	extern int strcasecmp(const char *, const char *);
 #endif /* DECL_STRCASECMP */
 #ifdef DECL_STRCHR
-	extern char *strchr (const char *, int);
-	extern char *strrchr (const char *, int);
+	extern char *strchr(const char *, int);
+	extern char *strrchr(const char *, int);
 #endif /* DECL_STRCHR */
 #ifdef DECL_STRFTIME
-	extern int strftime (char *, int, char *, struct tm *);
+	extern int strftime(char *, int, char *, struct tm *);
 #endif /* DECL_STRFTIME */
 #ifdef DECL_STRNCASECMP
-	extern int strncasecmp (const char *, const char *, size_t);
+	extern int strncasecmp(const char *, const char *, size_t);
 #endif /* DECL_STRNCASECMP */
 #ifdef DECL_STRTOL
-	extern long strtol (const char *, char **, int);
+	extern long strtol(const char *, char **, int);
 #endif /* DECL_STRTOL */
 #ifdef DECL_SYSTEM
-	extern int system (char *);
+	extern int system(char *);
 #endif /* DECL_SYSTEM */
 #ifdef DECL_TEMPNAM
-	extern char *tempnam (const char *, const char *);
+	extern char *tempnam(const char *, const char *);
 #endif /* DECL_TEMPNAM */
 #ifdef DECL_TGETENT
-	extern int tgetent (char *, char *);
+	extern int tgetent(char *, char *);
 #endif /* DECL_TGETENT */
 #ifdef DECL_TGETFLAG
-	extern int tgetflag (char *);
+	extern int tgetflag(char *);
 #endif /* DECL_TGETFLAG */
 #if defined(DECL_TGETNUM)
-	extern int tgetnum (char *);
+	extern int tgetnum(char *);
 #endif /* DECL_TGETNUM */
 #ifdef DECL_TGETSTR
-	extern char *tgetstr (char *, char **);
+	extern char *tgetstr(char *, char **);
 #endif /* DECL_TGETSTR */
 #ifdef DECL_TGOTO
-	extern char *tgoto (char *, int, int);
+	extern char *tgoto(char *, int, int);
 #endif /* DECL_TGOTO */
 #ifdef DECL_TIGETFLAG
-	extern int tigetflag (char *);
+	extern int tigetflag(char *);
 #endif /* DECL_TIGETFLAG */
 #if defined(DECL_TIGETNUM)
-	extern int tigetnum (char *);
+	extern int tigetnum(char *);
 #endif /* DECL_TGETNUM */
 #ifdef DECL_TIGETSTR
-	extern char *tigetstr (char *, char **);
+	extern char *tigetstr(char *, char **);
 #endif /* DECL_TIGETSTR */
 #ifdef DECL_TIME
-	extern time_t time (time_t *);
+	extern time_t time(time_t *);
 #endif /* DECL_TIME */
 #if defined(DECL_TOLOWER) && !defined(tolower)
-	extern int tolower (int);
+	extern int tolower(int);
 #endif /* DECL_TOLOWER && !tolower */
 #if defined(DECL_TOUPPER) && !defined(toupper)
-	extern int toupper (int);
+	extern int toupper(int);
 #endif /* DECL_TOUPPER && !toupper */
 #ifdef DECL_TPARM
-	extern char *tparm (const char *, ...);
+	extern char *tparm(const char *, ...);
 #endif /* DECL_TPARM */
 #ifdef DECL_TPUTS
-	extern int tputs (char *, int, OutcPtr);
+	extern int tputs(char *, int, OutcPtr);
 #endif /* DECL_TPUTS */
 #ifdef DECL_UNGETC
-	extern int ungetc (int, FILE *);
+	extern int ungetc(int, FILE *);
 #endif /* DECL_UNGETC */
-#ifdef DECL_USLEEP
-	extern void usleep(unsigned long);
-#endif /* DECL_USLEEP */
+
+#if 0 /* SUSv2 changed that to "int usleep(unsigned long)" */
+#	ifdef DECL_USLEEP
+		extern void usleep(unsigned long);
+#	endif /* DECL_USLEEP */
+#endif /* 0 */
+
 #ifdef DECL_VSNPRINTF
-	extern int vsnprintf (char *, size_t, const char *, va_list);
+	extern int vsnprintf(char *, size_t, const char *, va_list);
 #endif /* DECL_VSNPRINTF */
 #ifdef DECL_VSPRINTF
 	extern int vsprintf(char *, char *, va_list);
@@ -333,7 +339,6 @@ extern char dead_article[PATH_LEN];
 extern char dead_articles[PATH_LEN];
 extern char default_organization[PATH_LEN];
 extern char default_signature[PATH_LEN];
-extern char global_attributes_file[PATH_LEN];
 extern char global_config_file[PATH_LEN];
 extern char homedir[PATH_LEN];
 extern char index_maildir[PATH_LEN];
@@ -353,14 +358,18 @@ extern char mail_active_file[PATH_LEN];
 extern char mail_news_user[LEN];
 extern char mailbox[PATH_LEN];
 extern char mailer[PATH_LEN];
-extern char mailgroups_file[PATH_LEN];
+#ifdef HAVE_MH_MAIL_HANDLING
+	extern char mailgroups_file[PATH_LEN];
+#endif /* HAVE_MH_MAIL_HANDLING */
 extern char mesg[LEN];
 extern char newnewsrc[PATH_LEN];
 extern char news_active_file[PATH_LEN];
 extern char newsgroups_file[PATH_LEN];
 extern char newsrc[PATH_LEN];
-extern char novrootdir[PATH_LEN];
-extern char novfilename[PATH_LEN];
+#ifndef NNTP_ONLY
+	extern char novrootdir[PATH_LEN];
+	extern char novfilename[PATH_LEN];
+#endif /* !NNTP_ONLY */
 extern char page_header[LEN];
 extern char posted_info_file[PATH_LEN];
 extern char posted_msgs_file[PATH_LEN];
@@ -387,25 +396,28 @@ extern constext *help_page[];
 extern constext *help_select[];
 extern constext *help_thread[];
 extern constext *txt_colors[];
+extern constext *txt_confirm_choices[];
 extern constext *txt_kill_level_type[];
 #ifdef CHARSET_CONVERSION
 	extern constext *txt_mime_charsets[];
 #endif /* CHARSET_CONVERSION */
+extern constext *txt_mailbox_formats[];
 extern constext *txt_marks[];
 extern constext *txt_mime_encodings[NUM_MIME_ENCODINGS];
 extern constext *txt_onoff[];
 extern constext *txt_post_process_type[];
+extern constext *txt_quote_style_type[];
 extern constext *txt_show_from[];
 extern constext *txt_sort_a_type[];	/* a=articles */
 extern constext *txt_sort_t_type[];	/* t=threads */
 extern constext *txt_strip_bogus_type[];
 extern constext *txt_thread[];
+extern constext *txt_thread_score_type[];
 extern constext *txt_wildcard_type[];
 extern constext txt_1_resp[];
 extern constext txt_7bit[];
 extern constext txt_8bit[];
 extern constext txt_active_file_is_empty[];
-extern constext txt_added_groups[];
 extern constext txt_all[];
 extern constext txt_all_groups[];
 extern constext txt_append_overwrite_quit[];
@@ -446,10 +458,10 @@ extern constext txt_autosubscribed[];
 extern constext txt_autosubscribing_groups[];
 extern constext txt_bad_active_file[];
 extern constext txt_bad_article[];
+extern constext txt_bad_attrib[];
 extern constext txt_bad_command[];
 extern constext txt_base64[];
 extern constext txt_batch_update_unavail[];
-extern constext txt_batch_update_failed[];
 extern constext txt_begin_of_art[];
 extern constext txt_cancel_article[];
 extern constext txt_cancelling_art[];
@@ -464,6 +476,7 @@ extern constext txt_cannot_open[];
 extern constext txt_cannot_open_for_saving[];
 extern constext txt_cannot_post[];
 extern constext txt_cannot_post_group[];
+extern constext txt_cannot_retrieve[];
 extern constext txt_cannot_write_index[];
 extern constext txt_cannot_write_to_directory[];
 extern constext txt_catchup_all_read_groups[];
@@ -480,6 +493,7 @@ extern constext txt_choose_post_process_type[];
 extern constext txt_color_off[];
 extern constext txt_color_on[];
 extern constext txt_command_failed[];
+extern constext txt_confirm_select_on_exit[];
 extern constext txt_connecting[];
 extern constext txt_connecting_port[];
 extern constext txt_copyright_notice[];
@@ -489,10 +503,12 @@ extern constext txt_creating_newsrc[];
 extern constext txt_default[];
 extern constext txt_delete_processed_files[];
 extern constext txt_deleting[];
+extern constext txt_disconnecting[];
 extern constext txt_end_of_art[];
 extern constext txt_end_of_arts[];
 extern constext txt_end_of_groups[];
 extern constext txt_end_of_thread[];
+extern constext txt_enter_getart_limit[];
 extern constext txt_enter_message_id[];
 extern constext txt_enter_next_thread[];
 extern constext txt_enter_option_num[];
@@ -537,7 +553,6 @@ extern constext txt_error_header_line_bad_charset[];
 extern constext txt_error_header_line_bad_encoding[];
 extern constext txt_error_header_line_blank[];
 extern constext txt_error_header_line_colon[];
-extern constext txt_error_header_line_comma[];
 extern constext txt_error_header_line_empty[];
 #ifndef FOLLOW_USEFOR_DRAFT
 	extern constext txt_error_header_line_comma[];
@@ -559,9 +574,6 @@ extern constext txt_error_unknown_dlevel[];
 extern constext txt_esc[];
 extern constext txt_exiting[];
 extern constext txt_external_mail_done[];
-#ifndef HAVE_LIBUU
-	extern constext txt_extracting_archive[];
-#endif /* !HAVE_LIBUU */
 extern constext txt_extracting_shar[];
 extern constext txt_failed_to_connect_to_server[];
 extern constext txt_feed_pattern[];
@@ -569,6 +581,7 @@ extern constext txt_filesystem_full[];
 extern constext txt_filesystem_full_backup[];
 extern constext txt_filter_file[];
 extern constext txt_filter_global_rules[];
+extern constext txt_filter_rule_created[];
 extern constext txt_filter_score[];
 extern constext txt_filter_score_help[];
 extern constext txt_filter_text_type[];
@@ -577,8 +590,10 @@ extern constext txt_followup_poster[];
 extern constext txt_from_line_only[];
 extern constext txt_from_line_only_case[];
 extern constext txt_full[];
+extern constext txt_gethostbyname[];
 extern constext txt_global[];
 extern constext txt_group[];
+extern constext txt_group_aliased[];
 extern constext txt_group_is_moderated[];
 extern constext txt_group_plural[];
 extern constext txt_group_select_com[];
@@ -620,15 +635,14 @@ extern constext txt_help_article_search_forwards[];
 extern constext txt_help_article_show_raw[];
 extern constext txt_help_article_skip_quote[];
 extern constext txt_help_article_toggle_formfeed[];
-#ifdef HAVE_COLOR
-	extern constext txt_help_article_toggle_highlight[];
-#endif /* HAVE_COLOR */
+extern constext txt_help_article_toggle_highlight[];
 extern constext txt_help_article_toggle_rot13[];
 extern constext txt_help_article_toggle_tabwidth[];
 extern constext txt_help_article_toggle_tex2iso[];
 extern constext txt_help_article_toggle_uue[];
 extern constext txt_help_article_view_attachments[];
 extern constext txt_help_bug[];
+extern constext txt_help_filter_comment[];
 extern constext txt_help_filter_from[];
 extern constext txt_help_filter_lines[];
 extern constext txt_help_filter_msgid[];
@@ -665,6 +679,7 @@ extern constext txt_help_global_search_auth_backwards[];
 extern constext txt_help_global_search_auth_forwards[];
 extern constext txt_help_global_search_body[];
 extern constext txt_help_global_search_body_comment[];
+extern constext txt_help_global_search_repeat[];
 extern constext txt_help_global_search_subj_backwards[];
 extern constext txt_help_global_search_subj_forwards[];
 #ifndef NO_SHELL_ESCAPE
@@ -720,7 +735,6 @@ extern constext txt_help_select_last_group[];
 extern constext txt_help_select_mark_group_unread[];
 extern constext txt_help_select_move_group[];
 extern constext txt_help_select_next_unread_group[];
-extern constext txt_help_select_next_unread_group[];
 extern constext txt_help_select_quit[];
 extern constext txt_help_select_quit_no_write[];
 extern constext txt_help_select_read_group[];
@@ -736,6 +750,7 @@ extern constext txt_help_select_toggle_descriptions[];
 extern constext txt_help_select_toggle_read_groups[];
 extern constext txt_help_select_unsubscribe[];
 extern constext txt_help_select_unsubscribe_pattern[];
+extern constext txt_help_select_sort_active[];
 extern constext txt_help_select_yank_active[];
 extern constext txt_help_thread_article_by_num[];
 extern constext txt_help_thread_catchup[];
@@ -749,7 +764,6 @@ extern constext txt_help_title_disp[];
 extern constext txt_help_title_misc[];
 extern constext txt_help_title_navi[];
 extern constext txt_help_title_ops[];
-extern constext txt_hit_space_for_more[];
 extern constext txt_index_page_com[];
 extern constext txt_info_add_kill[];
 extern constext txt_info_add_select[];
@@ -828,7 +842,6 @@ extern constext txt_no_filename[];
 extern constext txt_no_group[];
 extern constext txt_no_groups[];
 extern constext txt_no_groups_to_read[];
-extern constext txt_no_groups_to_yank_in[];
 extern constext txt_no_last_message[];
 extern constext txt_no_mail_address[];
 extern constext txt_no_match[];
@@ -869,8 +882,10 @@ extern constext txt_post_newsgroups[];
 extern constext txt_post_processing[];
 extern constext txt_post_processing_finished[];
 extern constext txt_post_subject[];
-extern constext txt_post_via_builtin_inews[];
-extern constext txt_post_via_builtin_inews_only[];
+#ifdef NNTP_INEWS
+	extern constext txt_post_via_builtin_inews[];
+	extern constext txt_post_via_builtin_inews_only[];
+#endif /* NNTP_INEWS */
 extern constext txt_posted_info_file[];
 extern constext txt_posting[];
 extern constext txt_postpone_repost[];
@@ -937,6 +952,7 @@ extern constext txt_search_forwards[];
 extern constext txt_searching[];
 extern constext txt_searching_body[];
 extern constext txt_select_art[];
+extern constext txt_select_comment[];
 extern constext txt_select_config_file_option[];
 extern constext txt_select_from[];
 extern constext txt_select_group[];
@@ -979,9 +995,10 @@ extern constext txt_tinrc_filter[];
 extern constext txt_tinrc_header[];
 extern constext txt_tinrc_info_in_last_line[];
 extern constext txt_tinrc_newnews[];
-extern constext txt_tinrc_post_process_view[];
+extern constext txt_toggled_high[];
 extern constext txt_toggled_rot13[];
 extern constext txt_toggled_tex2iso[];
+extern constext txt_toggled_tabwidth[];
 extern constext txt_type_h_for_help[];
 extern constext txt_unlimited_time[];
 extern constext txt_unread[];
@@ -993,9 +1010,8 @@ extern constext txt_untagged_art[];
 extern constext txt_untagged_thread[];
 extern constext txt_unthreading_arts[];
 extern constext txt_updated[];
-#ifndef HAVE_UUDECODE
-	extern constext txt_uuencode_not_supported[];
-#endif /* !HAVE_UUDECODE */
+extern constext txt_url_open[];
+extern constext txt_url_done[];
 extern constext txt_value_out_of_range[];
 extern constext txt_view_attachment[];
 extern constext txt_warn_art_line_too_long[];
@@ -1018,19 +1034,19 @@ extern constext txt_warn_wrong_sig_format[];
 extern constext txt_warn_xref_not_supported[];
 extern constext txt_writing_attributes_file[];
 extern constext txt_x_resp[];
-extern constext txt_yanking_all_groups[];
-extern constext txt_yanking_sub_groups[];
+extern constext txt_yanked_groups[];
+extern constext txt_yanked_none[];
+extern constext txt_yanked_sub_groups[];
+extern constext txt_show_unread[];
 extern constext txt_yes[];
 extern constext txt_you_have_mail[];
 
 #ifndef DISABLE_PRINTING
-	extern constext txt_help_o[];
 	extern constext txt_print[];
 	extern constext txt_printing[];
 #endif /* !DISABLE_PRINTING */
 
 #ifndef DONT_HAVE_PIPING
-	extern constext txt_help_pipe[];
 	extern constext txt_no_command[];
 #endif /* !DONT_HAVE_PIPING */
 
@@ -1043,7 +1059,6 @@ extern constext txt_you_have_mail[];
 
 #ifdef HAVE_LIBUU
 	extern constext txt_libuu_saved[];
-	extern constext txt_libuu_success[];
 	extern constext txt_libuu_error_missing[];
 	extern constext txt_libuu_error_no_begin[];
 	extern constext txt_libuu_error_no_data[];
@@ -1053,9 +1068,9 @@ extern constext txt_uu_error_decode[];
 extern constext txt_uu_error_no_end[];
 extern constext txt_uu_success[];
 
-#if defined(NNTP_ABLE) || defined(NNTP_ONLY)
+#if defined(NNTP_ABLE) && !defined(NNTP_ONLY)
 	extern constext txt_cannot_open_active_file[];
-#endif /* NNTP_ABLE || NNTP_ONLY */
+#endif /* NNTP_ABLE && !NNTP_ONLY */
 
 #ifndef NO_SHELL_ESCAPE
 	extern constext txt_shell_escape[];
@@ -1070,8 +1085,8 @@ extern int cCOLS;
 extern int cLINES;
 extern int curr_line;
 extern int debug;
-extern int group_hash[TABLE_SIZE];
 extern int groupname_len;
+extern int i_key_search_last;
 extern int iso2asc_supported;
 extern int last_resp;
 extern int max_active;
@@ -1116,7 +1131,10 @@ extern struct regex_cache url_regex;
 extern struct regex_cache mail_regex;
 extern struct regex_cache news_regex;
 extern struct regex_cache shar_regex;
-
+extern struct regex_cache slashes_regex;
+extern struct regex_cache stars_regex;
+extern struct regex_cache underscores_regex;
+extern struct regex_cache strokes_regex;
 #ifdef HAVE_COLOR
 	extern struct regex_cache quote_regex;
 	extern struct regex_cache quote_regex2;
@@ -1166,32 +1184,36 @@ extern t_menu *currmenu;
 
 extern t_openartinfo pgart;
 
-#define HIST_SIZE		15
-#define HIST_OTHER		 0
-#define HIST_ART_SEARCH		 1
-#define HIST_AUTHOR_SEARCH	 2
-#define HIST_GOTO_GROUP		 3
-#define HIST_GROUP_SEARCH	 4
-#define HIST_MAIL_ADDRESS	 5
-#define HIST_MESSAGE_ID		 6
-#define HIST_MOVE_GROUP		 7
-#define HIST_PIPE_COMMAND	 8
-#define HIST_POST_NEWSGROUPS	 9
-#define HIST_POST_SUBJECT	10
-#define HIST_REGEX_PATTERN	11
-#define HIST_REPOST_GROUP	12
-#define HIST_SAVE_FILE		13
-#define HIST_SELECT_PATTERN	14
-#define HIST_SHELL_COMMAND	15
-#define HIST_SUBJECT_SEARCH	16
-#define HIST_CONFIG_SEARCH	17
-#define HIST_HELP_SEARCH	18
-#define HIST_MAXNUM		18	/* must always be the same as the highest HIST_ value except HIST_NONE */
-#define HIST_NONE		(HIST_MAXNUM+1)
+enum {
+	HIST_OTHER = 0,
+	HIST_ART_SEARCH,
+	HIST_AUTHOR_SEARCH,
+	HIST_GOTO_GROUP,
+	HIST_GROUP_SEARCH,
+	HIST_MAIL_ADDRESS,
+	HIST_MESSAGE_ID,
+	HIST_MOVE_GROUP,
+	HIST_PIPE_COMMAND,
+	HIST_POST_NEWSGROUPS,
+	HIST_POST_SUBJECT,
+	HIST_REGEX_PATTERN,
+	HIST_REPOST_GROUP,
+	HIST_SAVE_FILE,
+	HIST_SELECT_PATTERN,
+	HIST_SHELL_COMMAND,
+	HIST_SUBJECT_SEARCH,
+	HIST_CONFIG_SEARCH,
+	HIST_HELP_SEARCH,
+	HIST_URL
+};
+/* must always be the same as the highest HIST_ value except HIST_NONE */
+#define HIST_MAXNUM		HIST_URL
+#define HIST_NONE		(HIST_MAXNUM + 1)
+#define HIST_SIZE		15	/* # items in each history */
 
-extern int hist_last[HIST_MAXNUM+1];
-extern int hist_pos[HIST_MAXNUM+1];
-extern char *input_history[HIST_MAXNUM+1][HIST_SIZE+1];
+extern int hist_last[HIST_MAXNUM + 1];
+extern int hist_pos[HIST_MAXNUM + 1];
+extern char *input_history[HIST_MAXNUM + 1][HIST_SIZE + 1];
 
 
 /* defines for GNKSA checking */
@@ -1237,10 +1259,6 @@ extern char *input_history[HIST_MAXNUM+1][HIST_SIZE+1];
 	extern constext txt_pipe_to_command[];
 	extern constext txt_piping[];
 #else
-#	ifdef VMS
-		extern constext txt_pipe_to_command[];
-		extern constext txt_piping[];
-#	endif /* VMS */
 	extern constext txt_piping_not_enabled[];
 #endif /* !DONT_HAVE_PIPING */
 
@@ -1248,15 +1266,12 @@ extern char *input_history[HIST_MAXNUM+1][HIST_SIZE+1];
 	extern constext txt_warn_cancel_forgery[];
 #endif /* FORGERY */
 
+extern t_bool word_highlight;
 #ifdef HAVE_COLOR
-	extern constext txt_help__[];
-	extern constext txt_help_color[];
 	extern constext txt_tinrc_colors[];
-	extern constext txt_toggled_high[];
 	extern int default_bcol;
 	extern int default_fcol;
 	extern t_bool use_color;
-	extern t_bool word_highlight;
 #	ifdef USE_CURSES
 		extern constext txt_no_colorterm[];
 #	endif /* USE_CURSES */
@@ -1274,7 +1289,6 @@ extern char *input_history[HIST_MAXNUM+1][HIST_SIZE+1];
 #endif /* HAVE_METAMAIL */
 
 #ifdef HAVE_PGP_GPG
-	extern constext txt_help_ctrl_g[];
 	extern constext txt_pgp_add[];
 	extern constext txt_pgp_mail[];
 	extern constext txt_pgp_news[];
@@ -1311,10 +1325,6 @@ extern constext txt_processing_saved_arts[];
 	extern constext txt_warn_posting_etiquette[];
 #endif /* NO_ETIQUETTE */
 
-#ifndef NO_SHELL_ESCAPE
-	extern constext txt_help_shell[];
-#endif /* NO_SHELL_ESCAPE */
-
 #if !defined(USE_CURSES)
 	extern struct t_screen *screen;
 #endif /* !USE_CURSES */
@@ -1347,15 +1357,14 @@ extern struct opttxt txt_beginner_level;
 extern struct opttxt txt_cache_overview_files;
 extern struct opttxt txt_catchup_read_groups;
 extern struct opttxt txt_color_options;
-extern struct opttxt txt_confirm_action;
-extern struct opttxt txt_confirm_to_quit;
+extern struct opttxt txt_confirm_choice;
 extern struct opttxt txt_display_options;
 extern struct opttxt txt_draw_arrow;
 extern struct opttxt txt_editor_format;
 extern struct opttxt txt_expert_options;
 extern struct opttxt txt_filter_days;
+extern struct opttxt txt_filtering_options;
 extern struct opttxt txt_force_screen_redraw;
-extern struct opttxt txt_full_page_scroll;
 extern struct opttxt txt_getart_limit;
 extern struct opttxt txt_getart_limit_options;
 extern struct opttxt txt_group_catchup_on_exit;
@@ -1371,6 +1380,7 @@ extern struct opttxt txt_mail_8bit_header;
 extern struct opttxt txt_mail_address;
 extern struct opttxt txt_mail_mime_encoding;
 extern struct opttxt txt_mail_quote_format;
+extern struct opttxt txt_mailbox_format;
 extern struct opttxt txt_maildir;
 extern struct opttxt txt_mailer_format;
 extern struct opttxt txt_mark_saved_read;
@@ -1390,16 +1400,18 @@ extern struct opttxt txt_printer;
 extern struct opttxt txt_process_only_unread;
 extern struct opttxt txt_prompt_followupto;
 extern struct opttxt txt_quote_chars;
-extern struct opttxt txt_quote_empty_lines;
-extern struct opttxt txt_quote_signatures;
+extern struct opttxt txt_quote_style;
 extern struct opttxt txt_recent_time;
 extern struct opttxt txt_reread_active_file_secs;
-extern struct opttxt txt_save_to_mmdf_mailbox;
 extern struct opttxt txt_savedir;
 extern struct opttxt txt_saving_options;
+extern struct opttxt txt_score_limit_kill;
+extern struct opttxt txt_score_limit_select;
+extern struct opttxt txt_score_kill;
+extern struct opttxt txt_score_select;
+extern struct opttxt txt_scroll_lines;
 extern struct opttxt txt_show_author;
 extern struct opttxt txt_show_description;
-extern struct opttxt txt_show_last_line_prev_page;
 extern struct opttxt txt_show_lines;
 extern struct opttxt txt_show_only_unread_arts;
 extern struct opttxt txt_show_only_unread_groups;
@@ -1408,29 +1420,34 @@ extern struct opttxt txt_show_signatures;
 extern struct opttxt txt_sigdashes;
 extern struct opttxt txt_sigfile;
 extern struct opttxt txt_signature_repost;
+extern struct opttxt txt_slashes_regex;
 extern struct opttxt txt_sort_article_type;
 extern struct opttxt txt_sort_threads_type;
 extern struct opttxt txt_space_goto_next_unread;
 extern struct opttxt txt_spamtrap_warning_addresses;
+extern struct opttxt txt_stars_regex;
 extern struct opttxt txt_start_editor_offset;
 extern struct opttxt txt_strip_blanks;
 extern struct opttxt txt_strip_bogus;
 extern struct opttxt txt_strip_newsrc;
 extern struct opttxt txt_strip_re_regex;
 extern struct opttxt txt_strip_was_regex;
-extern struct opttxt txt_tab_after_X_selection;
+extern struct opttxt txt_strokes_regex;
 extern struct opttxt txt_tab_goto_next_unread;
 extern struct opttxt txt_tex2iso_conv;
 extern struct opttxt txt_thread_articles;
 extern struct opttxt txt_thread_catchup_on_exit;
+extern struct opttxt txt_thread_score;
+extern struct opttxt txt_underscores_regex;
 extern struct opttxt txt_unlink_article;
-extern struct opttxt txt_use_getart_limit;
+extern struct opttxt txt_url_handler;
 extern struct opttxt txt_use_mailreader_i;
 extern struct opttxt txt_use_mouse;
 extern struct opttxt txt_wildcard;
+extern struct opttxt txt_word_highlight;
+extern struct opttxt txt_wrap_column;
 extern struct opttxt txt_xpost_quote_format;
 #ifdef CHARSET_CONVERSION
-	extern struct opttxt txt_mm_local_charset;
 	extern struct opttxt txt_mm_network_charset;
 #endif /* CHARSET_CONVERSION */
 #ifdef HAVE_COLOR
@@ -1456,11 +1473,15 @@ extern struct opttxt txt_xpost_quote_format;
 	extern struct opttxt txt_col_from;
 	extern struct opttxt txt_col_title;
 	extern struct opttxt txt_col_signature;
-	extern struct opttxt txt_word_highlight;
 	extern struct opttxt txt_word_h_display_marks;
 	extern struct opttxt txt_col_markstar;
 	extern struct opttxt txt_col_markdash;
+	extern struct opttxt txt_col_markslash;
+	extern struct opttxt txt_col_markstroke;
 #endif /* HAVE_COLOR */
+#ifdef HAVE_ICONV_OPEN_TRANSLIT
+	extern struct opttxt txt_translit;
+#endif /* HAVE_ICONV_OPEN_TRANSLIT */
 #ifdef HAVE_KEYPAD
 	extern struct opttxt txt_use_keypad;
 #endif /* HAVE_KEYPAD */
