@@ -3,10 +3,10 @@
  *  Module    : keymap.h
  *  Author    : J. Faultless, D. Nimmich
  *  Created   : 1999
- *  Updated   : 2000-06-10
+ *  Updated   : 2002-01-07
  *  Notes     :
  *
- * Copyright (c) 1999-2002 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 1999-2003 Jason Faultless <jason@radar.tele2.co.uk>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -82,6 +82,7 @@ struct k_global {
 	t_keynode Eight;
 	t_keynode Nine;
 	t_keynode SearchSubjB;				/* '?' */
+	t_keynode SearchRepeat;				/* '\\' */
 	t_keynode SearchAuthB;				/* 'A' */
 	t_keynode SearchBody;				/* 'B' */
 	t_keynode ToggleHelpDisplay;		/* 'H' */
@@ -236,9 +237,7 @@ struct k_page {
 	t_keynode MarkThdUnread;			/* 'Z' */
 	t_keynode QuickAutoSel;				/* '[' */
 	t_keynode QuickKill;					/* ']' */
-#ifdef HAVE_COLOR
 	t_keynode ToggleHighlight;			/* '_' */
-#endif /* HAVE_COLOR */
 	t_keynode Catchup;					/* 'c' */
 	t_keynode EditArticle;				/* 'e' */
 	t_keynode FollowupQuote;			/* 'f' */
@@ -323,6 +322,7 @@ struct k_select {
 	t_keynode ReadGrp;					/* '\n' */
 	t_keynode ReadGrp2;					/* '\r' */
 	t_keynode ResetNewsrc;				/* ctrl('R') */
+	t_keynode SortActive;				/* '.' */
 	t_keynode CatchupNextUnread;		/* 'C' */
 	t_keynode NextUnreadGrp;			/* 'N' */
 	t_keynode BugReport;					/* 'R' */
