@@ -2,11 +2,11 @@
  *  Project   : tin - a Usenet reader
  *  Module    : rfc2046.h
  *  Author    : Jason Faultless <jason@radar.tele2.co.uk>
- *  Created   : 18/02/2000
- *  Updated   : 28/02/2000
+ *  Created   : 2000-02-18
+ *  Updated   : 2002-04-08
  *  Notes     : rfc2046 MIME article definitions
  *
- * Copyright (c) 2000-2002 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 2000-2003 Jason Faultless <jason@radar.tele2.co.uk>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Chris Blum.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -41,7 +38,7 @@
 #	define RFC2046_H 1
 
 /* The version of MIME we conform to */
-#	define MIME_SUPPORTED_VERSION	1.0
+#	define MIME_SUPPORTED_VERSION	"1.0"
 
 /* These must track the array definitions in lang.c */
 #	define NUM_CONTENT_TYPES	7
@@ -51,10 +48,11 @@
 #	define TYPE_MULTIPART		1
 #	define TYPE_APPLICATION		2
 
-
 #	define ENCODING_7BIT			0
 #	define ENCODING_QP			1
 #	define ENCODING_BASE64		2
+#	define ENCODING_8BIT			3
+#	define ENCODING_BINARY		4
 #	define ENCODING_UUE			5
 
 #	define DISP_INLINE			0
@@ -76,6 +74,7 @@ typedef struct param
 	struct param *next;
 } t_param;
 
+
 /*
  * Describes the properties of an article or article attachment
  * We re-use this to describe uuencoded sections
@@ -120,10 +119,11 @@ struct t_header
 	char *ftnto;			/* Old X-Comment-To: (Used by FIDO) */
 	char *authorids;		/* Author-IDs: (USEFOR, 2nd Son of 1036) Obsolete, USEFOR kicked it */
 	t_param *persist;		/* P-ersistent headers (USEFOR, 2nd Son of 1036) Obsolete, USEFOR kicked it */
-	t_bool mime:1;			/* Is Mime-Version: defined */
+	t_bool mime:1;			/* Is Mime-Version: defined - TODO: change to version number */
 	t_part *ext;			/* Extended Mime header information */
 };
 
+
 /* flags for lineinfo.flags */
 /* Primary colours */
 #	define C_HEADER	0x001
@@ -142,12 +142,14 @@ struct t_header
 #	define C_NEWS		0x400 /* Contains news|nntp: */
 #	define C_CTRLL		0x800	/* Contains ^L */
 
+
 typedef struct lineinfo
 {
 	long offset;			/* Offset of this line */
 	int flags;				/* Info about this line */
 } t_lineinfo;
 
+
 /*
  * Oddball collection of information about the open article
  */
