@@ -3,10 +3,10 @@
  *  Module    : tcurses.h
  *  Author    : Thomas Dickey
  *  Created   : 1997-03-02
- *  Updated   : 2001-11-10
+ *  Updated   : 2002-12-18
  *  Notes     : curses #include files, #defines & struct's
  *
- * Copyright (c) 1997-2002 Thomas Dickey <dickey@invisible-island.net>
+ * Copyright (c) 1997-2003 Thomas Dickey <dickey@invisible-island.net>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,7 +47,14 @@
 #		include <xcurses.h>
 #		define getattrs(w) (w)->_attrs
 #	elif defined(HAVE_NCURSESW_NCURSES_H)
+#		ifndef _XOPEN_SOURCE_EXTENDED
+#			define _XOPEN_SOURCE_EXTENDED 1
+#		endif /* _XOPEN_SOURCE_EXTENDED */
 #		include <ncursesw/ncurses.h>
+#		/* we need a recent ncursesw for wide-char */
+#		if (NCURSES_VERSION_MAJOR >= 5) && (NCURSES_VERSION_MINOR >= 3)
+#			define HAVE_NCURSESW 1
+#		endif /* NCURSES_VERSION_MAJOR >= 5 && NCURSES_VERSION_MINOR >=3 */
 #	elif defined(HAVE_NCURSES_H)
 #		include <ncurses.h>
 #	elif defined(HAVE_NCURSES_NCURSES_H)
@@ -80,25 +87,33 @@
 #define CleartoEOS()			clrtobot()
 #define ScrollScreen(lines_to_scroll)	scrl(lines_to_scroll)
 #define SetScrollRegion(top,bottom)	setscrreg(top, bottom)
-#define WriteLine(row,buffer)		write_line(row,buffer)
+#define WriteLine(row,buffer)		write_line(row, buffer)
 
 #define HpGlitch(func)			/*nothing*/
 
-extern int cmdReadCh (void);
+extern int cmdReadCh(void);
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	extern wint_t cmdReadWch(void);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 extern char *screen_contents(int row, int col, char *buffer);
-extern void MoveCursor(int row,int col);
+extern int my_innstr(char *str, int n);
+extern void MoveCursor(int row, int col);
 extern void my_erase(void);
 extern void my_fflush(FILE *stream);
 extern void my_fputc(int ch, FILE *stream);
 extern void my_fputs(const char *str, FILE *stream);
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	extern void my_fputwc(wint_t wc, FILE *fp);
+	extern wint_t ReadWch(void);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 extern void my_fprintf(FILE *stream, const char *fmt, ...)
-#ifdef __GNUC__
+#if defined(__GNUC__) && !defined(printf)
 	__attribute__((format(printf,2,3)))
 #endif /* __GNUC__ */
 	;
 extern void my_printf(const char *fmt, ...)
-#ifdef __GNUC__
+#if defined(__GNUC__) && !defined(printf)
 	__attribute__((format(printf,1,2)))
 #endif /* __GNUC__ */
 	;
@@ -110,17 +125,29 @@ extern void write_line(int row, char *buffer);
 
 #ifdef NEED_TERM_H
 #	include <curses.h>
-#	include <term.h>
+#	ifdef HAVE_NCURSES_TERM_H
+#		include <ncurses/term.h>
+#	else
+#		include <term.h>
+#	endif /* HAVE_NCURSES_TERM_H */
 #else
 #	ifdef HAVE_TERMCAP_H
-#	include <termcap.h>
+#		include <termcap.h>
 #	endif /* HAVE_TERMCAP_H */
 #endif /* NEED_TERM_H */
 
 #define cCRLF				"\r\n"
 
-#define my_fputc(ch, stream)		fputc (ch, stream)
-#define my_fputs(str, stream)		fputs (str, stream)
+#define my_fputc(ch, stream)		fputc(ch, stream)
+#define my_fputs(str, stream)		fputs(str, stream)
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+#	define my_fputwc(wc, stream)	{ \
+		if (fwide(stream, 0) <= 0) \
+			fprintf(stream, "%lc", wc); \
+		else \
+			fputwc(wc, stream); \
+	}
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 #define my_printf			printf
 #define my_fprintf			fprintf
