@@ -3,10 +3,10 @@
  *  Module    : tin.h
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2001-07-22
+ *  Updated   : 2003-01-16
  *  Notes     : #include files, #defines & struct's
  *
- * Copyright (c) 1997-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1997-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Iain Lea, Rich Skrenta.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -66,21 +63,30 @@
 #	define INET6
 #endif /* HAVE_GETADDRINFO && HAVE_GAI_STRERROR && ENABLE_IPV6 */
 
+/* Locale support in Mac OS X doesn't work yet, so turn it off */
+#ifdef MAC_OS_X
+#	ifndef NO_LOCALE
+#		define NO_LOCALE 1
+#	endif /* !NO_LOCALE */
+#endif /* MAC_OS_X */
+
 /*
  * Native Language Support.
  */
-#ifdef HAVE_LOCALE_H
-#	include <locale.h>
-#endif /* HAVE_LOCALE_H */
-#ifndef HAVE_SETLOCALE
-#	define setlocale(Category, Locale) /* empty */
-#endif /* !HAVE_SETLOCALE */
+#ifndef NO_LOCALE
+#	ifdef HAVE_LOCALE_H
+#		include <locale.h>
+#	endif /* HAVE_LOCALE_H */
+#	ifndef HAVE_SETLOCALE
+#		define setlocale(Category, Locale) /* empty */
+#	endif /* !HAVE_SETLOCALE */
+#endif /* !NO_LOCALE */
 
 #define N_(Str) Str
 
 #ifdef ENABLE_NLS
 #	include <libintl.h>
-#	define _(Text) gettext (Text)
+#	define _(Text)	gettext(Text)
 #else
 #	undef bindtextdomain
 #	define bindtextdomain(Domain, Directory) /* empty */
@@ -102,7 +108,7 @@
 #	if !defined(__GNUC__)
 #		undef M_UNIX
 #		define M_AMIGA
-#		define SIG_ARGS /*nothing, since compiler doesn't handle it*/
+#		define SIG_ARGS /* nothing, since compiler doesn't handle it */
 #		undef DECL_SIG_CONST
 #	endif /* !__GNUC__ */
 #endif /* __amiga__ || __amiga */
@@ -116,7 +122,7 @@ enum resizer { cNo, cYes, cRedraw };
 #	ifdef __DECC
 #		include <unixio.h>
 #	else
-#		ifndef __VMS_VER  /* assume old types.h */
+#		ifndef __VMS_VER	/* assume old types.h */
 			typedef unsigned short mode_t;
 #			undef HAVE_STRFTIME
 #		endif /* !__VMS_VER */
@@ -161,18 +167,22 @@ enum resizer { cNo, cYes, cRedraw };
 #		endif /* VAXC */
 #		include <types.h>
 #	endif /* !MULTINET */
-#	define FOPEN_OPTS       , "fop=cif"
-#	define JOINPATH			joindir
+#	define FOPEN_OPTS		, "fop=cif"
+#	define JOINPATH		joindir
 #else
 #	define FOPEN_OPTS
-#	define JOINPATH			joinpath
+#	define JOINPATH		joinpath
 #endif /* VMS */
 
 #include <stdio.h>
 #ifdef HAVE_ERRNO_H
 #	include	<errno.h>
 #else
-#	include	<sys/errno.h>
+#	ifdef HAVE_SYS_ERRNO_H
+#		include	<sys/errno.h>
+#	else
+#		error "No errno.h or sys/errno.h found"
+#	endif /* HAVE_SYS_ERRNO_H */
 #endif /* HAVE_ERRNO_H */
 #if !defined(errno)
 #	ifdef DECL_ERRNO
@@ -183,12 +193,16 @@ enum resizer { cNo, cYes, cRedraw };
 #ifdef HAVE_STDDEF_H
 #	include <stddef.h>
 #endif /* HAVE_STDDEF_H */
-#include <sys/types.h>
+#ifdef HAVE_SYS_TYPES_H
+#	include <sys/types.h>
+#endif /* HAVE_SYS_TYPES_H */
 
 #ifdef M_AMIGA
 #	include "include:stat.h"	/* FIXME: Problem with AmiTCP-includes, AmiTCP's fstat() needs */
 #else									/* a running TCP-Stack. OTOH fstat() ist used with local spool */
-#	include <sys/stat.h>
+#	ifdef HAVE_SYS_STAT_H
+#		include <sys/stat.h>
+#	endif /* HAVE_SYS_STAT_H */
 #endif /* M_AMIGA */
 
 #ifdef TIME_WITH_SYS_TIME
@@ -290,10 +304,6 @@ enum resizer { cNo, cYes, cRedraw };
 #	include <prototypes.h>
 #endif /* HAVE_PROTOTYPES_H */
 
-#if defined(HAVE_LOCALE_H) && !defined(NO_LOCALE)
-#	include <locale.h>
-#endif /* HAVE_LOCALE_H && !NO_LOCALE */
-
 #ifdef HAVE_SYS_UTSNAME_H
 #	include <sys/utsname.h>
 #endif /* HAVE_SYS_UTSNAME_H */
@@ -322,12 +332,16 @@ enum resizer { cNo, cYes, cRedraw };
 #	endif /* NEED_TIMEVAL_FIX */
 #endif /* HAVE_SYS_SELECT_H */
 
-#ifdef HAVE_STROPTS_H
+#if defined(HAVE_STROPTS_H) && !defined(__dietlibc__)
 #	include <stropts.h>
-#endif /* HAVE_STROPTS_H */
+#endif /* HAVE_STROPTS_H && !__dietlibc__ */
 
 #ifdef HAVE_POLL_H
 #	include <poll.h>
+#else
+#	ifdef HAVE_SYS_POLL_H
+#		include <sys/poll.h>
+#	endif /* HAVE_SYS_POLL_H */
 #endif /* HAVE_POLL_H */
 
 /*
@@ -371,7 +385,7 @@ enum resizer { cNo, cYes, cRedraw };
 #endif /* !HAVE_UNLINK */
 
 /*
- * If native OS has'nt defined STDIN_FILENO be a smartass and do it
+ * If native OS hasn't defined STDIN_FILENO be a smartass and do it
  */
 #if !defined(STDIN_FILENO)
 #	define STDIN_FILENO	0
@@ -397,9 +411,19 @@ enum resizer { cNo, cYes, cRedraw };
 
 /*
  * any pgp/gpp support possible and wanted
+ * sort out possible conflicts: gpg is prefered over pgp5 over pgp
  */
 #if defined(HAVE_PGP) || defined(HAVE_PGPK) || defined(HAVE_GPG)
 #	define HAVE_PGP_GPG 1
+#	if defined(HAVE_PGP) && defined(HAVE_PGPK)
+#		undef HAVE_PGP
+#	endif /* HAVE_PGP && HAVE_PGPK */
+#	if defined(HAVE_PGPK) && defined(HAVE_GPG)
+#		undef HAVE_PGPK
+#	endif /* HAVE_PGPK && HAVE_GPG */
+#	if defined(HAVE_PGP) && defined(HAVE_GPG)
+#		undef HAVE_PGP
+#	endif /* HAVE_PGP && HAVE_GPG */
 #endif /* HAVE_PGP || HAVE_PGPK || HAVE_GPG */
 
 
@@ -418,8 +442,8 @@ enum resizer { cNo, cYes, cRedraw };
 #define FAKE_NNTP_FP		(FILE *) 9999
 
 /*
- *  Max time between the first character of a VT terminal escape sequence
- *  for special keys and the following characters to arrive (msec)
+ * Max time between the first character of a VT terminal escape sequence
+ * for special keys and the following characters to arrive (msec)
  */
 #define SECOND_CHARACTER_DELAY	200
 
@@ -501,7 +525,7 @@ enum resizer { cNo, cYes, cRedraw };
 #	else
 #		define DEFAULT_PRINTER	"/usr/ucb/lpr"
 #		define DEFAULT_SUM	"sum"
-#	endif /* __386BSD__ || __bsdi__ || __NetBSD__ ||__FreeBSD__ || __OpenBSD__ */
+#	endif /* __386BSD__ || __bsdi__ || __NetBSD__ || __FreeBSD__ || __OpenBSD__ */
 #	ifdef DGUX
 #		define USE_INVERSE_HACK
 #	endif /* DGUX */
@@ -542,9 +566,6 @@ enum resizer { cNo, cYes, cRedraw };
 #		define DEFAULT_PRINTER	"/bin/lp"
 #		define READ_CHAR_HACK
 #	endif /* _AIX */
-#	ifdef SCO_UNIX
-#		define HAVE_MMDF_MAILER
-#	endif /* SCO_UNIX */
 #	ifdef sinix
 #		define DEFAULT_PRINTER	"/bin/lpr"
 #	endif /* sinix */
@@ -651,7 +672,8 @@ enum resizer { cNo, cYes, cRedraw };
 #define CONFIG_FILE	"tinrc"
 #define DEFAULT_MAILDIR	"Mail"
 #define DEFAULT_SAVEDIR	"News"
-#define DEFAULT_INEWS_PROG "--internal"
+#define DEFAULT_URL_HANDLER "url_handler.sh"
+
 
 /* MMDF-mailbox seperator */
 #ifndef MMDFHDRTXT
@@ -669,13 +691,30 @@ enum resizer { cNo, cYes, cRedraw };
 #	define DEFAULT_QUOTE_REGEX3	"^\\s{0,3}(?:(?:[\\]{}>|:)]|\\w{1,3}[>|])\\s*){3}"
 #endif /* HAVE_COLOR */
 
+/* case insensitive */
+#if 0 /* single words only */
+#	define DEFAULT_SLASHES_REGEX	"(?:^|(?<=\\s))/[^\\s/]+/(?:(?=[,.!?;]?\\s)|$)"
+#	define DEFAULT_STARS_REGEX	"(?:^|(?<=\\s))\\*[^\\s*]+\\*(?:(?=[,.!?;]?\\s)|$)"
+#	define DEFAULT_UNDERSCORES_REGEX	"(?:^|(?<=\\s))_[^\\s_]+_(?:(?=[,.!?;]?\\s)|$)"
+#	define DEFAULT_STROKES_REGEX	"(?:^|(?<=\\s))-[^-\\s]+-(?:(?=[,.!?;]?\\s)|$)"
+#else /* multiple words */
+#	define DEFAULT_SLASHES_REGEX	"(?:^|(?<=\\s))/(?(?=[^-*/_\\s][^/\\s])[^-*/_\\s][^/]*[^-*/_\\s]|[^/\\s])/(?:(?=[,.!?;]?\\s)|$)"
+#	define DEFAULT_STARS_REGEX	"(?:^|(?<=\\s))\\*(?(?=[^-*/_\\s][^*\\s])[^-*/_\\s][^*]*[^-*/_\\s]|[^*\\s])\\*(?:(?=[,.!?;]?\\s)|$)"
+#	define DEFAULT_UNDERSCORES_REGEX	"(?:^|(?<=\\s))_(?(?=[^-*/_\\s][^_\\s])[^-*/_\\s][^_]*[^-*/_\\s]|[^_\\s])_(?:(?=[,.!?;]?\\s)|$)"
+#	define DEFAULT_STROKES_REGEX	"(?:^|(?<=\\s))-(?(?=[^-*/_\\s][^-\\s])[^-*/_\\s][^-]*[^-*/_\\s]|[^-\\s])-(?:(?=[,.!?;]?\\s)|$)"
+#endif /* 0 */
+
 /* case sensitive && ^-anchored */
 #define DEFAULT_STRIP_RE_REGEX	"(?:R[eE](?:\\^\\d+|\\[\\d\\])?|A[wW]|Odp|Sv):\\s"
 /* case sensitive */
 #define DEFAULT_STRIP_WAS_REGEX	".\\((?:[Ww]a[rs]|[Bb]y[l]o):.*\\)\\s*$"
+/*
+ * overkill regexp for balanced '()':
+ * #define DEFAULT_STRIP_WAS_REGEX	".\\((?:[Ww]a[rs]|[Bb]y[l]o):(?:(?:[^)(])*(?:\\([^)(]*\\))*)+\\)\\s*$"
+ */
 
-/* case insensitive & ^-anchored */
-#define UUBEGIN_REGEX	"begin\\s+[0-7]{3,4}\\s+"
+/* case sensitive & ^-anchored */
+#define UUBEGIN_REGEX	"begin\\s\\s?[0-7]{3,4}\\s+"
 /* case sensitive & ^-anchored */
 #define UUBODY_REGEX	"(?:`|.[\\x20-\\x60]{1,61})$"
 
@@ -692,6 +731,8 @@ enum resizer { cNo, cYes, cRedraw };
 /*
  * case insensitive
  * split out ftp (only ftp allows username:passwd@, RFC 1738)?
+ *
+ * NOTE: the "-(?!-)" assertion must be removed when IDN is introduced
  */
 #if 0 /* this one is ok for IPv4 */
 #	define URL_REGEX	"\\b(?:https?|ftp|gopher)://(?:[^:@/]*(?::[^:@/]*)?@)?(?:(?:[^\\W_](?:(?:-(?!-)|[^\\W_]){0,61}[^\\W_])?\\.)+[a-z]{2,6}\\.?|localhost|(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?))(?::\\d+)?(?:/[^)\\>\"\\s]*|$|(?=[)\\>\"\\s]))"
@@ -704,22 +745,25 @@ enum resizer { cNo, cYes, cRedraw };
  */
 /* #define MAIL_REGEX	"\\b(?:mailto:(?:(?:[-\\w$.+!*'(),;/?:@&=]|(?:%[\\da-f]{2}))+))" */
 #define MAIL_REGEX	"\\b(?:mailto:(?:[-\\w$.+!*'(),;/?:@&=]|%[\\da-f]{2})+)"
-
 /*
  * case insensitive
  */
 #define NEWS_REGEX	"\\b(?:s?news|nntp):[^\\s@]+[@.][^\\s@]+(?:$|(?=[\\s.><,\"/():]))\\b"
+#if 0 /* not implemented */
 /*
- * case insensitive, no implemented
+ * case insensitive
  */
-#define TELNET_REGEX	"\\btelnet://(?:[^:@/]*(?::[^:@/]*)?@)?(?:(?:[^\\W_](?:(?:-(?!-)|[^\\W_]){0,61}[^\\W_])?\\.)+[a-z]{2,6}\\.?||localhost|(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?))(?::\\d+)?/?"
+#	define TELNET_REGEX	"\\btelnet://(?:[^:@/]*(?::[^:@/]*)?@)?(?:(?:[^\\W_](?:(?:-(?!-)|[^\\W_]){0,61}[^\\W_])?\\.)+[a-z]{2,6}\\.?||localhost|(?:(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?)\\.){3}(?:2[0-4]\\d|25[0-5]|[01]?\\d\\d?))(?::\\d+)?/?"
+#endif /* 0 */
 
 
 #define FILTER_FILE	"filter"
-#define GROUP_TIMES_FILE	"group.times"
+/* editor offset for filter-file; TODO: doesn't match for german filter-file */
+#define FILTER_FILE_OFFSET	32
 #define INPUT_HISTORY_FILE	".inputhistory"
-#define MAILGROUPS_FILE	"mailgroups"
-#define MSG_HEADERS_FILE	"headers"
+#ifdef HAVE_MH_MAIL_HANDLING
+#	define MAILGROUPS_FILE	"mailgroups"
+#endif /* HAVE_MH_MAIL_HANDLING */
 #define NEWSRC_FILE	".newsrc"
 #define NEWSRCTABLE_FILE	"newsrctable"
 /* ifdef APPEND_PID (default) NEWNEWSRC_FILE will be .newnewsrc<pid> */
@@ -733,9 +777,11 @@ enum resizer { cNo, cYes, cRedraw };
 #define POSTPONED_FILE	"postponed.articles"
 #define SUBSCRIPTIONS_FILE	"subscriptions"
 
-#define _CONF_FROMHOST	"fromhost"
-#define _CONF_ORGANIZATION	"organization"
-#define _CONF_SERVER	"server"
+#ifdef USE_INN_NNTPLIB
+#	define _CONF_FROMHOST	"fromhost"
+#	define _CONF_ORGANIZATION	"organization"
+#	define _CONF_SERVER	"server"
+#endif /* USE_INN_NNTPLIB */
 
 #define SIGDASHES "-- \n"
 
@@ -753,6 +799,41 @@ enum resizer { cNo, cYes, cRedraw };
 #	endif /* HAVE_ICONV_H */
 #endif /* HAVE_ICONV && !LOCAL_CHARSET */
 
+#ifdef HAVE_LANGINFO_H
+#	include <langinfo.h>
+#else
+#	ifdef HAVE_NL_TYPES_H
+#		include <nl_types.h>
+#	else
+		typedef int nl_item;
+#	endif /* HAVE_NL_TYPES_H */
+#endif /* HAVE_LANGINFO_H */
+#ifndef CODESET
+#	define CODESET ((nl_item) 1)
+#endif /* CODESET */
+
+#ifdef CHARSET_CONVERSION
+#	define IS_LOCAL_CHARSET(c)	(!strncasecmp(tinrc.mm_local_charset, c, strlen(c)))
+#else
+#	define IS_LOCAL_CHARSET(c)	(!strncasecmp(tinrc.mm_charset, c, strlen(c)))
+#endif /* CHARSET_CONVERSION */
+
+/* TODO: move up to the 'right' place */
+#ifdef HAVE_SYS_FILE_H
+#	include <sys/file.h>
+#endif /* HAVE_SYS_FILE_H */
+
+#ifdef HAVE_LIBUTF8_H
+#	include <libutf8.h>
+#else
+#	ifdef HAVE_WCHAR_H
+#		include <wchar.h>
+#	endif /* HAVE_WCHAR_H */
+#	ifdef HAVE_WCTYPE_H
+#		include <wctype.h>
+#	endif /* HAVE_WCTYPE_H */
+#endif /* HAVE_LIBUTF8_H */
+
 #ifndef MAX
 #	define MAX(a,b)	(((a) > (b)) ? (a) : (b))
 #endif /* !MAX */
@@ -765,16 +846,8 @@ enum resizer { cNo, cYes, cRedraw };
 #	define forever	for(;;)
 #endif /* !forever */
 
-#ifndef nop
-#	define nop	((void)0)
-#endif /* !nop */
-
-#ifndef nobreak
-#	define nobreak
-#endif /* !nobreak */
-
 /* safe strcpy into fixed-legth buffer */
-#define STRCPY(dst, src)	(dst[sizeof(dst) - 1] = '\0', strncpy(dst, src, sizeof(dst) -1))
+#define STRCPY(dst, src)	(dst[sizeof(dst) - 1] = '\0', strncpy(dst, src, sizeof(dst) - 1))
 
 #define STRCMPEQ(s1, s2)	(strcmp((s1), (s2)) == 0)
 #define STRNCMPEQ(s1, s2, n)	(strncmp((s1), (s2), n) == 0)
@@ -785,7 +858,7 @@ enum resizer { cNo, cYes, cRedraw };
 #	define PATH_LEN	256
 #endif /* VMS || M_AMIGA */
 
-#if defined(M_UNIX)
+#ifdef M_UNIX
 #	ifndef MAXPATHLEN
 #		define MAXPATHLEN	256
 #	endif /* !MAXPATHLEN */
@@ -799,14 +872,13 @@ enum resizer { cNo, cYes, cRedraw };
 
 #define MODULO_COUNT_NUM	50
 #define TABLE_SIZE	1409
-#define MAX_PAGES	2000	/* maximum article pages */
 
 #define DAY	(60*60*24)		/* Seconds in a day */
 
 #define ctrl(c)	((c) & 0x1F)
 
 #ifndef DEFAULT_ISO2ASC
-#	define DEFAULT_ISO2ASC	"-1 "	/* ISO -> Ascii charset conversion */
+#	define DEFAULT_ISO2ASC	"-1 "	/* ISO -> ASCII charset conversion */
 #endif /* !DEFAULT_ISO2ASC */
 
 #ifndef DEFAULT_COMMENT
@@ -859,22 +931,28 @@ enum resizer { cNo, cYes, cRedraw };
  * Return values for tin_errno
  */
 #define TIN_ABORT		1			/* User requested abort or timeout */
-#define TIN_TIMEOUT		2			/* Client side timeout */
+
+#define NUM_CONFIRM_CHOICES	8	/* confirm what? */
+#define TINRC_CONFIRM_ACTION	(tinrc.confirm_choice == 1 || tinrc.confirm_choice == 4 || tinrc.confirm_choice == 5 || tinrc.confirm_choice == 7)
+#define TINRC_CONFIRM_TO_QUIT	(tinrc.confirm_choice == 3 || tinrc.confirm_choice == 4 || tinrc.confirm_choice == 6 || tinrc.confirm_choice == 7)
+#define TINRC_CONFIRM_SELECT	(tinrc.confirm_choice == 2 || tinrc.confirm_choice == 5 || tinrc.confirm_choice == 6 || tinrc.confirm_choice == 7)
 
 /*
  * Number of MIME Encodings
  */
 #define NUM_MIME_ENCODINGS	4
 
-#define MIME_ENCODING_8BIT	0
+#define MIME_ENCODING_8BIT	0	/* not used */
 #define MIME_ENCODING_BASE64	1
 #define MIME_ENCODING_QP	2
 #define MIME_ENCODING_7BIT	3
 
 #ifdef CHARSET_CONVERSION			/* can/should do charset conversion via iconv() */
-#	define NUM_MIME_CHARSETS 27	/* # known 'outgoing' charsets */
+#	define NUM_MIME_CHARSETS 28	/* # known 'outgoing' charsets */
 #endif /* CHARSET_CONVERSION */
 
+#define NUM_MAILBOX_FORMATS 3		/* MBOX0, MBOXRD, MMDF */
+
 /*
  * Number of charset-traslation tables (iso2asci)
  */
@@ -888,16 +966,17 @@ enum resizer { cNo, cYes, cRedraw };
 
 /*
  * Maximal permissible word mark type
+ * 0 = nothing, 1 = mark, 2 = space
  */
-#define MAX_MARK		3
+#define MAX_MARK	2
 
 /* Line number (starting at 0) of 1st non-header data on the screen */
 /* ie, size of header */
 #define INDEX_TOP	2
 
-#define GROUP_MATCH(s1, pat, case)		(wildmat (s1, pat, case))
+#define GROUP_MATCH(s1, pat, case)		(wildmat(s1, pat, case))
 
-#define REGEX_MATCH(s1, pat, case)	(wildcard_func (s1, pat, case))
+#define REGEX_MATCH(s1, pat, case)	(wildcard_func(s1, pat, case))
 #define REGEX_FMT (tinrc.wildcard ? "%s" : "*%s*")
 
 #define IGNORE_ART(i)	((tinrc.kill_level != KILL_THREAD && arts[i].killed) || (arts[i].thread == ART_EXPIRED))
@@ -906,7 +985,7 @@ enum resizer { cNo, cYes, cRedraw };
  * Is this part text/plain ?
  */
 #define IS_PLAINTEXT(x) \
-			(x->type == TYPE_TEXT && strcmp("plain", x->subtype) == 0)
+			(x->type == TYPE_TEXT && strcasecmp("plain", x->subtype) == 0)
 
 /* TRUE if basenote has responses */
 #define HAS_FOLLOWUPS(i)	(arts[base[i]].thread != -1)
@@ -963,8 +1042,8 @@ enum resizer { cNo, cYes, cRedraw };
 
 /*
  * used in curses.c and signal.c
- *     it's useless trying to run tin below these sizes
- *     (values acquired by testing ;-) )
+ * it's useless trying to run tin below these sizes
+ * (values acquired by testing ;-) )
  */
 #define MIN_LINES_ON_TERMINAL		 8
 #define MIN_COLUMNS_ON_TERMINAL		50
@@ -1014,6 +1093,13 @@ enum resizer { cNo, cYes, cRedraw };
 #define SHOW_FROM_NAME		2
 #define SHOW_FROM_BOTH		3
 
+/*
+ * Values for thread_score
+ */
+#define THREAD_SCORE_MAX	0
+#define THREAD_SCORE_SUM	1
+#define THREAD_SCORE_WEIGHT	2
+
 /*
  * used in feed.c & save.c
  */
@@ -1034,6 +1120,8 @@ enum resizer { cNo, cYes, cRedraw };
 #define SORT_ARTICLES_BY_DATE_ASCEND	6
 #define SORT_ARTICLES_BY_SCORE_DESCEND	7
 #define SORT_ARTICLES_BY_SCORE_ASCEND	8
+#define SORT_ARTICLES_BY_LINES_DESCEND	9
+#define SORT_ARTICLES_BY_LINES_ASCEND	10
 
 /*
  * used in art.c
@@ -1052,9 +1140,9 @@ enum resizer { cNo, cYes, cRedraw };
 /*
  * Different values of strip_bogus - the ways to handle bogus groups
  */
-#define BOGUS_KEEP		0
+#define BOGUS_KEEP		0	/* not used */
 #define BOGUS_REMOVE		1
-#define BOGUS_ASK		2
+#define BOGUS_SHOW		2
 
 /*
  * Different extents to which we can hide killed articles
@@ -1064,10 +1152,11 @@ enum resizer { cNo, cYes, cRedraw };
 #define KILL_NOTHREAD	2		/* Kill all articles, never show them */
 
 /*
- * used in help.c
+ * Various types of quoting behaviour
  */
-#define HELP_INFO		0
-#define POST_INFO		1
+#define QUOTE_COMPRESS	1		/* Compress quotes */
+#define QUOTE_SIGS		2		/* Quote signatures */
+#define QUOTE_EMPTY		4		/* Quote empty lines */
 
 
 /*
@@ -1174,14 +1263,6 @@ enum resizer { cNo, cYes, cRedraw };
 
 #define SCORE_MAX		10000
 
-#define SCORE_DEFAULT		100
-#define SCORE_KILL		-(SCORE_DEFAULT)
-#define SCORE_SELECT		SCORE_DEFAULT
-
-/* TODO: the next two should be configurable at runtime */
-#define SCORE_LIM_KILL		-50
-#define SCORE_LIM_SEL		50
-
 #define FILTER_SUBJ_CASE_SENSITIVE		0
 #define FILTER_SUBJ_CASE_IGNORE		1
 #define FILTER_FROM_CASE_SENSITIVE		2
@@ -1266,9 +1347,9 @@ enum resizer { cNo, cYes, cRedraw };
  * construct a mask for the byte containing the low bit, AND or OR it in;
  * use memset to fill in the intervening bytes efficiently; then construct
  * a mask for the byte containing the high bit, and AND or OR this mask
- * in.	Masks are constructed by left-shift of 0xff (to set high-order bits
+ * in. Masks are constructed by left-shift of 0xff (to set high-order bits
  * to 1), negating a left-shift of 0xfe (to set low-order bits to 1), and
- * the various negations and combinations of the same.	This procedure is
+ * the various negations and combinations of the same. This procedure is
  * complex, but 1 to 2 orders of magnitude faster than a shift inside a
  * loop for each bit inside a loop for each individual byte.
  *
@@ -1306,8 +1387,7 @@ typedef unsigned char	t_bitmap;
 /*
  *	struct t_msgid - message id
  */
-struct t_msgid
-{
+struct t_msgid {
 	struct t_msgid *next;		/* Next in hash chain */
 	struct t_msgid *parent;		/* Message-id followed up to */
 	struct t_msgid *sibling;	/* Next followup to parent */
@@ -1328,10 +1408,8 @@ struct t_msgid
  * article.inthread:
  *	FALSE for the first article in a thread, TRUE for all
  *	following articles in thread
- *
  */
-struct t_article
-{
+struct t_article {
 	long artnum;			/* Article number in spool directory for group */
 	char *subject;			/* Subject: line from mail header */
 	char *from;			/* From: line from mail header (address) */
@@ -1350,20 +1428,19 @@ struct t_article
 	int tagged;			/* 0 = not tagged, >0 = tagged */
 	int thread;
 	int score;			/* score article has reached after filtering */
-	unsigned int inthread:1;	/* 0 = thread head, 1 = thread follower */
-	unsigned int status:2;		/* 0 = read, 1 = unread, 2 = will return */
-	unsigned int killed:1;		/* 0 = not killed, 1 = killed */
-	unsigned int selected:1;	/* 0 = not selected, 1 = selected */
-	unsigned int zombie:1;		/* 1 = was alive (unread) before 'X' command */
+	unsigned int status:2;	/* 0 = read, 1 = unread, 2 = will return */
+	unsigned int killed:1;	/* 0 = not killed, 1 = killed */
+	unsigned int zombie:1;	/* 1 = was alive (unread) before 'X' command */
 	unsigned int delete_it:1;	/* 1 = delete art when leaving group [mail group] */
-	unsigned int inrange:1;		/* 1 = article selected via # range command */
+	unsigned int inthread:1;	/* 0 = thread head, 1 = thread follower */
+	t_bool selected:1;	/* FALSE = not selected, TRUE = selected */
+	t_bool inrange:1;	/* TRUE = article selected via # range command */
 };
 
 /*
  * struct t_attribute - configurable attributes on a per group basis
  */
-struct t_attribute
-{
+struct t_attribute {
 	char *maildir;				/* mail dir if other than ~/Mail */
 	char *savedir;				/* save dir if other than ~/News */
 	char *savefile;			/* save articles to specified file */
@@ -1403,43 +1480,51 @@ struct t_attribute
 	unsigned int post_proc_type:2;		/* 0=none, 1=shar, 2=uudecode */
 	unsigned int x_comment_to:1;		/* insert X-Comment-To: in Followup */
 	unsigned int tex2iso_conv:1;		/* Convert TeX2ISO */
+#ifdef CHARSET_CONVERSION
+	int mm_network_charset;				/* network charset */
+	char *undeclared_charset;			/* charset of articles without MIME charset declaration */
+#endif /* CHARSET_CONVERSION */
 };
 
+/*
+ * TODO: turn longs to unsigned long long or at least unsigned long
+ */
 /*
  * struct t_newsrc - newsrc related info.
  */
-struct t_newsrc
-{
-	t_bool present;		/* update newsrc ? */
+struct t_newsrc {
+	t_bool present:1;		/* update newsrc ? */
 	long num_unread;		/* unread articles in group */
-	long xmax;				/* newsrc max */
-	long xmin;				/* newsrc min */
+	long xmax;			/* newsrc max */
+	long xmin;			/* newsrc min */
 	long xbitlen;			/* bitmap length (max-min+1) */
 	t_bitmap *xbitmap;	/* bitmap read/unread (max-min+1+7)/8 */
 };
 
+/*
+ * TODO: turn longs to unsigned long long or at least unsigned long
+ */
 /*
  * struct t_group - newsgroup info from active file
  */
-struct t_group
-{
-	char *name;				/* newsgroup / mailbox name */
+struct t_group {
+	char *name;			/* newsgroup / mailbox name */
 	char *aliasedto;		/* =new.group in active file, NULL if not */
 	char *description;	/* text from NEWSLIBDIR/newsgroups file */
 	char *spooldir;		/* groups spool directory */
 	char moderated;		/* state of group moderation */
-	long count;				/* article number count */
-	long xmax;				/* max. article number */
-	long xmin;				/* min. article number */
-	unsigned int type:4;			/* grouptype - newsgroup / mailbox / savebox */
-	t_bool inrange:1;			/* TRUE if group selected via # range command */
-	t_bool read_during_session:1;		/* TRUE if group entered during session */
-	t_bool art_was_posted:1;		/* TRUE if art was posted to group */
-	t_bool subscribed:1;			/* TRUE if subscribed to group */
-	t_bool newgroup:1;				/* TRUE if group was new this session */
-	t_bool bogus:1;					/* TRUE if group is not in active list */
-	int next;				/* next active entry in hash chain */
-	struct t_newsrc newsrc;				/* newsrc bitmap specific info. */
+	long count;			/* article number count */
+	long xmax;			/* max. article number */
+	long xmin;			/* min. article number */
+	unsigned int type:4;		/* grouptype - newsgroup / mailbox / savebox */
+	t_bool inrange:1;		/* TRUE if group selected via # range command */
+	t_bool read_during_session:1;	/* TRUE if group entered during session */
+	t_bool art_was_posted:1;	/* TRUE if art was posted to group */
+	t_bool subscribed:1;		/* TRUE if subscribed to group */
+	t_bool newgroup:1;		/* TRUE if group was new this session */
+	t_bool bogus:1;			/* TRUE if group is not in active list */
+	int next;			/* next active entry in hash chain */
+	struct t_newsrc newsrc;		/* newsrc bitmap specific info. */
 	struct t_attribute *attribute;	/* group specific attributes */
 	struct t_filters *glob_filter;	/* points to filter array */
 };
@@ -1447,43 +1532,49 @@ struct t_group
 /*
  * used in hashstr.c
  */
-struct t_hashnode
-{
-	struct t_hashnode *next;		/* chain for spillover */
-	int aptr;				/* used in subject threading */
+struct t_hashnode {
+	struct t_hashnode *next;	/* chain for spillover */
+	int aptr;			/* used in subject threading */
 	char txt[1];			/* stub for the string data, \0 terminated */
 };
 
 /*
  * used in filter.c
  *
- *  Create 2 filter arrays - global & local. Local will be part of group_t
- *  structure and will have priority over global filter. Should help to
- *  speed kill/selecting within a group. The long value number that is in
- *  ~/.tin/kill will be replaced by group name so that it is more human
- *  readable and that if hash routine is changed it will still work.
+ * Create 2 filter arrays - global & local. Local will be part of group_t
+ * structure and will have priority over global filter. Should help to
+ * speed kill/selecting within a group. The long value number that is in
+ * ~/.tin/kill will be replaced by group name so that it is more human
+ * readable and that if hash routine is changed it will still work.
  *
- *  Add time period to filter_t struct to allow timed kills & auto-selection
- *  Default kill & select time 28 days. Store as a long and compare when
- *  loading against present time. If time secs is passed set flag to save
- *  filter file and don't load expired entry. Renamed to filter because of
- *  future directions in adding other retrieval methods to present kill &
- *  auto selection.
+ * Add time period to filter_t struct to allow timed kills & auto-selection
+ * Default kill & select time 28 days. Store as a long and compare when
+ * loading against present time. If time secs is passed set flag to save
+ * filter file and don't load expired entry. Renamed to filter because of
+ * future directions in adding other retrieval methods to present kill &
+ * auto selection.
  *
- *  Also seperate kill/select screen to allow ^K=kill ^A=auto-select
+ * Also seperate kill/select screen to allow ^K=kill ^A=auto-select
  */
-struct t_filters
-{
+struct t_filters {
 	int max;
 	int num;
 	struct t_filter *filter;
 };
 
+/*
+ * struct t_filter_comment: allow multiple comment-lines in filter-file.
+ */
+struct t_filter_comment {
+	char *text;			/* One line of comment. */
+	struct t_filter_comment *next;	/* points to next comment-entry */
+};
+
 /*
  * struct t_filter - local & global filtering (ie. kill & auto-selection)
  */
-struct t_filter
-{
+struct t_filter {
+	struct t_filter_comment *comment;
 	char *scope;			/* NULL='*' (all groups) or 'comp.os.*' */
 	char *subj;			/* Subject: line */
 	char *from;			/* From: line */
@@ -1501,7 +1592,6 @@ struct t_filter
 	time_t time;			/* expire time in seconds */
 	struct t_filter *next;		/* next rule valid in group */
 	unsigned int inscope:4;		/* if group matches scope ie. 'comp.os.*' */
-	unsigned int type:2;		/* kill/auto select */
 	unsigned int icase:2;		/* Case sensitive filtering */
 	unsigned int fullref:4;		/* use full references or last entry only */
 };
@@ -1509,8 +1599,8 @@ struct t_filter
 /*
  * struct t_filter_rule - provides parameters to build filter rule from
  */
-struct t_filter_rule
-{
+struct t_filter_rule {
+	struct t_filter_comment *comment;
 	char text[PATH_LEN];
 	char scope[PATH_LEN];
 	int counter;
@@ -1518,14 +1608,13 @@ struct t_filter_rule
 	int fullref;
 	int lines_cmp;
 	int lines_num;
+	int score;
+	int expire_time;
 	t_bool from_ok:1;
 	t_bool lines_ok:1;
 	t_bool msgid_ok:1;
 	t_bool subj_ok:1;
 	t_bool check_string:1;
-	int type;
-	int score;
-	int expire_time;
 };
 
 /*
@@ -1537,32 +1626,29 @@ struct regex_cache {
 	pcre_extra *extra;
 };
 
-struct t_save
-{
+struct t_save {
 	char *path;
 	char *file;					/* => file part of *path */
 	struct t_article *artptr;	/* => article in arts[] */
-	t_bool saved;				/* Set if saved okay */
-	t_bool is_mailbox;			/* Set if path is a mailbox */
+	t_bool saved:1;				/* Set if saved okay */
+	t_bool is_mailbox:1;			/* Set if path is a mailbox */
 };
 
 #ifndef USE_CURSES
-struct t_screen
-{
+struct t_screen {
 	char *col;
 };
 #endif /* !USE_CURSES */
 
-struct t_posted
-{
+struct t_posted {
 	char date[10];
 	char group[80];
 	char action;
 	char subj[120];
 };
 
-struct t_art_stat
-{
+struct t_art_stat {
+	char art_mark;		/* mark to use for this thread - not used for groups */
 	int total;		/* total article count */
 	int unread;		/* number of unread articles (does not include seen) arts */
 	int seen;		/* number of seen articles (ART_WILL_RETURN) */
@@ -1571,20 +1657,18 @@ struct t_art_stat
 	int selected_total;	/* total selected count */
 	int selected_unread;	/* selected and unread */
 	int selected_seen;	/* selected and seen */
-	char art_mark;		/* mark to use for this thread - not used for groups */
 	int score;		/* maximum score */
-	time_t time;		/* latest time */
 	int multipart_total; /* 0=not multipart, >0 = number of articles in the multipart */
 	int multipart_have; /* number of articles we actually have found */
 	int multipart_compare_len; /* length of subject which contains non-specific multipart info */
+	time_t time;		/* latest time */
 };
 
 
 /*
  * Used for detecting changes in active file size on different news servers
  */
-struct t_newnews
-{
+struct t_newnews {
 	char *host;
 	time_t time;
 };
@@ -1646,15 +1730,14 @@ typedef struct {
  */
 typedef struct {
 	int art;
-	t_bool ignore_unavail;
+	t_bool ignore_unavail:1;
 } t_pagerinfo;
 
 
 /*
  * Time functions.
  */
-typedef struct _TIMEINFO
-{
+typedef struct _TIMEINFO {
 	time_t time;
 	long usec;
 	long tzone;
@@ -1678,8 +1761,8 @@ typedef struct {
 	char *test;
 	char *x11bitmap;
 	int textualnewlines;
-	t_bool needsterminal;
-	t_bool copiousoutput;
+	t_bool needsterminal:1;
+	t_bool copiousoutput:1;
 } t_mailcap;
 
 
@@ -1699,15 +1782,19 @@ typedef struct {
 #	else
 		typedef void *t_comptype;
 #	endif /* __STDC__ */
+#else
+#	ifdef HAVE_COMPTYPE_CHAR
+		typedef char *t_comptype;
+#	endif /* HAVE_COMPTYPE_CHAR */
 #endif /* HAVE_COMPTYPE_VOID */
 
-#ifdef HAVE_COMPTYPE_CHAR
-	typedef char *t_comptype;
-#endif /* HAVE_COMPTYPE_CHAR */
+#define _CDECL
 
-#	define _CDECL
-#	define _FAR_
+#ifdef VMS
+#	define SEPDIR ']'
+#else
 #	define SEPDIR	'/'
+#endif /* VMS */
 
 /*
  * mouse buttons for use in xterm
@@ -1731,7 +1818,6 @@ typedef struct {
 #	define METAMAIL_CMD		"%s -e -p -m \"tin\""
 #	define TMPDIR			"T:"
 #	define KEY_PREFIX		0x9b
-extern void joinpath (char *result, const char *dir, const char *file);
 #endif /* M_AMIGA */
 #ifdef VMS
 #	define REDIRECT_OUTPUT	""
@@ -1745,7 +1831,6 @@ extern void joinpath (char *result, const char *dir, const char *file);
 #	ifdef	HAVE_KEY_PREFIX
 #		define KEY_PREFIX	0x9b
 #	endif /* HAVE_KEY_PREFIX */
-extern void joinpath (char *result, const char *dir, const char *file);
 extern void joindir (char *result, const char *dir, const char *file);
 #endif /* VMS */
 
@@ -1757,7 +1842,7 @@ extern void joindir (char *result, const char *dir, const char *file);
 #	define TIN_EDITOR_FMT_ON		"%E +%N %F"
 #	define MAILER_FORMAT		"%M -oi -t < %F"
 #	define METAMAIL_CMD		"%s -e -p -m \"tin\""
-#	define TMPDIR	"/tmp/"
+#	define TMPDIR	_PATH_TMP
 #	ifdef	HAVE_KEY_PREFIX
 #		define KEY_PREFIX		0x8f: case 0x9b
 #	endif /* HAVE_KEY_PREFIX */
@@ -1778,7 +1863,7 @@ extern void joindir (char *result, const char *dir, const char *file);
 #	define ENV_VAR_SHELL		""
 #endif /* !ENV_VAR_SHELL */
 #ifndef TIN_EDITOR_FMT_ON
-#	define TIN_EDITOR_FMT_ON		""
+#	define TIN_EDITOR_FMT_ON		TIN_EDITOR_FMT_OFF
 #endif /* !TIN_EDITOR_FMT_ON */
 #ifndef MAILER_FORMAT
 #	define MAILER_FORMAT		""
@@ -1787,7 +1872,7 @@ extern void joindir (char *result, const char *dir, const char *file);
 #	define METAMAIL_CMD		""
 #endif /* !METAMAIL_CMD */
 #ifndef TMPDIR
-#	define TMPDIR		""
+#	define TMPDIR	_PATH_TMP
 #endif /* !TMPDIR */
 
 #if !defined(S_ISDIR)
@@ -1859,26 +1944,26 @@ extern void joindir (char *result, const char *dir, const char *file);
 /* Various function redefinitions */
 #ifdef USE_DBMALLOC
 #	define my_malloc(size)	malloc(size)
+#	define my_calloc(nmemb, size)	calloc((nmemb), (size))
 #	define my_realloc(ptr, size)	realloc((ptr), (size))
 #else
 #	define my_malloc(size)	my_malloc1(__FILE__, __LINE__, (size))
+#	define my_calloc(nmemb, size)	my_calloc1(__FILE__, __LINE__, (nmemb), (size))
 #	define my_realloc(ptr, size)	my_realloc1(__FILE__, __LINE__, (ptr), (size))
 #endif /* USE_DBMALLOC */
 
-#define SIZEOF(array)	((int)(sizeof array / sizeof array[0]))
+#define ARRAY_SIZE(array)	((int) (sizeof(array) / sizeof(array[0])))
 
-#if 0
-#	define FreeIfNeeded(p)	if (p != (char *)0) free((char *)p)
-#	define FreeAndNull(p)	if (p != (char *)0) { free((char *)p); p = (char *)0; }
-#else
-#	define FreeIfNeeded(p)	if (p != NULL) free((void *)p)
-#	define FreeAndNull(p)	if (p != NULL) { free((void *)p); p = NULL; }
-#endif /* 0 */
+#define FreeIfNeeded(p)	if (p != NULL) free((void *) p)
+#define FreeAndNull(p)	if (p != NULL) { free((void *) p); p = NULL; }
 
 #define BlankIfNull(p)	((p) ? (p) : "")
 
 #define my_group_find(x)	add_my_group(x, FALSE)
-#define my_group_add(x)	add_my_group(x, TRUE)
+#define my_group_add(x)		add_my_group(x, TRUE)
+#define for_each_group(x)	for (x = 0; x < num_active; x++)
+#define for_each_art(x)		for (x = 0; x < top_art; x++)
+#define for_each_art_in_thread(x, y)	for (x = (int) base[y]; x >= 0; x = arts[x].thread)
 
 /*
  * Cast for the (few!) places where we need to examine 8-bit characters w/o
@@ -1890,7 +1975,6 @@ extern void joindir (char *result, const char *dir, const char *file);
 /*
  * function prototypes & extern definitions
  */
-/* #include	"filebug.h" */
 
 #ifndef SIG_ARGS
 #	if defined(__STDC__)
@@ -1929,13 +2013,13 @@ extern void joindir (char *result, const char *dir, const char *file);
 #	define OUTC_FUNCTION(func)	OUTC_RETTYPE func (c) int c;
 #endif /* __STDC__ || __cplusplus */
 
-typedef	OUTC_RETTYPE (*OutcPtr) (OUTC_ARGS);
+typedef OUTC_RETTYPE (*OutcPtr) (OUTC_ARGS);
 
 #ifdef M_AMIGA
-	typedef	struct __tcpbuf TCP;
+	typedef struct __tcpbuf	TCP;
 #	include "amigatcp.h"
 #else
-	typedef	FILE	TCP;
+	typedef FILE TCP;
 #endif /* M_AMIGA */
 
 #ifndef EXTERN_H
@@ -1964,8 +2048,8 @@ typedef void (*BodyPtr) (char *, FILE *, int);
 #	undef strrchr
 #	undef NSET1
 #	undef NSET0
-#	define NSET1(n,b) memset(n+NOFFSET(b), n[NOFFSET(b)] | NTEST(n,b), 1)
-#	define NSET0(n,b) memset(n+NOFFSET(b), n[NOFFSET(b)] & ~NTEST(n,b), 1)
+#	define NSET1(n,b) memset(n + NOFFSET(b), n[NOFFSET(b)] | NTEST(n,b), 1)
+#	define NSET0(n,b) memset(n + NOFFSET(b), n[NOFFSET(b)] & ~NTEST(n,b), 1)
 #	include <dbmalloc.h> /* dbmalloc 1.4 */
 #endif /* USE_DBMALLOC */
 
@@ -1975,23 +2059,23 @@ typedef void (*BodyPtr) (char *, FILE *, int);
 #endif /* USE_DMALLOC */
 
 #ifdef DOALLOC
-	extern char *doalloc (char *, size_t);
-	extern char *docalloc (size_t, size_t);
-	extern void	dofree (char *);
+	extern char *doalloc(char *, size_t);
+	extern char *docalloc(size_t, size_t);
+	extern void	dofree(char *);
 #	undef malloc
 #	undef realloc
 #	undef calloc
 #	undef free
-#	define malloc(n)		doalloc((char *)0, n)
+#	define malloc(n)	doalloc((char *) 0, n)
 #	define realloc		doalloc
 #	define calloc		docalloc
 #	define free		dofree
-	extern void	fail_alloc ( char *, char * );
-	extern void	Trace ( char *, ... );
-	extern void	Elapsed ( char * );
-	extern void	WalkBack ( void );
-	extern void	show_alloc ( void );
-	extern void	no_leaks ( void );
+	extern void	fail_alloc(char *, char *);
+	extern void	Trace(char *, ...);
+	extern void	Elapsed(char *);
+	extern void	WalkBack(void);
+	extern void	show_alloc(void);
+	extern void	no_leaks(void);
 #endif /* DOALLOC */
 
 #ifdef __DECC		/* VMS */
@@ -2015,7 +2099,7 @@ typedef void (*BodyPtr) (char *, FILE *, int);
  * argument can't be a pointer and if argument is changed on return
  * we must unlik the tmp-file ourself
  */
-#	define my_tmpfile_only(a)  my_tmpfile(a, sizeof(a)-1, FALSE, (char *) 0)
+#	define my_tmpfile_only(a)	my_tmpfile(a, sizeof(a) - 1, FALSE, (char *) 0)
 #endif /* !my_tmpfile_only */
 
 
@@ -2038,7 +2122,7 @@ typedef void (*BodyPtr) (char *, FILE *, int);
 
 /*
  * We force this include-ordering since socks.h contains redefinitions of
- * functions that probably are prototyped via other includes.  The socks.h
+ * functions that probably are prototyped via other includes. The socks.h
  * definitions have to be included everywhere, since they're making wrappers
  * for the stdio functions as well as the network functions.
  */
@@ -2074,8 +2158,10 @@ extern struct tm *localtime(time_t *);
 #	include <mss.h>
 #	undef my_malloc
 #	undef my_realloc
+#	undef my_calloc
 #	define my_malloc(size)	malloc(size)
 #	define my_realloc(ptr, size)	realloc((ptr), (size))
+#	define my_calloc(nmemb, size) calloc((nmemb), (size))
 #endif /* MSS */
 
 /* libcanlock */
@@ -2085,9 +2171,9 @@ extern struct tm *localtime(time_t *);
 
 /* FXIME: use autoconf here to detect if stat(2)-structure has st_mtime */
 #ifndef M_AMIGA
-#	define FILE_CHANGED(file)  file_mtime(file)
+#	define FILE_CHANGED(file)	file_mtime(file)
 #else
-#	define FILE_CHANGED(file)  file_size(file)
+#	define FILE_CHANGED(file)	file_size(file)
 #endif /* M_AMIGA */
 
 /* snprintf(), vsnprintf() */
@@ -2098,27 +2184,18 @@ extern struct tm *localtime(time_t *);
 #	define vsnprintf	plp_vsnprintf
 #endif /* HAVE_VSNPRINTF */
 
-/* GCC-specific attributes */
-#if defined(__GNUC__) && !defined(__APPLE_CC__)
+/* gcc-specific attributes */
+#if defined(__GNUC__) && !defined(__cplusplus) && !defined(__APPLE_CC__)
 #	define UNUSED(x) x __attribute__((unused))
 #else
 #	define UNUSED(x) x
-#endif /* __GNUC__ && !__APPLE_CC__ */
-
-/* can we use mblen()? */
-#if defined(HAVE_MBLEN) && defined(HAVE_SETLOCALE) && defined(MB_CUR_MAX) && !defined (NO_LOCALE)
-#	define ENABLE_MBLEN 1
-#endif /* HAVE_MBLEN && HAVE_SETLOCALE && MB_CUR_MAX && !NO_LOCALE */
+#endif /* __GNUC__ && !__cplusplus && !__APPLE_CC__ */
 
 /* init_selfinfo() needs MM_CHARSET */
 #ifndef MM_CHARSET
 #	define MM_CHARSET "US-ASCII"
 #endif /* !MM_CHARSET */
 
-/* TODO: move up to the 'right' place */
-#ifdef HAVE_SYS_FILE_H
-#	include <sys/file.h>
-#endif /* HAVE_SYS_FILE_H */
 
 #if !defined(SEEK_SET)
 #	define SEEK_SET 0L
@@ -2139,10 +2216,4 @@ extern struct tm *localtime(time_t *);
 #	define TIN_LETTER_NAME	"letter."
 #endif /* !VMS */
 
-/* THREAD_WEIGHT without THREAD_SUM doesn't make sense */
-#if defined(THREAD_WEIGHT) && !defined(THREAD_SUM)
-#	define THREAD_SUM
-#endif /* THREAD_WEIGHT && !THREAD_SUM */
-
-
 #endif /* !TIN_H */
