@@ -3,10 +3,10 @@
  *  Module    : version.h
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 1999-11-13
+ *  Updated   : 2002-04-26
  *  Notes     :
  *
- * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Iain Lea.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -43,11 +40,11 @@
 
 #	define PRODUCT		"tin"
 #	ifndef TIN_AUTOCONF_H
-#		define VERSION		"1.5.11"
+#		define VERSION		"1.5.16"
 #	endif /* !TIN_AUTOCONF_H */
-#	define RELEASEDATE	"20020130"
-#	define RELEASENAME	"Toxicity"
-#	define TINRC_VERSION	"1.2"
+#	define RELEASEDATE	"20030125"
+#	define RELEASENAME	"Bubbles"
+#	define TINRC_VERSION	"1.3.1"
 
 #	ifdef M_AMIGA
 #		define OSNAME	"AMIGA"
