@@ -7,6 +7,6 @@
 # that many releases anyway.
 
 PCRE_MAJOR=3
-PCRE_MINOR=4
-PCRE_DATE=22-Aug-2000
+PCRE_MINOR=9
+PCRE_DATE=02-Jan-2002
 PCRE_VERSION=${PCRE_MAJOR}.${PCRE_MINOR}
