@@ -3,10 +3,10 @@
  *  Module    : cook.c
  *  Author    : J. Faultless
  *  Created   : 2000-03-08
- *  Updated   :
+ *  Updated   : 2003-01-16
  *  Notes     : Split from page.c
  *
- * Copyright (c) 2000-2002 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 2000-2003 Jason Faultless <jason@radar.tele2.co.uk>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Jason Faultless
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -55,18 +52,18 @@
 #define STRIP_ALTERNATIVE(x) \
 			(tinrc.alternative_handling && \
 			(x)->hdr.ext->type == TYPE_MULTIPART && \
-			strcmp("alternative", (x)->hdr.ext->subtype) == 0)
+			strcasecmp("alternative", (x)->hdr.ext->subtype) == 0)
 
-#define MATCH_REGEX(x,y,z)	(pcre_exec (x.re, x.extra, y, z, 0, 0, NULL, 0) >= 0)
+#define MATCH_REGEX(x,y,z)	(pcre_exec(x.re, x.extra, y, z, 0, 0, NULL, 0) >= 0)
 
-static void put_cooked (t_bool wrap_lines, int flags, const char *fmt, ...);
-static void set_rest (char **rest, const char *ptr);
-static int put_rest (char **rest, char *dest, int max_line_len);
-static int read_decoded_base64_line (FILE *file, char *line, const int max_line_len, const int max_lines_to_read, const char *charset, char **rest);
-static int read_decoded_qp_line (FILE *file, char *line, const int max_line_len, const int max_lines_to_read, const char *charset, char **rest);
-static t_part *new_uue (t_part **part, char *name);
-static void process_text_body_part (t_bool wrap_lines, FILE *in, t_part *part);
-static t_bool header_wanted (const char *line);
+static int put_rest(char **rest, char *dest, int max_line_len);
+static int read_decoded_base64_line(FILE *file, char *line, const int max_line_len, const int max_lines_to_read, char **rest);
+static int read_decoded_qp_line(FILE *file, char *line, const int max_line_len, const int max_lines_to_read, char **rest);
+static t_bool header_wanted(const char *line);
+static t_part *new_uue(t_part **part, char *name);
+static void process_text_body_part(t_bool wrap_lines, FILE *in, t_part *part);
+static void put_cooked(t_bool wrap_lines, int flags, const char *fmt, ...);
+static void set_rest(char **rest, const char *ptr);
 
 /*
  * These are used globally within this module for access to the context
@@ -76,44 +73,55 @@ static int cook_width;
 static t_bool hide_uue;
 static t_openartinfo *art;
 
+
 /*
  * Rewrite frombuf into tobuf to a maximum length
  * Handle backspace, expand tabs, expand control chars to a literal ^[A-Z]
  * Allows \n through
  * Return TRUE if line contains a ^L (form-feed)
+ * Anything after the (lenght-2)th character in the output string is
+ * dropped, if the resultinglast character is not \n, \n is added
+ * afterwards.
  */
 t_bool
-expand_ctrl_chars (
+expand_ctrl_chars(
 	char *to,
 	const char *from,
 	int length,
 	size_t lcook_width)
 {
-	const char *p;
+	const char *p, *end;
 	char *q;
+	int i, j, space;
 	t_bool ctrl_L = FALSE;
 
-	for (p = from, q = to; *p && q < &to[length]; p++) {
+	for (p = from, q = to, end = p, space = length - 2; space && *p; end = p, p++) {
 		if (*p == '\t') {			/* Expand tabs */
-			int i, j;
-
 			i = q - to;
 			j = ((i + lcook_width) / lcook_width) * lcook_width;
-/*			j = (i|(tabwidth - 1)) + 1; TODO more efficient ? */
+/*			j = (i | (tabwidth - 1)) + 1; TODO more efficient? */
 
-			while (i++ < j)
+			while (space && i++ < j) {
 				*q++ = ' ';
-
-		} else if (((*p) & 0xFF) < ' ' && *p != '\n') {	/* Literal ctrl chars */
+				space--;
+			}
+		} else if (((*p) & 0xFF) < ' ' && *p != '\n' && (!IS_LOCAL_CHARSET("Big5") || *p != 27)) {	/* Literal ctrl chars */
 			*q++ = '^';
-			*q++ = ((*p) & 0xFF) + '@';
+			if (--space) {
+				*q++ = ((*p) & 0xFF) + '@';
+				space--;
+			}
 			if (*p == '\f')					/* ^L detected */
 				ctrl_L = TRUE;
-		} else
+		} else {
 			*q++ = *p;
+			space--;
+		}
 	}
-	*q = '\0';
+	if (*end != '\n')
+		*q++ = '\n';	/* Force last char of string to be \n */
 
+	*q = '\0';
 	return ctrl_L;
 }
 
@@ -123,10 +131,9 @@ expand_ctrl_chars (
  * Update the line count and the array of line offsets
  * Extend the lineoffset array as needed in CHUNK amounts.
  * flags are 'hints' to the pager about line content
- * TODO do word wrap of long lines ?
  */
 static void
-put_cooked (
+put_cooked(
 	t_bool wrap_lines,
 	int flags,
 	const char *fmt,
@@ -134,49 +141,82 @@ put_cooked (
 {
 	char *p, *bufp;
 	char buf[LEN];
+	int wrap_column;
+	int space;
 	static int overflow = 0;
 	static int saved_flags = 0;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	int bytes;
+	wint_t *wp;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 	va_list ap;
 
 	va_start(ap, fmt);
-	vsnprintf (buf, sizeof(buf) - 1, fmt, ap);
+	vsnprintf(buf, sizeof(buf) - 1, fmt, ap);
 
-	bufp = buf;
+	if (tinrc.wrap_column < 0)
+		wrap_column = ((tinrc.wrap_column > -cCOLS) ? cCOLS + tinrc.wrap_column : cCOLS);
+	else
+		wrap_column = ((tinrc.wrap_column > 0) ? tinrc.wrap_column : cCOLS);
 
-	for (p = bufp; *p; p++) {
-		if (*p == '\n' || ((overflow + p - bufp >= cCOLS) && wrap_lines)) {
-			fwrite(bufp, p - bufp, 1, art->cooked);
+	p = bufp = buf;
 
-			fputs("\n", art->cooked);
-			bufp = p;
-			overflow = 0;
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	wp = my_malloc ((MB_CUR_MAX + 1) * sizeof(wint_t));
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
-			/* Skip over newline if that's what split the current buffer */
-			if (*p == '\n')
-				++bufp;
-
-			if (art->cooked_lines == 0) {
-				art->cookl = my_malloc(sizeof(t_lineinfo)*CHUNK);
-				art->cookl[0].offset = 0;
+	while (*p) {
+		if (wrap_lines) {
+			space = wrap_column;
+			while (space > 0 && *p && *p != '\n') {
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+				if ((bytes = mbtowc((wchar_t *) wp, p, MB_CUR_MAX)) > 0) {
+					if ((space -= wcwidth(*wp)) < 0)
+						break;
+					p += bytes;
+				} else
+					p++;
+#else
+				p++;
+				space--;
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 			}
-
-			/*
-			 * Pick up flags from a previous partial write
-			 */
-			art->cookl[art->cooked_lines].flags = flags | saved_flags;
-			saved_flags = 0;
-			art->cooked_lines++;
-
-			/*
-			 * Grow the array of lines if needed - we resize it properly at the end
-			 */
-			if (art->cooked_lines % CHUNK == 0)
-				art->cookl = my_realloc ((char *)art->cookl, sizeof(t_lineinfo) * CHUNK * ((art->cooked_lines / CHUNK) + 1));
-
-			art->cookl[art->cooked_lines].offset = ftell(art->cooked);
+		} else {
+			while (*p && *p != '\n')
+				p++;
 		}
+		fwrite(bufp, 1, p - bufp, art->cooked);
+		fputs("\n", art->cooked);
+		if (*p == '\n')
+			p++;
+		bufp = p;
+		overflow = 0;
+
+		if (art->cooked_lines == 0) {
+			art->cookl = my_malloc(sizeof(t_lineinfo) * CHUNK);
+			art->cookl[0].offset = 0;
+		}
+
+		/*
+		 * Pick up flags from a previous partial write
+		 */
+		art->cookl[art->cooked_lines].flags = flags | saved_flags;
+		saved_flags = 0;
+		art->cooked_lines++;
+
+		/*
+		 * Grow the array of lines if needed - we resize it properly at the end
+		 */
+		if (art->cooked_lines % CHUNK == 0)
+			art->cookl = my_realloc(art->cookl, sizeof(t_lineinfo) * CHUNK * ((art->cooked_lines / CHUNK) + 1));
+
+		art->cookl[art->cooked_lines].offset = ftell(art->cooked);
 	}
 
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+	free(wp);
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
+
 	/*
 	 * If there is anything left over, then it must be a non \n terminated
 	 * partial line from base64 decoding etc.. Dump it now and the rest of
@@ -190,7 +230,7 @@ put_cooked (
 		overflow += strlen(bufp);
 	}
 
-	va_end (ap);
+	va_end(ap);
 }
 
 
@@ -201,23 +241,23 @@ put_cooked (
  * later.
  */
 static void
-set_rest (
+set_rest(
 	char **rest,
 	const char *ptr)
 {
 	char *new_rest;
 	size_t new_rest_len;
 
-	new_rest_len = strlen (ptr);
+	new_rest_len = strlen(ptr);
 	if (new_rest_len) {
-		new_rest = my_malloc (new_rest_len + 1);
-		if (new_rest)
-			strcpy (new_rest, ptr);
+		new_rest = my_malloc(new_rest_len + 1);
+		strcpy(new_rest, ptr);
 		*rest = new_rest;
 	} else
 		*rest = NULL;	/* no rest anymore */
 }
 
+
 /*
  * FIXME: should go into rfc1521.c
  *
@@ -233,7 +273,7 @@ set_rest (
  * there was no newline in the rest.
  */
 static int
-put_rest (
+put_rest(
 	char **rest,
 	char *dest,
 	int max_line_len)
@@ -244,7 +284,7 @@ put_rest (
 
 	ptr = my_rest;
 	if (my_rest) {
-		if (strlen (my_rest) > 0) {
+		if (strlen(my_rest)) {
 			char c;
 
 			while ((c = *ptr++) && (c != '\n') && (put_chars < (max_line_len - 2))) {
@@ -271,16 +311,16 @@ put_rest (
 					 */
 					ptr--;
 				}
-				set_rest (rest, ptr);
-				FreeIfNeeded (my_rest);
+				set_rest(rest, ptr);
+				FreeIfNeeded(my_rest);
 			} else {
 				/* rest is now empty */
 				*rest = NULL;
-				FreeIfNeeded (my_rest);
+				FreeIfNeeded(my_rest);
 			}
 		} else {
 			/* empty rest, clean up */
-			free (my_rest);
+			free(my_rest);
 			*rest = NULL;
 		}
 	}
@@ -301,16 +341,16 @@ put_rest (
  * value on error.
  */
 static int
-read_decoded_base64_line (
+read_decoded_base64_line(
 	FILE *file,
 	char *line,
 	const int max_line_len,
 	const int max_lines_to_read,
-	const char *charset,
 	char **rest)
 {
 	char *dest = line;
 	char *ptr;
+	char c;
 	char buf2[HEADER_LEN];	/* holds the entire decoded line */
 	char buf[HEADER_LEN];	/* holds the entire encoded line; allowed are 76 char + CRLF (cf. RFC 2045, section 6.8), so "this should be enough for everyone" */
 	int count = 0;
@@ -354,8 +394,6 @@ read_decoded_base64_line (
 	 * Ok, now read a new line from the original article.
 	 */
 	do {
-		char c;
-
 		if (fgets(buf, sizeof(buf), file) == NULL) {
 			/*
 			 * Premature end of file (or file error), leave loop. To prevent
@@ -372,7 +410,7 @@ read_decoded_base64_line (
 			return max_lines_to_read;
 		}
 		lines_read++;
-		count = mmdecode (buf, 'b', '\0', buf2, charset);
+		count = mmdecode(buf, 'b', '\0', buf2);
 		buf2[count] = '\0';
 		ptr = &buf2[0];
 		while ((c = *ptr++) && (c != '\n') && (put_chars < (max_line_len - 2))) {
@@ -388,11 +426,11 @@ read_decoded_base64_line (
 			*dest = '\0';
 			if (c != '\n')
 				ptr--;
-			set_rest (rest, ptr);
+			set_rest(rest, ptr);
 			return lines_read;
 		}
 	} while ((lines_read < max_lines_to_read) && (put_chars < max_line_len));
-	set_rest (rest, ptr);	/* should not be needed */
+	set_rest(rest, ptr);	/* should not be needed */
 	return lines_read;
 }
 
@@ -401,7 +439,7 @@ read_decoded_base64_line (
  * FIXME: should go into rfc1521.c
  *
  * Read a logical quoted-printable encoded line into the specified line
- * buffer.  Quoted-printable lines can be split over several physical lines,
+ * buffer. Quoted-printable lines can be split over several physical lines,
  * so this function collects all affected lines, concatenates and decodes
  * them.
  *
@@ -409,12 +447,11 @@ read_decoded_base64_line (
  * value on error.
  */
 static int
-read_decoded_qp_line (
+read_decoded_qp_line(
 	FILE *file,
 	char *line,							/* where to copy the decoded line */
-	const int max_line_len,			/* maximum line length  */
+	const int max_line_len,			/* maximum line length */
 	const int max_lines_to_read,	/* don't read more physical lines than told here */
-	const char *charset,				/* local charset */
 	char **rest)						/* rest of line if line is too small */
 {
 	char *buf, *buf2, *endptr;
@@ -432,15 +469,14 @@ read_decoded_qp_line (
 	 * If there was anything left over from the last invokation, put it out
 	 * now.
 	 */
-	put_chars = put_rest (rest, dest, max_line_len);
+	put_chars = put_rest(rest, dest, max_line_len);
 	if (put_chars && (line[put_chars - 1] == '\n'))
 		return 0;	/* we didn't read any new lines but filled the line */
 
 	dest = &line[put_chars];
 	if ((max_lines_to_read < 1) || (max_line_len < 2))
 		return max_lines_to_read;
-	if (!(buf = my_malloc (max_buf_len)))
-		return -1;	/* oops, no memory */
+	buf = my_malloc((size_t) max_buf_len);
 	*buf = '\0';
 	endptr = buf;
 	do {
@@ -472,13 +508,14 @@ read_decoded_qp_line (
 			 */
 			break;
 		endptr = &buf[buflen - 1];
+		buf[buflen] = '\0';
 		c = *endptr;
 		/*
 		 * Strip trailing white space at the end of the line, but no more than
 		 * the number of characters that were newly read.
 		 * See RFC 2045, section 6.7, #3
 		 */
-		while ((chars_added > 0) && ((c == ' ') || (c == '\t') || (c == '\n'))) {
+		while ((chars_added > 0) && ((c == ' ') || (c == '\t') || (c == '\n') || (c == '\r'))) {
 			c = *--endptr;
 			buflen--;
 			chars_added--;
@@ -507,24 +544,21 @@ read_decoded_qp_line (
 	 * buffer where the invoking function expects it. Don't decode directly
 	 * to the buffer of the other function to prevent buffer overruns.
 	 */
-	buf2 = my_malloc (strlen(buf) + 1);
-	if (buf2)
-		count = mmdecode (buf, 'q', '\0', buf2, charset);
-	else
-		count = -1;
+	buf2 = my_malloc(strlen(buf) + 1);
+	count = mmdecode(buf, 'q', '\0', buf2);
 
 	if (count >= 0) {
 		buf2[count] = '\0';
 		ptr = buf2;
-	} else	/* error in encoding or no memory, copy raw line */
+	} else	/* error in encoding: copy raw line */
 		ptr = buf;
 
 	strncpy(dest, ptr, max_line_len - put_chars - 1);
 	line[max_line_len - 1] = '\0'; /* be sure to terminate string */
-	if ((signed int)(strlen(ptr) + put_chars) > max_line_len - 1)
-		set_rest (rest, &ptr[max_line_len - put_chars - 1]);
-	FreeIfNeeded (buf);
-	FreeIfNeeded (buf2);
+	if ((signed int) (strlen(ptr) + put_chars) > max_line_len - 1)
+		set_rest(rest, &ptr[max_line_len - put_chars - 1]);
+	free(buf);
+	free(buf2);
 	return lines_read;
 }
 
@@ -558,7 +592,7 @@ new_uue(
 	 * If an extension is present, try and add a Content-Type
 	 */
 	if ((name = strrchr(name, '.')) != NULL)
-		lookup_mimetype (name + 1, ptr);
+		lookup_mimetype(name + 1, ptr);
 
 	return ptr;
 }
@@ -578,10 +612,10 @@ get_filename(
 
 	if (!(name = get_param(ptr, "filename"))) {
 		if (!(name = get_param(ptr, "name")))
-		return NULL;
+			return NULL;
 	}
 
-	/* Use basename() ? */
+	/* TODO: Use base_name()? */
 	if (((p = strrchr(name, '/'))) || ((p = strrchr(name, '\\'))))
 		return p + 1;
 
@@ -598,54 +632,46 @@ process_text_body_part(
 	FILE *in,
 	t_part *part)
 {
-	const char *charset;
-	char *line;
 	char *rest = NULL;
-	char buf[LEN], to[LEN];
+	char *line, *buf;
+	char to[LEN];
+	int max_line_len = LEN;
 	int flags, len, lines_left;
-	t_bool decode = TRUE;
 	t_bool in_sig = FALSE;			/* Set when in sig portion */
 	t_bool in_uue = FALSE;			/* Set when in uuencoded section */
 	t_part *curruue = NULL;
 
+	line = buf = my_malloc(max_line_len);
 	fseek(in, part->offset, SEEK_SET);
 
-	if ((charset = get_param(part->params, "charset")) == NULL)
-		decode = FALSE;				/* Impossible in practice, since it defaults */
-
 	if (part->encoding == ENCODING_BASE64)
-		(void) mmdecode(NULL, 'b', 0, NULL, NULL);		/* flush */
+		(void) mmdecode(NULL, 'b', 0, NULL);		/* flush */
 
 	lines_left = part->line_count;
 	while ((lines_left > 0) || rest) {
 		switch (part->encoding) {
 			case ENCODING_BASE64:
-				lines_left -= read_decoded_base64_line (in, buf, sizeof(buf), lines_left, charset, &rest);
+				lines_left -= read_decoded_base64_line(in, buf, max_line_len, lines_left, &rest);
 				line = buf;
 				break;
 
 			case ENCODING_QP:
-				lines_left -= read_decoded_qp_line (in, buf, sizeof(buf), lines_left, charset, &rest);
+				lines_left -= read_decoded_qp_line(in, buf, max_line_len, lines_left, &rest);
 				line = buf;
 				break;
 
 			default:
-				line = fgets (buf, (int) sizeof (buf), in);
+				line = fgets(buf, max_line_len, in);
 				lines_left--;
 				break;
 		}
 		if (!(line && strlen(line)))
 			break;	/* premature end of file, file error etc. */
 
-#if defined(LOCAL_CHARSET) || defined(MAC_OS_X)
-		if (decode)
-			buffer_to_local (line);
-#else
-#	ifdef CHARSET_CONVERSION
-		if (decode)
-			 buffer_to_local (line, charset, tinrc.mm_local_charset);
-#	endif /* CHARSET_CONVERSION */
-#endif /* LOCAL_CHARSET || MAC_OS_X */
+		/* convert network to local charset, tex2iso, iso2asc etc. */
+		process_charsets(&line, &max_line_len, get_param(part->params, "charset"), tinrc.mm_local_charset, CURR_GROUP.attribute->tex2iso_conv && art->tex2iso);
+		/* we might have realloced line, so make sure we use this new buffer */
+		buf = line;
 
 		/*
 		 * Detect and skip signatures if necessary
@@ -655,32 +681,32 @@ process_text_body_part(
 		len = (int) strlen(line);
 
 		if (!in_sig) {
-			if (strcmp (line, SIGDASHES) == 0) {
+			if (strcmp(line, SIGDASHES) == 0) {
 				in_sig = TRUE;
 				if (in_uue) {
 					in_uue = FALSE;
-					put_cooked (wrap_lines, C_UUE, txt_uue, "incomplete ", curruue->line_count, get_filename(curruue->params));
+					put_cooked(wrap_lines, C_UUE, txt_uue, "incomplete ", curruue->line_count, get_filename(curruue->params));
 				}
 			}
 		}
 
 		if (hide_uue) {
 			int offsets[6];
-			int size_offsets = (int) (sizeof(offsets)/sizeof(int));
+			int size_offsets = ARRAY_SIZE(offsets);
 			t_bool is_uubody = FALSE;		/* Set if this line looks like a uuencoded line */
 
 			/*
 			 * Look for the start or the end of a uuencoded section
 			 */
-			if (pcre_exec (uubegin_regex.re, uubegin_regex.extra, line, len, 0, 0, offsets, size_offsets) != PCRE_ERROR_NOMATCH) {
+			if (pcre_exec(uubegin_regex.re, uubegin_regex.extra, line, len, 0, 0, offsets, size_offsets) != PCRE_ERROR_NOMATCH) {
 				in_uue = TRUE;
 				curruue = new_uue(&part, line + offsets[1]);
 				continue;
-			} else if (strncmp (line, "end\n", 4) == 0) {
+			} else if (strncmp(line, "end\n", 4) == 0) {
 				if (in_uue) {
 					in_uue = FALSE;
-					/* TODO include type/subtype/encoding in txt_uue as in txt_attach ? */
-					put_cooked (wrap_lines, C_UUE, txt_uue, "", curruue->line_count, get_filename(curruue->params));
+					/* TODO include type/subtype/encoding in txt_uue as in txt_attach? */
+					put_cooked(wrap_lines, C_UUE, txt_uue, "", curruue->line_count, get_filename(curruue->params));
 					continue;				/* To stop 'end' line appearing */
 				}
 			}
@@ -689,7 +715,7 @@ process_text_body_part(
 			 * See if this line looks like a uuencoded line
 			 */
 			if (MATCH_REGEX (uubody_regex, line, len)) {
-				int sum = (((*line) - ' ') & 077) *4/3;		/* uuencode octet checksum */
+				int sum = (((*line) - ' ') & 077) * 4 / 3;		/* uuencode octet checksum */
 
 				/*
 				 * sum = 0 in a uubody only on the last line, a single `
@@ -713,7 +739,7 @@ process_text_body_part(
 						fprintf(stderr, "not a uue line while reading a uue body?\n");
 #endif /* DEBUG_ART */
 						in_uue = FALSE;
-						put_cooked (wrap_lines, C_UUE, txt_uue, "incomplete ", curruue->line_count, get_filename(curruue->params));
+						put_cooked(wrap_lines, C_UUE, txt_uue, "incomplete ", curruue->line_count, get_filename(curruue->params));
 					}
 				}
 			} else {
@@ -732,60 +758,50 @@ process_text_body_part(
 		if (in_uue || (in_sig && !tinrc.show_signatures))
 			continue;
 
-		flags = (in_sig) ? C_SIG : C_BODY;
+		flags = in_sig ? C_SIG : C_BODY;
 
 #ifdef HAVE_COLOR
 		if (quote_regex3.re) {
-			if (MATCH_REGEX (quote_regex3, line, len))
+			if (MATCH_REGEX(quote_regex3, line, len))
 				flags |= C_QUOTE3;
 			else if (quote_regex2.re) {
-				if (MATCH_REGEX (quote_regex2, line, len))
+				if (MATCH_REGEX(quote_regex2, line, len))
 					flags |= C_QUOTE2;
 				else if (quote_regex.re) {
-					if (MATCH_REGEX (quote_regex, line, len))
+					if (MATCH_REGEX(quote_regex, line, len))
 						flags |= C_QUOTE1;
 				}
 			}
 		}
 #endif /* HAVE_COLOR */
 
-		if (MATCH_REGEX (url_regex, line, len))
+		if (MATCH_REGEX(url_regex, line, len))
 			flags |= C_URL;
-		if (MATCH_REGEX (mail_regex, line, len))
+		if (MATCH_REGEX(mail_regex, line, len))
 			flags |= C_MAIL;
-		if (MATCH_REGEX (news_regex, line, len))
+		if (MATCH_REGEX(news_regex, line, len))
 			flags |= C_NEWS;
 
-		if ((CURR_GROUP.attribute->tex2iso_conv) && art->tex2iso) {
-			char texbuf[LEN];
-			strcpy (texbuf, line);
-			convert_tex2iso (texbuf, line);
-		}
-
-		if (iso2asc_supported >= 0) {
-			char isobuf[LEN];
-			strcpy (isobuf, line);
-			convert_iso2asc (isobuf, line, iso2asc_supported);
-		}
-
-#if 1
-/* Basically: if (!(my_isprint(*c) || *c==8 || *c==9 || *c==12)) */
-/* How about if !isprint() && !isctrl() - expand_ctrl_chars is done at display time */
-		convert_body2printable (line);
-#endif /* 1 */
-/* TODO integrate above into expand_ctrl_chars */
-
-		if (expand_ctrl_chars (to, line, sizeof(to), cook_width))
+		/*
+		 * Basically, c_b2p() does: if (!(my_isprint(*c) || *c==8 || *c==9 || *c==12))
+		 * It is only used here
+		 * How about if !isprint() && !isctrl() - expand_ctrl_chars is done at display time.
+		 * TODO: integrate into expand_ctrl_chars
+		 */
+		convert_body2printable(line);
+		if (expand_ctrl_chars(to, line, sizeof(to) - 1, cook_width))
 			flags |= C_CTRLL;				/* Line contains form-feed */
 
-		put_cooked (wrap_lines, flags, "%s", to);
+		put_cooked(wrap_lines && (!IS_LOCAL_CHARSET("Big5")), flags, "%s", to);
 	} /* while */
 
 	/*
 	 * Were we reading uue and ran off the end ?
 	 */
 	if (in_uue)
-		put_cooked (wrap_lines, C_UUE, txt_uue, "incomplete ", curruue->line_count, get_filename(curruue->params));
+		put_cooked(wrap_lines, C_UUE, txt_uue, "incomplete ", curruue->line_count, get_filename(curruue->params));
+
+	free(line);
 }
 
 
@@ -800,22 +816,22 @@ header_wanted(
 	int i;
 	t_bool ret = FALSE;
 
-	if (num_headers_to_display && (news_headers_to_display_array[0][0] == '*')) {
+	if (num_headers_to_display && (news_headers_to_display_array[0][0] == '*'))
 		ret = TRUE; /* wild do */
-	} else {
+	else {
 		for (i = 0; i < num_headers_to_display; i++) {
-			if (!strncasecmp (line, news_headers_to_display_array[i], strlen (news_headers_to_display_array[i]))) {
+			if (!strncasecmp(line, news_headers_to_display_array[i], strlen(news_headers_to_display_array[i]))) {
 				ret = TRUE;
 				break;
 			}
 		}
 	}
 
-	if (num_headers_to_not_display && (news_headers_to_not_display_array[0][0] == '*')) {
+	if (num_headers_to_not_display && (news_headers_to_not_display_array[0][0] == '*'))
 		ret = FALSE; /* wild don't: doesn't make sense! */
-	} else {
+	else {
 		for (i = 0; i < num_headers_to_not_display; i++) {
-			if (!strncasecmp (line, news_headers_to_not_display_array[i], strlen (news_headers_to_not_display_array[i]))) {
+			if (!strncasecmp(line, news_headers_to_not_display_array[i], strlen(news_headers_to_not_display_array[i]))) {
 				ret = FALSE;
 				break;
 			}
@@ -833,10 +849,9 @@ dump_cooked(
 	void)
 {
 	int i;
+	char *line;
 
 	for (i = 0; i < art->cooked_lines; i++) {
-		char *line;
-
 		fseek(art->cooked, art->cookl[i].offset, SEEK_SET);
 		line = tin_fgets(art->cooked, FALSE);
 		fprintf(stderr, "[%3d] %4ld %3x [%s]\n", i, art->cookl[i].offset, art->cookl[i].flags, line);
@@ -874,8 +889,8 @@ cook_article(
 	t_bool uue)
 {
 	char *line;
-	int header_put = FALSE;
 	struct t_header *hdr = &artinfo->hdr;
+	t_bool header_put = FALSE;
 
 	art = artinfo;				/* Global saves lots of passing artinfo around */
 	cook_width = tabs;
@@ -891,22 +906,22 @@ cook_article(
 	/*
 	 * Put down just the headers we want
 	 */
-	while ((line = tin_fgets (artinfo->raw, TRUE)) != (char *) 0) {
-		if (line[0] == '\0') {				/* End of headers ? */
+	while ((line = tin_fgets(artinfo->raw, TRUE)) != NULL) {
+		if (line[0] == '\0') {				/* End of headers? */
 			if (STRIP_ALTERNATIVE(artinfo)) {
 				if (header_wanted(_(txt_info_x_conversion_note))) {
 					header_put = TRUE;
-					put_cooked (wrap_lines, C_HEADER, _(txt_info_x_conversion_note));
+					put_cooked(wrap_lines, C_HEADER, _(txt_info_x_conversion_note));
 				}
 			}
 			if (header_put)
-				put_cooked (TRUE, 0, "\n");		/* put a newline after headers */
+				put_cooked(TRUE, 0, "\n");		/* put a newline after headers */
 			break;
 		}
 
 		if (header_wanted(line)) {	/* Put cooked data */
 			header_put = TRUE;
-			put_cooked (wrap_lines, C_HEADER, "%s\n", rfc1522_decode(line));
+			put_cooked(wrap_lines, C_HEADER, "%s\n", rfc1522_decode(line));
 		}
 	}
 
@@ -919,9 +934,10 @@ cook_article(
 	 */
 	if (hdr->mime && hdr->ext->type == TYPE_MULTIPART) {
 		t_part *ptr;
+		const char *name;
 
 		for (ptr = hdr->ext->next; ptr != NULL; ptr = ptr->next) {
-			const char *name = get_filename(ptr->params);
+			name = get_filename(ptr->params);
 
 			/*
 			 * Ignore non text/plain sections with alternative handling
@@ -929,7 +945,7 @@ cook_article(
 			if (STRIP_ALTERNATIVE(artinfo) && !IS_PLAINTEXT(ptr))
 				continue;
 
-			put_cooked (wrap_lines, C_ATTACH, _(txt_attach),
+			put_cooked(wrap_lines, C_ATTACH, _(txt_attach),
 				ptr->depth * 4, "",
 				content_types[ptr->type], ptr->subtype,
 				content_encodings[ptr->encoding], ptr->line_count,
@@ -948,11 +964,11 @@ cook_article(
 		else {						/* Non-textual main body */
 			const char *name = get_filename(hdr->ext->params);
 
-			put_cooked (wrap_lines, C_ATTACH, _(txt_attach),
+			put_cooked(wrap_lines, C_ATTACH, _(txt_attach),
 					0, "",
 					content_types[hdr->ext->type], hdr->ext->subtype,
 					content_encodings[hdr->ext->encoding], hdr->ext->line_count,
-					(name)? _(", name: ") : "", (name) ? name : "");	/* FIXME: -> lang.c */
+					(name) ? _(", name: ") : "", (name) ? name : "");	/* FIXME: -> lang.c */
 		}
 	}
 
@@ -961,9 +977,8 @@ cook_article(
 #endif /* DEBUG_ART */
 
 	if (art->cooked_lines > 0)
-		art->cookl = my_realloc ((char *)art->cookl, sizeof(t_lineinfo) * art->cooked_lines);
+		art->cookl = my_realloc(art->cookl, sizeof(t_lineinfo) * art->cooked_lines);
 
 	rewind(art->cooked);
-
 	return TRUE;
 }
