@@ -3,7 +3,7 @@
  *  Module    : curses.c
  *  Author    : D. Taylor & I. Lea
  *  Created   : 1986-01-01
- *  Updated   : 2001-07-22
+ *  Updated   : 2002-12-18
  *  Notes     : This is a screen management library borrowed with permission
  *              from the Elm mail system. This library was hacked to provide
  *              what tin needs.
@@ -25,6 +25,9 @@
 #ifdef USE_CURSES
 
 #define ReadCh cmdReadCh
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+#	define ReadWch cmdReadWch
+#endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 
 void my_dummy(void) { }	/* ANSI C requires non-empty file */
 t_bool have_linescroll = TRUE;	/* USE_CURSES always allows line scrolling */
@@ -55,14 +58,14 @@ t_bool have_linescroll = TRUE;	/* USE_CURSES always allows line scrolling */
 
 int cLINES = DEFAULT_LINES_ON_TERMINAL - 1;
 int cCOLS  = DEFAULT_COLUMNS_ON_TERMINAL;
-int _hp_glitch = FALSE;		/* standout not erased by overwriting on HP terms */
+int _hp_glitch = FALSE;		/* stdout not erased by overwriting on HP terms */
 static int _inraw = FALSE;	/* are we IN rawmode? */
 static int xclicks = FALSE;	/* do we have an xterm? */
 t_bool have_linescroll = FALSE;
 
-#define BACKSPACE        '\b'
-#define VERY_LONG_STRING 2500
-#define TTYIN            0
+#define BACKSPACE	'\b'
+#define VERY_LONG_STRING	2500
+#define TTYIN	0
 
 #ifdef HAVE_CONFIG_H
 
@@ -90,7 +93,7 @@ t_bool have_linescroll = FALSE;
 #				define USE_SGTTY 1
 #				define TTY struct sgttyb
 #			else
-				please-fix-me(thanks)
+#				error "No termios.h, termio.h or sgtty.h found"
 #			endif /* HAVE_SGTTY_H */
 #		endif /* HAVE_TERMIO_H */
 #	endif /* HAVE_TERMIOS_H && HAVE_TCGETATTR && HAVE_TCSETATTR */
@@ -177,18 +180,18 @@ static int _columns, _line, _lines;
 #	undef SET_TTY
 #	undef GET_TTY
 #	if USE_POSIX_TERMIOS
-#		define SET_TTY(arg) tcsetattr (TTYIN, TCSANOW, arg)
-#		define GET_TTY(arg) tcgetattr (TTYIN, arg)
+#		define SET_TTY(arg) tcsetattr(TTYIN, TCSANOW, arg)
+#		define GET_TTY(arg) tcgetattr(TTYIN, arg)
 #	else
 #		if USE_TERMIO
-#			define SET_TTY(arg) ioctl (TTYIN, TCSETAW, arg)
-#			define GET_TTY(arg) ioctl (TTYIN, TCGETA, arg)
+#			define SET_TTY(arg) ioctl(TTYIN, TCSETAW, arg)
+#			define GET_TTY(arg) ioctl(TTYIN, TCGETA, arg)
 #		else
 #			if USE_SGTTY
 #				define SET_TTY(arg) stty(TTYIN, arg)
 #				define GET_TTY(arg) gtty(TTYIN, arg)
 #			else
-				please-fix-me(thanks)
+#				error "No termios.h, termio.h or sgtty.h found"
 #			endif /* USE_SGTTY */
 #		endif /* USE_TERMIO */
 #	endif /* USE_POSIX_TERMIOS */
@@ -200,15 +203,19 @@ static int in_inverse;			/* 1 when in inverse, 0 otherwise */
 /*
  * Local prototypes
  */
-static void ScreenSize (int *num_lines, int *num_columns);
-static int input_pending (int delay);
+static int input_pending(int delay);
+static void ScreenSize(int *num_lines, int *num_columns);
+static void xclick(int state);
+#if !defined(VMS) && defined(M_AMIGA)
+	static int AmiReadCh(int getscrsize);
+#endif /* !VMS && M_AMIGA */
 
 
 /*
- *  returns the number of lines and columns on the display.
+ * returns the number of lines and columns on the display.
  */
 static void
-ScreenSize (
+ScreenSize(
 	int *num_lines,
 	int *num_columns)
 {
@@ -217,7 +224,7 @@ ScreenSize (
 	if (!_columns)
 		_columns = DEFAULT_COLUMNS_ON_TERMINAL;
 
-	*num_lines = _lines - 1;		/* assume index from zero*/
+	*num_lines = _lines - 1;		/* assume index from zero */
 	*num_columns = _columns;		/* assume index from one */
 }
 
@@ -226,13 +233,14 @@ ScreenSize (
  * get screen size from termcap entry & setup sizes
  */
 void
-setup_screen (void)
+setup_screen(
+	void)
 {
 	_line = 1;
-	ScreenSize (&cLINES, &cCOLS);
+	ScreenSize(&cLINES, &cCOLS);
 	cmd_line = FALSE;
-	Raw (TRUE);
-	set_win_size (&cLINES, &cCOLS);
+	Raw(TRUE);
+	set_win_size(&cLINES, &cCOLS);
 }
 
 #ifdef M_UNIX
@@ -243,7 +251,7 @@ setup_screen (void)
 #		define TGETSTR(b,bufp)    tigetstr(b)
 #		define TGETNUM(b)         tigetnum(b) /* may be tigetint() */
 #		define TGETFLAG(b)        tigetflag(b)
-#		define NO_CAP(s)           (s == 0 || s == (char *)-1)
+#		define NO_CAP(s)          (s == 0 || s == (char *) -1)
 #		if !defined(HAVE_TIGETNUM) && defined(HAVE_TIGETINT)
 #			define tigetnum tigetint
 #		endif /* !HAVE_TIGETNUM && HAVE_TIGETINT */
@@ -255,7 +263,7 @@ setup_screen (void)
 #		define TGETSTR(a, bufp)   tgetstr(a, bufp)
 #		define TGETNUM(a)         tgetnum(a)
 #		define TGETFLAG(a)        tgetflag(a)
-#		define NO_CAP(s)           (s == 0)
+#		define NO_CAP(s)          (s == 0)
 #	endif /* USE_TERMINFO */
 
 #	ifdef HAVE_TPARM
@@ -269,7 +277,8 @@ extern char PC;			/* used in 'tputs()' */
 #	endif /* HAVE_EXTERN_TCAP_PC */
 
 int
-get_termcaps (void)
+get_termcaps(
+	void)
 {
 	static struct {
 		char **value;
@@ -307,28 +316,28 @@ get_termcaps (void)
 	char the_termname[40], *p;
 	unsigned n;
 
-	if ((p = getenv ("TERM")) == (char *) 0) {
-		my_fprintf (stderr, _(txt_no_term_set), tin_progname);
-		return (FALSE);
+	if ((p = getenv("TERM")) == NULL) {
+		my_fprintf(stderr, _(txt_no_term_set), tin_progname);
+		return FALSE;
 	}
 
 	my_strncpy(the_termname, p, sizeof(the_termname) - 1);
 
-#ifdef USE_TERMINFO
-	setupterm(the_termname, fileno(stdout), (int *)0);
-#else
-	if (tgetent (_terminal, the_termname) != 1) {
-		my_fprintf (stderr, _(txt_cannot_get_term_entry), tin_progname);
-		return (FALSE);
+#	ifdef USE_TERMINFO
+	setupterm(the_termname, fileno(stdout), (int *) 0);
+#	else
+	if (tgetent(_terminal, the_termname) != 1) {
+		my_fprintf(stderr, _(txt_cannot_get_term_entry), tin_progname);
+		return FALSE;
 	}
-#endif /* USE_TERMINFO */
+#	endif /* USE_TERMINFO */
 
 	/* load in all those pesky values */
-	for (n = 0; n < SIZEOF(table); n++) {
+	for (n = 0; n < ARRAY_SIZE(table); n++) {
 		*(table[n].value) = TGETSTR(table[n].capname, &ptr);
 	}
-	_lines          = TGETNUM (dCAPNAME("li", "lines"));
-	_columns        = TGETNUM (dCAPNAME("co", "cols"));
+	_lines          = TGETNUM(dCAPNAME("li", "lines"));
+	_columns        = TGETNUM(dCAPNAME("co", "cols"));
 	_hp_glitch      = TGETFLAG(dCAPNAME("xs", "xhp"));
 
 #	if defined(USE_TERMCAP) && defined(HAVE_EXTERN_TCAP_PC)
@@ -346,20 +355,20 @@ get_termcaps (void)
 	}
 
 	if (NO_CAP(_clearscreen)) {
-		my_fprintf (stderr, _(txt_no_term_clearscreen), tin_progname);
-		return (FALSE);
+		my_fprintf(stderr, _(txt_no_term_clearscreen), tin_progname);
+		return FALSE;
 	}
 	if (NO_CAP(_moveto)) {
-		my_fprintf (stderr, _(txt_no_term_cursor_motion), tin_progname);
-		return (FALSE);
+		my_fprintf(stderr, _(txt_no_term_cursor_motion), tin_progname);
+		return FALSE;
 	}
 	if (NO_CAP(_cleartoeoln)) {
-		my_fprintf (stderr, _(txt_no_term_clear_eol), tin_progname);
-		return (FALSE);
+		my_fprintf(stderr, _(txt_no_term_clear_eol), tin_progname);
+		return FALSE;
 	}
 	if (NO_CAP(_cleartoeos)) {
-		my_fprintf (stderr, _(txt_no_term_clear_eos), tin_progname);
-		return (FALSE);
+		my_fprintf(stderr, _(txt_no_term_clear_eos), tin_progname);
+		return FALSE;
 	}
 	if (_lines == -1)
 		_lines = DEFAULT_LINES_ON_TERMINAL;
@@ -368,7 +377,7 @@ get_termcaps (void)
 
 	if (_lines < MIN_LINES_ON_TERMINAL || _columns < MIN_COLUMNS_ON_TERMINAL) {
 		my_fprintf(stderr, _(txt_screen_too_small), tin_progname);
-		return (FALSE);
+		return FALSE;
 	}
 	/*
 	 * kludge to workaround no inverse
@@ -383,23 +392,26 @@ get_termcaps (void)
 		have_linescroll = FALSE;
 	else
 		have_linescroll = TRUE;
-	return (TRUE);
+	return TRUE;
 }
 
+
 int
-InitScreen (void)
+InitScreen(
+	void)
 {
-	InitWin ();
+	InitWin();
 #	ifdef HAVE_COLOR
 	postinit_colors();
 #	endif /* HAVE_COLOR */
-	return (TRUE);
+	return TRUE;
 }
 
 #else	/* !M_UNIX */
 
 int
-InitScreen (void)
+InitScreen(
+	void)
 {
 	char *ptr;
 
@@ -437,11 +449,11 @@ InitScreen (void)
 	 * you're using something other than an Amiga window
 	 */
 
-	if ((ptr = getenv ("LINES")) != 0) {
-		_lines = atol (ptr);
+	if ((ptr = getenv("LINES")) != 0) {
+		_lines = atol(ptr);
 	}
-	if ((ptr = getenv ("COLUMNS")) != 0) {
-		_columns = atol (ptr);
+	if ((ptr = getenv("COLUMNS")) != 0) {
+		_columns = atol(ptr);
 	}
 
 	/*
@@ -459,7 +471,7 @@ InitScreen (void)
 		_getwinsize = NULL;
 	}
 #	endif /* M_AMIGA */
-#	ifdef VMS /* moved from below InitWin () M.St. 22.01.98 */
+#	ifdef VMS /* moved from below InitWin() M.St. 22.01.98 */
 	{
 		int input_chan, status;
 		int item_code, eightbit;
@@ -477,12 +489,12 @@ InitScreen (void)
 			unsigned long tt_char : 24, scr_len : 8;
 			unsigned long tt2_char;
 		} tty;
-		$DESCRIPTOR (input_dsc, "TT");
+		$DESCRIPTOR(input_dsc, "TT");
 
-		status = SYS$ASSIGN (&input_dsc, &input_chan, 0, 0);
+		status = SYS$ASSIGN(&input_dsc, &input_chan, 0, 0);
 		if (!(status & 1))
-			LIB$STOP (status);
-		SYS$QIOW (0, input_chan, IO$_SENSEMODE, &tty, 0, 0, &tty.class, 12, 0, 0, 0, 0);
+			LIB$STOP(status);
+		SYS$QIOW(0, input_chan, IO$_SENSEMODE, &tty, 0, 0, &tty.class, 12, 0, 0, 0, 0);
 		item_code = DVI$_TT_EIGHTBIT;
 		status = LIB$GETDVI(&item_code, &input_chan, 0, &eightbit, 0, 0);
 		_columns = tty.scr_wid;
@@ -490,7 +502,7 @@ InitScreen (void)
 
 		if (eightbit) { /* if using eightbit then use CSI (octal 233) rather than ESC "[" */
 			_clearscreen = "\2331;1H\233J";
-			_moveto = "\233%d;%dH"; /* not a termcap string !*/
+			_moveto = "\233%d;%dH"; /* not a termcap string! */
 			_cleartoeoln = "\233K";
 			_cleartoeos = "\233J";
 			_setinverse = "\2337m";
@@ -536,50 +548,53 @@ InitScreen (void)
 
 	if (_lines < MIN_LINES_ON_TERMINAL || _columns < MIN_COLUMNS_ON_TERMINAL) {
 		my_fprintf(stderr, _(txt_screen_too_small), tin_progname);
-		return (FALSE);
+		return FALSE;
 	}
 
-	InitWin ();
+	InitWin();
 
-	Raw (FALSE);
+	Raw(FALSE);
 
 	have_linescroll = FALSE;
-	return (TRUE);
+	return TRUE;
 }
 
 #endif /* M_UNIX */
 
 
 void
-InitWin (void)
+InitWin(
+	void)
 {
 	if (_terminalinit) {
-		tputs (_terminalinit, 1, outchar);
+		tputs(_terminalinit, 1, outchar);
 		my_flush();
 	}
-	set_keypad_on ();
-	set_xclick_on ();
+	set_keypad_on();
+	set_xclick_on();
 }
 
 
 void
-EndWin (void)
+EndWin(
+	void)
 {
 	if (_terminalend) {
-		tputs (_terminalend, 1, outchar);
+		tputs(_terminalend, 1, outchar);
 		my_flush();
 	}
-	set_keypad_off ();
-	set_xclick_off ();
+	set_keypad_off();
+	set_xclick_off();
 }
 
 
 void
-set_keypad_on (void)
+set_keypad_on(
+	void)
 {
 #	ifdef HAVE_KEYPAD
 	if (tinrc.use_keypad && _keypadxmit) {
-		tputs (_keypadxmit, 1, outchar);
+		tputs(_keypadxmit, 1, outchar);
 		my_flush();
 	}
 #	endif /* HAVE_KEYPAD */
@@ -587,100 +602,105 @@ set_keypad_on (void)
 
 
 void
-set_keypad_off (void)
+set_keypad_off(
+	void)
 {
 #	ifdef HAVE_KEYPAD
 	if (tinrc.use_keypad && _keypadlocal) {
-		tputs (_keypadlocal, 1, outchar);
+		tputs(_keypadlocal, 1, outchar);
 		my_flush();
 	}
 #	endif /* HAVE_KEYPAD */
 }
 
-/*
- *  clear the screen
- */
 
+/*
+ * clear the screen
+ */
 void
-ClearScreen (void)
+ClearScreen(
+	void)
 {
-	tputs (_clearscreen, 1, outchar);
-	my_flush ();		/* clear the output buffer */
+	tputs(_clearscreen, 1, outchar);
+	my_flush();		/* clear the output buffer */
 	_line = 1;
 }
 
+
 /*
  *  move cursor to the specified row column on the screen.
  *  0,0 is the top left!
  */
-
 #ifdef M_UNIX
-
 void
-MoveCursor (
+MoveCursor(
 	int row,
 	int col)
 {
 	char *stuff;
 
 	stuff = tgoto(_moveto, col, row);
-	tputs (stuff, 1, outchar);
-	my_flush ();
+	tputs(stuff, 1, outchar);
+	my_flush();
 	_line = row + 1;
 }
 
 #else	/* !M_UNIX */
 
 void
-MoveCursor (
+MoveCursor(
 	int row,
 	int col)
 {
 	char stuff[12];
 
 	if (_moveto) {
-		sprintf (stuff, _moveto, row+1, col+1);
-		tputs (stuff, 1, outchar);
-		my_flush ();
+		sprintf(stuff, _moveto, row + 1, col + 1);
+		tputs(stuff, 1, outchar);
+		my_flush();
 		_line = row + 1;
 	}
 }
-
 #endif /* M_UNIX */
 
-/*
- *  clear to end of line
- */
-void
-CleartoEOLN (void)
-{
-	tputs (_cleartoeoln, 1, outchar);
-	my_flush ();	/* clear the output buffer */
-}
 
 /*
- *  clear to end of screen
+ * clear to end of line
  */
 void
-CleartoEOS (void)
+CleartoEOLN(
+	void)
+{
+	tputs(_cleartoeoln, 1, outchar);
+	my_flush();	/* clear the output buffer */
+}
+
+
+/*
+ * clear to end of screen
+ */
+void
+CleartoEOS(
+	void)
 {
 	int i;
 
 	if (_cleartoeos) {
-		tputs (_cleartoeos, 1, outchar);
+		tputs(_cleartoeos, 1, outchar);
 	} else {
-		for (i = _line - 1 ; i < _lines ; i++) {
-			MoveCursor (i, 0);
-			CleartoEOLN ();
+		for (i = _line - 1; i < _lines; i++) {
+			MoveCursor(i, 0);
+			CleartoEOLN();
 		}
 	}
-	my_flush ();	/* clear the output buffer */
+	my_flush();	/* clear the output buffer */
 }
 
+
 static int _topscrregion, _bottomscrregion;
 
 void
-SetScrollRegion (
+SetScrollRegion(
 	int topline,
 	int bottomline)
 {
@@ -690,15 +710,16 @@ SetScrollRegion (
 		return;
 	if (_scrollregion) {
 		stuff = TFORMAT(_scrollregion, topline, bottomline);
-		tputs (stuff, 1, outchar);
+		tputs(stuff, 1, outchar);
 		_topscrregion = topline;
 		_bottomscrregion = bottomline;
 	}
-	my_flush ();
+	my_flush();
 }
 
+
 void
-ScrollScreen (
+ScrollScreen(
 	int lines_to_scroll)
 {
 	int i;
@@ -709,27 +730,27 @@ ScrollScreen (
 		if (_scrollback) {
 			i = lines_to_scroll;
 			while (i++) {
-				MoveCursor (_topscrregion, 0);
-				tputs (_scrollback, 1, outchar);
+				MoveCursor(_topscrregion, 0);
+				tputs(_scrollback, 1, outchar);
 			}
 		}
 	} else
 		if (_scrollfwd) {
 			i = lines_to_scroll;
 			while (i--) {
-				MoveCursor (_bottomscrregion, 0);
-				tputs (_scrollfwd, 1, outchar);
+				MoveCursor(_bottomscrregion, 0);
+				tputs(_scrollfwd, 1, outchar);
 			}
 		}
-	my_flush ();
+	my_flush();
 }
 
 
 /*
- *  set inverse video mode
+ * set inverse video mode
  */
 void
-StartInverse (
+StartInverse(
 	void)
 {
 	in_inverse = 1;
@@ -739,21 +760,21 @@ StartInverse (
 			bcol(tinrc.col_invers_bg);
 			fcol(tinrc.col_invers_fg);
 		} else {
-			tputs (_setinverse, 1, outchar);
+			tputs(_setinverse, 1, outchar);
 		}
 #	else
-		tputs (_setinverse, 1, outchar);
+		tputs(_setinverse, 1, outchar);
 #	endif /* HAVE_COLOR */
 	}
-	my_flush ();
+	my_flush();
 }
 
 
 /*
- *  compliment of startinverse
+ * compliment of startinverse
  */
 void
-EndInverse (
+EndInverse(
 	void)
 {
 	in_inverse = 0;
@@ -763,21 +784,21 @@ EndInverse (
 			fcol(tinrc.col_normal);
 			bcol(tinrc.col_back);
 		} else {
-			tputs (_clearinverse, 1, outchar);
+			tputs(_clearinverse, 1, outchar);
 		}
 #	else
-		tputs (_clearinverse, 1, outchar);
+		tputs(_clearinverse, 1, outchar);
 #	endif /* HAVE_COLOR */
 	}
-	my_flush ();
+	my_flush();
 }
 
 
 /*
- *  toggle inverse video mode
+ * toggle inverse video mode
  */
 void
-ToggleInverse (
+ToggleInverse(
 	void)
 {
 	if (!in_inverse)
@@ -788,21 +809,21 @@ ToggleInverse (
 
 
 /*
- *  returns either 1 or 0, for ON or OFF
+ * returns either 1 or 0, for ON or OFF
  */
 int
 RawState(
 	void)
 {
-	return (_inraw);
+	return _inraw;
 }
 
 
 /*
- *  state is either TRUE or FALSE, as indicated by call
+ * state is either TRUE or FALSE, as indicated by call
  */
 void
-Raw (
+Raw(
 	int state)
 {
 #ifdef VMS
@@ -819,11 +840,11 @@ Raw (
 	rawcon (state);
 #	else
 	if (!state && _inraw) {
-		SET_TTY (&_original_tty);
+		SET_TTY(&_original_tty);
 		_inraw = 0;
 	} else if (state && !_inraw) {
-		GET_TTY (&_original_tty);
-		GET_TTY (&_raw_tty);
+		GET_TTY(&_original_tty);
+		GET_TTY(&_raw_tty);
 #		if USE_SGTTY
 		_raw_tty.sg_flags &= ~(ECHO | CRMOD);	/* echo off */
 		_raw_tty.sg_flags |= CBREAK;		/* raw on */
@@ -840,15 +861,16 @@ Raw (
 		_raw_tty.c_cc[VTIME] = '\0';	/* minimum time to wait for input */
 #			endif /* __FreeBSD__ */
 #		endif /* USE_SGTTY */
-		SET_TTY (&_raw_tty);
+		SET_TTY(&_raw_tty);
 		_inraw = 1;
 	}
 #	endif /* M_AMIGA && __SASC */
 #endif /* !VMS */
 }
 
+
 /*
- *  read a character with Raw mode set!
+ * read a character with Raw mode set!
  */
 
 #ifndef VMS
@@ -858,7 +880,7 @@ Raw (
 static int new_lines, new_columns;
 
 static int
-AmiReadCh (
+AmiReadCh(
 	int getscrsize)
 {
 	int result;
@@ -868,22 +890,22 @@ AmiReadCh (
 
 	while (getscrsize || !buflen) {
 PROFILE_OFF();
-		result = read (0, (char *)&buf[buflen], 1);
+		result = read(0, (char *) &buf[buflen], 1);
 PROFILE_ON();
 		if (result <= 0) return EOF;
 		buflen++;
 		if (buf[bufp] == KEY_PREFIX) {
 			do {
-				result = read (0, (char *)&buf[buflen], 1);
+				result = read(0, (char *) &buf[buflen], 1);
 				if (result <= 0) return EOF;
 			} while (buf[buflen++] < 0x40);
 
-			switch (buf[buflen-1])
-			{	char *ptr;
+			switch (buf[buflen-1]) {
+				char *ptr;
 				long class;
 
 				case 'r':		/* Window bounds report */
-					ptr = (char *)&buf[bufp+1];
+					ptr = (char *) &buf[bufp + 1];
 					strtol(ptr, &ptr, 10);
 					ptr++;
 					strtol(ptr, &ptr, 10);
@@ -897,7 +919,7 @@ PROFILE_ON();
 					break;
 
 				case '|':		/* Raw Input Events */
-					ptr = (char *)&buf[bufp+1];
+					ptr = (char *) &buf[bufp + 1];
 					class = strtol(ptr, &ptr, 10);
 					ptr++;
 					switch (class) {
@@ -935,13 +957,14 @@ PROFILE_ON();
 	}
 
 	ch = buf[bufp++];
-	if (bufp >= buflen) buflen = bufp = 0;
+	if (bufp >= buflen)
+		buflen = bufp = 0;
 	return ch;
 }
 
 
 int
-ReadCh (
+ReadCh(
 	void)
 {
 	return AmiReadCh(0);
@@ -949,13 +972,13 @@ ReadCh (
 
 
 void
-AmiGetWinSize (
+AmiGetWinSize(
 	int *lines,
 	int *columns)
 {
 	if (_getwinsize) {
-		tputs (_getwinsize, 1, outchar);	/* identify yourself */
-		my_flush ();
+		tputs(_getwinsize, 1, outchar);	/* identify yourself */
+		my_flush();
 		AmiReadCh(1);		/* Look for the identification */
 		*lines = new_lines;
 		*columns = new_columns;
@@ -968,71 +991,77 @@ AmiGetWinSize (
 /*
  *  output a character. From tputs... (Note: this CANNOT be a macro!)
  */
-OUTC_FUNCTION (
+OUTC_FUNCTION(
 	outchar)
 {
 #ifdef OUTC_RETURN
-	return fputc (c, stdout);
+	return fputc(c, stdout);
 #else
-	(void) fputc (c, stdout);
+	(void) fputc(c, stdout);
 #endif /* OUTC_RETURN */
 }
 
 
 /*
- *  setup to monitor mouse buttons if running in a xterm
+ * setup to monitor mouse buttons if running in a xterm
  */
 static void
-xclick (
+xclick(
 	int state)
 {
 	static int prev_state = 999;
 
 	if (xclicks && prev_state != state) {
 		if (state) {
-			tputs (_xclickinit, 1, outchar);
+			tputs(_xclickinit, 1, outchar);
 		} else {
-			tputs (_xclickend, 1, outchar);
+			tputs(_xclickend, 1, outchar);
 		}
-		my_flush ();
+		my_flush();
 		prev_state = state;
 	}
 }
 
 
 /*
- *  switch on monitoring of mouse buttons
+ * switch on monitoring of mouse buttons
  */
 void
-set_xclick_on (void)
+set_xclick_on(
+	void)
 {
-	if (tinrc.use_mouse) xclick (TRUE);
+	if (tinrc.use_mouse)
+		xclick(TRUE);
 }
 
 
 /*
- *  switch off monitoring of mouse buttons
+ * switch off monitoring of mouse buttons
  */
 void
-set_xclick_off (void)
+set_xclick_off(
+	void)
 {
-	if (tinrc.use_mouse) xclick (FALSE);
+	if (tinrc.use_mouse)
+		xclick(FALSE);
 }
 
 
 void
-cursoron (void)
+cursoron(
+	void)
 {
 	if (_cursoron)
-		tputs (_cursoron, 1, outchar);
+		tputs(_cursoron, 1, outchar);
 }
 
 
 void
-cursoroff (void)
+cursoroff(
+	void)
 {
 	if (_cursoroff)
-		tputs (_cursoroff, 1, outchar);
+		tputs(_cursoroff, 1, outchar);
 }
 
 
@@ -1040,32 +1069,75 @@ cursoroff (void)
  * Inverse 'size' chars at (row,col)
  */
 void
-highlight_string (
+highlight_string(
 	int row,
 	int col,
 	int size)
 {
 	char tmp[LEN];
 
-	MoveCursor (row, col);
-	my_strncpy (tmp, &(screen[row].col[col]), size);
+	MoveCursor(row, col);
+	my_strncpy(tmp, &(screen[row].col[col]), size);
 
-	StartInverse ();
-	my_fputs (tmp, stdout);
-	my_flush ();
-	EndInverse ();
+	StartInverse();
+	my_fputs(tmp, stdout);
+	my_flush();
+	EndInverse();
 
 	stow_cursor();
 }
 
 
+#ifdef HAVE_COLOR
+/*
+ * Color 'size' chars at (row,col) with 'color' and handle marks
+ */
+void
+word_highlight_string(
+	int row,
+	int col,
+	int size,
+	int color)
+{
+	char tmp[LEN];
+
+	MoveCursor(row, col);
+	my_strncpy(tmp, &(screen[row].col[col]), size);
+
+	/* safegurad against bogus regexps */
+	if ((tmp[0] == '*' && tmp[size - 1] == '*') ||
+		 (tmp[0] == '/' && tmp[size - 1] == '/') ||
+		 (tmp[0] == '_' && tmp[size - 1] == '_') ||
+		 (tmp[0] == '-' && tmp[size - 1] == '-')) {
+
+		switch (tinrc.word_h_display_marks) {
+			case 0: /* FIXME */
+				break;
+
+			case 2: /* print space */
+				tmp[0] = tmp[size - 1] = ' ';
+				break;
+
+			default:	/* print mark (case 1) */
+				break;
+		}
+	}
+	fcol(color);
+	my_fputs(tmp, stdout);
+	my_flush();
+	fcol(tinrc.col_text);
+	stow_cursor();
+}
+#endif /* HAVE_COLOR */
+
+
 /*
  * input_pending() waits for input during time given
- * by delay in msec. The original behaviour of input_pending()
+ * by delay in msec. The original behavior of input_pending()
  * (in art.c's threading code) is delay=0
  */
 static int
-input_pending (
+input_pending(
 	int delay)
 {
 #if 0
@@ -1095,9 +1167,9 @@ input_pending (
 	FD_SET(fd, &fdread);
 
 #		ifdef HAVE_SELECT_INTP
-	if (select (1, (int *)&fdread, NULL, NULL, &tvptr))
+	if (select(1, (int *) &fdread, NULL, NULL, &tvptr))
 #		else
-	if (select (1, &fdread, NULL, NULL, &tvptr))
+	if (select(1, &fdread, NULL, NULL, &tvptr))
 #		endif /* HAVE_SELECT_INTP */
 	{
 		if (FD_ISSET(fd, &fdread))
@@ -1111,7 +1183,7 @@ input_pending (
 	static struct pollfd fds[]= {{ STDIN_FILENO, POLLIN, 0 }};
 
 	Timeout = delay;
-	if (poll (fds, nfds, Timeout) < 0) /* Error on poll */
+	if (poll(fds, nfds, Timeout) < 0) /* Error on poll */
 		return FALSE;
 
 	switch (fds[0].revents) {
@@ -1134,7 +1206,7 @@ input_pending (
 
 
 int
-get_arrow_key (
+get_arrow_key(
 	int prech)
 {
 	int ch;
@@ -1165,7 +1237,7 @@ get_arrow_key (
 		wait_a_while(i) {
 			tvptr.tv_sec = 0;
 			tvptr.tv_usec = SECOND_CHARACTER_DELAY * 1000;
-			select (0, NULL, NULL, NULL, &tvptr);
+			select(0, NULL, NULL, NULL, &tvptr);
 			i++;
 		}
 #				else /* !HAVE_SELECT */
@@ -1187,9 +1259,9 @@ get_arrow_key (
 	}
 #		endif /* M_AMIGA */
 #	endif /* !VMS */
-	ch = ReadCh ();
+	ch = ReadCh();
 	if (ch == '[' || ch == 'O')
-		ch = ReadCh ();
+		ch = ReadCh();
 
 	switch (ch) {
 		case 'A':
@@ -1256,26 +1328,26 @@ get_arrow_key (
 			return KEYMAP_END;
 
 		case '2':		/* vt200 Ins */
-			(void) ReadCh ();	/* eat the ~ */
+			(void) ReadCh();	/* eat the ~ */
 			return KEYMAP_INS;
 
 		case '3':		/* vt200 Del */
-			(void) ReadCh ();	/* eat the ~ */
+			(void) ReadCh();	/* eat the ~ */
 			return KEYMAP_DEL;
 
 		case '5':		/* vt200 PgUp */
-			(void) ReadCh ();	/* eat the ~ (interesting use of words :) */
+			(void) ReadCh();	/* eat the ~ (interesting use of words :) */
 			return KEYMAP_PAGE_UP;
 
 		case '6':		/* vt200 PgUp */
-			(void) ReadCh ();	/* eat the ~ */
+			(void) ReadCh();	/* eat the ~ */
 			return KEYMAP_PAGE_DOWN;
 
 		case '1':		/* vt200 PgUp */
-			ch = ReadCh (); /* eat the ~ */
+			ch = ReadCh(); /* eat the ~ */
 			if (ch == '5') {	/* RS/6000 PgUp is 150g, PgDn is 154g */
-				ch1 = ReadCh ();
-				(void) ReadCh ();
+				ch1 = ReadCh();
+				(void) ReadCh();
 				if (ch1 == '0')
 					return KEYMAP_PAGE_UP;
 				if (ch1 == '4')
@@ -1284,20 +1356,20 @@ get_arrow_key (
 			return KEYMAP_HOME;
 
 		case '4':		/* vt200 PgUp */
-			(void) ReadCh ();	/* eat the ~ */
+			(void) ReadCh();	/* eat the ~ */
 			return KEYMAP_END;
 
 		case 'M':		/* xterminal button press */
-			xmouse = ReadCh () - ' ';	/* button */
-			xcol = ReadCh () - '!';		/* column */
-			xrow = ReadCh () - '!';		/* row */
+			xmouse = ReadCh() - ' ';	/* button */
+			xcol = ReadCh() - '!';		/* column */
+			xrow = ReadCh() - '!';		/* row */
 			return KEYMAP_MOUSE;
 
 		default:
 			return KEYMAP_UNKNOWN;
 	}
 }
-#endif /* !USE_CURSES */
+#endif /* USE_CURSES */
 
 
 /*
@@ -1305,25 +1377,26 @@ get_arrow_key (
  */
 #ifdef M_UNIX
 int
-ReadCh (void)
+ReadCh(
+	void)
 {
 	register int result;
-#		ifndef READ_CHAR_HACK
+#	ifndef READ_CHAR_HACK
 	char ch;
-#		endif /* READ_CHAR_HACK */
+#	endif /* READ_CHAR_HACK */
 
 	fflush(stdout);
-#		ifdef READ_CHAR_HACK
-#			undef getc
+#	ifdef READ_CHAR_HACK
+#		undef getc
 	while ((result = getc(stdin)) == EOF) {
 		if (feof(stdin))
 			break;
 
-#			ifdef EINTR
+#		ifdef EINTR
 		if (ferror(stdin) && errno != EINTR)
-#			else
+#		else
 		if (ferror(stdin))
-#			endif /* EINTR */
+#		endif /* EINTR */
 			break;
 
 		clearerr(stdin);
@@ -1331,23 +1404,105 @@ ReadCh (void)
 
 	return ((result == EOF) ? EOF : result & 0xFF);
 
-#		else
-#			ifdef EINTR
+#	else
+#		ifdef EINTR
 
-	allow_resize (TRUE);
-	while ((result = read (0, &ch, 1)) < 0 && errno == EINTR) {		/* spin on signal interrupts */
+	allow_resize(TRUE);
+	while ((result = read(0, &ch, 1)) < 0 && errno == EINTR) {		/* spin on signal interrupts */
 		if (need_resize) {
-			handle_resize ((need_resize == cRedraw) ? TRUE : FALSE);
+			handle_resize((need_resize == cRedraw) ? TRUE : FALSE);
 			need_resize = cNo;
 		}
 	}
-	allow_resize (FALSE);
-#			else
-	result = read (0, &ch, 1);
-#			endif /* EINTR */
+	allow_resize(FALSE);
+#		else
+	result = read(0, &ch, 1);
+#		endif /* EINTR */
 
 	return ((result <= 0) ? EOF : ch & 0xFF);
 
-#		endif /* READ_CHAR_HACK */
+#	endif /* READ_CHAR_HACK */
 }
+
+
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+wint_t
+ReadWch(
+	void)
+{
+	char *mbs = my_malloc(MB_CUR_MAX + 1);
+	int result, offset;
+	wchar_t wch = 0;
+
+	fflush(stdout);
+
+#		ifndef USE_CURSES
+		/* read only the first char */
+#			ifdef EINTR
+	allow_resize(TRUE);
+	while ((result = read(0, mbs, 1)) < 0 && errno == EINTR) { /* spin on signal interrupts */
+		if (need_resize) {
+			handle_resize((need_resize == cRedraw) ? TRUE :  FALSE);
+			need_resize = cNo;
+		}
+	}
+	allow_resize(FALSE);
+#			else
+	result = read(0, mbs, sizeof(mbs));
+#			endif /* EINTR */
+	if (result <= 0) {
+		free(mbs);
+		return WEOF;
+	}
+
+	/* Begin of an ESC-sequence. Let get_arrow_key() figure out which it is */
+	if (mbs[0] == ESC) {
+		free(mbs);
+		return (wint_t) ESC;
+	}
+
+	/* the pressed key can be represented in one byte */
+	if (!input_pending(0)) {
+		char out = mbs[0] & 0xff;
+
+		free(mbs);
+		return (wint_t) out;
+	}
+
+	/* read the rest of the multibyte-sequence */
+	offset = 1;
+#		else
+	/* read the complete input */
+	offset = 0;
+#		endif /* !USE_CURSES */
+
+#		ifdef EINTR
+	allow_resize(TRUE);
+	while ((result = read(0, mbs + offset, MB_CUR_MAX)) < 0 && errno == EINTR) { /* spin on signal interrupts */
+		if (need_resize) {
+			handle_resize((need_resize == cRedraw) ? TRUE :  FALSE);
+			need_resize = cNo;
+		}
+	}
+	allow_resize(FALSE);
+#		else
+	result = read(0, mbs + offset, MB_CUR_MAX);
+#		endif /* EINTR */
+
+	if (result <= 0) {
+		free(mbs);
+		return WEOF;
+	} else {
+		int res;
+
+		mbs[result + offset] = '\0';
+		res = mbtowc(&wch, mbs, MB_CUR_MAX);
+		free(mbs);
+		if (res == -1)
+			return WEOF;
+		else
+			return (wint_t) wch;
+	}
+}
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 #endif /* M_UNIX */
