@@ -3,10 +3,10 @@
  *  Module    : help.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2001-11-10
+ *  Updated   : 2003-01-07
  *  Notes     :
  *
- * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Iain Lea.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -48,11 +45,18 @@
 #	include "menukeys.h"
 #endif /* !MENUKEYS_H */
 
+
 typedef struct thp {
 	constext *helptext;
 	char key;
 } t_help_page;
 
+/*
+ * local prototypes
+ */
+static void make_help_page(FILE *fp, const t_help_page *helppage, const t_menukeys *menukeys);
+
+
 const char **info_help;
 static constext txt_help_empty_line[] = "";
 
@@ -78,6 +82,7 @@ static t_help_page select_help_page[] = {
 	{ txt_help_select_search_group_forwards, iKeySearchSubjF },
 	{ txt_help_select_search_group_backwards, iKeySearchSubjB },
 	{ txt_help_select_search_group_comment, 0 },
+	{ txt_help_global_search_repeat, iKeySearchRepeat },
 	{ txt_help_empty_line, 0 },
 	{ txt_help_title_disp, 0 },
 	{ txt_help_select_toggle_read_groups, iKeySelectToggleReadDisplay },
@@ -88,6 +93,7 @@ static t_help_page select_help_page[] = {
 	{ txt_help_global_toggle_color, iKeyToggleColor },
 #endif /* HAVE_COLOR */
 	{ txt_help_empty_line, 0 },
+	{ txt_help_select_sort_active, iKeySelectSortActive },
 	{ txt_help_select_yank_active, iKeySelectYankActive },
 	{ txt_help_select_sync_with_active, iKeySelectSyncWithActive },
 	{ txt_help_empty_line, 0 },
@@ -113,7 +119,7 @@ static t_help_page select_help_page[] = {
 	{ txt_help_empty_line, 0 },
 	{ txt_help_title_misc, 0 },
 	{ txt_help_select_quit, iKeyQuit },
-	{ txt_help_select_quit, iKeyQuitTin },
+	{ txt_help_global_quit_tin, iKeyQuitTin },
 	{ txt_help_select_quit_no_write, iKeySelectQuitNoWrite },
 	{ txt_help_global_help, iKeyHelp },
 	{ txt_help_global_toggle_mini_help, iKeyToggleHelpDisplay },
@@ -162,6 +168,7 @@ static t_help_page group_help_page[] = {
 	{ txt_help_global_search_auth_backwards, iKeySearchAuthB },
 	{ txt_help_global_search_body, iKeySearchBody },
 	{ txt_help_global_search_body_comment, 0 },
+	{ txt_help_global_search_repeat, iKeySearchRepeat },
 	{ txt_help_empty_line, 0 },
 	{ txt_help_title_disp, 0 },
 	{ txt_help_group_toggle_read_articles, iKeyGroupToggleReadUnread },
@@ -262,6 +269,7 @@ static t_help_page thread_help_page[] = {
 	{ txt_help_global_search_auth_backwards, iKeySearchAuthB },
 	{ txt_help_global_search_body, iKeySearchBody },
 	{ txt_help_global_search_body_comment, 0 },
+	{ txt_help_global_search_repeat, iKeySearchRepeat },
 	{ txt_help_empty_line, 0 },
 	{ txt_help_title_disp, 0 },
 	{ txt_help_global_toggle_info_line, iKeyToggleInfoLastLine },
@@ -355,6 +363,7 @@ static t_help_page page_help_page[] = {
 	{ txt_help_global_search_auth_backwards, iKeySearchAuthB },
 	{ txt_help_global_search_body, iKeySearchBody },
 	{ txt_help_global_search_body_comment, 0 },
+	{ txt_help_global_search_repeat, iKeySearchRepeat },
 	{ txt_help_empty_line, 0 },
 	{ txt_help_title_disp, 0 },
 	{ txt_help_global_toggle_info_line, iKeyToggleInfoLastLine },
@@ -363,8 +372,8 @@ static t_help_page page_help_page[] = {
 	{ txt_help_article_show_raw, iKeyPageToggleHeaders },
 #ifdef HAVE_COLOR
 	{ txt_help_global_toggle_color, iKeyToggleColor },
-	{ txt_help_article_toggle_highlight, iKeyPageToggleHighlight },
 #endif /* HAVE_COLOR */
+	{ txt_help_article_toggle_highlight, iKeyPageToggleHighlight },
 	{ txt_help_article_toggle_tex2iso, iKeyPageToggleTex2iso },
 	{ txt_help_article_toggle_tabwidth, iKeyPageToggleTabs },
 	{ txt_help_article_toggle_uue, iKeyPageToggleUue },
@@ -432,7 +441,7 @@ static t_help_page page_help_page[] = {
 
 
 static void
-make_help_page (
+make_help_page(
 	FILE *fp,
 	const t_help_page *helppage,
 	const t_menukeys *menukeys)
@@ -445,60 +454,63 @@ make_help_page (
 		return;
 
 	while (helppage->helptext) {
-		if (helppage->key == 0)
-			snprintf (buf, sizeof(buf), "%s", _(helppage->helptext));
-		else
-			snprintf (buf, sizeof(buf), "%s\t  %s",
-				printascii (key, map_to_local (helppage->key, menukeys)),
-				_(helppage->helptext));
+		if (helppage->key == 0) {
+			if (!strlen(helppage->helptext))	/* avoid empty string translations */
+				snprintf(buf, sizeof(buf) - 1, "%s", helppage->helptext);
+			else
+				snprintf(buf, sizeof(buf) - 1, "%s", _(helppage->helptext));
+		} else
+			snprintf(buf, sizeof(buf) - 1, "%s\t  %s", printascii(key, map_to_local(helppage->key, menukeys)), _(helppage->helptext));
 		buf[sizeof(buf) - 1] = '\0';
-		expand_ctrl_chars (helpline, buf, sizeof(helpline), 8);
-		fprintf (fp, "%s\n", helpline);
+		expand_ctrl_chars(helpline, buf, sizeof(helpline) - 1, 8);
+		fprintf(fp, "%s", helpline);
 		helppage++;
 	}
 }
 
+
 void
-show_help_page (
+show_help_page(
 	const int level,
 	const char *title)
 {
 	FILE *fp;
 
-	if (!(fp = tmpfile ()))
+	if (!(fp = tmpfile()))
 		return;
 
 	switch (level) {
 		case SELECT_LEVEL:
-			make_help_page (fp, select_help_page, &menukeymap.select_nav);
+			make_help_page(fp, select_help_page, &menukeymap.select_nav);
 			break;
 
 		case GROUP_LEVEL:
-			make_help_page (fp, group_help_page, &menukeymap.group_nav);
+			make_help_page(fp, group_help_page, &menukeymap.group_nav);
 			break;
 
 		case THREAD_LEVEL:
-			make_help_page (fp, thread_help_page, &menukeymap.thread_nav);
+			make_help_page(fp, thread_help_page, &menukeymap.thread_nav);
 			break;
 
 		case PAGE_LEVEL:
-			make_help_page (fp, page_help_page, &menukeymap.page_nav);
+			make_help_page(fp, page_help_page, &menukeymap.page_nav);
 			break;
 
 		case INFO_PAGER:
 		default: /* should not happen */
-			error_message (_(txt_error_unknown_dlevel));
-			fclose (fp);
+			error_message(_(txt_error_unknown_dlevel));
+			fclose(fp);
 			return;
 	}
 
-	info_pager (fp, title, TRUE);
-	fclose (fp);
+	info_pager(fp, title, TRUE);
+	fclose(fp);
 	return;
 }
 
+
 void
-show_mini_help (
+show_mini_help(
 	int level)
 {
 	char buf[LEN];
@@ -509,8 +521,8 @@ show_mini_help (
 	if (!tinrc.beginner_level)
 		return;
 
-	line = NOTESLINES + (MINI_HELP_LINES - 2);
-	bufs = (size_t) MIN((unsigned)cCOLS, (sizeof(buf) - 1));
+	line = NOTESLINES + MINI_HELP_LINES - 2;
+	bufs = (size_t) MIN((unsigned) cCOLS, (sizeof(buf) - 1));
 
 #ifdef HAVE_COLOR
 	fcol(tinrc.col_minihelp);
@@ -518,119 +530,122 @@ show_mini_help (
 
 	switch (level) {
 		case SELECT_LEVEL:
-			snprintf (buf, bufs, _(txt_mini_select_1),
-				printascii (key[0], map_to_local (iKeySelectEnterNextUnreadGrp, &menukeymap.select_nav)),
-				printascii (key[1], map_to_local (iKeySelectGoto, &menukeymap.select_nav)),
-				printascii (key[2], map_to_local (iKeySearchSubjF, &menukeymap.select_nav)),
-				printascii (key[3], map_to_local (iKeySelectCatchup, &menukeymap.select_nav)));
-			center_line (line, FALSE, buf);
-			snprintf (buf, bufs, _(txt_mini_select_2),
-				printascii (key[0], map_to_local (iKeyDown2, &menukeymap.select_nav)),
-				printascii (key[1], map_to_local (iKeyUp2, &menukeymap.select_nav)),
-				printascii (key[2], map_to_local (iKeyHelp, &menukeymap.select_nav)),
-				printascii (key[3], map_to_local (iKeySelectMoveGrp, &menukeymap.select_nav)),
-				printascii (key[4], map_to_local (iKeyQuit, &menukeymap.select_nav)),
-				printascii (key[5], map_to_local (iKeySelectToggleReadDisplay, &menukeymap.select_nav)));
-			center_line (line + 1, FALSE, buf);
-			snprintf (buf, bufs, _(txt_mini_select_3),
-				printascii (key[0], map_to_local (iKeySelectSubscribe, &menukeymap.select_nav)),
-				printascii (key[1], map_to_local (iKeySelectSubscribePat, &menukeymap.select_nav)),
-				printascii (key[2], map_to_local (iKeySelectUnsubscribe, &menukeymap.select_nav)),
-				printascii (key[3], map_to_local (iKeySelectUnsubscribePat, &menukeymap.select_nav)),
-				printascii (key[4], map_to_local (iKeySelectYankActive, &menukeymap.select_nav)));
-			center_line (line + 2, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_select_1),
+				printascii(key[0], map_to_local(iKeySelectEnterNextUnreadGrp, &menukeymap.select_nav)),
+				printascii(key[1], map_to_local(iKeySelectGoto, &menukeymap.select_nav)),
+				printascii(key[2], map_to_local(iKeySearchSubjF, &menukeymap.select_nav)),
+				printascii(key[3], map_to_local(iKeySelectCatchup, &menukeymap.select_nav)));
+			center_line(line, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_select_2),
+				printascii(key[0], map_to_local(iKeyDown2, &menukeymap.select_nav)),
+				printascii(key[1], map_to_local(iKeyUp2, &menukeymap.select_nav)),
+				printascii(key[2], map_to_local(iKeyHelp, &menukeymap.select_nav)),
+				printascii(key[3], map_to_local(iKeySelectMoveGrp, &menukeymap.select_nav)),
+				printascii(key[4], map_to_local(iKeyQuit, &menukeymap.select_nav)),
+				printascii(key[5], map_to_local(iKeySelectToggleReadDisplay, &menukeymap.select_nav)));
+			center_line(line + 1, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_select_3),
+				printascii(key[0], map_to_local(iKeySelectSubscribe, &menukeymap.select_nav)),
+				printascii(key[1], map_to_local(iKeySelectSubscribePat, &menukeymap.select_nav)),
+				printascii(key[2], map_to_local(iKeySelectUnsubscribe, &menukeymap.select_nav)),
+				printascii(key[3], map_to_local(iKeySelectUnsubscribePat, &menukeymap.select_nav)),
+				printascii(key[4], map_to_local(iKeySelectYankActive, &menukeymap.select_nav)));
+			center_line(line + 2, FALSE, buf);
 			break;
+
 		case GROUP_LEVEL:
-			snprintf (buf, bufs, _(txt_mini_group_1),
-				printascii (key[0], map_to_local (iKeyGroupNextUnreadArtOrGrp, &menukeymap.group_nav)),
-				printascii (key[1], map_to_local (iKeySearchSubjF, &menukeymap.group_nav)),
-				printascii (key[2], map_to_local (iKeyGroupKill, &menukeymap.group_nav)));
-			center_line (line, FALSE, buf);
-			snprintf (buf, bufs, _(txt_mini_group_2),
-				printascii (key[0], map_to_local (iKeySearchAuthF, &menukeymap.group_nav)),
-				printascii (key[1], map_to_local (iKeyGroupCatchup, &menukeymap.group_nav)),
-				printascii (key[2], map_to_local (iKeyDown2, &menukeymap.group_nav)),
-				printascii (key[3], map_to_local (iKeyUp2, &menukeymap.group_nav)),
-				printascii (key[4], map_to_local (iKeyGroupMarkThdRead, &menukeymap.group_nav)),
-				printascii (key[5], map_to_local (iKeyGroupListThd, &menukeymap.group_nav)));
-			center_line (line + 1, FALSE, buf);
-			snprintf (buf, bufs, _(txt_mini_group_3),
+			snprintf(buf, bufs, _(txt_mini_group_1),
+				printascii(key[0], map_to_local(iKeyGroupNextUnreadArtOrGrp, &menukeymap.group_nav)),
+				printascii(key[1], map_to_local(iKeySearchSubjF, &menukeymap.group_nav)),
+				printascii(key[2], map_to_local(iKeyGroupKill, &menukeymap.group_nav)));
+			center_line(line, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_group_2),
+				printascii(key[0], map_to_local(iKeySearchAuthF, &menukeymap.group_nav)),
+				printascii(key[1], map_to_local(iKeyGroupCatchup, &menukeymap.group_nav)),
+				printascii(key[2], map_to_local(iKeyDown2, &menukeymap.group_nav)),
+				printascii(key[3], map_to_local(iKeyUp2, &menukeymap.group_nav)),
+				printascii(key[4], map_to_local(iKeyGroupMarkThdRead, &menukeymap.group_nav)),
+				printascii(key[5], map_to_local(iKeyGroupListThd, &menukeymap.group_nav)));
+			center_line(line + 1, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_group_3),
 #ifndef DONT_HAVE_PIPING
-				printascii (key[0], map_to_local (iKeyPipe, &menukeymap.group_nav)),
+				printascii(key[0], map_to_local(iKeyPipe, &menukeymap.group_nav)),
 #endif /* !DONT_HAVE_PIPING */
-				printascii (key[1], map_to_local (iKeyGroupMail, &menukeymap.group_nav)),
+				printascii(key[1], map_to_local(iKeyGroupMail, &menukeymap.group_nav)),
 #ifndef DISABLE_PRINTING
-				printascii (key[2], map_to_local (iKeyPrint, &menukeymap.group_nav)),
+				printascii(key[2], map_to_local(iKeyPrint, &menukeymap.group_nav)),
 #endif /* !DISABLE_PRINTING */
-				printascii (key[3], map_to_local (iKeyQuit, &menukeymap.group_nav)),
-				printascii (key[4], map_to_local (iKeyGroupToggleReadUnread, &menukeymap.group_nav)),
-				printascii (key[5], map_to_local (iKeyGroupSave, &menukeymap.group_nav)),
-				printascii (key[6], map_to_local (iKeyGroupTag, &menukeymap.group_nav)),
-				printascii (key[7], map_to_local (iKeyPost, &menukeymap.group_nav)));
-			center_line (line + 2, FALSE, buf);
+				printascii(key[3], map_to_local(iKeyQuit, &menukeymap.group_nav)),
+				printascii(key[4], map_to_local(iKeyGroupToggleReadUnread, &menukeymap.group_nav)),
+				printascii(key[5], map_to_local(iKeyGroupSave, &menukeymap.group_nav)),
+				printascii(key[6], map_to_local(iKeyGroupTag, &menukeymap.group_nav)),
+				printascii(key[7], map_to_local(iKeyPost, &menukeymap.group_nav)));
+			center_line(line + 2, FALSE, buf);
 			break;
+
 		case THREAD_LEVEL:
-			snprintf (buf, bufs, _(txt_mini_thread_1),
-				printascii (key[0], map_to_local (iKeyThreadReadNextArtOrThread, &menukeymap.thread_nav)),
-				printascii (key[1], map_to_local (iKeyThreadCatchup, &menukeymap.thread_nav)),
-				printascii (key[2], map_to_local (iKeyThreadToggleSubjDisplay, &menukeymap.thread_nav)));
-			center_line (line, FALSE, buf);
-			snprintf (buf, bufs, _(txt_mini_thread_2),
-				printascii (key[0], map_to_local (iKeyHelp, &menukeymap.thread_nav)),
-				printascii (key[1], map_to_local (iKeyDown2, &menukeymap.thread_nav)),
-				printascii (key[2], map_to_local (iKeyUp2, &menukeymap.thread_nav)),
-				printascii (key[3], map_to_local (iKeyQuit, &menukeymap.thread_nav)),
-				printascii (key[4], map_to_local (iKeyThreadTag, &menukeymap.thread_nav)),
-				printascii (key[5], map_to_local (iKeyThreadMarkArtUnread, &menukeymap.thread_nav)));
-			center_line (line + 1, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_thread_1),
+				printascii(key[0], map_to_local(iKeyThreadReadNextArtOrThread, &menukeymap.thread_nav)),
+				printascii(key[1], map_to_local(iKeyThreadCatchup, &menukeymap.thread_nav)),
+				printascii(key[2], map_to_local(iKeyThreadToggleSubjDisplay, &menukeymap.thread_nav)));
+			center_line(line, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_thread_2),
+				printascii(key[0], map_to_local(iKeyHelp, &menukeymap.thread_nav)),
+				printascii(key[1], map_to_local(iKeyDown2, &menukeymap.thread_nav)),
+				printascii(key[2], map_to_local(iKeyUp2, &menukeymap.thread_nav)),
+				printascii(key[3], map_to_local(iKeyQuit, &menukeymap.thread_nav)),
+				printascii(key[4], map_to_local(iKeyThreadTag, &menukeymap.thread_nav)),
+				printascii(key[5], map_to_local(iKeyThreadMarkArtUnread, &menukeymap.thread_nav)));
+			center_line(line + 1, FALSE, buf);
 			break;
+
 		case PAGE_LEVEL:
-			snprintf (buf, bufs, _(txt_mini_page_1),
-				printascii (key[0], map_to_local (iKeyPageNextUnread, &menukeymap.page_nav)),
-				printascii (key[1], map_to_local (iKeySearchSubjF, &menukeymap.page_nav)),
-				printascii (key[2], map_to_local (iKeyPageAutoKill, &menukeymap.page_nav)));
-			center_line (line, FALSE, buf);
-			snprintf (buf, bufs, _(txt_mini_page_2),
-				printascii (key[0], map_to_local (iKeySearchAuthF, &menukeymap.page_nav)),
-				printascii (key[1], map_to_local (iKeySearchBody, &menukeymap.page_nav)),
-				printascii (key[2], map_to_local (iKeyPageCatchup, &menukeymap.page_nav)),
-				printascii (key[3], map_to_local (iKeyPageFollowupQuote, &menukeymap.page_nav)),
-				printascii (key[4], map_to_local (iKeyPageKillThd, &menukeymap.page_nav)));
-			center_line (line + 1, FALSE, buf);
-			snprintf (buf, bufs, _(txt_mini_page_3),
+			snprintf(buf, bufs, _(txt_mini_page_1),
+				printascii(key[0], map_to_local(iKeyPageNextUnread, &menukeymap.page_nav)),
+				printascii(key[1], map_to_local(iKeySearchSubjF, &menukeymap.page_nav)),
+				printascii(key[2], map_to_local(iKeyPageAutoKill, &menukeymap.page_nav)));
+			center_line(line, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_page_2),
+				printascii(key[0], map_to_local(iKeySearchAuthF, &menukeymap.page_nav)),
+				printascii(key[1], map_to_local(iKeySearchBody, &menukeymap.page_nav)),
+				printascii(key[2], map_to_local(iKeyPageCatchup, &menukeymap.page_nav)),
+				printascii(key[3], map_to_local(iKeyPageFollowupQuote, &menukeymap.page_nav)),
+				printascii(key[4], map_to_local(iKeyPageKillThd, &menukeymap.page_nav)));
+			center_line(line + 1, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_page_3),
 #ifndef DONT_HAVE_PIPING
-				printascii (key[0], map_to_local (iKeyPipe, &menukeymap.page_nav)),
+				printascii(key[0], map_to_local(iKeyPipe, &menukeymap.page_nav)),
 #endif /* !DONT_HAVE_PIPING */
-				printascii (key[1], map_to_local (iKeyPageMail, &menukeymap.page_nav)),
+				printascii(key[1], map_to_local(iKeyPageMail, &menukeymap.page_nav)),
 #ifndef DISABLE_PRINTING
-				printascii (key[2], map_to_local (iKeyPrint, &menukeymap.page_nav)),
+				printascii(key[2], map_to_local(iKeyPrint, &menukeymap.page_nav)),
 #endif /* !DISABLE_PRINTING */
-				printascii (key[3], map_to_local (iKeyQuit, &menukeymap.page_nav)),
-				printascii (key[4], map_to_local (iKeyPageReplyQuote, &menukeymap.page_nav)),
-				printascii (key[5], map_to_local (iKeyPageSave, &menukeymap.page_nav)),
-				printascii (key[6], map_to_local (iKeyPageTag, &menukeymap.page_nav)),
-				printascii (key[7], map_to_local (iKeyPost, &menukeymap.page_nav)));
-			center_line (line + 2, FALSE, buf);
+				printascii(key[3], map_to_local(iKeyQuit, &menukeymap.page_nav)),
+				printascii(key[4], map_to_local(iKeyPageReplyQuote, &menukeymap.page_nav)),
+				printascii(key[5], map_to_local(iKeyPageSave, &menukeymap.page_nav)),
+				printascii(key[6], map_to_local(iKeyPageTag, &menukeymap.page_nav)),
+				printascii(key[7], map_to_local(iKeyPost, &menukeymap.page_nav)));
+			center_line(line + 2, FALSE, buf);
 			break;
 
 		case INFO_PAGER:
-			snprintf (buf, bufs, _(txt_mini_info_1),
-				printascii (key[0], map_to_local (iKeyUp, &menukeymap.info_nav)),
-				printascii (key[1], map_to_local (iKeyDown, &menukeymap.info_nav)),
-				printascii (key[2], map_to_local (iKeyPageUp, &menukeymap.info_nav)),
-				printascii (key[3], map_to_local (iKeyPageDown, &menukeymap.info_nav)),
-				printascii (key[4], map_to_local (iKeyFirstPage, &menukeymap.info_nav)),
-				printascii (key[5], map_to_local (iKeyLastPage, &menukeymap.info_nav)));
-			center_line (line, FALSE, buf);
-			snprintf (buf, bufs, _(txt_mini_info_2),
-				printascii (key[0], map_to_local (iKeySearchSubjF, &menukeymap.info_nav)),
-				printascii (key[1], map_to_local (iKeySearchSubjB, &menukeymap.info_nav)),
-				printascii (key[2], map_to_local (iKeyQuit, &menukeymap.info_nav)));
-			center_line (line + 1, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_info_1),
+				printascii(key[0], map_to_local(iKeyUp, &menukeymap.info_nav)),
+				printascii(key[1], map_to_local(iKeyDown, &menukeymap.info_nav)),
+				printascii(key[2], map_to_local(iKeyPageUp, &menukeymap.info_nav)),
+				printascii(key[3], map_to_local(iKeyPageDown, &menukeymap.info_nav)),
+				printascii(key[4], map_to_local(iKeyFirstPage, &menukeymap.info_nav)),
+				printascii(key[5], map_to_local(iKeyLastPage, &menukeymap.info_nav)));
+			center_line(line, FALSE, buf);
+			snprintf(buf, bufs, _(txt_mini_info_2),
+				printascii(key[0], map_to_local(iKeySearchSubjF, &menukeymap.info_nav)),
+				printascii(key[1], map_to_local(iKeySearchSubjB, &menukeymap.info_nav)),
+				printascii(key[2], map_to_local(iKeyQuit, &menukeymap.info_nav)));
+			center_line(line + 1, FALSE, buf);
 			break;
 
 		default: /* should not happen */
-			error_message (_(txt_error_unknown_dlevel));
+			error_message(_(txt_error_unknown_dlevel));
 			break;
 	}
 #ifdef HAVE_COLOR
@@ -640,10 +655,10 @@ show_mini_help (
 
 
 void
-toggle_mini_help (
+toggle_mini_help(
 	int level)
 {
-	tinrc.beginner_level = !tinrc.beginner_level;
-	set_win_size (&cLINES, &cCOLS);
-	show_mini_help (level);
+	tinrc.beginner_level = bool_not(tinrc.beginner_level);
+	set_noteslines(cLINES);
+	show_mini_help(level);
 }
