@@ -3,10 +3,10 @@
  *  Module    : init.c
  *  Author    : I. Lea
  *  Created   : 1991-04-01
- *  Updated   : 2001-11-10
+ *  Updated   : 2003-01-22
  *  Notes     :
  *
- * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Iain Lea.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -98,15 +95,19 @@ char mail_active_file[PATH_LEN];
 char mail_news_user[LEN];		/* mail new news to this user address */
 char mailbox[PATH_LEN];			/* system mailbox for each user */
 char mailer[PATH_LEN];			/* mail program */
-char mailgroups_file[PATH_LEN];
+#ifdef HAVE_MH_MAIL_HANDLING
+	char mailgroups_file[PATH_LEN];
+#endif /* HAVE_MH_MAIL_HANDLING */
 char **news_headers_to_display_array;	/* array of which headers to display */
 char **news_headers_to_not_display_array;	/* array of which headers to not display */
 char newnewsrc[PATH_LEN];
 char news_active_file[PATH_LEN];
 char newsgroups_file[PATH_LEN];
 char newsrc[PATH_LEN];
-char novrootdir[PATH_LEN];		/* root directory of nov index files */
-char novfilename[PATH_LEN];		/* file name of a single nov index files */
+#ifndef NNTP_ONLY
+	char novrootdir[PATH_LEN];		/* root directory of nov index files */
+	char novfilename[PATH_LEN];		/* file name of a single nov index files */
+#endif /* !NNTP_ONLY */
 char page_header[LEN];			/* page header of pgm name and version */
 char posted_info_file[PATH_LEN];
 char posted_msgs_file[PATH_LEN];
@@ -119,7 +120,7 @@ char subscriptions_file[PATH_LEN];
 char txt_help_bug_report[LEN];		/* address to send bug reports to */
 char userid[PATH_LEN];
 
-char proc_ch_default;			/* set in change_config_file () */
+char proc_ch_default;			/* set in change_config_file() */
 
 #ifdef VMS
 	char rcdir_asfile[PATH_LEN];	/* rcdir expressed as dev:[dir]tin.dir, for stat() */
@@ -128,6 +129,7 @@ char proc_ch_default;			/* set in change_config_file () */
 int groupname_len = 0;			/* 'runtime' copy of groupname_max_len */
 int hist_last[HIST_MAXNUM + 1];
 int hist_pos[HIST_MAXNUM + 1];
+int i_key_search_last;			/* for repeated search */
 int iso2asc_supported;			/* Convert ISO-Latin1 to Ascii */
 int num_headers_to_display;		/* num headers to display -- swp */
 int num_headers_to_not_display;		/* num headers to not display -- swp */
@@ -138,9 +140,10 @@ int xmouse, xrow, xcol;			/* xterm button pressing information */
 
 #ifdef HAVE_COLOR
 	t_bool use_color;		/* enables/disables ansi-color support under linux-console and color-xterm */
-	t_bool word_highlight;		/* word highlighting on/off */
 #endif /* HAVE_COLOR */
 
+t_bool word_highlight;		/* word highlighting on/off */
+
 pid_t process_id;			/* Useful to have around for .suffixes */
 mode_t real_umask;
 
@@ -155,7 +158,7 @@ t_bool got_sig_pipe = FALSE;
 t_bool filtered_articles;		/* locally killed / auto-selected articles */
 t_bool list_active;
 t_bool newsrc_active;
-t_bool post_article_and_exit;		/* quick post of an article then exit (elm like) */
+t_bool post_article_and_exit;		/* quick post of an article then exit(elm like) */
 t_bool post_postponed_and_exit;		/* post postponed articles and exit */
 t_bool reread_active_for_posted_arts;
 t_bool read_local_newsgroups_file;	/* read newsgroups file locally or via NNTP */
@@ -172,6 +175,7 @@ t_bool xref_supported = TRUE;
 #endif /* LOCAL_CHARSET */
 #ifdef NNTP_ABLE
 	t_bool force_auth_on_conn_open = FALSE;	/* authenticate on connection startup */
+	unsigned short nntp_tcp_port;
 #endif /* NNTP_ABLE */
 
 /* Currently active menu parameters */
@@ -184,16 +188,19 @@ char *input_history[HIST_MAXNUM + 1][HIST_SIZE + 1];
 	struct utsname system_info;
 #endif /* HAVE_SYS_UTSNAME_H */
 
+#if 0
 #ifndef M_AMIGA
 	static struct passwd *myentry;
 	static struct passwd pwdentry;
 #endif /* !M_AMIGA */
+#endif /* 0 */
 
 struct regex_cache
-					strip_re_regex, strip_was_regex,
-					uubegin_regex, uubody_regex,
-					url_regex, mail_regex, news_regex,
-					shar_regex
+		strip_re_regex, strip_was_regex,
+		uubegin_regex, uubody_regex,
+		url_regex, mail_regex, news_regex,
+		shar_regex,
+		slashes_regex, stars_regex, underscores_regex, strokes_regex
 #ifdef HAVE_COLOR
 		, quote_regex, quote_regex2, quote_regex3
 #endif /* HAVE_COLOR */
@@ -226,8 +233,8 @@ struct t_config tinrc = {
 #endif /* !DISABLE_PRINTING */
 	"1-.",	/* default_range_group */
 	"1-.",	/* default_range_select */
-	"0-.",	/* default_range_thread */
-	"",		/* default_regex_pattern */
+	"1-.",	/* default_range_thread */
+	"",		/* default_pattern */
 	"",		/* default_repost_group */
 	"savefile.tin",		/* default_save_file */
 	"",		/* default_search_art */
@@ -239,27 +246,41 @@ struct t_config tinrc = {
 	"",		/* default_shell_command */
 	"In article %M you wrote:",		/* mail_quote_format */
 	"",		/* maildir */
+#ifdef SCO_UNIX
+	2,			/* mailbox_format = MMDF */
+#else
+	0,			/* mailbox_format = MBOXO */
+#endif /* SCO_UNIX */
 	"",		/* mail_address */
+#ifndef CHARSET_CONVERSION
 	"",		/* mm_charset, defaults to $MM_CHARSET */
-#ifdef CHARSET_CONVERSION
-	"",		/* mm_local_charset, defaults to mm_charset */
-	0,			/* mm_network_charset, defaults to US-ASCII */
-#endif /* CHARSET_CONVERSION */
+#else
+	-1,		/* mm_network_charset, defaults to $MM_CHARSET */
+#endif /* !CHARSET_CONVERSION */
+	"",		/* mm_local_charset, display charset */
+#ifdef HAVE_ICONV_OPEN_TRANSLIT
+	FALSE,	/* translit */
+#endif /* HAVE_ICONV_OPEN_TRANSLIT */
 	"Newsgroups Followup-To Summary Keywords X-Comment-To",		/* news_headers_to_display */
 	"",		/* news_headers_to_not_display */
 	"%F wrote:",		/* news_quote_format */
 	DEFAULT_COMMENT,	/* quote_chars */
 #ifdef HAVE_COLOR
 	"",		/* quote_regex */
-	"",		/* quote_regex 2nd level*/
+	"",		/* quote_regex 2nd level */
 	"",		/* quote_regex >= 3rd level */
 #endif /* HAVE_COLOR */
+	"",		/* slashes_regex */
+	"",		/* stars_regex */
+	"",		/* underscores_regex */
+	"",		/* strokes_regex */
 	"",		/* sigfile */
 	"",		/* strip_re_regex */
 	"",		/* strip_was_regex */
 	"",		/* savedir */
 	"",		/* spamtrap_warning_addresses */
-	"In %G %F wrote:",				/* xpost_quote_format */
+	DEFAULT_URL_HANDLER,	/* url_handler */
+	"In %G %F wrote:",			/* xpost_quote_format */
 	DEFAULT_FILTER_DAYS,			/* filter_days */
 	FILTER_SUBJ_CASE_SENSITIVE,		/* default_filter_kill_header */
 	FILTER_SUBJ_CASE_SENSITIVE,		/* default_filter_select_header */
@@ -273,12 +294,18 @@ struct t_config tinrc = {
 	MIME_ENCODING_7BIT,		/* post_mime_encoding */
 	POST_PROC_NONE,			/* post_process */
 	REREAD_ACTIVE_FILE_SECS,	/* reread_active_file_secs */
+	1,		/* scroll_lines */
 	SHOW_FROM_NAME,				/* show_author */
 	SORT_ARTICLES_BY_DATE_ASCEND,		/* sort_article_type */
 	SORT_THREADS_BY_SCORE_DESCEND,		/* sort_threads_type */
-	BOGUS_ASK,		/* strip_bogus */
-	THREAD_MAX,		/* thread_articles */
+	BOGUS_SHOW,		/* strip_bogus */
+	THREAD_BOTH,		/* thread_articles */
+	THREAD_SCORE_MAX,	/* thread_score */
 	0,		/* Default to wildmat, not regex */
+	-50,		/* score_limit_kill */
+	50,		/* score_limit_select */
+	-100,		/* score_kill */
+	100,		/* score_select */
 #ifdef HAVE_COLOR
 	0,		/* col_back (initialised later) */
 	0,		/* col_from (initialised later) */
@@ -290,6 +317,8 @@ struct t_config tinrc = {
 	0,		/* col_normal (initialised later) */
 	0,		/* col_markdash (initialised later) */
 	0,		/* col_markstar (initialised later) */
+	0,		/* col_markslash (initialised later) */
+	0,		/* col_markstroke (initialised later) */
 	0,		/* col_message (initialised later) */
 	0,		/* col_newsheaders (initialised later) */
 	0,		/* col_quote (initialised later) */
@@ -301,7 +330,10 @@ struct t_config tinrc = {
 	0,		/* col_text (initialised later) */
 	0,		/* col_title (initialised later) */
 	2,		/* word_h_display_marks */
+#endif /* HAVE_COLOR */
 	TRUE,		/* word_highlight */
+	0,		/* wrap_column */
+#ifdef HAVE_COLOR
 	FALSE,		/* use_color */
 #endif /* HAVE_COLOR */
 	TRUE,		/* add_posted_to_filter */
@@ -316,15 +348,13 @@ struct t_config tinrc = {
 	TRUE,		/* beginner_level */
 	FALSE,		/* cache_overview_files */
 	FALSE,		/* catchup_read_groups */
-	TRUE,		/* confirm_action */
-	TRUE,		/* confirm_to_quit */
+	4,		/* confirm_choice */
 #ifdef USE_INVERSE_HACK
 	TRUE,		/* draw_arrow */
 #else
 	FALSE,		/* draw_arrow */
 #endif /* USE_INVERSE_HACK */
 	FALSE,		/* force_screen_redraw */
-	TRUE,		/* full_page_scroll */
 	TRUE,		/* group_catchup_on_exit */
 	FALSE,		/* info_in_last_line */
 #ifdef USE_INVERSE_HACK
@@ -346,15 +376,8 @@ struct t_config tinrc = {
 #endif /* !DISABLE_PRINTING */
 	FALSE,		/* process_only_unread */
 	FALSE,		/* prompt_followupto */
-	TRUE,		/* quote_empty_lines */
-	FALSE,		/* quote_signatures */
-#ifdef HAVE_MMDF_MAILER
-	TRUE,		/* save_to_mmdf_mailbox */
-#else
-	FALSE,		/* save_to_mmdf_mailbox */
-#endif /* HAVE_MMDF_MAILER */
+	QUOTE_COMPRESS|QUOTE_EMPTY,	/* quote_style */
 	TRUE,		/* show_description */
-	FALSE,		/* show_last_line_prev_page */
 	TRUE,		/* show_lines */
 	TRUE,		/* show_only_unread_arts */
 	FALSE,		/* show_only_unread_groups */
@@ -371,21 +394,19 @@ struct t_config tinrc = {
 #endif /* M_UNIX */
 	TRUE,		/* strip_blanks */
 	FALSE,		/* strip_newsrc */
-	FALSE,		/* tab_after_X_selection */
 	TRUE,		/* tab_goto_next_unread */
 	FALSE,		/* tex2iso_conv */
 	TRUE,		/* thread_catchup_on_exit */
 	TRUE,		/* unlink_article */
 	"",		/* inews_prog */
-	FALSE,		/* use_getart_limit */
 	FALSE,		/* use_mailreader_i */
 	FALSE,		/* use_mouse */
 #ifdef HAVE_KEYPAD
 	FALSE,		/* use_keypad */
 #endif /* HAVE_KEYPAD */
 #ifdef HAVE_METAMAIL
-	TRUE,		/* ask_for_metamail */
-	TRUE,		/* use_metamail */
+	FALSE,		/* ask_for_metamail */
+	FALSE,		/* use_metamail */
 #endif /* HAVE_METAMAIL */
 	FALSE,		/* default_filter_kill_case */
 	FALSE,		/* default_filter_kill_expire */
@@ -413,6 +434,8 @@ static const struct {
 	{ &tinrc.col_invers_fg,   7 },
 	{ &tinrc.col_markdash,   13 },
 	{ &tinrc.col_markstar,   11 },
+	{ &tinrc.col_markslash,  14 },
+	{ &tinrc.col_markstroke, 12 },
 	{ &tinrc.col_message,     6 },
 	{ &tinrc.col_minihelp,    3 },
 	{ &tinrc.col_newsheaders, 9 },
@@ -428,30 +451,33 @@ static const struct {
 };
 
 static void
-preinit_colors (
+preinit_colors(
 	void)
 {
 	size_t n;
 
-	for (n = 0; n < SIZEOF(our_colors); n++)
+	for (n = 0; n < ARRAY_SIZE(our_colors); n++)
 		*(our_colors[n].colorp) = DFT_INIT;
 }
 
+
 void
-postinit_colors (
+postinit_colors(
 	void)
 {
 	size_t n;
 
-	for (n = 0; n < SIZEOF(our_colors); n++) {
+	for (n = 0; n < ARRAY_SIZE(our_colors); n++) {
 		if (*(our_colors[n].colorp) == DFT_INIT) {
 			switch (our_colors[n].color_dft) {
 				case DFT_FORE:
 					*(our_colors[n].colorp) = default_fcol;
 					break;
+
 				case DFT_BACK:
 					*(our_colors[n].colorp) = default_bcol;
 					break;
+
 				default:
 					*(our_colors[n].colorp) = our_colors[n].color_dft;
 					break;
@@ -462,22 +488,21 @@ postinit_colors (
 }
 #endif /* HAVE_COLOR */
 
-#ifdef NNTP_ABLE
-	unsigned short nntp_tcp_port;
-#endif /* NNTP_ABLE */
-
 
 /*
  * Get users home directory, userid, and a bunch of other stuff!
  */
 void
-init_selfinfo (
+init_selfinfo(
 	void)
 {
 	char *ptr;
 	const char *cptr;
 	FILE *fp;
 	struct stat sb;
+#ifndef M_AMIGA
+	struct passwd *myentry;
+#endif /* !M_AMIGA */
 
 	host_name[0] = '\0';
 	domain_name[0] = '\0';
@@ -492,79 +517,83 @@ init_selfinfo (
 #	endif /* HAVE_SYS_UTSNAME_H */
 #endif /* !M_AMIGA */
 
-	if ((cptr = get_host_name()) != (char *) 0)
-		strcpy (host_name, cptr);
+	if ((cptr = get_host_name()) != NULL)
+		strcpy(host_name, cptr);
 
 #ifdef DOMAIN_NAME
-	if ((cptr = get_domain_name()) != (char *) 0)
-		strcpy (domain_name, cptr);
+	if ((cptr = get_domain_name()) != NULL)
+		strcpy(domain_name, cptr);
 #endif /* DOMAIN_NAME */
 
 #ifdef HAVE_GETHOSTBYNAME
 	if (domain_name[0] == '\0') {
 		cptr = ((host_name[0] == '\0') ? get_fqdn((char *) 0) : get_fqdn(host_name));
-		if (cptr != (char *)NULL)
-			strcpy (domain_name, cptr);
+		if (cptr != (char *) 0)
+			strcpy(domain_name, cptr);
 	}
 #endif /* HAVE_GETHOSTBYNAME */
 
-	process_id = getpid ();
+	process_id = getpid();
 
-	real_umask = umask (0);
-	(void) umask (real_umask);
+	real_umask = umask(0);
+	(void) umask(real_umask);
 
 #if defined(HAVE_SETLOCALE) && defined(LC_ALL) && !defined(NO_LOCALE)
-	setlocale (LC_ALL, "");
+	if (!setlocale(LC_ALL, "")) {
+		/*
+		 * TODO: issue a warning here like
+		 *       "Can't set the specified locale! Check $LANG, $LC_CTYPE, $LC_ALL"
+		 */
+		;
+	}
 #endif /* HAVE_SETLOCALE && LC_ALL && !NO_LOCALE */
 
-
 /* FIXME: move to get_user_name() [header.c] */
 #ifndef M_AMIGA
-	if ((myentry = getpwuid (getuid ())) != (struct passwd *) 0) {
-		memcpy (&pwdentry, myentry, sizeof (struct passwd));
-		myentry = &pwdentry;
+#	ifndef VMS
+	if ((myentry = getpwuid(getuid())) == NULL) {
+		error_message(_(txt_error_passwd_missing));
+		giveup();
 	}
-#	ifdef VMS
+#	else
 	/* TODO: get_user_name() entirely reles on $USER */
-	if (((ptr = getlogin ()) != (char *) 0) && strlen (ptr))
-		myentry = getpwnam (ptr);
-#	endif /* VMS */
-
-	if (myentry != (struct passwd *) 0)
-		strcpy (userid, myentry->pw_name);
+	if (((ptr = getlogin()) != NULL) && strlen(ptr))
+		myentry = getpwnam(ptr);
 	else {
-		error_message (_(txt_error_passwd_missing));
-		tin_done (EXIT_FAILURE);
+		error_message(_(txt_error_passwd_missing));
+		giveup();
 	}
+#	endif /* !VMS */
+
+	strcpy(userid, myentry->pw_name);
 
 #	ifdef VMS
-	lower (userid);
+	lower(userid);
 #	endif /* VMS */
 
-	if ((ptr = getenv ("TIN_HOMEDIR")) != (char *) 0) {
-		my_strncpy (homedir, ptr, sizeof (homedir));
-	} else if ((ptr = getenv ("HOME")) != (char *) 0) {
-		my_strncpy (homedir, ptr, sizeof (homedir));
+	if ((ptr = getenv("TIN_HOMEDIR")) != NULL) {
+		my_strncpy(homedir, ptr, sizeof(homedir));
+	} else if ((ptr = getenv("HOME")) != NULL) {
+		my_strncpy(homedir, ptr, sizeof(homedir));
 	} else if (!myentry) {
-		strcpy (homedir, "/tmp");
+		strcpy(homedir, "/tmp");
 	} else
-		my_strncpy (homedir, myentry->pw_dir, sizeof (homedir));
+		my_strncpy(homedir, myentry->pw_dir, sizeof(homedir));
 
 #else
 	/* TODO: get_user_name() uses $USER, not $USERNAME */
-	if ((ptr = getenv ("USERNAME")) != (char *) 0) {
-		my_strncpy (userid, ptr, sizeof (userid));
-	} else {
-		error_message (_(txt_env_var_not_found), "USERNAME");
-		tin_done (EXIT_FAILURE);
+	if ((ptr = getenv("USERNAME")) == NULL) {
+		error_message(_(txt_env_var_not_found), "USERNAME");
+		giveup();
 	}
-	/* TODO: why not also check for TIN_HOME? */
-	if ((ptr = getenv ("HOME")) != (char *) 0) {
-		my_strncpy (homedir, ptr, sizeof (homedir));
-	} else {
-		error_message (_(txt_env_var_not_found), "HOME");
-		tin_done (EXIT_FAILURE);
+	my_strncpy(userid, ptr, sizeof(userid));
+
+	/* TODO: why not also check for $TIN_HOMEDIR? */
+	if ((ptr = getenv("HOME")) == NULL) {
+		error_message(_(txt_env_var_not_found), "HOME");
+		giveup();
 	}
+	my_strncpy(homedir, ptr, sizeof(homedir));
 #endif /* !M_AMIGA */
 
 	cmdline_nntpserver[0] = '\0';
@@ -577,7 +606,7 @@ init_selfinfo (
 	disable_sender = FALSE;
 #endif /* MAC_OS_X */
 	filtered_articles = FALSE;
-	iso2asc_supported = atoi (get_val ("ISO2ASC", DEFAULT_ISO2ASC));
+	iso2asc_supported = atoi(get_val("ISO2ASC", DEFAULT_ISO2ASC));
 	if (iso2asc_supported > NUM_ISO_TABLES)
 		iso2asc_supported = 0;
 	list_active = FALSE;
@@ -592,30 +621,31 @@ init_selfinfo (
 	force_reread_active_file = TRUE;
 	reread_active_for_posted_arts = TRUE;
 	batch_mode = FALSE;
-	check_for_new_newsgroups = !batch_mode;
+	check_for_new_newsgroups = TRUE;
 	wildcard_func = wildmat;
 #ifdef HAVE_METAMAIL
 #	ifdef M_AMIGA
 		/* for all those AmigaElm users ... ;-) */
-		tinrc.use_metamail = (getenv ("NoMetaMail") != NULL) ? TRUE : FALSE;
+		tinrc.use_metamail = (getenv("NoMetaMail") != NULL) ? TRUE : FALSE;
 #	endif /* M_AMIGA */
 #endif /* HAVE_METAMAIL */
 
 #ifdef HAVE_COLOR
 	preinit_colors();
 	use_color = FALSE;
-	word_highlight = TRUE;
 #endif /* HAVE_COLOR */
 
+	word_highlight = TRUE;
+
 	index_maildir[0] = '\0';
 	index_newsdir[0] = '\0';
 	index_savedir[0] = '\0';
 	newsrc[0] = '\0';
 
-	sprintf (page_header, "%s %s release %s (\"%s\") [%s%s]",
+	sprintf(page_header, "%s %s release %s (\"%s\") [%s%s]",
 		tin_progname, VERSION, RELEASEDATE, RELEASENAME, OSNAME,
 		(iso2asc_supported >= 0 ? " ISO2ASC" : ""));
-	sprintf (cvers, txt_copyright_notice, page_header);
+	sprintf(cvers, txt_copyright_notice, page_header);
 
 	default_organization[0] = '\0';
 	proc_ch_default = 'n';
@@ -623,7 +653,7 @@ init_selfinfo (
 	news_headers_to_not_display_array = NULL;
 
 	/* TODO: nuke $BUG_ADDRESS entirely? */
-	strcpy (bug_addr, get_val("BUG_ADDRESS", BUG_REPORT_ADDRESS));
+	strcpy(bug_addr, get_val("BUG_ADDRESS", BUG_REPORT_ADDRESS));
 
 	bug_nntpserver1[0] = '\0';
 	bug_nntpserver2[0] = '\0';
@@ -637,7 +667,7 @@ init_selfinfo (
 	 */
 
 #ifdef INEWSDIR
-	strcpy (inewsdir, INEWSDIR);
+	strcpy(inewsdir, INEWSDIR);
 #else
 	inewsdir[0] = '\0';
 #endif /* INEWSDIR */
@@ -649,16 +679,16 @@ init_selfinfo (
 #endif /* apollo */
 
 #ifdef USE_INN_NNTPLIB
-	ptr = GetConfigValue (_CONF_ORGANIZATION);
-	if (ptr != (char *) 0)
-		my_strncpy (default_organization, ptr, sizeof (default_organization));
+	ptr = GetConfigValue(_CONF_ORGANIZATION);
+	if (ptr != NULL)
+		my_strncpy(default_organization, ptr, sizeof(default_organization));
 #endif /* USE_INN_NNTPLIB */
 
 #ifndef NNTP_ONLY
-	strcpy (libdir, get_val("TIN_LIBDIR", NEWSLIBDIR)); /* moved inside ifdef */
-	strcpy (novrootdir, get_val ("TIN_NOVROOTDIR", NOVROOTDIR));
-	strcpy (novfilename, get_val ("TIN_NOVFILENAME", OVERVIEW_FILE));
-	strcpy (spooldir, get_val ("TIN_SPOOLDIR", SPOOLDIR));
+	strcpy(libdir, get_val("TIN_LIBDIR", NEWSLIBDIR)); /* moved inside ifdef */
+	strcpy(novrootdir, get_val("TIN_NOVROOTDIR", NOVROOTDIR));
+	strcpy(novfilename, get_val("TIN_NOVFILENAME", OVERVIEW_FILE));
+	strcpy(spooldir, get_val("TIN_SPOOLDIR", SPOOLDIR));
 #endif /* !NNTP_ONLY */
 	/* clear news_active_file, active_time_file, newsgroups_file */
 	news_active_file[0] = '\0';
@@ -669,7 +699,7 @@ init_selfinfo (
 	/*
 	 * Setup default keymaps
 	 */
-	build_keymaps ();
+	build_keymaps();
 
 	/*
 	 * read the global site config file to override some default
@@ -678,165 +708,187 @@ init_selfinfo (
 	(void) read_site_config();
 
 	/*
-	 * the site_confog-file was the last chance to set the domainname
-	 *  if it's still unset exit tin.
+	 * the site_config-file was the last chance to set the domainname
+	 * if it's still unset exit tin.
 	 */
 	if (domain_name[0] == '\0') {
-		error_message (txt_error_no_domain_name);
-		tin_done(EXIT_FAILURE);
+		error_message(txt_error_no_domain_name);
+		giveup();
+		/*
+		 * TODO: ask the user to fall into no posting mode (can_post=FALSE)
+		 *       instead of exeting.
+		 */
 	}
 
 	/*
-	 * only set the following variables, if they weren't set from
-	 * within read_site_config()
+	 * only set the following variables if they weren't set from within
+	 * read_site_config()
+	 *
+	 * TODO: do we really want that read_site_config() overwrites
+	 * values given in env-vars? ($MM_CHARSET, $TIN_ACTIVEFILE)
 	 */
-	if (!*news_active_file)
-		joinpath (news_active_file, libdir, get_val ("TIN_ACTIVEFILE", ACTIVE_FILE));
+	if (!*news_active_file) /* TODO: really prepend libdir here in case of $TIN_ACTIVEFILE is set? */
+		joinpath(news_active_file, libdir, get_val("TIN_ACTIVEFILE", ACTIVE_FILE));
 	if (!*active_times_file)
-		joinpath (active_times_file, libdir, ACTIVE_TIMES_FILE);
+		joinpath(active_times_file, libdir, ACTIVE_TIMES_FILE);
 	if (!*newsgroups_file)
-		joinpath (newsgroups_file, libdir, NEWSGROUPS_FILE);
+		joinpath(newsgroups_file, libdir, NEWSGROUPS_FILE);
 	if (!*subscriptions_file)
-		joinpath (subscriptions_file, libdir, SUBSCRIPTIONS_FILE);
+		joinpath(subscriptions_file, libdir, SUBSCRIPTIONS_FILE);
 	if (!*default_organization) {
 		char buf[LEN];
 
-		joinpath (buf, libdir, "organization");
+		joinpath(buf, libdir, "organization");
 		if ((fp = fopen(buf, "r")) != NULL) {
-			if (fgets (buf, (int) sizeof (buf), fp) != (char *) 0) {
-				ptr = strrchr (buf, '\n');
-				if (ptr != (char *) 0)
+			if (fgets(buf, (int) sizeof(buf), fp) != NULL) {
+				ptr = strrchr(buf, '\n');
+				if (ptr != NULL)
 					*ptr = '\0';
 			}
-			fclose (fp);
-			my_strncpy (default_organization, buf, sizeof (default_organization));
+			fclose(fp);
+			my_strncpy(default_organization, buf, sizeof(default_organization));
 		}
 	}
 
 	/*
 	 * Formerly get_mm_charset(), read_site_config() may set mm_charset
 	 */
-	STRCPY(tinrc.mm_charset, get_val("MM_CHARSET", MM_CHARSET));
+#ifndef CHARSET_CONVERSION
+	if (!*tinrc.mm_charset)
+		STRCPY(tinrc.mm_charset, get_val("MM_CHARSET", MM_CHARSET));
+#else
+	if (tinrc.mm_network_charset < 0) {
+		size_t space = 255;
 
+		ptr = my_malloc(space + 1);
+		strcpy(ptr, "mm_network_charset=");
+		space -= strlen(ptr);
+		strncat(ptr, get_val("MM_CHARSET", MM_CHARSET), space);
+		if ((space -= strlen(ptr)) > 0) {
+			strncat(ptr, "\n", space);
+			match_list(ptr, "mm_network_charset=", txt_mime_charsets, NUM_MIME_CHARSETS, &tinrc.mm_network_charset);
+		}
+		free(ptr);
+	}
+#endif /* !CHARSET_CONVERSION */
+
+#ifdef TIN_DEFAULTS_DIR
+	joinpath(global_attributes_file, TIN_DEFAULTS_DIR, ATTRIBUTES_FILE);
+	joinpath(global_config_file, TIN_DEFAULTS_DIR, CONFIG_FILE);
+#else
 	/* read_site_config() might have changed the value of libdir */
-	/* FIXME: we'd better use TIN_DEFAULTS_DIR instead of TIN_LIBDIR here */
-	joinpath (global_attributes_file, libdir, ATTRIBUTES_FILE);
-	joinpath (global_config_file, libdir, CONFIG_FILE);
+	joinpath(global_attributes_file, libdir, ATTRIBUTES_FILE);
+	joinpath(global_config_file, libdir, CONFIG_FILE);
+#endif /* TIN_DEFAULTS_DIR */
 
 #ifdef VMS
-	joindir (rcdir, homedir, RCDIR); /* we're naming a directory here */
-	joinpath (rcdir_asfile, homedir, RCDIR);	/* for stat() */
+	joindir(rcdir, homedir, RCDIR); /* we're naming a directory here */
+	joinpath(rcdir_asfile, homedir, RCDIR);	/* for stat() */
 	strcat(rcdir_asfile, ".DIR");
-	if (stat (rcdir_asfile, &sb) == -1)
+	if (stat(rcdir_asfile, &sb) == -1)
 #else
-	joinpath (rcdir, homedir, RCDIR);
-	if (stat (rcdir, &sb) == -1)
+	joinpath(rcdir, homedir, RCDIR);
+	if (stat(rcdir, &sb) == -1)
 #endif /* VMS */
 	{
 		created_rcdir = TRUE;
-		my_mkdir (rcdir, (mode_t)(S_IRWXU));
+		my_mkdir(rcdir, (mode_t) (S_IRWXU));
 	}
-#if defined(M_UNIX) || defined (M_AMIGA) || defined(VMS)
-	strcpy (tinrc.mailer_format, MAILER_FORMAT);
+#if defined(M_UNIX) || defined(M_AMIGA) || defined(VMS)
+	strcpy(tinrc.mailer_format, MAILER_FORMAT);
 #else
-	strcpy (tinrc.mailer_format, mailer);
+	strcpy(tinrc.mailer_format, mailer);
 #endif /* M_UNIX || M_AMIGA || VMS */
 #ifndef DISABLE_PRINTING
-	strcpy (tinrc.printer, DEFAULT_PRINTER);
+	strcpy(tinrc.printer, DEFAULT_PRINTER);
 #	ifdef M_AMIGA
 	if (tin_bbs_mode)
 		strcpy(tinrc.printer, DEFAULT_BBS_PRINTER);
 #	endif /* M_AMIGA */
 #endif /* !DISABLE_PRINTING */
-#if defined(NNTP_ABLE) || defined(NNTP_ONLY)
-	strcpy (tinrc.inews_prog, DEFAULT_INEWS_PROG);
-#endif /* NNTP_ABLE) || NNTP_ONLY */
-	strcpy (mailer, get_val (ENV_VAR_MAILER, DEFAULT_MAILER));
-	joinpath (article, homedir, TIN_ARTICLE_NAME);
+	strcpy(tinrc.inews_prog, PATH_INEWS);
+	strcpy(mailer, get_val(ENV_VAR_MAILER, DEFAULT_MAILER));
+	joinpath(article, homedir, TIN_ARTICLE_NAME);
 #ifdef APPEND_PID
-	sprintf (article + strlen(article), ".%d", (int) process_id);
+	sprintf(article + strlen(article), ".%d", (int) process_id);
 #endif /* APPEND_PID */
-	joinpath (dead_article, homedir, "dead.article");
-	joinpath (dead_articles, homedir, "dead.articles");
+	joinpath(dead_article, homedir, "dead.article");
+	joinpath(dead_articles, homedir, "dead.articles");
 	JOINPATH(tinrc.maildir, homedir, DEFAULT_MAILDIR);
 	JOINPATH(tinrc.savedir, homedir, DEFAULT_SAVEDIR);
-	joinpath (tinrc.sigfile, homedir, ".Sig");
-	joinpath (default_signature, homedir, ".signature");
+	joinpath(tinrc.sigfile, homedir, ".Sig");
+	joinpath(default_signature, homedir, ".signature");
 
 	if (!index_newsdir[0])
-		JOINPATH(index_newsdir, get_val ("TIN_INDEX_NEWSDIR", rcdir), INDEX_NEWSDIR);
-#if 0
-	 else {
-		if (stat (index_newsdir, &sb) == -1)
-			my_mkdir (index_newsdir, (mode_t)S_IRWXUGO);
-	}
-#endif /* 0 */
-
-	JOINPATH(index_maildir, get_val ("TIN_INDEX_MAILDIR", rcdir), INDEX_MAILDIR);
-	if (stat (index_maildir, &sb) == -1)
-		my_mkdir (index_maildir, (mode_t)S_IRWXUGO);
-	joinpath (index_savedir, get_val ("TIN_INDEX_SAVEDIR", rcdir), INDEX_SAVEDIR);
-
-	if (stat (index_savedir, &sb) == -1)
-		my_mkdir (index_savedir, (mode_t)S_IRWXUGO);
-	joinpath (local_attributes_file, rcdir, ATTRIBUTES_FILE);
-	joinpath (local_config_file, rcdir, CONFIG_FILE);
-	joinpath (filter_file, rcdir, FILTER_FILE);
-	joinpath (local_input_history_file, rcdir, INPUT_HISTORY_FILE);
-	joinpath (local_newsrctable_file, rcdir, NEWSRCTABLE_FILE);
-	joinpath (local_newsgroups_file, rcdir, NEWSGROUPS_FILE);
-	joinpath (mail_active_file, rcdir, ACTIVE_MAIL_FILE);
+		JOINPATH(index_newsdir, get_val("TIN_INDEX_NEWSDIR", rcdir), INDEX_NEWSDIR);
+	JOINPATH(index_maildir, get_val("TIN_INDEX_MAILDIR", rcdir), INDEX_MAILDIR);
+	if (stat(index_maildir, &sb) == -1)
+		my_mkdir(index_maildir, (mode_t) S_IRWXU);
+	JOINPATH(index_savedir, get_val("TIN_INDEX_SAVEDIR", rcdir), INDEX_SAVEDIR);
+	if (stat(index_savedir, &sb) == -1)
+		my_mkdir(index_savedir, (mode_t) S_IRWXU);
+	joinpath(local_attributes_file, rcdir, ATTRIBUTES_FILE);
+	joinpath(local_config_file, rcdir, CONFIG_FILE);
+	joinpath(filter_file, rcdir, FILTER_FILE);
+	joinpath(local_input_history_file, rcdir, INPUT_HISTORY_FILE);
+	joinpath(local_newsrctable_file, rcdir, NEWSRCTABLE_FILE);
+	joinpath(local_newsgroups_file, rcdir, NEWSGROUPS_FILE);
+	joinpath(mail_active_file, rcdir, ACTIVE_MAIL_FILE);
 #ifdef VMS
-	joinpath (mailbox, DEFAULT_MAILBOX, "MAIL.TXT");
+	joinpath(mailbox, DEFAULT_MAILBOX, "MAIL.TXT");
 #else
-	joinpath (mailbox, DEFAULT_MAILBOX, userid);
+	joinpath(mailbox, DEFAULT_MAILBOX, userid);
 #endif /* VMS */
-	joinpath (mailgroups_file, rcdir, MAILGROUPS_FILE);
-	joinpath (newsrc, homedir, NEWSRC_FILE);
-	joinpath (newnewsrc, homedir, NEWNEWSRC_FILE);
+#ifdef HAVE_MH_MAIL_HANDLING
+	joinpath(mailgroups_file, rcdir, MAILGROUPS_FILE);
+#endif /* HAVE_MH_MAIL_HANDLING */
+	joinpath(newsrc, homedir, NEWSRC_FILE);
+	joinpath(newnewsrc, homedir, NEWNEWSRC_FILE);
 #ifdef APPEND_PID
 	sprintf(newnewsrc + strlen(newnewsrc), "%d", (int) process_id);
 #endif /* APPEND_PID */
-	joinpath (posted_info_file, rcdir, POSTED_FILE);
-	joinpath (posted_msgs_file, tinrc.maildir, POSTED_FILE);
-	joinpath (postponed_articles_file, rcdir, POSTPONED_FILE);
-	joinpath (save_active_file, rcdir, ACTIVE_SAVE_FILE);
+	joinpath(posted_info_file, rcdir, POSTED_FILE);
+	joinpath(posted_msgs_file, tinrc.maildir, POSTED_FILE);
+	joinpath(postponed_articles_file, rcdir, POSTPONED_FILE);
+	joinpath(save_active_file, rcdir, ACTIVE_SAVE_FILE);
 
 #ifdef HAVE_LONG_FILE_NAMES
-	sprintf (lock_file, "%stin.%s.LCK", TMPDIR, userid);
+	sprintf(lock_file, "%stin.%s.LCK", TMPDIR, userid);
 #else
-	sprintf (lock_file, "%s%s.LCK", TMPDIR, userid);
+	sprintf(lock_file, "%s%s.LCK", TMPDIR, userid);
 #endif /* HAVE_LONG_FILE_NAMES */
 
 #ifdef NNTP_ABLE
-	nntp_tcp_port = (unsigned short) atoi (get_val ("NNTPPORT", NNTP_TCP_PORT));
+	nntp_tcp_port = (unsigned short) atoi(get_val("NNTPPORT", NNTP_TCP_PORT));
 #endif /* NNTP_ABLE */
 
-	if (stat (posted_info_file, &sb) == -1) {
-		if ((fp = fopen (posted_info_file, "w")) != (FILE *) 0) {
-			fprintf (fp, txt_posted_info_file);
-			fclose (fp);
+	if (stat(posted_info_file, &sb) == -1) {
+		if ((fp = fopen(posted_info_file, "w")) != NULL) {
+			fprintf(fp, txt_posted_info_file);
+			fchmod(fileno(fp), (mode_t) (S_IRUSR|S_IWUSR));
+			fclose(fp);
 		}
 	}
 
-	if (stat (local_attributes_file, &sb) == -1)
-		write_attributes_file (local_attributes_file);
+	if (stat(local_attributes_file, &sb) == -1)
+		write_attributes_file(local_attributes_file);
 
 	init_postinfo();
-	sprintf (txt_help_bug_report, _(txt_help_bug), bug_addr);
+	sprintf(txt_help_bug_report, _(txt_help_bug), bug_addr);
 
 #ifdef HAVE_PGP_GPG
 	init_pgp();
 #endif /* HAVE_PGP_GPG */
 }
 
+
 /*
  * If we're caching overview files and the user specified an NNTP server
  * with the '-g' option, make the directory name specific to the NNTP server
  * and make sure the directory exists.
  */
 void
-set_up_private_index_cache (
+set_up_private_index_cache(
 	void)
 {
 	struct stat sb;
@@ -849,20 +901,20 @@ set_up_private_index_cache (
 		char *to;
 		int c;
 
-		to = index_newsdir + strlen (index_newsdir);
+		to = index_newsdir + strlen(index_newsdir);
 		*(to++) = '-';
 		for (from = cmdline_nntpserver; (c = *from) != 0; ++from)
 			*(to++) = tolower(c);
 		*to = 0;
 	}
 
-	if (stat (index_newsdir, &sb) == -1)
-		my_mkdir (index_newsdir, (mode_t)S_IRWXU); /* was S_IRWXUGO */
+	if (stat(index_newsdir, &sb) == -1)
+		my_mkdir(index_newsdir, (mode_t) S_IRWXU);
 
 #	ifdef DEBUG
-	debug_nntp ("set_up_private_index_cache", index_newsdir);
+	debug_nntp("set_up_private_index_cache", index_newsdir);
 #	endif /* DEBUG */
-	joinpath (local_newsgroups_file, index_newsdir, NEWSGROUPS_FILE);
+	joinpath(local_newsgroups_file, index_newsdir, NEWSGROUPS_FILE);
 	return;
 }
 
@@ -871,26 +923,26 @@ set_up_private_index_cache (
  * Create default mail & save directories if they do not exist
  */
 t_bool
-create_mail_save_dirs (
+create_mail_save_dirs(
 	void)
 {
 	t_bool created = FALSE;
 	char path[PATH_LEN];
 	struct stat sb;
 
-	if (!strfpath (tinrc.maildir, path, sizeof (path), &CURR_GROUP))
-		joinpath (path, homedir, DEFAULT_MAILDIR);
+	if (!strfpath(tinrc.maildir, path, sizeof(path), NULL))
+		joinpath(path, homedir, DEFAULT_MAILDIR);
 
-	if (stat (path, &sb) == -1) {
-		my_mkdir (path, (mode_t)(S_IRWXU|S_IRUGO|S_IXUGO));
+	if (stat(path, &sb) == -1) {
+		my_mkdir(path, (mode_t) (S_IRWXU));
 		created = TRUE;
 	}
 
-	if (!strfpath (tinrc.savedir, path, sizeof (path), &CURR_GROUP))
-		joinpath (path, homedir, DEFAULT_SAVEDIR);
+	if (!strfpath(tinrc.savedir, path, sizeof(path), NULL))
+		joinpath(path, homedir, DEFAULT_SAVEDIR);
 
-	if (stat (path, &sb) == -1) {
-		my_mkdir (path, (mode_t)(S_IRWXU|S_IRUGO|S_IXUGO));
+	if (stat(path, &sb) == -1) {
+		my_mkdir(path, (mode_t) (S_IRWXU));
 		created = TRUE;
 	}
 
@@ -908,10 +960,10 @@ create_mail_save_dirs (
  * Sven Paulus <sven@tin.org>, 26-Jan-'98
  */
 static int
-read_site_config (
+read_site_config(
 	void)
 {
-	FILE *fp = (FILE *)0;
+	FILE *fp = (FILE *) 0;
 	char buf[LEN];
 	static const char *tin_defaults[] = { TIN_DEFAULTS };
 	int i = 0;
@@ -920,7 +972,7 @@ read_site_config (
 	 * try to find tin.defaults in some different locations
 	 */
 	while (tin_defaults[i] != NULL) {
-		sprintf(buf, "%s/tin.defaults", tin_defaults[i++]);
+		joinpath(buf, tin_defaults[i++], "tin.defaults");
 		if ((fp = fopen(buf, "r")) != NULL)
 			break;
 	}
@@ -933,45 +985,49 @@ read_site_config (
 		if (*buf == '#' || *buf == ';' || *buf == ' ')
 			continue;
 #ifndef NNTP_ONLY
-		if (match_string (buf, "spooldir=", spooldir, sizeof (spooldir)))
+		if (match_string(buf, "spooldir=", spooldir, sizeof(spooldir)))
 			continue;
-		if (match_string (buf, "overviewdir=", novrootdir, sizeof (novrootdir)))
+		if (match_string(buf, "overviewdir=", novrootdir, sizeof(novrootdir)))
 			continue;
-		if (match_string (buf, "overviewfile=", novfilename, sizeof (novfilename)))
+		if (match_string(buf, "overviewfile=", novfilename, sizeof(novfilename)))
 			continue;
 #endif /* !NNTP_ONLY */
-		if (match_string (buf, "activefile=", news_active_file, sizeof (news_active_file)))
+		if (match_string(buf, "activefile=", news_active_file, sizeof(news_active_file)))
 			continue;
-		if (match_string (buf, "activetimesfile=", active_times_file, sizeof (active_times_file)))
+		if (match_string(buf, "activetimesfile=", active_times_file, sizeof(active_times_file)))
 			continue;
-		if (match_string (buf, "newsgroupsfile=", newsgroups_file, sizeof (newsgroups_file)))
+		if (match_string(buf, "newsgroupsfile=", newsgroups_file, sizeof(newsgroups_file)))
 			continue;
-		if (match_string (buf, "newslibdir=", libdir, sizeof (libdir)))
+		if (match_string(buf, "newslibdir=", libdir, sizeof(libdir)))
 			continue;
-		if (match_string (buf, "subscriptionsfile=", subscriptions_file, sizeof (subscriptions_file)))
+		if (match_string(buf, "subscriptionsfile=", subscriptions_file, sizeof(subscriptions_file)))
 			continue;
-		if (match_string (buf, "domainname=", domain_name, sizeof (domain_name)))
+		if (match_string(buf, "domainname=", domain_name, sizeof(domain_name)))
 			continue;
-		if (match_string (buf, "inewsdir=", inewsdir, sizeof (inewsdir)))
+		if (match_string(buf, "inewsdir=", inewsdir, sizeof(inewsdir)))
 			continue;
-		if (match_string (buf, "bugaddress=", bug_addr, sizeof (bug_addr)))
+		if (match_string(buf, "bugaddress=", bug_addr, sizeof(bug_addr)))
 			continue;
-		if (match_string (buf, "organization=", default_organization, sizeof (default_organization)))
+		if (match_string(buf, "organization=", default_organization, sizeof(default_organization)))
 			continue;
-		if (match_string (buf, "mm_charset=", tinrc.mm_charset, sizeof (tinrc.mm_charset)))
+#ifndef CHARSET_CONVERSION
+		if (match_string(buf, "mm_charset=", tinrc.mm_charset, sizeof(tinrc.mm_charset)))
 			continue;
-		if (match_list (buf, "post_mime_encoding=", txt_mime_encodings, NUM_MIME_ENCODINGS, &tinrc.post_mime_encoding))
+#else
+		if (match_list(buf, "mm_charset=", txt_mime_charsets, NUM_MIME_CHARSETS, &tinrc.mm_network_charset))
 			continue;
-		if (match_list (buf, "mail_mime_encoding=", txt_mime_encodings, NUM_MIME_ENCODINGS, &tinrc.mail_mime_encoding))
+#endif /* !CHARSET_CONVERSION */
+		if (match_list(buf, "post_mime_encoding=", txt_mime_encodings, NUM_MIME_ENCODINGS, &tinrc.post_mime_encoding))
 			continue;
-		if (match_boolean (buf, "disable_gnksa_domain_check=", &disable_gnksa_domain_check))
+		if (match_list(buf, "mail_mime_encoding=", txt_mime_encodings, NUM_MIME_ENCODINGS, &tinrc.mail_mime_encoding))
 			continue;
-		if (match_boolean (buf, "disable_sender=", &disable_sender))
+		if (match_boolean(buf, "disable_gnksa_domain_check=", &disable_gnksa_domain_check))
+			continue;
+		if (match_boolean(buf, "disable_sender=", &disable_sender))
 			continue;
 	}
 
 	fclose(fp);
-
 	return 0;
 }
 
@@ -980,33 +1036,47 @@ read_site_config (
  * set defaults if needed to avoid empty regexp
  */
 void
-postinit_regexp (
+postinit_regexp(
 	void)
 {
 	if (!strlen(tinrc.strip_re_regex))
 		STRCPY(tinrc.strip_re_regex, DEFAULT_STRIP_RE_REGEX);
-	compile_regex (tinrc.strip_re_regex, &strip_re_regex, PCRE_ANCHORED);
+	compile_regex(tinrc.strip_re_regex, &strip_re_regex, PCRE_ANCHORED);
 	if (!strlen(tinrc.strip_was_regex))
 		STRCPY(tinrc.strip_was_regex, DEFAULT_STRIP_WAS_REGEX);
-	compile_regex (tinrc.strip_was_regex, &strip_was_regex, 0);
+	compile_regex(tinrc.strip_was_regex, &strip_was_regex, 0);
+
 #ifdef HAVE_COLOR
 	if (!strlen(tinrc.quote_regex))
 		STRCPY(tinrc.quote_regex, DEFAULT_QUOTE_REGEX);
-	compile_regex (tinrc.quote_regex, &quote_regex, PCRE_CASELESS);
+	compile_regex(tinrc.quote_regex, &quote_regex, PCRE_CASELESS);
 	if (!strlen(tinrc.quote_regex2))
 		STRCPY(tinrc.quote_regex2, DEFAULT_QUOTE_REGEX2);
-	compile_regex (tinrc.quote_regex2, &quote_regex2, PCRE_CASELESS);
+	compile_regex(tinrc.quote_regex2, &quote_regex2, PCRE_CASELESS);
 	if (!strlen(tinrc.quote_regex3))
 		STRCPY(tinrc.quote_regex3, DEFAULT_QUOTE_REGEX3);
-	compile_regex (tinrc.quote_regex3, &quote_regex3, PCRE_CASELESS);
+	compile_regex(tinrc.quote_regex3, &quote_regex3, PCRE_CASELESS);
 #endif /* HAVE_COLOR */
 
-	compile_regex (UUBEGIN_REGEX, &uubegin_regex, PCRE_CASELESS|PCRE_ANCHORED);
-	compile_regex (UUBODY_REGEX, &uubody_regex, PCRE_ANCHORED);
+	if (!strlen(tinrc.slashes_regex))
+		STRCPY(tinrc.slashes_regex, DEFAULT_SLASHES_REGEX);
+	compile_regex(tinrc.slashes_regex, &slashes_regex, PCRE_CASELESS);
+	if (!strlen(tinrc.stars_regex))
+		STRCPY(tinrc.stars_regex, DEFAULT_STARS_REGEX);
+	compile_regex(tinrc.stars_regex, &stars_regex, PCRE_CASELESS);
+	if (!strlen(tinrc.strokes_regex))
+		STRCPY(tinrc.strokes_regex, DEFAULT_STROKES_REGEX);
+	compile_regex(tinrc.strokes_regex,&strokes_regex, PCRE_CASELESS);
+	if (!strlen(tinrc.underscores_regex))
+		STRCPY(tinrc.underscores_regex, DEFAULT_UNDERSCORES_REGEX);
+	compile_regex(tinrc.underscores_regex, &underscores_regex, PCRE_CASELESS);
 
-	compile_regex (URL_REGEX, &url_regex, PCRE_CASELESS);
-	compile_regex (MAIL_REGEX, &mail_regex, PCRE_CASELESS);
-	compile_regex (NEWS_REGEX, &news_regex, PCRE_CASELESS);
+	compile_regex(UUBEGIN_REGEX, &uubegin_regex, PCRE_ANCHORED);
+	compile_regex(UUBODY_REGEX, &uubody_regex, PCRE_ANCHORED);
 
-	compile_regex (SHAR_REGEX, &shar_regex, PCRE_ANCHORED);
+	compile_regex(URL_REGEX, &url_regex, PCRE_CASELESS);
+	compile_regex(MAIL_REGEX, &mail_regex, PCRE_CASELESS);
+	compile_regex(NEWS_REGEX, &news_regex, PCRE_CASELESS);
+
+	compile_regex(SHAR_REGEX, &shar_regex, PCRE_ANCHORED);
 }
