@@ -3,10 +3,10 @@
  *  Module    : keymap.c
  *  Author    : D. Nimmich, J. Faultless
  *  Created   : 2000-05-25
- *  Updated   : 2001-11-10
+ *  Updated   : 2003-01-07
  *  Notes     : This file contains key mapping routines and variables.
  *
- * Copyright (c) 2000-2002 Dirk Nimmich <nimmich@uni-muenster.de>
+ * Copyright (c) 2000-2003 Dirk Nimmich <nimmich@uni-muenster.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,7 @@
 #	include "menukeys.h"
 #endif /* !MENUKEYS_H */
 
-static int keymapsize (t_keynode *ptr[]);
+static size_t keymapsize(t_keynode *ptr[]);
 static t_bool check_duplicates(t_keynode *keyptr1, t_keynode *keyptr2);
 static t_bool processkey(t_keynode *keyptr, char *kname, char key);
 
@@ -84,6 +84,7 @@ static struct keymap Key = {
 		{ '8', '8', "" },
 		{ '9', '9', "" },
 		{ iKeySearchSubjB, iKeySearchSubjB, "SearchSubjB" },
+		{ iKeySearchRepeat, iKeySearchRepeat, "SearchRepeat" },
 		{ iKeySearchAuthB, iKeySearchAuthB, "SearchAuthB" },
 		{ iKeySearchBody, iKeySearchBody, "SearchBody" },
 		{ iKeyToggleHelpDisplay, iKeyToggleHelpDisplay, "ToggleHelpDisplay" },
@@ -235,9 +236,7 @@ static struct keymap Key = {
 		{ iKeyPageMarkThdUnread, iKeyPageMarkThdUnread, "MarkThdUnread" },
 		{ iKeyPageQuickAutoSel, iKeyPageQuickAutoSel, "QuickAutoSel" },
 		{ iKeyPageQuickKill, iKeyPageQuickKill, "QuickKill" },
-#ifdef HAVE_COLOR
 		{ iKeyPageToggleHighlight, iKeyPageToggleHighlight, "ToggleHighlight" },
-#endif /* HAVE_COLOR */
 		{ iKeyPageCatchup, iKeyPageCatchup, "Catchup" },
 		{ iKeyPageEditArticle, iKeyPageEditArticle, "EditArticle" },
 		{ iKeyPageFollowupQuote, iKeyPageFollowupQuote, "FollowupQuote" },
@@ -315,6 +314,7 @@ static struct keymap Key = {
 		{ iKeySelectReadGrp, iKeySelectReadGrp, "ReadGrp" },
 		{ iKeySelectReadGrp2, iKeySelectReadGrp2, "ReadGrp2" },
 		{ iKeySelectResetNewsrc, iKeySelectResetNewsrc, "ResetNewsrc" },
+		{ iKeySelectSortActive, iKeySelectSortActive, "SortActive" },
 		{ iKeySelectCatchupNextUnread, iKeySelectCatchupNextUnread, "CatchupNextUnread" },
 		{ iKeySelectNextUnreadGrp, iKeySelectNextUnreadGrp, "NextUnreadGrp" },
 		{ iKeySelectBugReport, iKeySelectBugReport, "BugReport" },
@@ -389,7 +389,7 @@ static t_keynode *keys_config_change[] = {
 	&Key.Config.FirstPage2, &Key.Global.LastPage, &Key.Config.LastPage2,
 	&Key.Global.PageUp, &Key.Global.PageUp2, &Key.Global.PageUp3,
 	&Key.Global.PageDown, &Key.Global.PageDown2, &Key.Global.PageDown3,
-	&Key.Global.SearchSubjF, &Key.Global.SearchSubjB, &Key.Config.Select,
+	&Key.Global.SearchSubjF, &Key.Global.SearchSubjB, &Key.Global.SearchRepeat, &Key.Config.Select,
 	&Key.Config.Select2, &Key.Global.RedrawScr, &Key.Global.One,
 	&Key.Global.Two, &Key.Global.Three, &Key.Global.Four, &Key.Global.Five,
 	&Key.Global.Six, &Key.Global.Seven, &Key.Global.Eight, &Key.Global.Nine,
@@ -448,7 +448,7 @@ static t_keynode *keys_group_nav[] = {
 #endif /* !DISABLE_PRINTING */
 	&Key.Group.Repost, &Key.Group.Save, &Key.Group.AutoSaveTagged,
 	&Key.Global.SetRange, &Key.Global.SearchAuthF, &Key.Global.SearchAuthB,
-	&Key.Global.SearchSubjF, &Key.Global.SearchSubjB, &Key.Global.SearchBody,
+	&Key.Global.SearchSubjF, &Key.Global.SearchSubjB, &Key.Global.SearchRepeat, &Key.Global.SearchBody,
 	&Key.Group.ReadBasenote, &Key.Group.ReadBasenote2,
 	&Key.Group.NextUnreadArtOrGrp, &Key.Global.PageDown, &Key.Global.PageDown2,
 	&Key.Global.PageDown3, &Key.Group.AutoSel, &Key.Group.Kill,
@@ -483,7 +483,7 @@ static t_keynode *keys_info_nav[] = {
 	&Key.Global.PageUp2, &Key.Global.PageUp3, &Key.Global.FirstPage,
 	&Key.Help.FirstPage2, &Key.Global.LastPage, &Key.Help.LastPage2,
 	&Key.Global.ToggleHelpDisplay, &Key.Global.SearchSubjF,
-	&Key.Global.SearchSubjB, &Key.Global.Quit, NULL };
+	&Key.Global.SearchSubjB, &Key.Global.SearchRepeat, &Key.Global.Quit, NULL };
 
 static t_keynode *keys_nrctbl_create[] = {
    &Key.Global.Abort, &Key.Nrctbl.Quit, &Key.Nrctbl.Alternative,
@@ -508,8 +508,8 @@ static t_keynode *keys_page_nav[] = {
 #endif /* !DISABLE_PRINTING */
 	&Key.Page.Repost, &Key.Page.Save, &Key.Page.AutoSaveTagged,
 	&Key.Global.SearchAuthF, &Key.Global.SearchAuthB, &Key.Global.SearchSubjF,
-	&Key.Global.SearchSubjB, &Key.Global.SearchBody, &Key.Page.TopThd,
-	&Key.Page.BotThd, &Key.Page.NextThd, &Key.Page.NextThd2,
+	&Key.Global.SearchSubjB, &Key.Global.SearchRepeat, &Key.Global.SearchBody,
+	&Key.Page.TopThd, &Key.Page.BotThd, &Key.Page.NextThd, &Key.Page.NextThd2,
 #ifdef HAVE_PGP_GPG
 	&Key.Page.PGPCheckArticle,
 #endif /* HAVE_PGP_GPG */
@@ -533,9 +533,7 @@ static t_keynode *keys_page_nav[] = {
 	&Key.Global.Postponed, &Key.Global.Postponed2, &Key.Global.DisplayPostHist,
 	&Key.Page.MarkArtUnread, &Key.Page.SkipIncludedText,
 	&Key.Global.ToggleInfoLastLine,
-#ifdef HAVE_COLOR
 	&Key.Page.ToggleHighlight,
-#endif /* HAVE_COLOR */
 	&Key.Page.ViewAttach, &Key.Page.ViewUrl, NULL };
 
 static t_keynode *keys_pgp_mail[] = {
@@ -616,10 +614,10 @@ static t_keynode *keys_select_nav[] = {
 	&Key.Global.PageUp2, &Key.Global.PageUp3, &Key.Global.PageDown,
 	&Key.Global.PageDown2, &Key.Global.PageDown3, &Key.Global.Up,
 	&Key.Global.Up2, &Key.Global.Down, &Key.Global.Down2, &Key.Global.SetRange,
-	&Key.Global.SearchSubjF, &Key.Global.SearchSubjB,
+	&Key.Global.SearchSubjF, &Key.Global.SearchSubjB, &Key.Global.SearchRepeat,
 	&Key.Select.ReadGrp, &Key.Select.ReadGrp2, &Key.Select.EnterNextUnreadGrp,
 	&Key.Select.EnterNextUnreadGrp2, &Key.Global.RedrawScr,
-	&Key.Select.ResetNewsrc, &Key.Select.Catchup,
+	&Key.Select.ResetNewsrc, &Key.Select.SortActive, &Key.Select.Catchup,
 	&Key.Select.CatchupNextUnread, &Key.Select.ToggleDescriptions,
 	&Key.Select.Goto, &Key.Global.Help, &Key.Global.ToggleHelpDisplay,
 	&Key.Global.ToggleInverseVideo,
@@ -654,7 +652,7 @@ static t_keynode *keys_thread_nav[] = {
 	&Key.Thread.CatchupNextUnread, &Key.Thread.MarkArtRead,
 	&Key.Thread.ToggleSubjDisplay, &Key.Global.Help,
 	&Key.Global.LookupMessage, &Key.Global.SearchBody,
-	&Key.Global.SearchSubjF, &Key.Global.SearchSubjB,
+	&Key.Global.SearchSubjF, &Key.Global.SearchSubjB, &Key.Global.SearchRepeat,
 	&Key.Global.SearchAuthF, &Key.Global.SearchAuthB,
 	&Key.Global.ToggleHelpDisplay, &Key.Global.ToggleInverseVideo,
 #ifdef HAVE_COLOR
@@ -696,16 +694,18 @@ t_menukeymap menukeymap = {
 	{ keys_thread_nav, NULL, NULL }
 };
 
+
 /*
  * Return the number of keys in a menu
  */
-static int
-keymapsize (
+static size_t
+keymapsize(
 	t_keynode *ptr[])
 {
-	int i = 0;
+	size_t i = 0;
 
-	if (!ptr) return 0;
+	if (!ptr)
+		return i;
 
 	while (*ptr) {
 		i++;
@@ -714,29 +714,30 @@ keymapsize (
 	return i;
 }
 
+
 /*
  * Compile keymaps for faster access and conversion
  */
 void
-build_keymaps (
+build_keymaps(
 	void)
 {
 	char *dkey, *lkey;
-	int cnt = sizeof (menukeymap) / sizeof (t_menukeys);
-	int size;
+	int cnt = sizeof(menukeymap) / sizeof(t_menukeys);
+	size_t size;
 	t_keynode *keyptr;
 	t_menukeys *menuptr = &menukeymap.config_change;
 
 	while (cnt--) {
-		size = keymapsize (menuptr->keys);
+		size = keymapsize(menuptr->keys);
 		if (menuptr->defaultkeys)
-			menuptr->defaultkeys = my_realloc (menuptr->defaultkeys, size + 1);
+			menuptr->defaultkeys = my_realloc(menuptr->defaultkeys, size + 1);
 		else
-			menuptr->defaultkeys = my_malloc (size + 1);
+			menuptr->defaultkeys = my_malloc(size + 1);
 		if (menuptr->localkeys)
-			menuptr->localkeys = my_realloc (menuptr->localkeys, size + 1);
+			menuptr->localkeys = my_realloc(menuptr->localkeys, size + 1);
 		else
-			menuptr->localkeys = my_malloc (size + 1);
+			menuptr->localkeys = my_malloc(size + 1);
 		dkey = menuptr->defaultkeys;
 		lkey = menuptr->localkeys;
 		dkey[size] = '\0';
@@ -751,15 +752,16 @@ build_keymaps (
 	ch_post_process = &menukeymap.feed_post_process_type.defaultkeys[2];
 }
 
+
 /*
  * convert a local key to the internal (default) mapping
  */
 int
-map_to_default (
+map_to_default(
 	const char key,
 	const t_menukeys *menukeys)
 {
-	char *ptr = strchr (menukeys->localkeys, key);
+	char *ptr = strchr(menukeys->localkeys, key);
 
 	if (ptr)
 		return menukeys->defaultkeys[ptr - menukeys->localkeys];
@@ -767,15 +769,16 @@ map_to_default (
 	return 0;
 }
 
+
 /*
  * convert an internal (default) key to a local one
  */
 int
-map_to_local (
+map_to_local(
 	const char key,
 	const t_menukeys *menukeys)
 {
-	char *ptr = strchr (menukeys->defaultkeys, key);
+	char *ptr = strchr(menukeys->defaultkeys, key);
 
 	if (ptr)
 		return menukeys->localkeys[ptr - menukeys->defaultkeys];
@@ -783,32 +786,31 @@ map_to_local (
 	return 0;
 }
 
+
 /*
  * Free all memory for keymaps.
  */
 void
-free_keymaps (
+free_keymaps(
 	void)
 {
-	int cnt = sizeof (menukeymap) / sizeof (t_menukeys);
+	int cnt = sizeof(menukeymap) / sizeof(t_menukeys);
 	t_menukeys *menuptr = &menukeymap.config_change;
 
 	while (cnt--) {
-		FreeIfNeeded (menuptr->localkeys);
-		FreeIfNeeded (menuptr->defaultkeys);
+		FreeIfNeeded(menuptr->localkeys);
+		FreeIfNeeded(menuptr->defaultkeys);
 		menuptr++;
 	}
 }
 
-/* TODO -> tin.h */
-#define KEYMAP_FILE	"keymap"
 
 /*
  * Render ch in human readable ASCII
  * Is there no lib function to do this ?
  */
 char *
-printascii (
+printascii(
 	char *buf,
 	int ch)
 {
@@ -816,28 +818,29 @@ printascii (
 		buf[0] = ch;
 		buf[1] = '\0';
 	} else if (ch == '\t') {	/* TAB */
-		strcpy (buf, _(txt_tab));
+		strcpy(buf, _(txt_tab));
 	} else if ((ch == '\n') || (ch == '\r')) {	/* LF, CR */
-		strcpy (buf, _(txt_cr));
+		strcpy(buf, _(txt_cr));
 	} else if (ch == ESC) {		/* Escape */
-		strcpy (buf, _(txt_esc));
+		strcpy(buf, _(txt_esc));
 	} else if (iscntrl(ch)) {	/* Control keys */
 		buf[0] = '^';
 		buf[1] = (ch & 0xFF) + '@';
 		buf[2] = '\0';
 	} else if (ch == ' ')		/* SPACE */
-		strcpy (buf, _(txt_space));
+		strcpy(buf, _(txt_space));
 	else
-		strcpy (buf, "???");	/* Never happens? */
+		strcpy(buf, "???");	/* Never happens? */
 
 	return buf;
 }
 
+
 /*
- * Find any key clashes between groups keyptr1 and keyptr2.  This is just an
- * ascending brute force search.  We need a pointer to the tag node in order
+ * Find any key clashes between groups keyptr1 and keyptr2. This is just an
+ * ascending brute force search. We need a pointer to the tag node in order
  * to report errors correctly, the pointers start 1 above this on the first
- * key definition.  If we are checking against the same keygroup, then ptr2
+ * key definition. If we are checking against the same keygroup, then ptr2
  * starts at (ptr1+1) else the algorithm doesn't work!
  */
 static t_bool
@@ -851,13 +854,12 @@ check_duplicates(
 	for (ptr1 = keyptr1 + 1; ptr1->localkey != '\0'; ++ptr1) {
 		for (ptr2 = (keyptr1 == keyptr2) ? ptr1 + 1 : keyptr2 + 1; ptr2->localkey != '\0'; ++ptr2) {
 			if (ptr1->localkey == ptr2->localkey) {
-				fprintf (stderr, _(txt_keymap_conflict), printascii (buf, ptr1->localkey),
+				fprintf(stderr, _(txt_keymap_conflict), printascii(buf, ptr1->localkey),
 								keyptr1->t, ptr1->t, keyptr2->t, ptr2->t);
 				return FALSE;
 			}
 		}
 	}
-
 	return TRUE;
 }
 
@@ -877,16 +879,16 @@ processkey(
 	char was[10], is[10];
 #endif /* DEBUG */
 
-	strcpy (buf, keyptr->t);	/* Preload the groupname for speed */
+	strcpy(buf, keyptr->t);	/* Preload the groupname for speed */
 	keyptr++;						/* Advance to 1st key */
-	i = strlen (buf);
+	i = strlen(buf);
 
 	for (; keyptr->t != NULL; ++keyptr) {
-		strcpy (buf + i, keyptr->t);
+		strcpy(buf + i, keyptr->t);
 
-		if (strcasecmp (kname, buf) == 0) {
+		if (strcasecmp(kname, buf) == 0) {
 #ifdef DEBUG
-			fprintf (stderr, _(txt_keymap_redef), buf, printascii (was, keyptr->localkey), printascii (is, key));
+			fprintf(stderr, _(txt_keymap_redef), buf, printascii(was, keyptr->localkey), printascii(is, key));
 #endif /* DEBUG */
 			keyptr->localkey = key;
 			return TRUE;
@@ -896,56 +898,51 @@ processkey(
 	return FALSE;
 }
 
-#define KEYSEPS		" \t\n"
 
+#define KEYSEPS		" \t\n"
+/* TODO -> tin.h */
+#define KEYMAP_FILE	"keymap"
 t_bool
-read_keymap_file (
+read_keymap_file(
 	void)
 {
 	FILE *fp = (FILE *) 0;
 	char *line, *keydef, *kname;
 	char *map, *ptr;
-	const char *ptr2;
 	char buf[LEN], buff[LEN];
 	char key;
 	int i;
 	t_bool ret = TRUE;
 
-	if (!batch_mode)
-		wait_message (0, _(txt_reading_keymap_file));
-
 	/*
 	 * checks TIN_HOMEDIR/HOME/TIN_DEFAULTS_DIR
-	 * for keymap."locale" or keymap
+	 * for KEYMAP_FILE."locale" or KEYMAP_FILE
 	 *
 	 * locale is first match from LC_ALL, LC_CTYPE, LC_MESSAGES, LANG
 	 *
+	 * TODO: fix possible buf-overflows
 	 */
-	ptr2 = get_val("TIN_HOMEDIR", get_val("HOME", homedir));
 	/* get locale suffix */
 	map = my_strdup(get_val("LC_ALL", get_val("LC_CTYPE", get_val("LC_MESSAGES", get_val("LANG", "")))));
 	if (strlen(map)) {
-		if ((ptr = strchr (map, '.')))
-				*ptr = '\0';
-		snprintf(buff, sizeof(buff) - 1, "%s/.tin/keymap.%s", ptr2, map);
-		if (strfpath (buff, buf, sizeof(buf), &CURR_GROUP))
-			fp = fopen (buf, "r");
+		if ((ptr = strchr(map, '.')))
+			*ptr = '\0';
+		snprintf(buff, sizeof(buff) - 1, "%s.%s", KEYMAP_FILE, map);
+		joinpath(buf, rcdir, buff);
+		fp = fopen(buf, "r");
 	}
 	if (!fp) {
-		snprintf(buff, sizeof(buff) - 1, "%s/.tin/keymap", ptr2);
-		if (strfpath (buff, buf, sizeof(buf), &CURR_GROUP))
-			fp = fopen (buf, "r");
+		joinpath(buf, rcdir, KEYMAP_FILE);
+		fp = fopen(buf, "r");
 	}
 #ifdef TIN_DEFAULTS_DIR
 	if (strlen(map) && !fp) {
-		snprintf(buff, sizeof(buff) - 1, "%s/keymap.%s", TIN_DEFAULTS_DIR, map);
-		if (strfpath (buff, buf, sizeof(buf), &CURR_GROUP))
-			fp = fopen (buf, "r");
+		joinpath(buf, TIN_DEFAULTS_DIR, buff);
+		fp = fopen(buf, "r");
 	}
 	if (!fp) {
-		snprintf(buff, sizeof(buff) - 1, "%s/keymap", TIN_DEFAULTS_DIR);
-		if (strfpath (buff, buf, sizeof(buf), &CURR_GROUP))
-			fp = fopen (buf, "r");
+		joinpath(buf, TIN_DEFAULTS_DIR, KEYMAP_FILE);
+		fp = fopen(buf, "r");
 	}
 #endif /* TIN_DEFAULTS_DIR */
 
@@ -954,21 +951,21 @@ read_keymap_file (
 	if (!fp)
 		return FALSE;
 
-	while ((line = fgets (buf, sizeof(buf), fp)) != NULL) {
+	while ((line = fgets(buf, sizeof(buf), fp)) != NULL) {
 		/*
 		 * Ignore blank and comment lines
 		 */
 		if (line[0] == '#' || line[0] == '\n')
 			continue;
 
-		kname = strtok (line, KEYSEPS);
-		keydef = strtok (NULL, KEYSEPS);
+		kname = strtok(line, KEYSEPS);
+		keydef = strtok(NULL, KEYSEPS);
 
 		/*
 		 * Warn about basic syntax errors
 		 */
 		if (keydef == NULL) {
-			fprintf (stderr, _(txt_keymap_missing_key), kname);
+			fprintf(stderr, _(txt_keymap_missing_key), kname);
 			ret = FALSE;
 			continue;
 		}
@@ -980,24 +977,24 @@ read_keymap_file (
 		 * TAB -> ^I
 		 * SPACE -> ' '
 		 */
-		if (strlen (keydef) > 1)
+		if (strlen(keydef) > 1)
 			switch (keydef[0]) {
 				case '^':
-					if (!(isupper ((int)keydef[1]))) {
-						fprintf (stderr, _(txt_keymap_invalid_key), keydef);
+					if (!(isupper((int)(unsigned char) keydef[1]))) {
+						fprintf(stderr, _(txt_keymap_invalid_key), keydef);
 						ret = FALSE;
 						continue;
 					}
-					key = ctrl (keydef[1]);
+					key = ctrl(keydef[1]);
 					break;
 				case 'S':		/* Only test 1st char - crude but effective */
 					key = ' ';
 					break;
 				case 'T':
-					key = ctrl ('I');
+					key = ctrl('I');
 					break;
 				default:
-					fprintf (stderr, _(txt_keymap_invalid_key), keydef);
+					fprintf(stderr, _(txt_keymap_invalid_key), keydef);
 					ret = FALSE;
 					continue;
 			}
@@ -1008,16 +1005,16 @@ read_keymap_file (
 		 * Try and locate the tagline in each keygroup
 		 */
 		for (i = 0; keygroups[i] != NULL; ++i) {
-			if (processkey (keygroups[i], kname, key))
+			if (processkey(keygroups[i], kname, key))
 				break;
 		}
 
 		/*
-		 * TODO: usefull? shared keymaps (NFS-Home) may differ
+		 * TODO: useful? shared keymaps (NFS-Home) may differ
 		 * depending on the OS (i.e. on tin has colr the other has not)
 		 */
 		if (keygroups[i] == NULL) {
-			fprintf (stderr, _(txt_keymap_invalid_name), kname);
+			fprintf(stderr, _(txt_keymap_invalid_name), kname);
 			ret = FALSE;
 			continue;
 		}
@@ -1028,13 +1025,13 @@ read_keymap_file (
 	 * Check for duplicates in each keygroup and vs. the global group
 	 */
 	for (i = 0; keygroups[i] != NULL; ++i) {
-		if (!(check_duplicates (keygroups[i], keygroups[i])))
+		if (!(check_duplicates(keygroups[i], keygroups[i])))
 			ret = FALSE;
-		if (i != 0 && !(check_duplicates (keygroups[i], keygroups[0])))
+		if (i != 0 && !(check_duplicates(keygroups[i], keygroups[0])))
 			ret = FALSE;
 	}
 
-	fclose (fp);
-	build_keymaps ();
+	fclose(fp);
+	build_keymaps();
 	return ret;
 }
