@@ -3,10 +3,10 @@
  *  Module    : main.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 2001-07-22
+ *  Updated   : 2002-12-06
  *  Notes     :
  *
- * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Iain Lea, Rich Skrenta
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -65,51 +62,57 @@ static char **cmdargs;
 static int num_cmdargs;
 static int max_cmdargs;
 
-static t_bool catchup = FALSE;			/* mark all arts read in all subscribed groups */
-static t_bool check_any_unread = FALSE;/* print/return status if any unread */
+static t_bool catchup = FALSE;		/* mark all arts read in all subscribed groups */
+static t_bool check_any_unread = FALSE;	/* print/return status if any unread */
 static t_bool mail_news = FALSE;		/* mail all arts to specified user */
 static t_bool save_news = FALSE;		/* save all arts to savedir structure */
-static t_bool start_any_unread = FALSE;/* only start if unread news */
+static t_bool start_any_unread = FALSE;	/* only start if unread news */
 
 
 /*
  * Local prototypes
  */
-static void read_cmd_line_options (int argc, char *argv[]);
-static void update_index_files (void);
-static void usage (char *theProgname);
-static void show_intro_page (void);
+static void read_cmd_line_options(int argc, char *argv[]);
+static void show_intro_page(void);
+static void update_index_files(void);
+static void usage(char *theProgname);
 
 
 /*
  * OK lets start the ball rolling...
  */
 int
-main (
+main(
 	int argc,
 	char *argv[])
 {
+	int count;
 	int num_cmd_line_groups;
 	int start_groupnum = 0;
-	int count;
 	t_bool tmp_no_write;
 
 	/* initialize locale support */
 #if defined(HAVE_SETLOCALE) && !defined(NO_LOCALE)
-	setlocale(LC_ALL, "");
+	if (!setlocale(LC_ALL, "")) {
+	/*
+	 * TODO: issue a warning here like
+	 *       "Can't set the specified locale! Check $LANG, $LC_CTYPE, $LC_ALL"
+	 */
+	 ;
+	}
 #	ifdef ENABLE_NLS
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 #	endif /* ENABLE_NLS */
 #endif /* HAVE_SETLOCALE && !NO_LOCALE */
 
-	set_signal_handlers ();
+	set_signal_handlers();
 
 	cmd_line = TRUE;
 	debug = 0;	/* debug OFF */
 
 #if defined(M_AMIGA) && defined(__SASC)
-	/* Call tzset() here!*/
+	/* Call tzset() here! */
 	_TZ = "GMT0";
 	tzset();
 	if (argc == 0) { /* we are running from the Workbench */
@@ -118,7 +121,7 @@ main (
 	}
 #endif /* M_AMIGA && __SASC */
 
-	base_name (argv[0], tin_progname);
+	base_name(argv[0], tin_progname);
 #ifdef VMS
 	argv[0] = tin_progname;
 #endif /* VMS */
@@ -133,7 +136,7 @@ main (
 #	ifdef NNTP_ABLE
 		read_news_via_nntp = TRUE;
 #	else
-		error_message (_(txt_option_not_enabled), "-DNNTP_ABLE");
+		error_message(_(txt_option_not_enabled), "-DNNTP_ABLE");
 		giveup();
 #	endif /* NNTP_ABLE */
 	}
@@ -142,17 +145,17 @@ main (
 	/*
 	 * Set up initial array sizes, char *'s: homedir, newsrc, etc.
 	 */
-	init_alloc ();
-	hash_init ();
-	init_selfinfo ();
-	init_group_hash ();
+	init_alloc();
+	hash_init();
+	init_selfinfo();
+	init_group_hash();
 
 	/*
 	 * Read user local & global config files
 	 * These override the compiled in defaults
 	 */
-	read_config_file (global_config_file, TRUE);
-	read_config_file (local_config_file, FALSE);
+	read_config_file(global_config_file, TRUE);
+	read_config_file(local_config_file, FALSE);
 
 	/*
 	 * avoid empty regexp
@@ -163,7 +166,7 @@ main (
 	 * Process envargs & command line options
 	 * These override the configured in values
 	 */
-	read_cmd_line_options (argc, argv);
+	read_cmd_line_options(argc, argv);
 
 	tmp_no_write = no_write; /* keep no_write */
 	no_write = TRUE;		 /* don't allow any writing back during startup */
@@ -171,8 +174,8 @@ main (
 	if (!batch_mode) {
 #ifdef M_UNIX
 #	ifndef USE_CURSES
-		if (!get_termcaps ()) {
-			error_message (_(txt_screen_init_failed), tin_progname);
+		if (!get_termcaps()) {
+			error_message(_(txt_screen_init_failed), tin_progname);
 			giveup();
 		}
 #	endif /* !USE_CURSES */
@@ -181,52 +184,57 @@ main (
 		/*
 		 * Init curses emulation
 		 */
-		if (!InitScreen ()) {
-			error_message (_(txt_screen_init_failed), tin_progname);
+		if (!InitScreen()) {
+			error_message(_(txt_screen_init_failed), tin_progname);
 			giveup();
 		}
 
-		EndInverse ();
+		EndInverse();
 
 		/*
 		 * This depends on various things in tinrc
 		 */
-		setup_screen ();
+		setup_screen();
 	}
 
 	if (!batch_mode || (batch_mode && verbose))
-		wait_message (0, "%s\n", cvers);
+		wait_message(0, "%s\n", cvers);
 
-	set_up_private_index_cache ();
+	set_up_private_index_cache();
 
 	/*
 	 * Connect to nntp server?
 	 */
-	if (read_news_via_nntp && !read_saved_news)
-		if (nntp_open () != 0)
-			giveup();
+	if (read_news_via_nntp) {
+		if (!read_saved_news) {
+			if (nntp_open())
+				giveup();
+		} else { /* set nntp_server if reading via -R as it's used in select.c */
+			nntp_server = getserverbyfile(NNTP_SERVER_FILE);
+			if (nntp_server == NULL)
+				nntp_server = my_strdup("reading saved news"); /* mem-leak ,-) */
+		}
+	}
 
 	/*
 	 * Check if overview indexes contain Xref: lines
 	 */
 	if (xover_supported)
-		xref_supported = overview_xref_support ();
+		xref_supported = overview_xref_support();
 
 #ifdef DEBUG_NEWSRC
-	unlink ("/tmp/BITMAP");
-/*	vNewsrcTestHarness ();*/
+	unlink("/tmp/BITMAP");
+/*	vNewsrcTestHarness(); */
 #endif /* DEBUG_NEWSRC */
 
 	/*
-	 * Read user specific keybindings
+	 * Read user specific keybindings and input history
 	 */
-	read_keymap_file ();
-
-	/*
-	 * Read input history
-	 */
-	if (!batch_mode)
-		read_input_history_file ();
+	if (!batch_mode) {
+		wait_message(0, _(txt_reading_keymap_file));
+		read_keymap_file();
+		read_input_history_file();
+	}
 
 	/*
 	 * Load the mail & news active files into active[]
@@ -236,90 +244,86 @@ main (
 	 */
 	no_write = tmp_no_write;
 	if (read_saved_news)
-		create_save_active_file ();
+		create_save_active_file();
 	no_write = TRUE;
 
 #ifdef HAVE_MH_MAIL_HANDLING
-	read_mail_active_file ();
+	read_mail_active_file();
 #endif /* HAVE_MH_MAIL_HANDLING */
 
 	/*
 	 * Initialise active[] and add new newsgroups to start of my_group[]
 	 */
 	selmenu.max = 0;
-	read_news_active_file ();
+	read_news_active_file();
 #ifdef DEBUG
 	debug_print_active();
 #endif /* DEBUG */
 
 	/*
 	 * Load the local & global group specific attribute files
-	 * FIXME: is global attributes still needed? if so use TIN_DEFAULTS_DIR
 	 */
-	if (!batch_mode /* || (batch_mode && verbose)*/) /* FIXME: NLS support missing */
-		wait_message (0, _(txt_reading_attributes_file), _(txt_global));
-	read_attributes_file (global_attributes_file, TRUE);
-	if (!batch_mode /* || (batch_mode && verbose)*/)
-		wait_message (0, _(txt_reading_attributes_file), "");
-	read_attributes_file (local_attributes_file, FALSE);
+	if (!batch_mode || (batch_mode && verbose))
+		wait_message(0, _(txt_reading_attributes_file), _(txt_global));
+	read_attributes_file(TRUE);
+	if (!batch_mode || (batch_mode && verbose))
+		wait_message(0, _(txt_reading_attributes_file), "");
+	read_attributes_file(FALSE);
 
 	/*
-	 * Read in users filter preferences file.
-	 * This has to be done before quick post
-	 * because the filters will be updated.
+	 * Read in users filter preferences file. This has to be done before
+	 * quick post because the filters might be updated.
 	 */
-	filtered_articles = read_filter_file (filter_file);
+	filtered_articles = read_filter_file(filter_file);
 
 #ifdef DEBUG
-	debug_print_filters ();
+	debug_print_filters();
 #endif /* DEBUG */
 
 	/*
-	 * Quick post an article & exit if -w specified
+	 * Quick post an article and exit if -w or -o specified
 	 */
 	if (post_article_and_exit || post_postponed_and_exit) {
-		quick_post_article (post_postponed_and_exit);
-		wait_message (2, _(txt_exiting));
-		tin_done (EXIT_SUCCESS);
+		no_write = tmp_no_write; /* restore original value */
+		quick_post_article(post_postponed_and_exit);
+		wait_message(2, _(txt_exiting));
+		no_write = TRUE; /* disable tinrc updates */
+		tin_done(EXIT_SUCCESS);
 	}
 
+	/* TODO: replace hardcoded key-name in txt_info_postponed */
 	if ((count = count_postponed_articles()))
 		wait_message(3, _(txt_info_postponed), count, PLURAL(count, txt_article));
 
 	/*
 	 * Read text descriptions for mail and/or news groups
 	 */
-	if (show_description && !batch_mode) {
-#ifdef HAVE_MH_MAIL_HANDLING
-		read_mailgroups_file ();
-#endif /* HAVE_MH_MAIL_HANDLING */
-		read_newsgroups_file ();
-	}
+	if (show_description && !batch_mode)
+		read_descriptions(TRUE);
 
-	if (create_mail_save_dirs ())
-		write_config_file (local_config_file);
+	if (create_mail_save_dirs())
+		write_config_file(local_config_file);
 
 	/*
 	 * Preloads active[] with command line groups. They will follow any
 	 * new newsgroups
 	 */
-	num_cmd_line_groups = read_cmd_line_groups ();
+	num_cmd_line_groups = read_cmd_line_groups();
 
-	backup_newsrc ();
+	backup_newsrc();
 
 	/*
 	 * Load my_groups[] from the .newsrc file. We append these groups to any
 	 * new newsgroups and command line newsgroups already loaded
 	 */
 
-	/* TODO:
+	/*
+	 * TODO:
 	 * if (num_cmd_line_groups != 0 && check_any_unread)
 	 * don't read newsrc.
 	 * This makes -Z handle command line newsgroups. Test & document
 	 */
-
-	read_newsrc_lines = read_newsrc (newsrc, FALSE);
-
+	read_newsrc_lines = read_newsrc(newsrc, FALSE);
 	no_write = tmp_no_write; /* restore old value */
 
 	/*
@@ -328,17 +332,17 @@ main (
 	if (num_cmd_line_groups)
 		tinrc.show_only_unread_groups = FALSE;
 	else
-		toggle_my_groups (tinrc.show_only_unread_groups, "");
+		toggle_my_groups(NULL);
 
 	/*
 	 * Check/start if any new/unread articles
 	 */
 	if (check_any_unread)
-		exit (check_start_save_any_news (CHECK_ANY_NEWS, catchup));
+		exit(check_start_save_any_news(CHECK_ANY_NEWS, catchup));
 
 	if (start_any_unread) {
 		batch_mode = TRUE;			/* Suppress some unwanted on-screen garbage */
-		if ((start_groupnum = check_start_save_any_news (START_ANY_NEWS, catchup)) == -1)
+		if ((start_groupnum = check_start_save_any_news(START_ANY_NEWS, catchup)) == -1)
 			giveup();				/* No new/unread news so exit */
 		batch_mode = FALSE;
 	}
@@ -347,19 +351,25 @@ main (
 	 * Mail any new articles to specified user
 	 * or
 	 * Save any new articles to savedir structure for later reading
+	 *
+	 * FIXME: this currentyl doen't work, see comments in
+	 *        check_start_save_any_news()
+	 * TODO: should we temporarely set
+	 *       getart_limit=-1,thread_arts=0,sort_art_type=0
+	 *       for speed reasons?
 	 */
 	if (mail_news || save_news) {
-		do_update (FALSE);
-		check_start_save_any_news (mail_news ? MAIL_ANY_NEWS : SAVE_ANY_NEWS, catchup);
-		tin_done (EXIT_SUCCESS);
+		do_update(FALSE);
+		check_start_save_any_news(mail_news ? MAIL_ANY_NEWS : SAVE_ANY_NEWS, catchup);
+		tin_done(EXIT_SUCCESS);
 	}
 
 	/*
 	 * Catchup newsrc file (-c option)
 	 */
-	if (catchup) {
-		catchup_newsrc_file ();
-		tin_done (EXIT_SUCCESS);
+	if (catchup && batch_mode) {
+		catchup_newsrc_file();
+		tin_done(EXIT_SUCCESS);
 	}
 
 	/*
@@ -367,7 +377,7 @@ main (
 	 * Only the -u batch_mode case will get this far
 	 */
 	if (batch_mode)
-		update_index_files ();
+		update_index_files();
 
 	/*
 	 * If first time print welcome screen and auto-subscribe
@@ -375,17 +385,17 @@ main (
 	 * or via NNTP if reading news remotely (LIST SUBSCRIBE)
 	 */
 	if (created_rcdir && !batch_mode)
-		show_intro_page ();
+		show_intro_page();
 
 #ifdef USE_CURSES
 	/* Turn scrolling off now the startup messages have been displayed */
-	scrollok (stdscr, FALSE);
+	scrollok(stdscr, FALSE);
 #endif /* USE_CURSES */
 
 	/*
 	 * Work loop
 	 */
-	selection_page (start_groupnum, num_cmd_line_groups);
+	selection_page(start_groupnum, num_cmd_line_groups);
 	return 0; /* not reached */
 }
 
@@ -400,23 +410,23 @@ main (
 #endif /* M_AMIGA */
 
 static void
-read_cmd_line_options (
+read_cmd_line_options(
 	int argc,
 	char *argv[])
 {
 	int ch;
 	t_bool newsrc_set = FALSE;
 
-	envargs (&argc, &argv, "TINRC");
+	envargs(&argc, &argv, "TINRC");
 
-	while ((ch = getopt (argc, argv, OPTIONS)) != -1) {
+	while ((ch = getopt(argc, argv, OPTIONS)) != -1) {
 		switch (ch) {
 #	ifndef M_AMIGA
 			case 'a':
 #		ifdef HAVE_COLOR
-				use_color = !use_color;
+				use_color = bool_not(use_color);
 #		else
-				error_message (_(txt_option_not_enabled), "-DHAVE_COLOR");
+				error_message(_(txt_option_not_enabled), "-DHAVE_COLOR");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -427,7 +437,7 @@ read_cmd_line_options (
 #		ifdef NNTP_ABLE
 				force_auth_on_conn_open = TRUE;
 #		else
-				error_message (_(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(_(txt_option_not_enabled), "-DNNTP_ABLE");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -441,7 +451,6 @@ read_cmd_line_options (
 
 			case 'c':
 				catchup = TRUE;
-				batch_mode = TRUE;
 				break;
 
 			case 'd':
@@ -450,27 +459,23 @@ read_cmd_line_options (
 
 			case 'D':		/* debug mode 1=NNTP 2=ALL */
 #ifdef DEBUG
-				debug = atoi (optarg);
-				debug_delete_files ();
+				debug = atoi(optarg);
+				debug_delete_files();
 #else
-				error_message (_(txt_option_not_enabled), "-DDEBUG");
+				error_message(_(txt_option_not_enabled), "-DDEBUG");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
 #endif /* DEBUG */
 				break;
 
-			case 'f':	/* active (tind) / newsrc (tin) file */
-				my_strncpy (newsrc, optarg, sizeof (newsrc));
+			case 'f':	/* newsrc (tin) file */
+				my_strncpy(newsrc, optarg, sizeof(newsrc));
 				newsrc_set = TRUE;
 				break;
 
 			case 'G':
 				tinrc.getart_limit = atoi(optarg);
-				if (tinrc.getart_limit != 0)
-					tinrc.use_getart_limit = TRUE;
-				else
-					tinrc.use_getart_limit = FALSE;
 				break;
 
 #	ifndef M_AMIGA
@@ -479,7 +484,7 @@ read_cmd_line_options (
 				my_strncpy(cmdline_nntpserver, optarg, sizeof(cmdline_nntpserver));
 				read_news_via_nntp = TRUE;
 #		else
-				error_message (_(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(_(txt_option_not_enabled), "-DNNTP_ABLE");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -488,17 +493,16 @@ read_cmd_line_options (
 #	endif /* !M_AMIGA */
 
 			case 'H':
-				show_intro_page ();
-				exit (EXIT_SUCCESS);
+				show_intro_page();
+				exit(EXIT_SUCCESS);
 				/* keep lint quiet: */
 				/* FALLTHROUGH */
 
 			case 'I':
 #ifndef NNTP_ONLY
-				my_strncpy (index_newsdir, optarg, sizeof (index_newsdir));
-/*				my_mkdir (index_newsdir, (mode_t)S_IRWXUGO);*/
+				my_strncpy(index_newsdir, optarg, sizeof(index_newsdir));
 #else
-				error_message (_(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(_(txt_option_not_enabled), "-DNNTP_ABLE");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -510,11 +514,11 @@ read_cmd_line_options (
 				break;
 
 			case 'm':
-				my_strncpy (tinrc.maildir, optarg, sizeof (tinrc.maildir));
+				my_strncpy(tinrc.maildir, optarg, sizeof(tinrc.maildir));
 				break;
 
 			case 'M':	/* mail new news to specified user */
-				my_strncpy (mail_news_user, optarg, sizeof (mail_news_user));
+				my_strncpy(mail_news_user, optarg, sizeof(mail_news_user));
 				mail_news = TRUE;
 				batch_mode = TRUE;
 				break;
@@ -524,13 +528,21 @@ read_cmd_line_options (
 				break;
 
 			case 'N':	/* mail new news to your posts */
-				my_strncpy (mail_news_user, userid, sizeof(userid));
+				my_strncpy(mail_news_user, userid, sizeof(userid));
 				mail_news = TRUE;
 				batch_mode = TRUE;
 				break;
 
 			case 'o':	/* post postponed articles & exit */
+#ifndef NO_POSTING
 				post_postponed_and_exit = TRUE;
+				check_for_new_newsgroups = FALSE;
+#else
+				error_message(_(txt_option_not_enabled), "-UNO_POSTING");
+				giveup();
+				/* keep lint quiet: */
+				/* NOTREACHED */
+#endif /* NO_POSTING */
 				break;
 
 #ifdef NNTP_ABLE
@@ -555,7 +567,7 @@ read_cmd_line_options (
 #	ifdef NNTP_ABLE
 				read_news_via_nntp = TRUE;
 #	else
-				error_message (_(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(_(txt_option_not_enabled), "-DNNTP_ABLE");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -567,11 +579,11 @@ read_cmd_line_options (
 				list_active = TRUE;
 				newsrc_active = FALSE;
 				check_for_new_newsgroups = FALSE;
-				my_strncpy (news_active_file, save_active_file, sizeof (news_active_file));
+				my_strncpy(news_active_file, save_active_file, sizeof(news_active_file));
 				break;
 
 			case 's':
-				my_strncpy (tinrc.savedir, optarg, sizeof (tinrc.savedir));
+				my_strncpy(tinrc.savedir, optarg, sizeof(tinrc.savedir));
 				break;
 
 			case 'S':	/* save new news to dir structure */
@@ -584,7 +596,7 @@ read_cmd_line_options (
 				batch_mode = TRUE;
 				show_description = FALSE;
 #	else
-				error_message (_(txt_option_not_enabled), "-DNNTP_ABLE");
+				error_message(_(txt_option_not_enabled), "-DNNTP_ABLE");
 				giveup();
 				/* keep lint quiet: */
 				/* NOTREACHED */
@@ -597,15 +609,15 @@ read_cmd_line_options (
 
 			case 'V':
 #if defined(__DATE__) && defined(__TIME__)
-				error_message (_("Version: %s release %s (\"%s\") %s %s"),
+				error_message(_("Version: %s release %s (\"%s\") %s %s"),
 					VERSION, RELEASEDATE, RELEASENAME, __DATE__, __TIME__);
 #else
-				error_message (_("Version: %s release %s (\"%s\")"),
+				error_message(_("Version: %s release %s (\"%s\")"),
 					VERSION, RELEASEDATE, RELEASENAME);
 #endif /* __DATE__ && __TIME__ */
 /*
- * FIXME: make the rest below a function, so we can also use it
- * inside post.c:mail_bug_report()
+ * FIXME: make the rest below a function (i.e. to fill a struct), so we
+ * can also use it inside post.c:mail_bug_report()
  */
 #ifdef SYSTEM_NAME
 				error_message("Platform:");
@@ -639,7 +651,7 @@ read_cmd_line_options (
 
 				error_message("Characteristics:");
 				error_message("\t"
-/* TODO: complete list and do some usefull grouping */
+/* TODO: complete list and do some useful grouping */
 #ifdef NNTP_ONLY
 				"+NNTP_ONLY "
 #else
@@ -654,16 +666,32 @@ read_cmd_line_options (
 #else
 				"-NO_POSTING "
 #endif /* NO_POSTING */
+#ifdef BROKEN_LISTGROUP
+				"+BROKEN_LISTGROUP "
+#else
+				"-BROKEN_LISTGROUP "
+#endif /* BROKEN_LISTGROUP */
+#ifdef XHDR_XREF
+				"+XHDR_XREF"
+#else
+				"-XHDR_XREF"
+#endif /* XHDR_XREF */
+				"\n\t"
 #ifdef HAVE_FASCIST_NEWSADMIN
 				"+HAVE_FASCIST_NEWSADMIN "
 #else
 				"-HAVE_FASCIST_NEWSADMIN "
 #endif /* HAVE_FASCIST_NEWSADMIN */
 #ifdef ENABLE_IPV6
-				"+ENABLE_IPV6"
+				"+ENABLE_IPV6 "
 #else
-				"-ENABLE_IPV6"
+				"-ENABLE_IPV6 "
 #endif /* ENABLE_IPV6 */
+#ifdef HAVE_COREFILE
+				"+HAVE_COREFILE"
+#else
+				"-HAVE_COREFILE"
+#endif /* HAVE_COREFILE */
 				"\n\t"
 #ifdef NO_SHELL_ESCAPE
 				"+NO_SHELL_ESCAPE "
@@ -692,10 +720,31 @@ read_cmd_line_options (
 				"-HAVE_LONG_FILE_NAMES "
 #endif /* HAVE_LONG_FILE_NAMES */
 #ifdef APPEND_PID
-				"+APPEND_PID"
+				"+APPEND_PID "
 #else
-				"-APPEND_PID"
+				"-APPEND_PID "
 #endif /* APPEND_PID */
+#ifdef HAVE_MH_MAIL_HANDLING
+				"+HAVE_MH_MAIL_HANDLING"
+#else
+				"-HAVE_MH_MAIL_HANDLING"
+#endif /* HAVE_MH_MAIL_HANDLING */
+				"\n\t"
+#ifdef HAVE_ISPELL
+				"+HAVE_ISPELL "
+#else
+				"-HAVE_ISPELL "
+#endif /* HAVE_ISPELL */
+#ifdef HAVE_METAMAIL
+				"+HAVE_METAMAIL "
+#else
+				"-HAVE_METAMAIL "
+#endif /* HAVE_METAMAIL */
+#ifdef HAVE_SUM
+				"+HAVE_SUM "
+#else
+				"-HAVE_SUM "
+#endif /* HAVE_SUM */
 				"\n\t"
 #ifdef HAVE_COLOR
 				"+HAVE_COLOR "
@@ -727,24 +776,83 @@ read_cmd_line_options (
 				"+MIME_STRICT_CHARSET "
 #else
 				"-MIME_STRICT_CHARSET "
-#endif /* MIME_BREAK_LONG_LINES */
+#endif /* MIME_STRICT_CHARSET */
+#ifdef CHARSET_CONVERSION
+				"+CHARSET_CONVERSION "
+#else
+				"-CHARSET_CONVERSION "
+#endif /* CHARSET_CONVERSION */
+				"\n\t"
+#ifdef LOCAL_CHARSET
+				"+LOCAL_CHARSET "
+#else
+				"-LOCAL_CHARSET "
+#endif /* LOCAL_CHARSET */
 #ifdef NO_LOCALE
 				"+NO_LOCALE "
 #else
 				"-NO_LOCALE "
 #endif /* NO_LOCALE */
+				"\n\t"
+#ifdef USE_CANLOCK
+				"+USE_CANLOCK "
+#else
+				"-USE_CANLOCK "
+#endif /* USE_CANLOCK */
+#ifdef EVIL_INSIDE
+				"+EVIL_INSIDE "
+#else
+				"-EVIL_INSIDE "
+#endif /* EVIL_INSIDE */
+#ifdef FORGERY
+				"+FORGERY "
+#else
+				"-FORGERY "
+#endif /* FORGERY */
+#ifdef TINC_DNS
+				"+TINC_DNS "
+#else
+				"-TINC_DNS "
+#endif /* TINC_DNS */
+#ifdef ENFORCE_RFC1034
+				"+ENFORCE_RFC1034"
+#else
+				"-ENFORCE_RFC1034"
+#endif /* ENFORCE_RFC1034  */
 				);
-				exit (EXIT_SUCCESS);
+				error_message("\t"
+#ifdef REQUIRE_BRACKETS_IN_DOMAIN_LITERAL
+				"+REQUIRE_BRACKETS_IN_DOMAIN_LITERAL "
+#else
+				"-REQUIRE_BRACKETS_IN_DOMAIN_LITERAL "
+#endif /* REQUIRE_BRACKETS_IN_DOMAIN_LITERAL */
+#ifdef FOLLOW_USEFOR_DRAFT
+				"+FOLLOW_USEFOR_DRAFT"
+#else
+				"-FOLLOW_USEFOR_DRAFT"
+#endif /* FOLLOW_USEFOR_DRAFT */
+				);
+				exit(EXIT_SUCCESS);
 				/* keep lint quiet: */
 				/* FALLTHROUGH */
 
 			case 'w':	/* post article & exit */
+#ifndef NO_POSTING
 				post_article_and_exit = TRUE;
-				no_write = TRUE;
-				newsrc_active = TRUE;
 				check_for_new_newsgroups = FALSE;
+#else
+				error_message(_(txt_option_not_enabled), "-UNO_POSTING");
+				giveup();
+				/* keep lint quiet: */
+				/* NOTREACHED */
+#endif /* NO_POSTING */
 				break;
 
+#if 0
+			case 'W':	/* reserved according to SUSV3 XDB Utility Syntax Guidelines, Guideline 3 */
+				break;
+#endif /* 0 */
+
 			case 'X':	/* don't save ~/.newsrc on exit */
 				no_write = TRUE;
 				break;
@@ -761,8 +869,8 @@ read_cmd_line_options (
 			case 'h':
 			case '?':
 			default:
-				usage (tin_progname);
-				exit (EXIT_SUCCESS);
+				usage(tin_progname);
+				exit(EXIT_SUCCESS);
 		}
 	}
 	cmdargs = argv;
@@ -777,13 +885,13 @@ read_cmd_line_options (
 			(void) uname(&uts);
 			get_newsrcname(newsrc, uts.nodename);
 #else
-			char nodenamebuf[32];
-#	ifdef M_AMIGA
-			my_strncpy (nodenamebuf, get_val ("NodeName", "PROBLEM_WITH_NODE_NAME"), sizeof (nodenamebuf));
-#	else
-			/* NeXT, Apollo */
+			char nodenamebuf[256]; /* SUSv2 limit; better use HOST_NAME_MAX */
+#	if defined(HAVE_GETHOSTNAME) && !defined(M_AMIGA)
 			(void) gethostname(nodenamebuf, sizeof(nodenamebuf));
-#	endif /* M_AMIGA */
+#	else
+			/* TODO: document $NodeName */
+			my_strncpy(nodenamebuf, get_val("NodeName", "PROBLEM_WITH_NODE_NAME"), sizeof(nodenamebuf));
+#	endif /* HAVE_GETHOSTNAME && !M_AMIGA */
 			get_newsrcname(newsrc, nodenamebuf);
 #endif /* HAVE_SYS_UTSNAME_H && HAVE_UNAME */
 		}
@@ -791,14 +899,18 @@ read_cmd_line_options (
 
 	/*
 	 * Sort out conflicts of options....
+	 * TODO: -> lang.c
 	 */
 	if (verbose && !batch_mode) {
-		wait_message(1, _("-v only useful for batch mode operations\n"));
+		wait_message(2, _("%s only useful for batch mode operations\n"), "-v");
 		verbose = FALSE;
 	}
-
+	if (catchup && !batch_mode) {
+		wait_message(2, _("%s only useful for batch mode operations\n"), "-c");
+		catchup = FALSE;
+	}
 	if (read_saved_news && batch_mode) {
-		wait_message(1, _("-R only useful without batch mode operations\n"));
+		wait_message(2, _("%s only useful without batch mode operations\n"), "-R");
 		read_saved_news = FALSE;
 	}
 
@@ -808,7 +920,7 @@ read_cmd_line_options (
 	 * for new newsgroups, trust our cached copy of the newsgroups file.
 	 */
 	if (read_news_via_nntp)
-		read_local_newsgroups_file = !check_for_new_newsgroups;
+		read_local_newsgroups_file = bool_not(check_for_new_newsgroups);
 #endif /* NNTP_ABLE */
 	/*
 	 * If we use neither list_active nor newsrc_active,
@@ -827,98 +939,97 @@ read_cmd_line_options (
  * FIXME: move strings to lang.c
  */
 static void
-usage (
+usage(
 	char *theProgname)
 {
-	error_message (_("A Usenet reader.\n\nUsage: %s [options] [newsgroup[,...]]"), theProgname);
+	error_message(_("A Usenet reader.\n\nUsage: %s [options] [newsgroup[,...]]"), theProgname);
 
 #	ifndef M_AMIGA
 #		ifdef HAVE_COLOR
-			error_message (_("  -a       toggle color flag"));
+			error_message(_("  -a       toggle color flag"));
 #		endif /* HAVE_COLOR */
 #		ifdef NNTP_ABLE
-			error_message (_("  -A       force authentication on connect"));
+			error_message(_("  -A       force authentication on connect"));
 #		endif /* NNTP_ABLE */
 #	else
-		error_message (_("  -B       BBS mode. File operations limited to home directories."));
+		error_message(_("  -B       BBS mode. File operations limited to home directories."));
 #	endif /* !M_AMIGA */
 
-	error_message (_("  -c       mark all news as read in subscribed newsgroups (batch mode)"));
-	error_message (_("  -d       don't show newsgroup descriptions"));
+	error_message(_("  -c       mark all news as read in subscribed newsgroups (batch mode)"));
+	error_message(_("  -d       don't show newsgroup descriptions"));
 
 #	ifdef DEBUG
-		error_message (_("  -D       debug mode 1=NNTP 2=ALL"));
+		error_message(_("  -D       debug mode 1=NNTP 2=ALL"));
 #	endif /* DEBUG */
 
-	error_message (_("  -f file  subscribed to newsgroups file [default=%s]"), newsrc);
-	error_message (_("  -G limit get only limit articles/group"));
+	error_message(_("  -f file  subscribed to newsgroups file [default=%s]"), newsrc);
+	error_message(_("  -G limit get only limit articles/group"));
 
 #	ifndef M_AMIGA
 #		ifdef NNTP_ABLE
 			/* FIXME, default should be $NNTPSERVER if set ... */
-			error_message (_("  -g serv  read news from NNTP server serv [default=%s]"), NNTP_DEFAULT_SERVER);
+			error_message(_("  -g serv  read news from NNTP server serv [default=%s]"), NNTP_DEFAULT_SERVER);
 #		endif /* NNTP_ABLE */
 #	endif /* !M_AMIGA */
 
-	error_message (_("  -h       this help message"));
-	error_message (_("  -H       help information about %s"), theProgname);
+	error_message(_("  -h       this help message"));
+	error_message(_("  -H       help information about %s"), theProgname);
 
 #	ifndef NNTP_ONLY
-		error_message (_("  -I dir   news index file directory [default=%s]"), index_newsdir);
+		error_message(_("  -I dir   news index file directory [default=%s]"), index_newsdir);
 #	endif /* !NNTP_ONLY */
 
 #	ifdef NNTP_ABLE
-		error_message (_("  -l       use only LISTGROUP instead of GROUP (-n) command"));
+		error_message(_("  -l       use only LISTGROUP instead of GROUP (-n) command"));
 #	else
-		error_message (_("  -l       TODO: document feature"));
+		error_message(_("  -l       TODO: document feature"));
 #	endif /* NNTP_ABLE */
 
-	error_message (_("  -m dir   mailbox directory [default=%s]"), tinrc.maildir);
-	error_message (_("  -M user  mail new news to specified user (batch mode)"));
+	error_message(_("  -m dir   mailbox directory [default=%s]"), tinrc.maildir);
+	error_message(_("  -M user  mail new news to specified user (batch mode)"));
 
 #	ifdef NNTP_ABLE
-		error_message (_("  -n       only read subscribed .newsrc groups from NNTP server"));
+		error_message(_("  -n       only read subscribed .newsrc groups from NNTP server"));
 #	else
-		error_message (_("  -n       TODO: document feature"));
+		error_message(_("  -n       TODO: document feature"));
 #	endif /* NNTP_ABLE */
 
-	error_message (_("  -N       mail new news to your posts (batch mode)"));
-	error_message (_("  -o       post all postponed articles and exit"));
+	error_message(_("  -N       mail new news to your posts (batch mode)"));
+	error_message(_("  -o       post all postponed articles and exit"));
 
 #	ifdef NNTP_ABLE
-		error_message (_("  -p port  use port as NNTP port [default=%d]"), nntp_tcp_port);
+		error_message(_("  -p port  use port as NNTP port [default=%d]"), nntp_tcp_port);
 #	endif /* NNTP_ABLE */
 
-	error_message (_("  -q       don't check for new newsgroups"));
+	error_message(_("  -q       don't check for new newsgroups"));
 
 #	ifdef NNTP_ABLE
-	error_message (_("  -Q       quick start. Same as -nqd"));
+	error_message(_("  -Q       quick start. Same as -nqd"));
 #	else
-	error_message (_("  -Q       quick start. Same as -qd"));
+	error_message(_("  -Q       quick start. Same as -qd"));
 #	endif /* NNTP_ABLE */
 
 #	ifdef NNTP_ABLE
 		if (!read_news_via_nntp)
-			error_message (_("  -r       read news remotely from default NNTP server"));
+			error_message(_("  -r       read news remotely from default NNTP server"));
 #	endif /* NNTP_ABLE */
 
-	error_message (_("  -R       read news saved by -S option"));
-	error_message (_("  -s dir   save news directory [default=%s]"), tinrc.savedir);
-	error_message (_("  -S       save new news for later reading (batch mode)"));
+	error_message(_("  -R       read news saved by -S option"));
+	error_message(_("  -s dir   save news directory [default=%s]"), tinrc.savedir);
+	error_message(_("  -S       save new news for later reading (batch mode)"));
 
 #	ifndef NNTP_ONLY
-		error_message (_("  -u       update index files (batch mode)"));
-		error_message (_("  -U       update index files in the background while reading news"));
+		error_message(_("  -u       update index files (batch mode)"));
 #	endif /* !NNTP_ONLY */
 
-	error_message (_("  -v       verbose output for batch mode options"));
-	error_message (_("  -V       print version & date information"));
-	error_message (_("  -w       post an article and exit"));
-	error_message (_("  -X       don't save any files on quit"));
-	error_message (_("  -z       start if any unread news"));
-	error_message (_("  -Z       return status indicating if any unread news (batch mode)"));
+	error_message(_("  -v       verbose output for batch mode options"));
+	error_message(_("  -V       print version & date information"));
+	error_message(_("  -w       post an article and exit"));
+	error_message(_("  -X       don't save any files on quit"));
+	error_message(_("  -z       start if any unread news"));
+	error_message(_("  -Z       return status indicating if any unread news (batch mode)"));
 
-	error_message (_("\nMail bug reports/comments to %s"), bug_addr);
+	error_message(_("\nMail bug reports/comments to %s"), bug_addr);
 }
 
 
@@ -926,20 +1037,20 @@ usage (
  * update index files
  */
 static void
-update_index_files (
+update_index_files(
 	void)
 {
 	if (!catchup && (read_news_via_nntp && xover_supported)) {
-		error_message (_(txt_batch_update_unavail), tin_progname);
-		tin_done (EXIT_FAILURE);
+		error_message(_(txt_batch_update_unavail), tin_progname);
+		tin_done(EXIT_FAILURE);
 	}
 
 	cCOLS = 132;							/* set because curses has not started */
 	batch_mode = TRUE;					/* -u handling... */
-	create_index_lock_file (lock_file);
+	create_index_lock_file(lock_file);
 	tinrc.thread_articles = THREAD_NONE;	/* stop threading to run faster */
-	do_update (catchup);
-	tin_done (EXIT_SUCCESS);
+	do_update(catchup);
+	tin_done(EXIT_SUCCESS);
 }
 
 
@@ -947,26 +1058,26 @@ update_index_files (
  * display page of general info. for first time user.
  */
 static void
-show_intro_page (
+show_intro_page(
 	void)
 {
 	char buf[4096];
 
 	if (!cmd_line) {
-		ClearScreen ();
-		center_line (0, TRUE, cvers);
-		Raw (FALSE);
+		ClearScreen();
+		center_line(0, TRUE, cvers);
+		Raw(FALSE);
 		my_printf("\n");
 	}
 
 	snprintf(buf, sizeof(buf) - 1, _(txt_intro_page), PRODUCT, PRODUCT, PRODUCT, bug_addr);
 
-	my_fputs (buf, stdout);
+	my_fputs(buf, stdout);
 	my_flush();
 
 	if (!cmd_line) {
-		Raw (TRUE);
-		prompt_continue ();
+		Raw(TRUE);
+		prompt_continue();
 	}
 }
 
@@ -976,42 +1087,41 @@ show_intro_page (
  * yank at startup. Return number of groups that were matched.
  */
 int
-read_cmd_line_groups (
+read_cmd_line_groups(
 	void)
 {
 	int matched = 0;
 	int num;
-	register int i;
+	int i;
 
 	if (num_cmdargs < max_cmdargs) {
 		selmenu.max = skip_newgroups();		/* Reposition after any newgroups */
 
 		for (num = num_cmdargs; num < max_cmdargs; num++) {
-			wait_message (0, _(txt_matching_cmd_line_groups), cmdargs[num]);
+			wait_message(0, _(txt_matching_cmd_line_groups), cmdargs[num]);
 
-			for (i = 0; i < num_active; i++) {
-				if (match_group_list (active[i].name, cmdargs[num])) {
-					if (my_group_add (active[i].name) != -1)
+			for_each_group(i) {
+				if (match_group_list(active[i].name, cmdargs[num])) {
+					if (my_group_add(active[i].name) != -1)
 						matched++;
 				}
 			}
 		}
 	}
-
 	return matched;
 }
 
 
 void
-giveup (
+giveup(
 	void)
 {
 	static int nested;
 
 	if (!cmd_line && !nested++) {
 		cursoron();
-		EndWin ();
-		Raw (FALSE);
+		EndWin();
+		Raw(FALSE);
 	}
-	exit (EXIT_FAILURE);
+	exit(EXIT_FAILURE);
 }
