@@ -3,10 +3,10 @@
  *  Module    : memory.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 1994-06-05
+ *  Updated   : 2003-01-21
  *  Notes     :
  *
- * Copyright (c) 1991-2001 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Iain Lea, Rich Skrenta.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -70,46 +67,46 @@ struct t_save *save;			/* sorts articles before saving them */
 /*
  * Local prototypes
  */
-static void free_newnews_array (void);
-static void free_active_arrays (void);
-static void free_if_not_default (char **attrib, char *deflt);
+static void free_active_arrays(void);
+static void free_if_not_default(char **attrib, char *deflt);
+static void free_newnews_array(void);
+
 
 /*
- *  Dynamic table management
- *  These settings are memory conservative:  small initial allocations
- *  and a 50% expansion on table overflow.  A fast vm system with
- *  much memory might want to start with higher initial allocations
- *  and a 100% expansion on overflow, especially for the arts[] array.
+ * Dynamic table management
+ * These settings are memory conservative: small initial allocations
+ * and a 50% expansion on table overflow. A fast vm system with
+ * much memory might want to start with higher initial allocations
+ * and a 100% expansion on overflow, especially for the arts[] array.
  */
-
 void
-init_alloc (
+init_alloc(
 	void)
 {
 	/*
 	 * active file arrays
 	 */
-	max_active = get_active_num ();
+	max_active = get_active_num();
 	max_newnews = DEFAULT_NEWNEWS_NUM;
 
-	active = (struct t_group *) my_malloc (sizeof(*active) * max_active);
-	newnews = (struct t_newnews *) my_malloc (sizeof(*newnews) * max_newnews);
-	my_group = (int *) my_malloc (sizeof(int) * max_active);
+	active = my_malloc(sizeof(*active) * max_active);
+	newnews = my_malloc(sizeof(*newnews) * max_newnews);
+	my_group = my_calloc(1, sizeof(int) * max_active);
 
 	/*
 	 * article headers array
 	 */
 	max_art = DEFAULT_ARTICLE_NUM;
 
-	arts = (struct t_article *) my_malloc (sizeof(*arts) * max_art);
-	base = (long *) my_malloc (sizeof(long) * max_art);
+	arts = my_malloc(sizeof(*arts) * max_art);
+	base = my_malloc(sizeof(long) * max_art);
 
 	/*
 	 * save file array
 	 */
 	max_save = DEFAULT_SAVE_NUM;
 
-	save = (struct t_save *) my_malloc (sizeof(*save) * max_save);
+	save = my_malloc(sizeof(*save) * max_save);
 
 #ifndef USE_CURSES
 	screen = (struct t_screen *) 0;
@@ -118,75 +115,70 @@ init_alloc (
 
 
 void
-expand_art (
+expand_art(
 	void)
 {
-	max_art += max_art / 2;		/* increase by 50% */
-
-	arts = (struct t_article *) my_realloc ((char *) arts, sizeof(*arts) * max_art);
-	base = (long *) my_realloc ((char *) base, sizeof(long) * max_art);
+	max_art += max_art >> 1;		/* increase by 50% */
+	arts = my_realloc(arts, sizeof(*arts) * max_art);
+	base = my_realloc(base, sizeof(long) * max_art);
 }
 
 
 void
-expand_active (
+expand_active(
 	void)
 {
-	max_active += max_active / 2;		/* increase by 50% */
-
-	if (active == (struct t_group *) 0) {
-		active = (struct t_group *) my_malloc (sizeof (*active) * max_active);
-		my_group = (int *) my_malloc (sizeof (int) * max_active);
+	max_active += max_active >> 1;		/* increase by 50% */
+	if (active == NULL) {
+		active = my_malloc(sizeof(*active) * max_active);
+		my_group = my_calloc(1, sizeof(int) * max_active);
 	} else {
-		active = (struct t_group *) my_realloc((char *) active, sizeof (*active) * max_active);
-		my_group = (int *) my_realloc((char *) my_group, sizeof (int) * max_active);
+		active = my_realloc(active, sizeof(*active) * max_active);
+		my_group = my_realloc(my_group, sizeof(int) * max_active);
 	}
 }
 
 
 void
-expand_save (
+expand_save(
 	void)
 {
-	max_save += max_save / 2;		/* increase by 50% */
-
-	save = (struct t_save *) my_realloc((char *) save,
-		sizeof (struct t_save) * max_save);
+	max_save += max_save >> 1;		/* increase by 50% */
+	save = my_realloc(save, sizeof(struct t_save) * max_save);
 }
 
 
 void
-expand_newnews (
+expand_newnews(
 	void)
 {
-	max_newnews += max_newnews / 2;			/* increase by 50% */
-
-	newnews = (struct t_newnews *) my_realloc((char *) newnews,
-		sizeof(struct t_newnews) * max_newnews);
+	max_newnews += max_newnews >> 1;			/* increase by 50% */
+	newnews = my_realloc(newnews, sizeof(struct t_newnews) * max_newnews);
 }
 
 
 #ifndef USE_CURSES
 void
-init_screen_array (
+init_screen_array(
 	t_bool allocate)
 {
 	int i;
 
 	if (allocate) {
-		screen = (struct t_screen *) my_malloc (
-			sizeof (struct t_screen) * cLINES+1);
+		screen = my_malloc(sizeof(struct t_screen) * cLINES + 1);
 
 		for (i = 0; i < cLINES; i++)
-			screen[i].col = (char *) my_malloc ((size_t)(cCOLS+2));
-
+#	if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+			screen[i].col = my_malloc((size_t) (cCOLS * MB_CUR_MAX + 2));
+#	else
+			screen[i].col = my_malloc((size_t) (cCOLS + 2));
+#	endif /* MULTIBYTE_ABLE && !NO_LOCALE */
 	} else {
-		if (screen != (struct t_screen *) 0) {
+		if (screen != NULL) {
 			for (i = 0; i < cLINES; i++)
 				FreeAndNull(screen[i].col);
 
-			free ((char *) screen);
-			screen = (struct t_screen *) 0;
+			FreeAndNull(screen);
 		}
 	}
 }
@@ -194,25 +186,20 @@ init_screen_array (
 
 
 void
-free_all_arrays (
+free_all_arrays(
 	void)
 {
-	hash_reclaim ();
+	hash_reclaim();
 
 #ifndef USE_CURSES
-	init_screen_array (FALSE);
+	init_screen_array(FALSE);
 #endif /* !USE_CURSES */
 
-	free_art_array ();
-	free_msgids ();
-
-	if (arts != (struct t_article *) 0) {
-		free ((char *) arts);
-		arts = (struct t_article *) 0;
-	}
-
-	free_all_filter_arrays();
-	free_active_arrays ();
+	free_art_array();
+	free_msgids();
+	FreeAndNull(arts);
+	free_filter_array(&glob_filter);
+	free_active_arrays();
 
 #ifdef HAVE_COLOR
 	FreeIfNeeded(quote_regex.re);
@@ -222,6 +209,14 @@ free_all_arrays (
 	FreeIfNeeded(quote_regex3.re);
 	FreeIfNeeded(quote_regex3.extra);
 #endif /* HAVE_COLOR */
+	FreeIfNeeded(slashes_regex.re);
+	FreeIfNeeded(slashes_regex.extra);
+	FreeIfNeeded(stars_regex.re);
+	FreeIfNeeded(stars_regex.extra);
+	FreeIfNeeded(strokes_regex.re);
+	FreeIfNeeded(strokes_regex.extra);
+	FreeIfNeeded(underscores_regex.re);
+	FreeIfNeeded(underscores_regex.extra);
 	FreeIfNeeded(strip_re_regex.re);
 	FreeIfNeeded(strip_re_regex.extra);
 	FreeIfNeeded(strip_was_regex.re);
@@ -239,38 +234,28 @@ free_all_arrays (
 	FreeIfNeeded(shar_regex.re);
 	FreeIfNeeded(shar_regex.extra);
 
-	if (base != (long *) 0) {
-		free ((char *) base);
-		base = (long *) 0;
+	FreeAndNull(base);
+
+	if (save != NULL) {
+		free_save_array();
+		FreeAndNull(save);
 	}
 
-	if (save != (struct t_save *) 0) {
-		free_save_array ();
-		if (save != (struct t_save *) 0) {
-			free ((char *) save);
-			save = (struct t_save *) 0;
-		}
+	if (newnews != NULL) {
+		free_newnews_array();
+		FreeAndNull(newnews);
 	}
-
-	if (newnews != (struct t_newnews *) 0) {
-		free_newnews_array ();
-		if (newnews != (struct t_newnews *) 0) {
-			free ((char *) newnews);
-			newnews = (struct t_newnews *) 0;
-		}
-	}
-
-	free_keymaps ();
+	free_keymaps();
 }
 
 
 void
-free_art_array (
+free_art_array(
 	void)
 {
 	register int i;
 
-	for (i = 0; i < top_art; i++) {
+	for_each_art(i) {
 		arts[i].artnum = 0L;
 		arts[i].thread = ART_EXPIRED;
 		arts[i].inthread = FALSE;
@@ -296,26 +281,23 @@ free_art_array (
  * in tinrc
  */
 static void
-free_if_not_default (
+free_if_not_default(
 	char **attrib,
 	char *deflt)
 {
-	/* Can't see how these attribs can be = NULL */
-	if (*attrib != (char *) 0 && *attrib != deflt) {
-		free ((char *) *attrib);
-		*attrib = (char *) 0;
-	}
+	if (*attrib != deflt)
+		FreeAndNull(*attrib);
 }
 
 
 void
-free_attributes_array (
+free_attributes_array(
 	void)
 {
 	register int i;
 	struct t_group *psGrp;
 
-	for (i = 0; i < num_active; i++) {
+	for_each_group(i) {
 		psGrp = &active[i];
 		if (psGrp->attribute && !psGrp->attribute->global) {
 			free_if_not_default(&psGrp->attribute->maildir, tinrc.maildir);
@@ -328,10 +310,6 @@ free_attributes_array (
 
 			FreeAndNull(psGrp->attribute->followup_to);
 
-#ifndef DISABLE_PRINTING
-			free_if_not_default(&psGrp->attribute->printer, tinrc.printer);
-#endif /* !DISABLE_PRINTING */
-
 			FreeAndNull(psGrp->attribute->quick_kill_scope);
 			FreeAndNull(psGrp->attribute->quick_select_scope);
 
@@ -347,7 +325,7 @@ free_attributes_array (
 			FreeAndNull(psGrp->attribute->ispell);
 #endif /* HAVE_ISPELL */
 
-			free ((char *) psGrp->attribute);
+			free(psGrp->attribute);
 		}
 		psGrp->attribute = (struct t_attribute *) 0;
 	}
@@ -355,52 +333,37 @@ free_attributes_array (
 
 
 static void
-free_active_arrays (
+free_active_arrays(
 	void)
 {
 	register int i;
 
-	if (my_group != (int *) 0) {			/* my_group[] */
-		free ((char *) my_group);
-		my_group = (int *) 0;
-	}
-
-	if (active != (struct t_group *) 0) {		/* active[] */
-		for (i = 0; i < num_active; i++) {
+	FreeAndNull(my_group);	/* my_group[] */
 
+	if (active != NULL) {	/* active[] */
+		for_each_group(i) {
 			FreeAndNull(active[i].name);
 			FreeAndNull(active[i].description);
 			FreeAndNull(active[i].aliasedto);
-
-			if (active[i].type == GROUP_TYPE_MAIL && active[i].spooldir != (char *) 0) {
-				free ((char *) active[i].spooldir);
-				active[i].spooldir = (char *) 0;
-			}
-			if (active[i].newsrc.xbitmap != 0) {
-				free ((char *) active[i].newsrc.xbitmap);
-				active[i].newsrc.xbitmap = 0;
+			if (active[i].type == GROUP_TYPE_MAIL) {
+				FreeAndNull(active[i].spooldir);
 			}
+			FreeAndNull(active[i].newsrc.xbitmap);
 		}
-
-		free_attributes_array ();
-
-		if (active != (struct t_group *) 0) {
-			free ((char *) active);
-			active = (struct t_group *) 0;
-		}
+		free_attributes_array();
+		FreeAndNull(active);
 	}
 	num_active = -1;
 }
 
 
 void
-free_save_array (
+free_save_array(
 	void)
 {
 	int i;
 
 	for (i = 0; i < num_save; i++) {
-
 		FreeAndNull(save[i].path);
 		/* file does NOT need to be freed */
 		save[i].file = NULL;
@@ -408,12 +371,12 @@ free_save_array (
 		save[i].saved = FALSE;
 		save[i].is_mailbox = FALSE;
 	}
-
 	num_save = 0;
 }
 
+
 static void
-free_newnews_array (
+free_newnews_array(
 	void)
 {
 	int i;
@@ -426,42 +389,65 @@ free_newnews_array (
 
 
 void *
-my_malloc1 (
+my_malloc1(
 	const char *file,
 	int line,
 	size_t size)
 {
-	char *p;
+	void *p;
 
 #ifdef DEBUG
-	vDbgPrintMalloc (TRUE, file, line, size);
+	vDbgPrintMalloc(TRUE, file, line, size);
 #endif /* DEBUG */
 
-	if ((p = (char *) malloc (size)) == (char *) 0) {
-		error_message (txt_out_of_memory, tin_progname, size, file, line);
+	if ((p = malloc(size)) == NULL) {
+		error_message(txt_out_of_memory, tin_progname, size, file, line);
 		giveup();
 	}
-	return (void *) p;
+	return p;
+}
+
+
+/*
+ * TODO: add fallback code with malloc(nmemb*size);memset(0,nmemb*size)?
+ */
+void *
+my_calloc1(
+	const char *file,
+	int line,
+	size_t nmemb,
+	size_t size)
+{
+	void *p;
+
+#ifdef DEBUG
+	vDbgPrintMalloc(TRUE, file, line, nmemb * size);
+#endif /* DEBUG */
+
+	if ((p = calloc(nmemb, size)) == NULL) {
+		error_message(txt_out_of_memory, tin_progname, nmemb * size, file, line);
+		giveup();
+	}
+	return p;
 }
 
 
 void *
-my_realloc1 (
+my_realloc1(
 	const char *file,
 	int line,
-	char *p,
+	void *p,
 	size_t size)
 {
-
 #ifdef DEBUG
-	vDbgPrintMalloc (FALSE, file, line, size);
+	vDbgPrintMalloc(FALSE, file, line, size);
 #endif /* DEBUG */
 
-	p = (char *) ((!p) ? (calloc (1, size)) : realloc (p, size));
+	p = ((!p) ? (calloc(1, size)) : realloc(p, size));
 
-	if (p == (char *) 0) {
-		error_message (txt_out_of_memory, tin_progname, size, file, line);
+	if (p == NULL) {
+		error_message(txt_out_of_memory, tin_progname, size, file, line);
 		giveup();
 	}
-	return (void *) p;
+	return p;
 }
