@@ -6,7 +6,7 @@
  *  Updated   : 1999-12-02
  *  Notes     : PGP support
  *
- * Copyright (c) 1995-2002 Steven J. Madsen <steve@erinet.com>
+ * Copyright (c) 1995-2003 Steven J. Madsen <steve@erinet.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Steven J. Madsen.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -55,7 +52,7 @@
  * NB: The '1' variations on DO_{SIGN,BOTH} are used when local-user name is
  * used and are valid only when signing
  */
-#	if defined(HAVE_PGP) /* && !defined(PGPNAME) */ /* pgp-2 */
+#	if defined(HAVE_PGP) /* pgp-2 */
 #		define PGPNAME		PATH_PGP
 #		define PGPDIR		".pgp"
 #		define PGP_PUBRING	"pubring.pgp"
@@ -69,7 +66,7 @@
 #		define DO_BOTH1		"%s %s -ates %s %s -u %s", PGPNAME, pgpopts, pt, mailto, mailfrom
 #	endif /* HAVE_PGP */
 
-#	if defined(HAVE_PGPK) /* && !defined(PGPNAME) */ /* pgp-5 */
+#	if defined(HAVE_PGPK) /* pgp-5 */
 #		define PGPNAME		"pgp"	/* FIXME: this is AFAIK not PATH_PGPK */
 #		define PGPDIR		".pgp"
 #		define PGP_PUBRING	"pubring.pkr"
@@ -83,7 +80,7 @@
 #		define DO_BOTH1		"%se %s -ats %s %s -u %s", PGPNAME, pgpopts, pt, mailto, mailfrom
 #	endif /* HAVE_PGPK */
 
-#	if defined(HAVE_GPG) && !defined(PGPNAME) /* gpg */
+#	if defined(HAVE_GPG) /* gpg */
 #		define PGPNAME		PATH_GPG
 #		define PGPDIR		".gnupg"
 #		define PGP_PUBRING	"pubring.gpg"
@@ -129,20 +126,19 @@ PGPNAME, pgpopts, pt, mailto, mailfrom, pt
 /*
  * local prototypes
  */
-static t_bool pgp_available (void);
-static void do_pgp (int what, const char *file, const char *mail_to);
-static void join_files (const char *file);
-static void pgp_append_public_key (char *file);
-static void split_file (const char *file);
+static t_bool pgp_available(void);
+static void do_pgp(int what, const char *file, const char *mail_to);
+static void join_files(const char *file);
+static void pgp_append_public_key(char *file);
+static void split_file(const char *file);
 
 static char pgp_data[PATH_LEN];
-static const char *pgpopts = "";
-
 static char hdr[PATH_LEN], pt[PATH_LEN], ct[PATH_LEN];
+static const char *pgpopts = "";
 
 
 void
-init_pgp (
+init_pgp(
 	void)
 {
 	char *ptr;
@@ -150,15 +146,15 @@ init_pgp (
 	pgpopts = get_val("PGPOPTS", "");
 
 #ifdef HAVE_GPG
-	if ((ptr = getenv("GNUPGHOME")) != (char *) 0)
-		my_strncpy (pgp_data, ptr, sizeof(pgp_data));
+	if ((ptr = getenv("GNUPGHOME")) != NULL)
+		my_strncpy(pgp_data, ptr, sizeof(pgp_data));
 	else
 #endif /* HAVE_GPG */
 	{
-		if ((ptr = getenv("PGPPATH")) != (char *) 0)
-			my_strncpy (pgp_data, ptr, sizeof(pgp_data));
+		if ((ptr = getenv("PGPPATH")) != NULL)
+			my_strncpy(pgp_data, ptr, sizeof(pgp_data));
 		else
-			joinpath (pgp_data, homedir, PGPDIR);
+			joinpath(pgp_data, homedir, PGPDIR);
 	}
 }
 
@@ -168,7 +164,7 @@ init_pgp (
  * This function is void, no way to return errs
  */
 static void
-join_files (
+join_files(
 	const char *file)
 {
 	FILE *art, *header, *text;
@@ -196,7 +192,7 @@ join_files (
  * file 'pt'
  */
 static void
-split_file (
+split_file(
 	const char *file)
 {
 	FILE *art, *header, *plaintext;
@@ -207,15 +203,15 @@ split_file (
 	snprintf(pt, sizeof(pt) - 1, PLAINTEXT, TMPDIR, process_id);
 	snprintf(ct, sizeof(ct) - 1, CIPHERTEXT, TMPDIR, process_id);
 
-	if ((art = fopen(file, "r")) == (FILE *) 0)
+	if ((art = fopen(file, "r")) == NULL)
 		return;
 
 	mask = umask((mode_t) (S_IRWXO|S_IRWXG));
 
-	if ((header = fopen(hdr, "w")) == (FILE *) 0)
+	if ((header = fopen(hdr, "w")) == NULL)
 		goto err_art;
 
-	if ((plaintext = fopen(pt, "w")) == (FILE *) 0)
+	if ((plaintext = fopen(pt, "w")) == NULL)
 		goto err_hdr;
 
 	fgets(buf, LEN, art);			/* Copy the hdr up to and including the \n */
@@ -236,14 +232,14 @@ err_art:
 
 
 static void
-do_pgp (
+do_pgp(
 	int what,
 	const char *file,
 	const char *mail_to)
 {
 	char cmd[LEN];
 	char mailfrom[LEN];
-	const char *mailto = (mail_to) ? mail_to : "";
+	const char *mailto = BlankIfNull(mail_to);
 
 	mailfrom[0] = '\0';
 
@@ -253,37 +249,36 @@ do_pgp (
 	 * <mailfrom> is valid only when signing and a local address exists
 	 */
 	if (what & PGP_SIGN) {
-		if ((CURR_GROUP.attribute->from) != (char *) 0)
-			strip_name (CURR_GROUP.attribute->from, mailfrom);
+		if ((CURR_GROUP.attribute->from) != NULL)
+			strip_name(CURR_GROUP.attribute->from, mailfrom);
 		if (strlen(mailfrom)) {
 			if (what & PGP_ENCRYPT)
-				sh_format (cmd, sizeof(cmd), DO_BOTH1);
+				sh_format(cmd, sizeof(cmd), DO_BOTH1);
 			else
-				sh_format (cmd, sizeof(cmd), DO_SIGN1);
+				sh_format(cmd, sizeof(cmd), DO_SIGN1);
 		} else {
 			if (what & PGP_ENCRYPT)
-				sh_format (cmd, sizeof(cmd), DO_BOTH);
+				sh_format(cmd, sizeof(cmd), DO_BOTH);
 			else
-				sh_format (cmd, sizeof(cmd), DO_SIGN);
+				sh_format(cmd, sizeof(cmd), DO_SIGN);
 		}
 	} else
-		sh_format (cmd, sizeof(cmd), DO_ENCRYPT);
+		sh_format(cmd, sizeof(cmd), DO_ENCRYPT);
 
-/*fprintf(stderr, "CHECK: !%s!\n", cmd);*/
 	invoke_cmd(cmd);
 	join_files(file);
 }
 
 
 static void
-pgp_append_public_key (
+pgp_append_public_key(
 	char *file)
 {
 	FILE *fp, *key;
 	char keyfile[PATH_LEN], cmd[LEN], buf[LEN];
 
-	if ((CURR_GROUP.attribute->from) != (char *) 0 && strlen(CURR_GROUP.attribute->from))
-		strip_name (CURR_GROUP.attribute->from, buf);
+	if ((CURR_GROUP.attribute->from) != NULL && strlen(CURR_GROUP.attribute->from))
+		strip_name(CURR_GROUP.attribute->from, buf);
 	else
 		snprintf(buf, sizeof(buf) - 1, "%s@%s", userid, host_name);
 
@@ -293,9 +288,9 @@ pgp_append_public_key (
  * TODO: I'm guessing the pgp append key command creates 'keyfile' and that
  * we should remove it
  */
-	sh_format (cmd, sizeof(cmd), APPEND_KEY, PGPNAME, pgpopts, keyfile, buf);
+	sh_format(cmd, sizeof(cmd), APPEND_KEY, PGPNAME, pgpopts, keyfile, buf);
 
-	if (invoke_cmd (cmd)) {
+	if (invoke_cmd(cmd)) {
 		if ((fp = fopen(file, "a")) != NULL) {
 			if ((key = fopen(keyfile, "r")) != NULL) {
 				fputc('\n', fp);			/* Add a blank line */
@@ -314,14 +309,14 @@ pgp_append_public_key (
  * Simply check for existence of keyring file
  */
 static t_bool
-pgp_available (
+pgp_available(
 	void)
 {
 	FILE *fp;
 	char keyring[PATH_LEN];
 
 	joinpath(keyring, pgp_data, PGP_PUBRING);
-	if ((fp = fopen(keyring, "r")) == (FILE *) 0) {
+	if ((fp = fopen(keyring, "r")) == NULL) {
 		wait_message(2, _(txt_pgp_not_avail), keyring);
 		return FALSE;
 	}
@@ -332,7 +327,7 @@ pgp_available (
 
 
 void
-invoke_pgp_mail (
+invoke_pgp_mail(
 	const char *nam,
 	char *mail_to)
 {
@@ -343,12 +338,12 @@ invoke_pgp_mail (
 	if (!pgp_available())
 		return;
 
-	ch = prompt_slk_response (ch_default, &menukeymap.pgp_mail,
+	ch = prompt_slk_response(ch_default, &menukeymap.pgp_mail,
 			_(txt_pgp_mail),
-			printascii (keyencrypt, map_to_local (iKeyPgpEncrypt, &menukeymap.pgp_mail)),
-			printascii (keysign, map_to_local (iKeyPgpSign, &menukeymap.pgp_mail)),
-			printascii (keyboth, map_to_local (iKeyPgpEncSign, &menukeymap.pgp_mail)),
-			printascii (keyquit, map_to_local (iKeyQuit, &menukeymap.pgp_mail)));
+			printascii(keyencrypt, map_to_local(iKeyPgpEncrypt, &menukeymap.pgp_mail)),
+			printascii(keysign, map_to_local(iKeyPgpSign, &menukeymap.pgp_mail)),
+			printascii(keyboth, map_to_local(iKeyPgpEncSign, &menukeymap.pgp_mail)),
+			printascii(keyquit, map_to_local(iKeyQuit, &menukeymap.pgp_mail)));
 	switch (ch) {
 		case iKeyAbort:
 		case iKeyQuit:
@@ -357,7 +352,7 @@ invoke_pgp_mail (
 		case iKeyPgpSign:
 #ifdef HAVE_PGPK
 			ClearScreen();
-			MoveCursor (cLINES - 7, 0);
+			MoveCursor(cLINES - 7, 0);
 #endif /* HAVE_PGPK */
 			do_pgp(PGP_SIGN, nam, NULL);
 			break;
@@ -365,7 +360,7 @@ invoke_pgp_mail (
 		case iKeyPgpEncSign:
 #ifdef HAVE_PGPK
 			ClearScreen();
-			MoveCursor (cLINES - 7, 0);
+			MoveCursor(cLINES - 7, 0);
 #endif /* HAVE_PGPK */
 			do_pgp(PGP_SIGN | PGP_ENCRYPT, nam, mail_to);
 			break;
@@ -381,7 +376,7 @@ invoke_pgp_mail (
 
 
 void
-invoke_pgp_news (
+invoke_pgp_news(
 	char *artfile)
 {
 	char ch, ch_default = iKeyPgpSign;
@@ -390,11 +385,11 @@ invoke_pgp_news (
 	if (!pgp_available())
 		return;
 
-	ch = prompt_slk_response (ch_default, &menukeymap.pgp_news,
+	ch = prompt_slk_response(ch_default, &menukeymap.pgp_news,
 				_(txt_pgp_news),
-				printascii (keysign, map_to_local (iKeyPgpSign, &menukeymap.pgp_news)),
-				printascii (keyinclude, map_to_local (iKeyPgpIncludekey, &menukeymap.pgp_news)),
-				printascii (keyquit, map_to_local (iKeyQuit, &menukeymap.pgp_news)));
+				printascii(keysign, map_to_local(iKeyPgpSign, &menukeymap.pgp_news)),
+				printascii(keyinclude, map_to_local(iKeyPgpIncludekey, &menukeymap.pgp_news)),
+				printascii(keyquit, map_to_local(iKeyQuit, &menukeymap.pgp_news)));
 	switch (ch) {
 		case iKeyAbort:
 		case iKeyQuit:
@@ -402,8 +397,8 @@ invoke_pgp_news (
 
 		case iKeyPgpSign:
 #ifdef HAVE_PGPK
-			info_message (" ");
-			MoveCursor (cLINES - 7, 0);
+			info_message(" ");
+			MoveCursor(cLINES - 7, 0);
 			my_printf("\n");
 #endif /* HAVE_PGPK */
 			do_pgp(PGP_SIGN, artfile, NULL);
@@ -411,8 +406,8 @@ invoke_pgp_news (
 
 		case iKeyPgpIncludekey:
 #ifdef HAVE_PGPK
-			info_message (" ");
-			MoveCursor (cLINES - 7, 0);
+			info_message(" ");
+			MoveCursor(cLINES - 7, 0);
 			my_printf("\n");
 #endif /* HAVE_PGPK */
 			do_pgp(PGP_SIGN, artfile, NULL);
@@ -426,7 +421,7 @@ invoke_pgp_news (
 
 
 t_bool
-pgp_check_article (
+pgp_check_article(
 	t_openartinfo *artinfo)
 {
 	FILE *art;
@@ -440,10 +435,10 @@ pgp_check_article (
 	joinpath(artfile, homedir, TIN_ARTICLE_NAME);
 
 #	ifdef APPEND_PID
-	snprintf (artfile + strlen(artfile), sizeof(artfile) - 1, ".%d", (int)process_id);
+	snprintf(artfile + strlen(artfile), sizeof(artfile) - 1, ".%d", (int) process_id);
 #	endif /* APPEND_PID */
 
-	if ((art = fopen(artfile, "w")) == (FILE *) 0) {
+	if ((art = fopen(artfile, "w")) == NULL) {
 		info_message(_(txt_cannot_open), artfile);
 		return FALSE;
 	}
@@ -451,9 +446,9 @@ pgp_check_article (
 
 	fgets(buf, LEN, artinfo->raw);		/* Copy the body whilst looking for SIG/KEY tags */
 	while (!feof(artinfo->raw)) {
-		if (!pgp_signed && strcmp(buf, PGP_SIG_TAG) == 0)
+		if (!pgp_signed && !strcmp(buf, PGP_SIG_TAG))
 			pgp_signed = TRUE;
-		if (!pgp_key && strcmp(buf, PGP_KEY_TAG) == 0)
+		if (!pgp_key && !strcmp(buf, PGP_KEY_TAG))
 			pgp_key = TRUE;
 		fputs(buf, art);
 		fgets(buf, LEN, artinfo->raw);
@@ -472,24 +467,24 @@ pgp_check_article (
 		/*
 		 * We don't use sh_format here else the redirection get misquoted
 		 */
-		snprintf (cmd, sizeof(cmd) - 1, CHECK_SIGN, PGPNAME, pgpopts, artfile, REDIRECT_PGP_OUTPUT);
+		snprintf(cmd, sizeof(cmd) - 1, CHECK_SIGN, PGPNAME, pgpopts, artfile, REDIRECT_PGP_OUTPUT);
 		invoke_cmd(cmd);
 		my_printf("\n");
 		Raw(TRUE);
 	}
 
 	if (pgp_key) {
-		if (prompt_yn (cLINES, _(txt_pgp_add), FALSE) == 1) {
-			Raw (FALSE);
+		if (prompt_yn(cLINES, _(txt_pgp_add), FALSE) == 1) {
+			Raw(FALSE);
 
-			sh_format (cmd, sizeof(cmd), ADD_KEY, PGPNAME, pgpopts, artfile);
+			sh_format(cmd, sizeof(cmd), ADD_KEY, PGPNAME, pgpopts, artfile);
 			invoke_cmd(cmd);
-			my_printf ("\n");
-			Raw (TRUE);
+			my_printf("\n");
+			Raw(TRUE);
 		}
 	}
 
-	prompt_continue ();
+	prompt_continue();
 	unlink(artfile);
 	return TRUE;
 }
