@@ -8,7 +8,7 @@
  *  Credits   : Richard Hodson <richard@macgyver.tele2.co.uk>
  *              hash_msgid, free_msgid
  *
- * Copyright (c) 1996-2002 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 1996-2003 Jason Faultless <jason@radar.tele2.co.uk>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -60,15 +60,15 @@ FILE *dbgfd;
 /*
  * local prototypes
  */
-static char * _get_references (struct t_msgid *refptr, int depth);
-static struct t_msgid *add_msgid (int key, const char *msgid, struct t_msgid *newparent);
-static struct t_msgid *find_next (struct t_msgid *ptr);
-static struct t_msgid *parse_references (char *r);
-static unsigned int hash_msgid (const char *key);
-static void add_to_parent (struct t_msgid *ptr);
-static void build_thread (struct t_msgid *ptr);
+static char *_get_references(struct t_msgid *refptr, int depth);
+static struct t_msgid *add_msgid(int key, const char *msgid, struct t_msgid *newparent);
+static struct t_msgid *find_next(struct t_msgid *ptr);
+static struct t_msgid *parse_references(char *r);
+static unsigned int hash_msgid(const char *key);
+static void add_to_parent(struct t_msgid *ptr);
+static void build_thread(struct t_msgid *ptr);
 #ifdef DEBUG_REFS
-	static void dump_thread (FILE *fp, struct t_msgid *msgid, int level);
+	static void dump_thread(FILE *fp, struct t_msgid *msgid, int level);
 	static void dump_msgid_thread(struct t_msgid *ptr, int level);
 	static void dump_msgid_threads(void);
 #endif /* DEBUG_REFS */
@@ -77,7 +77,7 @@ static void build_thread (struct t_msgid *ptr);
 #endif /* 0 */
 
 /*
- * Set if the the sorting algorithm goes 'upwards'
+ * Set if the sorting algorithm goes 'upwards'
  */
 static t_bool sort_ascend;
 
@@ -88,10 +88,10 @@ static struct t_msgid *msgids[MSGID_HASH_SIZE] = {0};
 
 /*
  * This part of the code deals with the cacheing and retrieval
- * of Message-id and References headers
+ * of Message-ID and References headers
  *
  * Rationale:
- *    Even though the Message-id is unique to an article, the References
+ *    Even though the Message-ID is unique to an article, the References
  *    field contains msgids from elsewhere in the group. As the expiry
  *    period increases, so does the redundancy of data.
  *    At the time of writing, comp.os.ms-windows.advocacy held ~850
@@ -117,7 +117,7 @@ static struct t_msgid *msgids[MSGID_HASH_SIZE] = {0};
  * MSGID_HASH_SIZE is a prime of order 2^11
  */
 static unsigned int
-hash_msgid (
+hash_msgid(
 	const char *key)
 {
 	unsigned int hash = 0;
@@ -132,11 +132,12 @@ hash_msgid (
 	return hash;
 }
 
+
 /*
  * Thread us into our parents' list of children.
  */
 static void
-add_to_parent (
+add_to_parent(
 	struct t_msgid *ptr)
 {
 	struct t_msgid *p;
@@ -147,7 +148,7 @@ add_to_parent (
 	/*
 	 * Trivial case - if we are the first child (followup)
 	 */
-	if (ptr->parent->child == NULL)	{
+	if (ptr->parent->child == NULL) {
 		ptr->parent->child = ptr;
 		return;
 	}
@@ -177,6 +178,7 @@ add_to_parent (
 	}
 }
 
+
 /*
  * Adds or updates a message id in the cache.
  * We return a ptr to the msgid, whether located or newly created.
@@ -202,7 +204,7 @@ add_to_parent (
  *        overview database.
  */
 static struct t_msgid *
-add_msgid (
+add_msgid(
 	int key,
 	const char *msgid,
 	struct t_msgid *newparent)
@@ -225,7 +227,6 @@ add_msgid (
 	 * Broken software will sometimes damage the case of a message-id.
 	 */
 	for (i = msgids[h]; i != NULL; i = i->next) {
-
 		if (strcasecmp(i->txt, msgid) != 0)				/* No match yet */
 			continue;
 
@@ -293,7 +294,7 @@ add_msgid (
 	/*
 	 * This is a new node, so build a structure for it
 	 */
-	ptr = (struct t_msgid *)my_malloc(sizeof(struct t_msgid) + strlen(msgid));
+	ptr = my_malloc(sizeof(struct t_msgid) + strlen(msgid));
 
 	strcpy(ptr->txt, msgid);
 	ptr->parent = newparent;
@@ -311,12 +312,13 @@ add_msgid (
 	return ptr;
 }
 
+
 /*
  * Find a Message-ID in the cache. Return ptr to this node, or NULL if
  * not found.
  */
 struct t_msgid *
-find_msgid (
+find_msgid(
 	const char *msgid)
 {
 	unsigned int h;
@@ -336,6 +338,7 @@ find_msgid (
 	return NULL;
 }
 
+
 /*
  * Take a raw line of references data and return a ptr to a linked list of
  * msgids, starting with the most recent entry. (Thus the list is reversed)
@@ -346,7 +349,7 @@ find_msgid (
  * The space saving vs. storing the refs as a single string is significant.
  */
 static struct t_msgid *
-parse_references (
+parse_references(
 	char *r)
 {
 	char *ptr;
@@ -378,6 +381,7 @@ parse_references (
 	return current;
 }
 
+
 /*
  * Reconstruct the References: field from the parent pointers
  * NB: The original Refs: can be no longer than HEADER_LEN (see open.c)
@@ -385,7 +389,7 @@ parse_references (
  *	    lists, which we strive to work around.
  */
 static char *
-_get_references (
+_get_references(
 	struct t_msgid *refptr,
 	int depth)
 {
@@ -398,7 +402,7 @@ _get_references (
 		if (depth > MAX_REFS)
 			error_message("Warning: Too many refs near to %s. Truncated\n", refptr->txt);
 #endif /* DEBUG_REFS */
-		refs = (char *) my_malloc(HEADER_LEN);
+		refs = my_malloc(HEADER_LEN);
 		len = 0;
 	} else
 		refs = _get_references(refptr->parent, depth + 1);
@@ -414,12 +418,13 @@ _get_references (
 	return refs;
 }
 
+
 /*
  * A wrapper to the above, null terminate the string and shrink it
  * to correct size
  */
 char *
-get_references (
+get_references(
 	struct t_msgid *refptr)
 {
 	char *refs;
@@ -429,22 +434,20 @@ get_references (
 		return NULL;
 
 	refs = _get_references(refptr, 1);
-
 	len = strlen(refs);
-
 	refs[len - 1] = '\0';
-
-	refs = (char *) my_realloc(refs, len);
+	refs = my_realloc(refs, len);
 
 	return refs;
 }
 
+
 /*
  * Clear the entire msgid cache, freeing up all chains. This is
  * normally only needed when entering a new group
  */
 void
-free_msgids (
+free_msgids(
 	void)
 {
 	int i;
@@ -466,9 +469,10 @@ free_msgids (
 	}
 }
 
+
 #if 0	/* But please don't remove it */
 static void
-dump_msgids (
+dump_msgids(
 	void)
 {
 	int i;
@@ -476,12 +480,9 @@ dump_msgids (
 
 	my_fprintf(stderr, "Dumping...\n");
 
-	for (i = 0; i<MSGID_HASH_SIZE; i++) {
-
+	for (i = 0; i < MSGID_HASH_SIZE; i++) {
 		if (msgids[i] != NULL) {
-
 			my_fprintf(stderr, "node %d", i);
-
 			for (ptr = msgids[i]; ptr != NULL; ptr = ptr->next)
 				my_fprintf(stderr, " -> %s", ptr->txt);
 
@@ -492,6 +493,7 @@ dump_msgids (
 }
 #endif /* 0 */
 
+
 /*
  * The rest of this code deals with reference threading
  *
@@ -530,13 +532,12 @@ dump_msgids (
  * the reference tree to locate the 'next' article in the thread.
  *
  */
-
 /*
  * Clear out all the article fields from the msgid hash prior to a
  * rethread.
  */
 void
-clear_art_ptrs (
+clear_art_ptrs(
 	void)
 {
 	int i;
@@ -548,13 +549,14 @@ clear_art_ptrs (
 	}
 }
 
+
 /*
  * Function to dump an ASCII tree map of a thread rooted at msgid.
  * Output goes to fp, level is the current depth of the tree.
  */
 #ifdef DEBUG_REFS
 static void
-dump_thread (
+dump_thread(
 	FILE *fp,
 	struct t_msgid *msgid,
 	int level)
@@ -587,13 +589,14 @@ dump_thread (
 	return;
 }
 
+
 /*
  * Dump out all the threads from the msgid point of view, show the
  * related article index in arts[] where possible
  * A thread is defined as a starting article with no parent
  */
 static void
-dump_msgid_thread (
+dump_msgid_thread(
 	struct t_msgid *ptr,
 	int level)
 {
@@ -608,8 +611,9 @@ dump_msgid_thread (
 	return;
 }
 
+
 static void
-dump_msgid_threads (
+dump_msgid_threads(
 	void)
 {
 	int i;
@@ -617,7 +621,7 @@ dump_msgid_threads (
 
 	fprintf(dbgfd, "Dump started.\n\n");
 
-	for (i = 0; i<MSGID_HASH_SIZE; i++) {
+	for (i = 0; i < MSGID_HASH_SIZE; i++) {
 		if (msgids[i] != NULL) {
 
 			for (ptr = msgids[i]; ptr != NULL; ptr = ptr->next) {
@@ -634,6 +638,7 @@ dump_msgid_threads (
 }
 #endif /* DEBUG_REFS */
 
+
 /*
  * Find the next message in the thread.
  * We descend children before siblings, and only return articles that
@@ -645,13 +650,13 @@ dump_msgid_threads (
  *  It doesn't point to an article OR
  *     (it's already threaded/expired OR it has been autokilled)
  */
-/* We SHOULD skip killed in the normal case */
-#	define SKIP_ART(ptr)	\
+#define SKIP_ART(ptr)	\
 	(ptr && (ptr->article == ART_UNAVAILABLE || \
-		(arts[ptr->article].thread != ART_NORMAL || (tinrc.kill_level != KILL_THREAD && arts[ptr->article].killed))))
+		(arts[ptr->article].thread != ART_NORMAL || \
+			(tinrc.kill_level == KILL_NOTHREAD && arts[ptr->article].killed))))
 
 static struct t_msgid *
-find_next (
+find_next(
 	struct t_msgid *ptr)
 {
 	static t_bool bottom = FALSE;
@@ -697,7 +702,7 @@ find_next (
 		 */
 		if (ptr->child == NULL && ptr->sibling == NULL) {
 
-			while(ptr != NULL && ptr->sibling == NULL)
+			while (ptr != NULL && ptr->sibling == NULL)
 				ptr = ptr->parent;
 
 			/*
@@ -714,12 +719,13 @@ find_next (
 	return ptr;
 }
 
+
 /*
  * Run the .thread and .inthread pointers through the members of this
  * thread.
  */
 static void
-build_thread (
+build_thread(
 	struct t_msgid *ptr)
 {
 	struct t_msgid *newptr;
@@ -727,7 +733,7 @@ build_thread (
 	/*
 	 * If the root article is gone/expired/killed, find the first valid one
 	 */
-	if (SKIP_ART (ptr))
+	if (SKIP_ART(ptr))
 		ptr = find_next(ptr);
 
 	/*
@@ -743,12 +749,13 @@ build_thread (
 
 }
 
+
 /*
  * Run a new set of threads through the base articles, using the
  * parent / child / sibling  / article pointers in the msgid hash.
  */
 void
-thread_by_reference (
+thread_by_reference(
 	void)
 {
 	int i;
@@ -762,7 +769,7 @@ thread_by_reference (
 	/*
 	 * Build threads starting from root msgids (ie without parent)
 	 */
-	for (i = 0; i<MSGID_HASH_SIZE; i++) {
+	for (i = 0; i < MSGID_HASH_SIZE; i++) {
 		if (msgids[i] != NULL) {
 			for (ptr = msgids[i]; ptr != NULL; ptr = ptr->next) {
 				if (ptr->parent == NULL)
@@ -775,7 +782,7 @@ thread_by_reference (
 	fprintf(dbgfd, "Full dump of threading info...\n");
 	fprintf(dbgfd, "%3s %3s %3s %3s : %3s %3s\n", "#", "Par", "Sib", "Chd", "In", "Thd");
 
-	for (i = 0; i < top_art; i++) {
+	for_each_art(i) {
 		fprintf(dbgfd, "%3d %3d %3d %3d : %3d %3d : %.50s %s\n", i,
 			(arts[i].refptr->parent) ? arts[i].refptr->parent->article : -2,
 			(arts[i].refptr->sibling) ? arts[i].refptr->sibling->article : -2,
@@ -787,9 +794,9 @@ thread_by_reference (
 #endif /* DEBUG_REFS */
 
 	return;
-
 }
 
+
 /*
  * Do the equivalent of subject threading, but only on the thread base
  * messages.
@@ -801,7 +808,7 @@ thread_by_reference (
  * correct.
  */
 void
-collate_subjects (
+collate_subjects(
 	void)
 {
 	int i, j, art;
@@ -811,8 +818,7 @@ collate_subjects (
 	 * Run through the root messages of each thread. We have to traverse
 	 * using arts[] and not msgids[] to preserve the sorting.
 	 */
-	for (i = 0; i < top_art; i++) {
-
+	for_each_art(i) {
 		/*
 		 * Ignore already threaded and expired arts
 		 */
@@ -822,12 +828,10 @@ collate_subjects (
 		/*
 		 * Get the contents of the magic marker in the hashnode
 		 */
-		h = (struct t_hashnode *)(arts[i].subject - sizeof(int) - sizeof(void *)); /* FIXME: cast increases required alignment of target type */
-
+		h = (struct t_hashnode *) (arts[i].subject - sizeof(int) - sizeof(void *)); /* FIXME: cast increases required alignment of target type */
 		j = h->aptr;
 
 		if (j != -1 && j < i) {
-
 			/*
 			 * Modified form of the subject threading - the only difference
 			 * is that we have to add later threads onto the end of the
@@ -856,6 +860,7 @@ collate_subjects (
 	return;
 }
 
+
 /*
  * Builds the reference tree:
  *
@@ -871,36 +876,38 @@ collate_subjects (
  * 4) free() up the msgid and refs headers once cached
  */
 void
-build_references (
+build_references(
 	struct t_group *group)
 {
+	char *s;
 	int i;
 	struct t_article *art;
-	char *s;
+	struct t_msgid *refs;
 
 	/*
 	 * The articles are currently unsorted, and are as they were put by setup_hard_base()
 	 */
 	if (group->attribute && group->attribute->sort_art_type != SORT_ARTICLES_BY_NOTHING)
-		sort_arts (group->attribute->sort_art_type);
+		sort_arts(group->attribute->sort_art_type);
 
 	if (group->attribute)
 		sort_ascend = (group->attribute->sort_art_type == SORT_ARTICLES_BY_SUBJ_ASCEND ||
 		               group->attribute->sort_art_type == SORT_ARTICLES_BY_FROM_ASCEND ||
 		               group->attribute->sort_art_type == SORT_ARTICLES_BY_DATE_ASCEND ||
-		               group->attribute->sort_art_type == SORT_ARTICLES_BY_SCORE_ASCEND);
+		               group->attribute->sort_art_type == SORT_ARTICLES_BY_SCORE_ASCEND ||
+			       group->attribute->sort_art_type == SORT_ARTICLES_BY_LINES_ASCEND);
 
 #ifdef DEBUG_REFS
 	dbgfd = fopen("Refs.dump", "w");
 	SETVBUF(dbgfd, NULL, _IONBF, 0);
-	fprintf (dbgfd, "MSGID phase\n");
+	fprintf(dbgfd, "MSGID phase\n");
 #endif /* DEBUG_REFS */
 
 	/*
 	 * Add the Message-ID headers to the cache, using the last Reference
 	 * as the parent
 	 */
-	for (i = 0; i < top_art; i++) {
+	for_each_art(i) {
 		art = &arts[i];
 
 		if (art->refs) {
@@ -938,9 +945,7 @@ build_references (
 	/*
 	 * Add the References data to the cache
 	 */
-	for (i = 0; i < top_art; i++) {
-		struct t_msgid *refs;
-
+	for_each_art(i) {
 		if (!arts[i].refs)						/* No refs - skip */
 			continue;
 
@@ -962,7 +967,4 @@ build_references (
 #ifdef DEBUG_REFS
 	fclose(dbgfd);
 #endif /* DEBUG_REFS */
-
 }
-
-/* end of refs.c */
