@@ -3,10 +3,10 @@
  *  Module    : rfc1524.c
  *  Author    : Urs Janssen <urs@tin.org>, Jason Faultless <jason@radar.tele2.co.uk>
  *  Created   : 2000-05-15
- *  Updated   : 2000-06-25
+ *  Updated   : 2002-06-18
  *  Notes     : mailcap parsing as defined in RFC 1524
  *
- * Copyright (c) 2000-2002 Urs Janssen <urs@tin.org>, Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 2000-2003 Urs Janssen <urs@tin.org>, Jason Faultless <jason@radar.tele2.co.uk>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,15 +41,16 @@
 #	include "rfc2046.h"
 #endif /* RFC2046_H */
 
+/* TODO: what about !unix systems? */
 #define DEFAULT_MAILCAPS "~/.mailcap:/etc/mailcap:/usr/etc/mailcap:/usr/local/etc/mailcap:/etc/mail/mailcap"
 
 /* maximum number of mailcap fields */
 #define MAILCAPFIELDS 13
 
 /* local prototypes */
-static char *expand_mailcap_meta (const char *mailcap, t_part *part, char *nametemplate, const char *path);
-static char *get_mailcap_field (char *mailcap);
-static t_mailcap *parse_mailcap_line (const char *mailcap, t_part *part, const char *path);
+static char *expand_mailcap_meta(const char *mailcap, t_part *part, char *nametemplate, const char *path);
+static char *get_mailcap_field(char *mailcap);
+static t_mailcap *parse_mailcap_line(const char *mailcap, t_part *part, const char *path);
 
 
 /*
@@ -60,7 +61,7 @@ static t_mailcap *parse_mailcap_line (const char *mailcap, t_part *part, const c
  * TODO: don't used fixed length buffers
  */
 t_mailcap *
-get_mailcap_entry (
+get_mailcap_entry(
 	t_part *part,
 	const char *path)
 {
@@ -75,24 +76,26 @@ get_mailcap_entry (
 
 	mailcaps[0] = '\0';
 	/* build list of mailcap files */
-	if ((ptr = getenv ("MAILCAPS")) != (char *) 0) {
+	if ((ptr = getenv("MAILCAPS")) != NULL) {
 		if (strlen(ptr)) {
 			STRCPY(mailcaps, ptr);
-			strncat(mailcaps, ":", sizeof(mailcaps) -1);
+			strncat(mailcaps, ":", sizeof(mailcaps) - 1);
 		}
 	}
-	strncat(mailcaps, DEFAULT_MAILCAPS, sizeof(mailcaps) -1);
+	strncat(mailcaps, DEFAULT_MAILCAPS, sizeof(mailcaps) - 1);
 
 	mailcap[0] = '\0';
 	wildcap[0] = '\0';
+	buf[0] = '\0';
+
 	ptr = buf;
 
 	nptr = strtok(mailcaps, ":");
-	while (nptr != (char *) 0) {
+	while (nptr != NULL) {
 		/* expand ~ and/or $HOME etc. */
-		if (strfpath (nptr, filename, sizeof (filename) - 1, &CURR_GROUP)) {
-			if ((fp = fopen(filename, "r")) != (FILE *) 0) {
-				while ((fgets (ptr, sizeof(buf) - strlen(buf), fp)) != NULL) {
+		if (strfpath(nptr, filename, sizeof(filename) - 1, &CURR_GROUP)) {
+			if ((fp = fopen(filename, "r")) != NULL) {
+				while ((fgets(ptr, sizeof(buf) - strlen(buf), fp)) != NULL) {
 					if (*ptr == '#' || *ptr == '\n')		/* skip comments & blank lines */
 						continue;
 
@@ -106,22 +109,22 @@ get_mailcap_entry (
 					else
 						ptr = buf;
 
-					if ((ptr2 = strchr(buf, '/')) != (char *) 0) {
-						if (!strncmp(ptr, content_types[part->type], strlen(ptr) - strlen(ptr2))) {
-							if (!strncmp(ptr + strlen(content_types[part->type]) + 1, part->subtype, strlen(part->subtype))) {
+					if ((ptr2 = strchr(buf, '/')) != NULL) {
+						if (!strncasecmp(ptr, content_types[part->type], strlen(ptr) - strlen(ptr2))) {
+							if (!strncasecmp(ptr + strlen(content_types[part->type]) + 1, part->subtype, strlen(part->subtype))) {
 								/* full match, so parse line and evaluate test if given. */
 								STRCPY(mailcap, ptr);
-								foo = parse_mailcap_line (mailcap, part, path);
-								if (foo != (t_mailcap *) 0) {
-									fclose (fp); /* perfect match with test succeded (if given) */
+								foo = parse_mailcap_line(mailcap, part, path);
+								if (foo != NULL) {
+									fclose(fp); /* perfect match with test succeded (if given) */
 									return foo;
 								}
 							} else {
 								if ((*(ptr2 + 1) == '*') || (*(ptr2 + 1) == ';')) { /* wildmat match */
 									if (!strlen(wildcap)) { /* we don't already have a wildmat match */
 										STRCPY(wildcap, buf);
-										foo = parse_mailcap_line (wildcap, part, path);
-										if (foo == (t_mailcap *) 0) /* test failed */
+										foo = parse_mailcap_line(wildcap, part, path);
+										if (foo == NULL) /* test failed */
 											wildcap[0] = '\0'; /* ignore match */
 									}
 								} /* else subtype missmatch, no action required */
@@ -129,11 +132,11 @@ get_mailcap_entry (
 						} /* else no match, no action required */
 					} /* else invalid mailcap line (no /), no action required */
 					if (strlen(wildcap)) {	/* we just had a wildmat match */
-						fclose (fp);
+						fclose(fp);
 						return foo;
 					}
-				} /* while ((fgets (ptr, ... */
-				fclose (fp);
+				} /* while ((fgets(ptr, ... */
+				fclose(fp);
 			}
 		} /* else strfpath() failed, no action required */
 		nptr = strtok(NULL, ":"); /* get next filename */
@@ -158,7 +161,7 @@ parse_mailcap_line(
 	t_mailcap *tmailcap;
 
 	/* malloc and init */
-	tmailcap = (t_mailcap *) my_malloc (sizeof(t_mailcap));
+	tmailcap = my_malloc(sizeof(t_mailcap));
 	tmailcap->type = (char *) 0;
 	tmailcap->command = (char *) 0;
 	tmailcap->needsterminal = FALSE;
@@ -178,21 +181,20 @@ parse_mailcap_line(
 	/* get required entrys */
 	ptr = get_mailcap_field(ptr);
 	blen = strlen(content_types[part->type]) + strlen(part->subtype) + 2;
-	buf = (char *) my_malloc(sizeof(char) * blen);
-	memset(buf, 0, blen);
-	sprintf (buf, "%s/%s", content_types[part->type], part->subtype);
+	buf = my_calloc(1, blen);
+	sprintf(buf, "%s/%s", content_types[part->type], part->subtype);
 	tmailcap->type = buf;
 	ptr += strlen(ptr) + 1;
-	if ((ptr = get_mailcap_field(ptr)) != (char *) 0) {
+	if ((ptr = get_mailcap_field(ptr)) != NULL) {
 		tmailcap->command = ptr;
 		ptr += strlen(ptr) + 1;
 	} else { /* required filed missing */
-		free (optr);
-		free_mailcap (tmailcap);
+		free(optr);
+		free_mailcap(tmailcap);
 		return ((t_mailcap *) 0);
 	}
 
-	while ((ptr = get_mailcap_field(ptr)) != (char *) 0) {
+	while ((ptr = get_mailcap_field(ptr)) != NULL) {
 		if (i-- <= 0) /* number of possible fields exhausted */
 			break;
 		if (!strncasecmp(ptr, "needsterminal", 13)) {
@@ -245,30 +247,30 @@ parse_mailcap_line(
 	 * expand metas - we do it in a 2nd pass to be able to honor
 	 * nametemplate
 	 */
-	if (tmailcap->command != (char *) 0)
+	if (tmailcap->command != NULL)
 		tmailcap->command = expand_mailcap_meta(tmailcap->command, part, tmailcap->nametemplate, path);
-	if (tmailcap->description != (char *) 0)
+	if (tmailcap->description != NULL)
 		tmailcap->description = expand_mailcap_meta(tmailcap->description, part, tmailcap->nametemplate, path);
-	if (tmailcap->test != (char *) 0)
+	if (tmailcap->test != NULL)
 		tmailcap->test = expand_mailcap_meta(tmailcap->test, part, tmailcap->nametemplate, path);
-	if (tmailcap->compose != (char *) 0)
+	if (tmailcap->compose != NULL)
 		tmailcap->compose = expand_mailcap_meta(tmailcap->compose, part, tmailcap->nametemplate, path);
-	if (tmailcap->composetyped != (char *) 0)
+	if (tmailcap->composetyped != NULL)
 		tmailcap->composetyped = expand_mailcap_meta(tmailcap->composetyped, part, tmailcap->nametemplate, path);
-	if (tmailcap->edit != (char *) 0)
+	if (tmailcap->edit != NULL)
 		tmailcap->edit = expand_mailcap_meta(tmailcap->edit, part, tmailcap->nametemplate, path);
-	if (tmailcap->print != (char *) 0)
+	if (tmailcap->print != NULL)
 		tmailcap->print = expand_mailcap_meta(tmailcap->print, part, tmailcap->nametemplate, path);
-	if (tmailcap->x11bitmap != (char *) 0)
+	if (tmailcap->x11bitmap != NULL)
 		tmailcap->x11bitmap = expand_mailcap_meta(tmailcap->x11bitmap, part, tmailcap->nametemplate, path);
 
-	free (optr);
+	free(optr);
 
-	if (tmailcap->test != (char *) 0) { /* test field given */
+	if (tmailcap->test != NULL) { /* test field given */
 		/* TODO: EndWin()/InitWin() around system needed? */
-		/* TODO: use invoke_cmd() ? */
-		if (system(tmailcap->test) != 0) { /* test failed? */
-			free_mailcap (tmailcap);
+		/* TODO: use invoke_cmd()? */
+		if (system(tmailcap->test)) { /* test failed? */
+			free_mailcap(tmailcap);
 			return ((t_mailcap *) 0);
 		}
 	}
@@ -279,7 +281,7 @@ parse_mailcap_line(
 /*
  * extract fields - called from parse_mailcap_line()
  *
- * TODO: add handling for signlequotes
+ * TODO: add handling for singlequotes
  */
 static char *
 get_mailcap_field(
@@ -291,15 +293,15 @@ get_mailcap_field(
 
 	ptr = str_trim(mailcap);
 
-	while(*ptr != '\0') {
+	while (*ptr != '\0') {
 		switch (*ptr) {
 			case '\\':
-				backquote = !backquote;
+				backquote = bool_not(backquote);
 				break;
 
 			case '"':
 				if (!backquote)
-					doublequote = !doublequote;
+					doublequote = bool_not(doublequote);
 				backquote = FALSE;
 				break;
 
@@ -323,21 +325,22 @@ get_mailcap_field(
 }
 
 
-/*
- * expand metas - called from parse_mailcap_line()
- *
- * TODO: expand %F, %n
- */
 #define CHECK_SPACE(minlen) { \
 	while (space <= (minlen)) { /* need more space ? */ \
 		olen = strlen(line); \
 		space += linelen; \
 		linelen <<= 1; \
-		line = (char *) my_realloc((void *) line, linelen); \
+		line = my_realloc(line, linelen); \
 		memset(line + olen, 0, linelen - olen); \
 	} \
 }
 
+
+/*
+ * expand metas - called from parse_mailcap_line()
+ *
+ * TODO: expand %F, %n
+ */
 static char *
 expand_mailcap_meta(
 	const char *mailcap,
@@ -351,15 +354,13 @@ expand_mailcap_meta(
 	t_bool percent = FALSE;
 	size_t linelen, space, olen;
 
-	if ((ptr = strchr(mailcap, '%')) == (char *) 0) /* nothing to expand */
+	if ((ptr = strchr(mailcap, '%')) == NULL) /* nothing to expand */
 		return my_strdup(mailcap); /* waste of mem, but simplyfies the frees */
 
-	linelen = sizeof(char) * LEN * 2;		/* initial maxlen */
-	space = linelen - 1;							/* available space in string */
-	line = (char *) my_malloc (linelen);	/* initial malloc */
-	memset (line, 0, linelen);
+	linelen = LEN * 2;					/* initial maxlen */
+	space = linelen - 1;					/* available space in string */
+	line = my_calloc(1, linelen);
 	lptr = line;
-
 	ptr = mailcap;
 
 	while (*ptr != '\0') {
@@ -367,18 +368,18 @@ expand_mailcap_meta(
 		 * to avoid reallocs() for the all the single char cases
 		 * we do a check here
 		 */
-		if (space < (sizeof(char) * 10)) { /* 'worst'case are two chars ... */
+		if (space < 10) {				/* 'worst'case are two chars ... */
 			olen = strlen(line);		/* get current legth of string */
 			space += linelen;			/* recalc available space */
 			linelen <<= 1;				/* double maxlen */
-			line = (char *) my_realloc((void *) line, linelen);
-			memset (line + olen, 0, linelen - olen); /* weed out junk */
+			line = my_realloc(line, linelen);
+			memset(line + olen, 0, linelen - olen); /* weed out junk */
 			lptr = line + olen;		/* adjust pointer to current position */
 		}
 
 		switch (*ptr) {
 			case '\\':
-				quote = !quote;
+				quote = bool_not(quote);
 				break;
 
 			case '%':
@@ -386,7 +387,7 @@ expand_mailcap_meta(
 					percent = TRUE;
 				else {
 					*lptr++ = '%';
-					space -= sizeof(char);
+					space--;
 					quote = FALSE;
 				}
 				break;
@@ -396,22 +397,21 @@ expand_mailcap_meta(
 					char *end;
 
 					percent = FALSE;
-					if ((end = strchr (ptr, '}')) != (char *) 0) {
-						if (part->params != (t_param *) 0) {
+					if ((end = strchr(ptr, '}')) != NULL) {
+						if (part->params != NULL) {
 							char *parameter;
 							const char *value;
 
-							parameter = (char *) my_malloc (end - ptr + 1);
-							memset (parameter, 0, end - ptr + 1);
-							strncpy (parameter, ptr + 1, end - ptr - 1); /* extract paramter name */
+							parameter = my_calloc(1, end - ptr + 1);
+							strncpy(parameter, ptr + 1, end - ptr - 1); /* extract paramter name */
 
-							if ((value = get_param (part->params, parameter)) != (char *) 0) { /* match ? */
+							if ((value = get_param(part->params, parameter)) != NULL) { /* match? */
 								CHECK_SPACE(strlen(value));
-								strcat (line, value);
+								strcat(line, value);
 								lptr = line + strlen(line);
 								space -= strlen(line);
 							}
-						free (parameter);
+						free(parameter);
 						}
 						ptr = end;	/* skip past closing } */
 					}
@@ -419,7 +419,7 @@ expand_mailcap_meta(
 				}
 				/* FALLTHROUGH */
 
-#if 0
+#if 0 /* TODO */
 			case 'F':
 				if (percent) {
 					percent = FALSE;
@@ -439,17 +439,14 @@ expand_mailcap_meta(
 				if (percent) {
 					char *nptr = (char *) 0;
 
-					if (nametemplate)
-						nptr = expand_mailcap_meta(nametemplate, part, (char *) 0, path);
-
-					if (nptr == (char *) 0)
-						nptr = (char *) path;
-
-					CHECK_SPACE(strlen(nptr) + 2);
-					strcat(line, nptr);
-
-					if (nametemplate) /* recursiv expand_mailcap_meta() calls need extra free */
+					if (nametemplate && (nptr = expand_mailcap_meta(nametemplate, part, (char *) 0, path)) != 0) {
+						CHECK_SPACE(strlen(nptr) + 2);
+						strcat(line, nptr);
 						free(nptr);
+					} else {
+						CHECK_SPACE(strlen(path) + 2);
+						strcat(line, path);
+					}
 
 					lptr = line + strlen(line);
 					space -= strlen(line);
@@ -475,19 +472,19 @@ expand_mailcap_meta(
 				if (quote) { /* last char was \ */
 					*lptr = '\\';
 					lptr++;
-					space -= sizeof(char);
+					space--;
 					quote = FALSE;
 				}
 				if (percent) { /* unknow %x sequence */
 					*lptr = '%';
 					lptr++;
-					space -= sizeof(char);
+					space--;
 					percent = FALSE;
 					quote = FALSE;
 				}
 				*lptr = *ptr;
 				lptr++;
-				space -= sizeof(char);
+				space--;
 		}
 		ptr++;
 	}
@@ -499,7 +496,7 @@ expand_mailcap_meta(
  * frees the malloced space
  */
 void
-free_mailcap (
+free_mailcap(
 	t_mailcap *tmailcap)
 {
 	FreeIfNeeded(tmailcap->type);
@@ -512,5 +509,5 @@ free_mailcap (
 	FreeIfNeeded(tmailcap->print);
 	FreeIfNeeded(tmailcap->test);
 	FreeIfNeeded(tmailcap->x11bitmap);
-	FreeIfNeeded((char *)tmailcap);
+	FreeIfNeeded(tmailcap);
 }
