@@ -2,11 +2,11 @@
  *  Project   : tin - a Usenet reader
  *  Module    : rfc2045.c
  *  Author    : Chris Blum <chris@resolution.de>
- *  Created   : September '95
- *  Updated   : 1998-01-03
+ *  Created   : 1995-09-01
+ *  Updated   : 2002-03-19
  *  Notes     : RFC 2045/2047 encoding
  *
- * Copyright (c) 1995-2002 Chris Blum <chris@resolution.de>
+ * Copyright (c) 1995-2003 Chris Blum <chris@resolution.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Chris Blum.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -45,11 +42,11 @@
 /*
  * local prototypes
  */
-static unsigned char bin2hex (unsigned int x);
+static unsigned char bin2hex(unsigned int x);
 
 
 static unsigned char
-bin2hex (
+bin2hex(
 	unsigned int x)
 {
 	if (x < 10)
@@ -58,8 +55,8 @@ bin2hex (
 }
 
 
-#define HI4BITS(c) (unsigned char)(*EIGHT_BIT(c) >> 4)
-#define LO4BITS(c) (unsigned char)(*c & 0xf)
+#define HI4BITS(c) ((unsigned char) (*EIGHT_BIT(c) >> 4))
+#define LO4BITS(c) ((unsigned char) (*c & 0xf))
 
 /*
  * A MIME replacement for fputs. e can be 'b' for base64, 'q' for
@@ -70,7 +67,7 @@ bin2hex (
  *       certainly break any binary format ...
  */
 void
-rfc1521_encode (
+rfc1521_encode(
 	char *line,
 	FILE *f,
 	int e)
@@ -87,7 +84,7 @@ rfc1521_encode (
 			b = buffer;
 			*buffer = '\0';
 		}
-		if (!line) {				  /* flush */
+		if (!line) {		/* flush */
 			if (bits) {
 				if (xpos >= 73) {
 					*b++ = '\n';
@@ -178,7 +175,7 @@ rfc1521_encode (
 					}
 					l++;
 				}
-				if (!*l) {			  /* trailing whitespace must be encoded */
+				if (!*l) {		/* trailing whitespace must be encoded */
 					*b++ = '=';
 					*b++ = bin2hex(HI4BITS(line));
 					*b++ = bin2hex(LO4BITS(line));
@@ -199,7 +196,7 @@ rfc1521_encode (
 				line++;
 			}
 			if (xpos > 72 && *line != '\n') {	/* 72 +3 [worst case] + equal sign = 76 :-) */
-				*b++ = '=';			  /* break long lines with a 'soft line break' */
+				*b++ = '=';		/* break long lines with a 'soft line break' */
 				*b++ = '\n';
 				*b++ = '\0';
 				fputs(buffer, f);
@@ -222,17 +219,14 @@ rfc1521_encode (
  * NOT to be used for News posting, which is made certain
  * by setting tinrc.post_mime_encoding to 8bit
  */
-
 #define KSC 1
 #define ASCII 0
 #define isksc(c)	((unsigned char) (c) > (unsigned char) '\240' && \
 						(unsigned char) (c) < (unsigned char) '\377')
 #define SI '\017'
 #define SO '\016'
-
-
 void
-rfc1557_encode (
+rfc1557_encode(
 	char *line,
 	FILE *f,
 	int UNUSED(e))		/* dummy argument : not used */
@@ -245,21 +239,21 @@ rfc1557_encode (
 		iskorean = FALSE;
 		return;
 	}
-	if (!iskorean) {				  /* search for KS C 5601 character(s) in line */
+	if (!iskorean) {		/* search for KS C 5601 character(s) in line */
 		while (line[i]) {
 			if (isksc(line[i])) {
-				iskorean = TRUE;		  /* found KS C 5601 */
-				fprintf(f, "\033$)C\n");	/* put out the designator */
+				iskorean = TRUE;		/* found KS C 5601 */
+				fprintf(f, "\033$)C\n");		/* put out the designator */
 				break;
 			}
 			i++;
 		}
 	}
-	if (!iskorean) {				  /* KS C 5601 doesn't appear, yet -  no conversion */
+	if (!iskorean) {		/* KS C 5601 doesn't appear, yet -  no conversion */
 		fputs(line, f);
 		return;
 	}
-	i = 0;							  /* back to the beginning of the line */
+	i = 0;		/* back to the beginning of the line */
 	while (line[i] && line[i] != '\n') {
 		if (mode == ASCII && isksc(line[i])) {
 			fputc(SO, f);
@@ -281,13 +275,12 @@ rfc1557_encode (
 		fputc(SI, f);
 	if (line[i] == '\n')
 		fputc('\n', f);
-	return;
 }
 
 
 #if 0 /* Not yet implemented */
 void
-rfc1468_encode (
+rfc1468_encode(
 	char *line,
 	FILE *f,
 	int UNUSED(e))		/* dummy argument: not used */
@@ -299,7 +292,7 @@ rfc1468_encode (
 
 /* Not yet implemented */
 void
-rfc1922_encode (
+rfc1922_encode(
 	char *line,
 	FILE *f,
 	int UNUSED(e))		/* dummy argument: not used */
