@@ -2,11 +2,11 @@
  *  Project   : tin - a Usenet reader
  *  Module    : rfc2047.c
  *  Author    : Chris Blum <chris@resolution.de>
- *  Created   : September '95
- *  Updated   : 1998-04-05
+ *  Created   : 1995-09-01
+ *  Updated   : 2003-01-18
  *  Notes     : MIME header encoding/decoding stuff
  *
- * Copyright (c) 1995-2002 Chris Blum <chris@resolution.de>
+ * Copyright (c) 1995-2003 Chris Blum <chris@resolution.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Chris Blum.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -41,6 +38,9 @@
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
+#ifndef RFC2046_H
+#	include "rfc2046.h"
+#endif /* !RFC2046_H */
 
 #define isreturn(c) ((c) == '\r' || ((c) == '\n'))
 
@@ -51,7 +51,7 @@
  * in unstructured headers like Subject, Keyword and Summary
  * c.f. RFC 2047
  */
-#define isbetween(c, s) (isspace((unsigned char)c) || ((s) && ((c) == '(' || (c) == ')' || (c) == '"')))
+#define isbetween(c, s) (isspace((unsigned char) c) || ((s) && ((c) == '(' || (c) == ')' || (c) == '"')))
 
 #define NOT_RANKED 255
 
@@ -74,18 +74,18 @@ static int quoteflag;
 /*
  * local prototypes
  */
-static int do_b_encode (char *w, char *b, int max_ewsize, t_bool isstruct_head);
-static int rfc1522_do_encode (char *what, char **where, t_bool break_long_line);
-static int sizeofnextword (char *w);
-static int which_encoding (char *w);
-static t_bool contains_nonprintables (char *w, t_bool isstruct_head);
-static unsigned hex2bin (int x);
-static void build_base64_rank_table (void);
-static void str2b64 (char *from, char *to);
+static int do_b_encode(char *w, char *b, int max_ewsize, t_bool isstruct_head);
+static int rfc1522_do_encode(char *what, char **where, const char *charset, t_bool break_long_line);
+static int sizeofnextword(char *w);
+static int which_encoding(char *w, const char *charset);
+static t_bool contains_nonprintables(char *w, t_bool isstruct_head);
+static unsigned hex2bin(int x);
+static void build_base64_rank_table(void);
+static void str2b64(char *from, char *to);
 
 
 static void
-build_base64_rank_table (
+build_base64_rank_table(
 	void)
 {
 	int i;
@@ -101,7 +101,7 @@ build_base64_rank_table (
 
 
 static unsigned
-hex2bin (
+hex2bin(
 	int x)
 {
 	if (x >= '0' && x <= '9')
@@ -119,27 +119,16 @@ hex2bin (
  * Return number of bytes decoded into 'where' or -1.
  */
 int
-mmdecode (
+mmdecode(
 	const char *what,
 	int encoding,
 	int delimiter,
-	char *where,
-	const char *charset)
+	char *where)
 {
 	char *t;
-	t_bool decode_gt128 = FALSE;
-
-#ifdef MIME_STRICT_CHARSET
-#	ifdef CHARSET_CONVERSION
-	if (charset && !strcasecmp(charset, tinrc.mm_local_charset))
-#	else
-	if (charset && !strcasecmp(charset, tinrc.mm_charset))
-#	endif /* CHARSET_CONVERSION */
-#endif /* MIME_STRICT_CHARSET */
-		decode_gt128 = TRUE;
 
 	t = where;
-	encoding = tolower((unsigned char)encoding);
+	encoding = tolower((unsigned char) encoding);
 	if (encoding == 'q') {		/* quoted-printable */
 		int x;
 		unsigned hi, lo;
@@ -168,8 +157,6 @@ mmdecode (
 			if (hi == 255 || lo == 255)
 				return -1;
 			x = (hi << 4) + lo;
-			if (x >= 128 && !decode_gt128)
-				x = '?';
 			*EIGHT_BIT(t)++ = x;
 		}
 		return t - where;
@@ -194,8 +181,6 @@ mmdecode (
 			bits += 6;
 			if (bits >= 8) {
 				x = (pattern >> (bits - 8)) & 0xff;
-				if (x >= 128 && !decode_gt128)
-					x = '?';
 				*t++ = x;
 				bits -= 8;
 			}
@@ -212,21 +197,38 @@ mmdecode (
  * format
  */
 char *
-rfc1522_decode (
+rfc1522_decode(
 	const char *s)
 {
-	const char *c, *d;
+	char *c, *sc;
+	const char *d;
 	char *t;
-	static char buffer[2048];
+#define BUFFER_LEN 2048
+	static char buffer[BUFFER_LEN];
+	int max_len;
 	char charset[1024];
 	char encoding;
 	t_bool adjacentflag = FALSE;
 
 	charset[0] = '\0';
-	c = s;
+	c = my_strdup(s);
+	max_len = strlen(c) + 1;
 	t = buffer;
 
-	while (*c && t - buffer < 2048) {
+	/*
+	 * remove non-ASCII chars if MIME_STRICT_CHARSET is set
+	 * must be changed if UTF-8 becomes default charset for headers:
+	 *
+	 * process_charsets(c, "UTF-8", tinrc.mm_local_charset, FALSE);
+	 */
+#ifndef CHARSET_CONVERSION
+	process_charsets(&c, &max_len, "US-ASCII", tinrc.mm_local_charset, FALSE);
+#else
+	process_charsets(&c, &max_len, (CURR_GROUP.attribute->undeclared_charset) ? (CURR_GROUP.attribute->undeclared_charset) : "US-ASCII", tinrc.mm_local_charset, FALSE);
+#endif /* !CHARSET_CONVERSION */
+	sc = c;
+
+	while (*c && t - buffer < BUFFER_LEN) {
 		if (*c != '=') {
 			if (adjacentflag && isspace((unsigned char) *c)) {
 				const char *dd;
@@ -256,16 +258,29 @@ rfc1522_decode (
 				c++;
 				encoding = tolower((unsigned char)*c);
 				if (encoding == 'b')
-					(void) mmdecode((char *) 0, 'b', 0, (char *) 0, (char *) 0);		/* flush */
+					(void) mmdecode((char *) 0, 'b', 0, (char *) 0);	/* flush */
 				c++;
 				if (*c == '?') {
 					c++;
 					if ((e = strchr(c, '?'))) {
 						int i;
 
-						i = mmdecode(c, encoding, '?', t, charset);
+						i = mmdecode(c, encoding, '?', t);
 						if (i > 0) {
-							t += i;
+							int chars_to_copy;
+							char *tmpbuf;
+
+							max_len = i + 1;
+							tmpbuf = my_malloc(max_len);
+							strncpy(tmpbuf, t, i);
+							*(tmpbuf + i) = '\0';
+							process_charsets(&tmpbuf, &max_len, charset, tinrc.mm_local_charset, FALSE);
+							chars_to_copy = strlen(tmpbuf);
+							if (chars_to_copy > BUFFER_LEN - (t - buffer) - 1)
+								chars_to_copy = BUFFER_LEN - (t - buffer) - 1;
+							strncpy(t, tmpbuf, chars_to_copy);
+							free(tmpbuf);
+							t += chars_to_copy;
 							e++;
 							if (*e == '=')
 								e++;
@@ -276,18 +291,11 @@ rfc1522_decode (
 				}
 			}
 		}
-		while (d != c)
+		while (d != c && t - buffer < BUFFER_LEN - 1)
 			*t++ = *d++;
 	}
 	*t = '\0';
-
-#if defined(LOCAL_CHARSET) || defined(MAC_OS_X)
-	buffer_to_local(buffer);
-#else
-#	ifdef CHARSET_CONVERSION
-	buffer_to_local(buffer, charset, tinrc.mm_local_charset);
-#	endif /* CHARSET_CONVERSION */
-#endif /* LOCAL_CHARSET || MAC_OS_X */
+	free(sc);
 
 	return buffer;
 }
@@ -298,7 +306,7 @@ rfc1522_decode (
  * Woohyung Choi's(whchoi@cosmos.kaist.ac.kr) sdn2ks and ks2sdn
  */
 static void
-str2b64 (
+str2b64(
 	char *from,
 	char *to)
 {
@@ -325,21 +333,19 @@ str2b64 (
 
 
 static int
-do_b_encode (
+do_b_encode(
 	char *w,
 	char *b,
 	int max_ewsize,
 	t_bool isstruct_head)
 {
-
 	char tmp[60];				/* strings to be B encoded */
 	char *t = tmp;
 	int len8 = 0;				/* the number of trailing 8bit chars, which
-									   should be even(i.e. the first and second byte
-									   of wide_char should NOT be split into two
-									   encoded words) in order to be compatible with
-									   some CJK mail client */
-
+								   should be even(i.e. the first and second byte
+								   of wide_char should NOT be split into two
+								   encoded words) in order to be compatible with
+								   some CJK mail client */
 	int count = max_ewsize / 4 * 3;
 	t_bool isleading_between = TRUE;		/* are we still processing leading space */
 
@@ -370,10 +376,13 @@ do_b_encode (
  * header (i.e. two encoding won't be mixed in a single header line).
  * Mixing two encodings is not a violation of RFC 2047 but may break
  * some news/mail clients.
+ *
+ * mmnwcharset is ignored unless CHARSET_CONVERSION
  */
 static int
-which_encoding (
-	char *w)
+which_encoding(
+	char *w,
+	const char *charset)
 {
 	int chars = 0;
 	int schars = 0;
@@ -398,11 +407,7 @@ which_encoding (
 		 */
 		if (chars + 2 * (nonprint + schars) /* QP size */ >
 			 (chars * 4 + 3) / 3		/* B64 size */
-#ifdef CHARSET_CONVERSION
-			 || !strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-KR")
-#else
-			 || !strcasecmp(tinrc.mm_charset, "EUC-KR")
-#endif /* CHARSET_CONVERSION */
+			 || !strcasecmp(charset, "EUC-KR")
 			)
 			return 'B';
 		return 'Q';
@@ -413,7 +418,7 @@ which_encoding (
 
 /* now only checks if there's any 8bit chars in a given "fragment" */
 static t_bool
-contains_nonprintables (
+contains_nonprintables(
 	char *w,
 	t_bool isstruct_head)
 {
@@ -434,7 +439,7 @@ contains_nonprintables (
 			 * best would be using a regexp like
 			 * ^=\?\S+\?[qQbB]\?\S+\?=
 			 */
-			if (strstr(w, "?=") != (char *) 0)
+			if (strstr(w, "?=") != NULL)
 				nonprint = TRUE;
 		}
 		w++;
@@ -448,7 +453,7 @@ contains_nonprintables (
  * with rfc 2047 (rfc 1522)
  */
 static int
-sizeofnextword (
+sizeofnextword(
 	char *w)
 {
 	char *x;
@@ -466,6 +471,7 @@ static int
 rfc1522_do_encode(
 	char *what,
 	char **where,
+	const char *charset,
 	t_bool break_long_line)
 {
 	/*
@@ -491,23 +497,24 @@ rfc1522_do_encode(
 	 * (just checking that they don get longer than 76 characters),
 	 * then, in a second pass, we replace all SPACEs inside encoded
 	 * words by '_', break long lines, etc.
+	 *
+	 * *group is passed for evaluation of group-attributes,
+	 * set to NULL for e-mails
 	 */
-
-	int encoding;					  /* which encoding to use ('B' or 'Q') */
-	t_bool quoting = FALSE;		  /* currently inside quote block? */
-	t_bool any_quoting_done = FALSE;
-	t_bool isbroken_within = FALSE;	/* is word broken due to length restriction on encoded of word? */
-	t_bool rightafter_ew = FALSE;
+	char *c;
+	char *t;
+	char buf[2048];				/* buffer for encoded stuff */
+	char buf2[80];					/* buffer for this and that */
+	int encoding;					/* which encoding to use ('B' or 'Q') */
+	int ew_taken_len;
 	int column = 0;				/* current column */
 	int word_cnt = 0;
 	size_t ewsize = 0;			/* size of current encoded-word */
-	char buf[2048];				/* buffer for encoded stuff */
-	char buf2[80];				/* buffer for this and that */
-	char *c;
-	char *t;
+	t_bool quoting = FALSE;		/* currently inside quote block? */
+	t_bool any_quoting_done = FALSE;
+	t_bool isbroken_within = FALSE;	/* is word broken due to length restriction on encoded of word? */
 	t_bool isstruct_head = FALSE;		/* are we dealing with structured header? */
-	int ew_taken_len;
-
+	t_bool rightafter_ew = FALSE;
 /*
  * the list of structured header fields where '(' and ')' are
  * treated specially in rfc 1522 encoding
@@ -526,12 +533,8 @@ rfc1522_do_encode(
 	} while (*(++strptr) != 0);
 
 	t = buf;
-	encoding = which_encoding(what);
-#ifdef CHARSET_CONVERSION
-	ew_taken_len = strlen(txt_mime_charsets[tinrc.mm_network_charset]) + 7 /* =?c?E?d?= */;
-#else
-	ew_taken_len = strlen(tinrc.mm_charset) + 7;		/* the minimum encoded word length without any encoded text */
-#endif /* CHARSET_CONVERSION */
+	encoding = which_encoding(what, charset);
+	ew_taken_len = strlen(charset) + 7 /* =?c?E?d?= */;
 	while (*what) {
 		if (break_long_line)
 			word_cnt++;
@@ -543,15 +546,12 @@ rfc1522_do_encode(
 		if (contains_nonprintables(what, isstruct_head) || isbroken_within) {
 			if (encoding == 'Q') {
 				if (!quoting) {
-#ifdef CHARSET_CONVERSION
-					sprintf(buf2, "=?%s?%c?", txt_mime_charsets[tinrc.mm_network_charset], encoding);
-#else
-					sprintf(buf2, "=?%s?%c?", tinrc.mm_charset, encoding);
-#endif /* CHARSET_CONVERSION */
+					sprintf(buf2, "=?%s?%c?", charset, encoding);
 					ewsize = mystrcat(&t, buf2);
 					if (break_long_line) {
 						if (word_cnt == 2) {
-							/* Make sure we fit the first encoded
+							/*
+							 * Make sure we fit the first encoded
 							 * word in with the header keyword,
 							 * since we cannot break the line
 							 * directly after the keyword.
@@ -565,9 +565,9 @@ rfc1522_do_encode(
 				isbroken_within = FALSE;
 				while (*what && !isbetween(*what, isstruct_head)) {
 #if 0
-					if (is_EIGHT_BIT(what) || (strchr (RFC2047_ESPECIALS, *what)))
+					if (is_EIGHT_BIT(what) || (strchr(RFC2047_ESPECIALS, *what)))
 #else
-					if (is_EIGHT_BIT(what) || !isalnum((int)*what))
+					if (is_EIGHT_BIT(what) || !isalnum((int)(unsigned char) *what))
 #endif /* 0 */
 					{
 						sprintf(buf2, "=%2.2X", *EIGHT_BIT(what));
@@ -580,7 +580,8 @@ rfc1522_do_encode(
 						ewsize++;
 					}
 					what++;
-					/* Be sure to encode at least one char, even if
+					/*
+					 * Be sure to encode at least one char, even if
 					 * that overflows the line limit, otherwise, we
 					 * will be stuck in a loop (if this were in the
 					 * while condition above). (Can only happen in
@@ -592,12 +593,7 @@ rfc1522_do_encode(
 						break;
 					}
 				}
-#ifdef CHARSET_CONVERSION
-				if (!contains_nonprintables(what, isstruct_head) || ewsize >= 70 - strlen(txt_mime_charsets[tinrc.mm_network_charset]))
-#else
-				if (!contains_nonprintables(what, isstruct_head) || ewsize >= 70 - strlen(tinrc.mm_charset))
-#endif /* CHARSET_CONVERSION */
-				{
+				if (!contains_nonprintables(what, isstruct_head) || ewsize >= 70 - strlen(charset)) {
 					/* next word is 'clean', close encoding */
 					*t++ = '?';
 					*t++ = '=';
@@ -607,12 +603,7 @@ rfc1522_do_encode(
  * after the point where it's split should be encoded (i.e. even if
  * they are made of only 7bit chars)
  */
-#ifdef CHARSET_CONVERSION
-					if (ewsize >= 70 - strlen(txt_mime_charsets[tinrc.mm_network_charset]) && (contains_nonprintables(what, isstruct_head) || isbroken_within))
-#else
-					if (ewsize >= 70 - strlen(tinrc.mm_charset) && (contains_nonprintables(what, isstruct_head) || isbroken_within))
-#endif /* CHARSET_CONVERSION */
-					{
+					if (ewsize >= 70 - strlen(charset) && (contains_nonprintables(what, isstruct_head) || isbroken_within)) {
 						*t++ = ' ';
 						ewsize++;
 					}
@@ -631,21 +622,18 @@ rfc1522_do_encode(
 							ewsize += 3;
 						}
 						what++;
-					}					  /* end of while */
-				}						  /* end of else */
-			} else {					  /* end of Q encoding and beg. of B encoding */
+					}					/* end of while */
+				}						/* end of else */
+			} else {					/* end of Q encoding and beg. of B encoding */
 				/*
-				 * if what immediately precedes the current fragment with 8bit char is
-				 * encoded word, the leading spaces should be encoded together with
-				 * 8bit chars following them. No need to worry about '(',')' and '"'
-				 * as they're already excluded with contain_nonprintables used in outer if-clause
+				 * if what immediately precedes the current fragment with 8bit
+				 * char is encoded word, the leading spaces should be encoded
+				 * together with 8bit chars following them. No need to worry
+				 * about '(',')' and '"' as they're already excluded with
+				 * contain_nonprintables used in outer if-clause
 				 */
 				while (*what && (!isbetween(*what, isstruct_head) || rightafter_ew)) {
-#ifdef CHARSET_CONVERSION
-					sprintf(buf2, "=?%s?%c?", txt_mime_charsets[tinrc.mm_network_charset], encoding);
-#else
-					sprintf(buf2, "=?%s?%c?", tinrc.mm_charset, encoding);
-#endif /* CHARSET_CONVERSION */
+					sprintf(buf2, "=?%s?%c?", charset, encoding);
 					ewsize = mystrcat(&t, buf2);
 
 					if (word_cnt == 2)
@@ -662,7 +650,7 @@ rfc1522_do_encode(
 					any_quoting_done = TRUE;
 				}
 				rightafter_ew = TRUE;
-				word_cnt--;			  /* compensate double counting */
+				word_cnt--;		/* compensate double counting */
 				/*
 				 * if encoded word is followed by 7bit-only fragment, we need to
 				 * eliminate ' ' inserted in while-block above
@@ -671,38 +659,36 @@ rfc1522_do_encode(
 					t--;
 					ewsize--;
 				}
-			}							  /* end of B encoding */
+			}		/* end of B encoding */
 		} else {
 			while (*what && !isbetween(*what, isstruct_head))
-				*t++ = *what++;	  /* output word unencoded */
+				*t++ = *what++;		/* output word unencoded */
 			while (*what && isbetween(*what, isstruct_head))
-				*t++ = *what++;	  /* output trailing whitespace unencoded */
+				*t++ = *what++;		/* output trailing whitespace unencoded */
 			rightafter_ew = FALSE;
 		}
-	}									  /* end of pass 1 while loop */
+	}		/* end of pass 1 while loop */
 	*t = 0;
 	/* Pass 2: break long lines if there are MIME-sequences in the result */
 	c = buf;
 	if (break_long_line) {
 		column = 0;
 		if (any_quoting_done) {
-			word_cnt = 1;			  /*
-											* note, if the user has typed a
-										   * continuation line, we will consider the
-										   * initial whitespace to be delimiting
-										   * word one (well, just assume an empty
-										   * word).
-										   */
+			word_cnt = 1;			/*
+									 * note, if the user has typed a continuation
+									 * line, we will consider the initial
+									 * whitespace to be delimiting word one (well,
+									 * just assume an empty word).
+									 */
 			while (*c) {
 				if (isspace((unsigned char) *c)) {
-					/* According to rfc1522, header lines
-					 * containing encoded words are limited to 76
-					 * chars, but if the first line is too long
-					 * (due to a long header keyword), we cannot
-					 * stick to that, since we would break the line
-					 * directly after the keyword's colon, which is
-					 * not allowed. The same is necessary for a
-					 * continuation line with an unencoded word
+					/*
+					 * According to rfc1522, header lines containing encoded
+					 * words are limited to 76 chars, but if the first line is
+					 * too long (due to a long header keyword), we cannot stick
+					 * to that, since we would break the line directly after the
+					 * keyword's colon, which is not allowed. The same is
+					 * necessary for a continuation line with an unencoded word
 					 * that is too long.
 					 */
 					if (sizeofnextword(c) + column > 76 && word_cnt != 1) {
@@ -723,7 +709,7 @@ rfc1522_do_encode(
 			while (*c)
 				*((*where)++) = *c++;
 		}
-	} else {							  /* !break_long_line */
+	} else {		/* !break_long_line */
 		while (*c)
 			*((*where)++) = *c++;
 	}
@@ -733,47 +719,44 @@ rfc1522_do_encode(
 }
 
 
+/*
+ * calling code must free() the result if it's no longer needed
+ */
 char *
-rfc1522_encode (
+rfc1522_encode(
 	char *s,
+	const char *charset,
 	t_bool ismail)
 {
-	char *b;
+	char *b, *buf;
 	int x;
-	static char buf[2048];
-
 /*
- * break_long_line is FALSE for news posting unless MIME_BREAK_LONG_LINES
- * is defined, but it's TRUE for mail messages regardless of whether or not
+ * break_long_line is FALSE for news posting unless MIME_BREAK_LONG_LINES is
+ * defined, but it's TRUE for mail messages regardless of whether or not
  * MIME_BREAK_LONG_LINES is defined
  */
 #ifdef MIME_BREAK_LONG_LINES
 	t_bool break_long_line = TRUE;
 #else
-	t_bool break_long_line = FALSE;
+/*
+ * Even if MIME_BREAK_LONG_LINES is NOT defined, long headers in mail
+ * messages should be broken up in accordance with RFC 2047(1522)
+ */
+	t_bool break_long_line = ismail;
 #endif /* MIME_BREAK_LONG_LINES */
 
-/*
- * Even if MIME_BREAK_LONG_LINES is NOT defined,
- * long headers in mail messages should be broken up in
- * accordance with RFC 2047(1522)
- */
-#ifndef MIME_BREAK_LONG_LINES
-	if (ismail)
-		break_long_line = TRUE;
-#endif /* !MIME_BREAK_LONG_LINES */
-
-	b = buf;
-	x = rfc1522_do_encode(s, &b, break_long_line);
+	b = buf = my_malloc(2048);
+	x = rfc1522_do_encode(s, &b, charset, break_long_line);
 	quoteflag = quoteflag || x;
 	return buf;
 }
 
 
 void
-rfc15211522_encode (
+rfc15211522_encode(
 	const char *filename,
 	constext * mime_encoding,
+	struct t_group *group,
 	t_bool allow_8bit_header,
 	t_bool ismail)
 {
@@ -786,6 +769,16 @@ rfc15211522_encode (
 	t_bool body_encoding_needed = FALSE;
 	t_bool umlauts = FALSE;
 	BodyPtr body_encode;
+#if defined(LOCAL_CHARSET) || defined(CHARSET_CONVERSION)
+	int mmnwcharset = 0;
+
+#	ifdef CHARSET_CONVERSION
+	if (group) /* Posting */
+		mmnwcharset = group->attribute->mm_network_charset;
+	else /* E-Mail */
+		mmnwcharset = tinrc.mm_network_charset;
+#	endif /* CHARSET_CONVERSION */
+#endif /* LOCAL_CHARSET || CHARSET_CONVERSION */
 
 	if ((g = tmpfile()) == NULL)
 		return;
@@ -797,24 +790,38 @@ rfc15211522_encode (
 	quoteflag = 0;
 
 	while ((header = tin_fgets(f, TRUE))) {
-#if defined(LOCAL_CHARSET) || defined(MAC_OS_X) || defined(CHARSET_CONVERSION)
-		buffer_to_network(header);
-#endif /* LOCAL_CHARSET || MAC_OS_X || defined(CHARSET_CONVERSION) */
+#if defined(LOCAL_CHARSET) || defined(CHARSET_CONVERSION)
+		buffer_to_network(header, mmnwcharset);
+#endif /* LOCAL_CHARSET || CHARSET_CONVERSION */
 		if (*header == '\0')
 			break;
+
 		if (allow_8bit_header)
 			fputs(header, g);
-		else
-			fputs(rfc1522_encode(header, ismail), g);
+		else {
+			char *p;
+
+#ifdef CHARSET_CONVERSION
+			if (group)
+				p = rfc1522_encode(header, txt_mime_charsets[group->attribute->mm_network_charset], ismail);
+			else
+				p = rfc1522_encode(header, txt_mime_charsets[tinrc.mm_network_charset], ismail);
+#else
+			p = rfc1522_encode(header, tinrc.mm_charset, ismail);
+#endif /* CHARSET_CONVERSION */
+
+			fputs(p, g);
+			free(p);
+		}
 		fputc('\n', g);
 	}
 
 	fputc('\n', g);
 
 	while (fgets(buffer, 2048, f)) {
-#if defined(LOCAL_CHARSET) || defined(MAC_OS_X) || defined(CHARSET_CONVERSION)
-		buffer_to_network(buffer);
-#endif /* LOCAL_CHARSET || MAC_OS_X || defined(CHARSET_CONVERSION) */
+#if defined(LOCAL_CHARSET) || defined(CHARSET_CONVERSION)
+		buffer_to_network(buffer, mmnwcharset);
+#endif /* LOCAL_CHARSET || CHARSET_CONVERSION */
 		fputs(buffer, g);
 		/* see if there are any umlauts in the body... */
 		for (c = buffer; *c && !isreturn(*c); c++)
@@ -836,22 +843,21 @@ rfc15211522_encode (
 		fputs(buffer, f);
 
 	/* now add MIME headers as necessary */
-#if 0									  /* RFC1522 does not require MIME headers just because there are
-										     encoded header lines */
+#if 0	/* RFC1522 does not require MIME headers just because there are encoded header lines */
 	if (quoteflag || umlauts)
 #else
 	if (umlauts)
 #endif /* 0 */
 	{
-		fputs("MIME-Version: 1.0\n", f);
+		fprintf(f, "MIME-Version: %s\n", MIME_SUPPORTED_VERSION);
 		if (body_encoding_needed) {
 		/* added for CJK charsets like EUC-KR/JP/CN and others */
 #ifdef CHARSET_CONVERSION
-			if (!strncasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-", 4) &&
+			if (!strncasecmp(txt_mime_charsets[mmnwcharset], "EUC-", 4) &&
 				 !strcasecmp(mime_encoding, txt_7bit))
-				fprintf(f, "Content-Type: text/plain; charset=ISO-2022-%s\n", &txt_mime_charsets[tinrc.mm_network_charset][4]);
+				fprintf(f, "Content-Type: text/plain; charset=ISO-2022-%s\n", &txt_mime_charsets[mmnwcharset][4]);
 			else
-				fprintf(f, "Content-Type: text/plain; charset=%s\n", txt_mime_charsets[tinrc.mm_network_charset]);
+				fprintf(f, "Content-Type: text/plain; charset=%s\n", txt_mime_charsets[mmnwcharset]);
 #else
 			if (!strncasecmp(tinrc.mm_charset, "EUC-", 4) &&
 				 !strcasecmp(mime_encoding, txt_7bit))
@@ -885,7 +891,7 @@ rfc15211522_encode (
 /* For EUC-KR, 7bit means conversion to ISO-2022-KR specified in RFC 1557 */
 
 #ifdef CHARSET_CONVERSION
-		if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-KR"))
+		if (!strcasecmp(txt_mime_charsets[mmnwcharset], "EUC-KR"))
 #else
 		if (!strcasecmp(tinrc.mm_charset, "EUC-KR"))
 #endif /* CHARSET_CONVERSION */
@@ -893,11 +899,11 @@ rfc15211522_encode (
 
 #if 0
 /*
- * Not only EUC-JP but also other Japanese charsets such as
- * SJIS might need RFC 1468 encoding. To be confirmed.
+ * Not only EUC-JP but also other Japanese charsets such as SJIS might need
+ * RFC 1468 encoding. To be confirmed.
  */
 #	ifdef CHARSET_CONVERSION
-		else if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-JP"))
+		else if (!strcasecmp(txt_mime_charsets[mmnwcharset], "EUC-JP"))
 			body_encode = rfc1468_encode;
 #	else
 		else if (!strcasecmp(tinrc.mm_charset, "EUC-JP"))
@@ -905,11 +911,11 @@ rfc15211522_encode (
 #	endif /* CHARSET_CONVERSION */
 
 /*
- * Not only EUC-CN but also other Chinese charsets such as
- * BIG5 and EUC-TW might need RFC 1922 encoding. To be confirmed.
+ * Not only EUC-CN but also other Chinese charsets such as Big5 and EUC-TW
+ * might need RFC 1922 encoding. To be confirmed.
  */
 #	ifdef CHARSET_CONVERSION
-		else if (!strcasecmp(txt_mime_charsets[tinrc.mm_network_charset], "EUC-CN"))
+		else if (!strcasecmp(txt_mime_charsets[mmnwcharset], "EUC-CN"))
 			body_encode = rfc1922_encode;
 #	else
 		else if (!strcasecmp(tinrc.mm_charset, "EUC-CN"))
