@@ -3,10 +3,10 @@
  *  Module    : search.c
  *  Author    : I. Lea & R. Skrenta
  *  Created   : 1991-04-01
- *  Updated   : 1997-12-27
+ *  Updated   : 2003-01-07
  *  Notes     :
  *
- * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
+ * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>, Rich Skrenta <skrenta@pbm.com>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Iain Lea, Rich Skrenta.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -41,22 +38,27 @@
 #ifndef TIN_H
 #	include "tin.h"
 #endif /* !TIN_H */
-
-/*
- * Hold the last pattern used
- */
-static char tmpbuf[LEN];
+#ifndef MENUKEYS_H
+#	include "menukeys.h"
+#endif /* !MENUKEYS_H */
 
 /*
  * local prototypes
  */
-static char *get_search_pattern (t_bool forward, const char *fwd_msg, const char *bwd_msg, char *def, int which_hist);
+static char *get_search_pattern(t_bool *forward, t_bool repeat, const char *fwd_msg, const char *bwd_msg, char *def, int which_hist);
+static int author_search(int i, char *searchbuf);
+static int body_search(int i, char *searchbuf);
+static int subject_search(int i, char *searchbuf);
+static int search_group(t_bool forward, int current_art, char *searchbuff, int (*search_func) (int i, char *searchbuff));
+
 
 /*
  * The search function may place error text into mesg
  */
 #define MATCH_MSG	(mesg[0] ? mesg : _(txt_no_match))
 
+/* char i_key_search_last; */				/* for repeated search */
+
 /*
  * Kludge to maintain some internal state for body search
  */
@@ -68,7 +70,7 @@ int srch_lineno = -1;
  * of parameters all the time
  */
 static int srch_offsets[6];
-static int srch_offsets_size = sizeof(srch_offsets)/sizeof(int);
+static int srch_offsets_size = ARRAY_SIZE(srch_offsets);
 static struct regex_cache srch_regex;
 
 
@@ -77,72 +79,92 @@ static struct regex_cache srch_regex;
  * Return NULL if no pattern could be found
  */
 static char *
-get_search_pattern (
-	t_bool forward,
+get_search_pattern(
+	t_bool *forward,
+	t_bool repeat,
 	const char *fwd_msg,
 	const char *bwd_msg,
 	char *def,
 	int which_hist)
 {
-	sprintf (tmpbuf, (forward ? fwd_msg : bwd_msg), def);
+	static char tmpbuf[LEN];	/* Hold the last pattern used */
+	static char last_pattern[LEN];	/* last search pattern used; for repeated search */
+	static t_bool last_forward;
 
-	if (!prompt_string_default(tmpbuf, def, _(txt_no_search_string), which_hist))
-		return NULL;
+	if (repeat) {
+		*forward = last_forward;
+		my_strncpy(def, last_pattern, LEN);
+	}
+	else {
+		sprintf(tmpbuf, (*forward ? fwd_msg : bwd_msg), def);
 
-	wait_message (0, _(txt_searching));
+		if (!prompt_string_default(tmpbuf, def, _(txt_no_search_string), which_hist))
+			return NULL;
+
+		last_forward = *forward;
+		my_strncpy(last_pattern, def, LEN);
+
+		/* HIST_BODY_SEARCH doesn't exist, hence i_key_search_last is set directly in search_body() */
+		if (which_hist == HIST_AUTHOR_SEARCH)
+			i_key_search_last = *forward ? iKeySearchAuthF : iKeySearchAuthB;
+		else
+			i_key_search_last = *forward ? iKeySearchSubjF : iKeySearchSubjB;
+	}
+
+	wait_message(0, _(txt_searching));
 
 	stow_cursor();
 
 	if (tinrc.wildcard) {			/* ie, not wildmat() */
-		strcpy(def, quote_wild_whitespace (def));
+		strcpy(def, quote_wild_whitespace(def));
 		return def;
 	}
 
 	/*
 	 * A gross hack to simulate substrings with wildmat()
 	 */
-/* TODO somehow use REGEX_FMT here ? */
+/* TODO somehow use REGEX_FMT here? */
 	sprintf(tmpbuf, "*%s*", def);
 	return tmpbuf;
 }
 
 
 /*
- * Called by config.c
+ * called by config.c
  */
 int
-search_config (
+search_config(
 	t_bool forward,
+	t_bool repeat,
 	int current,
 	int last)
 {
-	int n;
-	int incr = forward ? 1 : -1;
-	int result = current;
 	char *buf;
+	int n;
+	int incr;
+	int result = current;
 
-	if (!(buf = get_search_pattern(
-				forward,
-				_(txt_search_forwards),
-				_(txt_search_backwards),
-				tinrc.default_search_config,
-				HIST_CONFIG_SEARCH
-	))) return result;
+	if (!(buf = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_config, HIST_CONFIG_SEARCH)))
+		return result;
+
+	incr = forward ? 1 : -1;
 
 	current += incr;
 	n = current;
 	do {
 		if (n < 0)
 			n = last;
-		else if (n > last)
-			n = 0;
-		if (REGEX_MATCH (option_table[n].txt->opt, buf, TRUE)) {
+		else {
+			if (n > last)
+				n = 0;
+		}
+		if (REGEX_MATCH(option_table[n].txt->opt, buf, TRUE)) {
 			result = n;
 			break;
 		}
 		n += incr;
 	} while (n != current);
-	clear_message ();
+	clear_message();
 	return result;
 }
 
@@ -153,8 +175,9 @@ search_config (
  * Return index into active of matching groupname or -1
  */
 int
-search_active (
-	t_bool forward)
+search_active(
+	t_bool forward,
+	t_bool repeat)
 {
 	char *buf;
 	char buf2[LEN];
@@ -162,23 +185,21 @@ search_active (
 	int i;
 
 	if (!selmenu.max) {
-		info_message (_(txt_no_groups));
+		info_message(_(txt_no_groups));
 		return -1;
 	}
 
-	if (!(buf = get_search_pattern(forward, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_group, HIST_GROUP_SEARCH)))
+	if (!(buf = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_group, HIST_GROUP_SEARCH)))
 		return -1;
 
 	i = selmenu.curr;
 
 	do {
 		if (forward) {
-			i++;
-			if (i >= selmenu.max)
+			if (++i >= selmenu.max)
 				i = 0;
 		} else {
-			i--;
-			if (i < 0)
+			if (--i < 0)
 				i = selmenu.max - 1;
 		}
 
@@ -186,17 +207,17 @@ search_active (
 		 * Get the group name & description into buf2
 		 */
 		if (show_description && active[my_group[i]].description) {
-			sprintf (buf2, "%s %s", active[my_group[i]].name, active[my_group[i]].description);
+			snprintf(buf2, sizeof(buf2) - 1, "%s %s", active[my_group[i]].name, active[my_group[i]].description);
 			ptr = buf2;
 		} else
 			ptr = active[my_group[i]].name;
 
-		if (REGEX_MATCH (ptr, buf, TRUE)) {
+		if (REGEX_MATCH(ptr, buf, TRUE)) {
 			return i;
 		}
 	} while (i != selmenu.curr);
 
-	info_message (MATCH_MSG);
+	info_message(MATCH_MSG);
 	return -1;
 }
 
@@ -206,31 +227,30 @@ search_active (
  * used only by search_body()
  * Returns:	1	String found
  *          0	Not found
- *			-1	User aborted search
+ *	        -1	User aborted search
  */
 static int
-body_search (
+body_search(
 	int i,
 	char *searchbuf)
 {
 	char *line;
 	char group_path[PATH_LEN];
-	int  code;
 	t_openartinfo artinfo;
 
 	if (!read_news_via_nntp || CURR_GROUP.type != GROUP_TYPE_NEWS)
-		make_group_path (CURR_GROUP.name, group_path);
+		make_group_path(CURR_GROUP.name, group_path);
 
-	sprintf(mesg, _(txt_searching_body), ++curr_cnt, total_cnt);
-
-	memset (&artinfo, 0, sizeof(t_openartinfo));
-	switch (art_open (TRUE, &arts[i], group_path, &artinfo)) {
+	memset(&artinfo, 0, sizeof(t_openartinfo));
+	switch (art_open(TRUE, &arts[i], group_path, &artinfo, FALSE)) {
 		case ART_ABORT:					/* User 'q'uit */
-			code = -1;
-			goto exit_search;
-		case ART_UNAVAILABLE:
-			code = 0;					/* Treat as string not present */
-			goto exit_search;
+			art_close(&artinfo);
+			return -1;
+
+		case ART_UNAVAILABLE:			/* Treat as string not present */
+			art_close(&artinfo);
+			info_message(_(txt_no_match));
+			return 0;
 	}
 
 	/*
@@ -238,28 +258,30 @@ body_search (
 	 */
 	for (i = 0; artinfo.cookl[i].flags & C_HEADER; ++i)
 		;
-	fseek (artinfo.cooked, artinfo.cookl[i].offset, SEEK_SET);
+	fseek(artinfo.cooked, artinfo.cookl[i].offset, SEEK_SET);
 
 	/*
 	 * Now search the body
 	 */
-	while ((line = tin_fgets (artinfo.cooked, FALSE)) != (char *) 0) {
+	sprintf(mesg, _(txt_searching_body), ++curr_cnt, total_cnt);
+	show_progress(mesg, curr_cnt, total_cnt);
+	while ((line = tin_fgets(artinfo.cooked, FALSE)) != NULL) {
 		if (tinrc.wildcard) {
-			if (pcre_exec (srch_regex.re, srch_regex.extra, line, strlen(line), 0, 0, srch_offsets, srch_offsets_size) != PCRE_ERROR_NOMATCH) {
+			if (pcre_exec(srch_regex.re, srch_regex.extra, line, strlen(line), 0, 0, srch_offsets, srch_offsets_size) != PCRE_ERROR_NOMATCH) {
 				srch_lineno = i;
-				art_close (&pgart);		/* Switch the pager over to matched art */
+				art_close(&pgart);		/* Switch the pager over to matched art */
 				pgart = artinfo;
 #ifdef DEBUG
 				if (debug == 2)
-					fprintf(stderr, "art_switch(%p = %p)\n", &pgart, &artinfo);
+					fprintf(stderr, "art_switch(%p = %p)\n", (void *) &pgart, (void *) &artinfo);
 #endif /* DEBUG */
 
 				return 1;
 			}
 		} else {
-			if (REGEX_MATCH (line, searchbuf, TRUE)) {
+			if (wildmatpos(line, searchbuf, TRUE, srch_offsets, srch_offsets_size)) {
 				srch_lineno = i;
-				art_close (&pgart);		/* Switch the pager over to matched art */
+				art_close(&pgart);		/* Switch the pager over to matched art */
 				pgart = artinfo;
 				return 1;
 			}
@@ -268,14 +290,13 @@ body_search (
 	}
 
 	if (tin_errno != 0) {			/* User abort */
-		code = -1;
-		goto exit_search;
+		art_close(&artinfo);
+		return -1;
 	}
 
-	code = 0;						/* Didn't find it */
-exit_search:
-	art_close (&artinfo);
-	return code;
+	art_close(&artinfo);
+/*	info_message(MATCH_MSG); */
+	return 0;
 }
 
 
@@ -284,19 +305,19 @@ exit_search:
  * 1 = found, 0 = not found
  */
 static int
-author_search (
+author_search(
 	int i,
 	char *searchbuf)
 {
 	char buf[LEN];
 	char *ptr = buf;
 
-	if (arts[i].name == (char *) 0)
+	if (arts[i].name == NULL)
 		ptr = arts[i].from;
 	else
-		snprintf (buf, sizeof(buf) - 1, "%s <%s>", arts[i].name, arts[i].from);
+		snprintf(buf, sizeof(buf) - 1, "%s <%s>", arts[i].name, arts[i].from);
 
-	return (REGEX_MATCH (ptr, searchbuf, TRUE)) ? 1 : 0;
+	return (REGEX_MATCH(ptr, searchbuf, TRUE)) ? 1 : 0;
 }
 
 
@@ -305,11 +326,11 @@ author_search (
  * 1 = found, 0 = not found
  */
 static int
-subject_search (
+subject_search(
 	int i,
 	char *searchbuf)
 {
-	return (REGEX_MATCH (arts[i].subject, searchbuf, TRUE)) ? 1 : 0;
+	return (REGEX_MATCH(arts[i].subject, searchbuf, TRUE)) ? 1 : 0;
 }
 
 
@@ -317,7 +338,7 @@ subject_search (
  * Returns index into arts[] of matching article or -1
  */
 static int
-search_group (
+search_group(
 	t_bool forward,
 	int current_art,
 	char *searchbuff,
@@ -327,39 +348,39 @@ search_group (
 	int i;
 
 	if (grpmenu.curr < 0) {
-		info_message (_(txt_no_arts));
+		info_message(_(txt_no_arts));
 		return -1;
 	}
 
 	if (!read_news_via_nntp || CURR_GROUP.type != GROUP_TYPE_NEWS)
-		make_group_path (CURR_GROUP.name, group_path);
+		make_group_path(CURR_GROUP.name, group_path);
 
 	i = current_art;
 
 	do {
 		if (forward) {
-			if ((i = next_response (i)) < 0)
+			if ((i = next_response(i)) < 0)
 				i = base[0];
 		} else {
-			if ((i = prev_response (i)) < 0)
-				i = find_response(grpmenu.max - 1, num_of_responses (grpmenu.max - 1));
+			if ((i = prev_response(i)) < 0)
+				i = find_response(grpmenu.max - 1, num_of_responses(grpmenu.max - 1));
 		}
 
 		/* Only search displayed articles */
 		if (CURR_GROUP.attribute->show_only_unread && arts[i].status != ART_UNREAD)
 			continue;
 
-		switch (search_func (i, searchbuff)) {
+		switch (search_func(i, searchbuff)) {
 			case 1:								/* Found */
-				clear_message ();
+				clear_message();
 				return i;
+
 			case -1:							/* User abort */
 				return -1;
 		}
-
 	} while (i != current_art);
 
-	info_message (MATCH_MSG);
+	info_message(_(txt_no_match));
 	return -1;
 }
 
@@ -369,42 +390,30 @@ search_group (
  * Returns index into arts[] of matching article or -1
  */
 int
-search (
+search(
 	int key,
 	int current_art,
-	t_bool forward)
+	t_bool forward,
+	t_bool repeat)
 {
 	char *buf = NULL;
 	int (*search_func) (int i, char *searchbuff) = author_search;
 
 	switch (key) {
 		case SEARCH_SUBJ:
-			if (!(buf = get_search_pattern(
-					forward,
-					_(txt_search_forwards),
-					_(txt_search_backwards),
-					tinrc.default_search_subject,
-					HIST_SUBJECT_SEARCH
-			))) return -1;
-
+			if (!(buf = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_subject, HIST_SUBJECT_SEARCH)))
+				return -1;
 			search_func = subject_search;
 			break;
 
 		case SEARCH_AUTH:
 		default:
-			if (!(buf = get_search_pattern(
-					forward,
-					_(txt_author_search_forwards),
-					_(txt_author_search_backwards),
-					tinrc.default_search_author,
-					HIST_AUTHOR_SEARCH
-			))) return -1;
-
+			if (!(buf = get_search_pattern(&forward, repeat, _(txt_author_search_forwards), _(txt_author_search_backwards), tinrc.default_search_author, HIST_AUTHOR_SEARCH)))
+				return -1;
 			search_func = author_search;
 			break;
 	}
-
-	return (search_group (forward, current_art, buf, search_func));
+	return (search_group(forward, current_art, buf, search_func));
 }
 
 
@@ -414,27 +423,23 @@ search (
  * If using regex's return vector of character offsets
  */
 int
-search_article (
+search_article(
 	t_bool forward,
+	t_bool repeat,
 	int start_line,
 	int lines,
 	t_lineinfo *line,
-	t_bool show_ctrl_l,
+	int reveal_ctrl_l_lines,
 	FILE *fp)
 {
 	char *pattern, *ptr;
-	struct regex_cache srch;
 	int i;
+	struct regex_cache srch;
 
-	if (!(pattern = get_search_pattern(
-				forward,
-				_(txt_search_forwards),
-				_(txt_search_backwards),
-				tinrc.default_search_art,
-				HIST_ART_SEARCH
-	))) return FALSE;
+	if (!(pattern = get_search_pattern(&forward, repeat, _(txt_search_forwards), _(txt_search_backwards), tinrc.default_search_art, HIST_ART_SEARCH)))
+		return FALSE;
 
-	if (tinrc.wildcard && !(compile_regex (pattern, &srch, PCRE_EXTENDED | PCRE_CASELESS)))
+	if (tinrc.wildcard && !(compile_regex(pattern, &srch, PCRE_EXTENDED | PCRE_CASELESS)))
 		return -1;
 
 	srch_lineno = -1;
@@ -442,41 +447,40 @@ search_article (
 
 	forever {
 		if (forward) {
-			i++;
-			if (i == lines)
+			if (++i == lines)
 				break;
 		} else {
-			i--;
-			if (i < 0)
+			if (--i < 0)
 				break;
 		}
 
-/* TODO consider not searching some line types ? - body search skips hdrs */
-		fseek (fp, line[i].offset, SEEK_SET);
-
 		/*
-		 * Don't search beyond ^L if hiding is enabled
+		 * TODO: consider not searching some line types?
+		 * 'B'ody search skips hdrs, '/' inside article does not.
 		 */
-		if (!show_ctrl_l && (line[i].flags&C_CTRLL))
+		fseek(fp, line[i].offset, SEEK_SET);
+
+		/* Don't search beyond ^L if hiding is enabled */
+		if ((line[i].flags&C_CTRLL) && i > reveal_ctrl_l_lines)
 			break;
 
 		ptr = tin_fgets(fp, FALSE);
 
 		if (tinrc.wildcard) {
-			if (pcre_exec (srch.re, srch.extra, ptr, strlen(ptr), 0, 0,
+			if (pcre_exec(srch.re, srch.extra, ptr, strlen(ptr), 0, 0,
 								srch_offsets, srch_offsets_size) != PCRE_ERROR_NOMATCH) {
 				srch_lineno = i;
 				return i;
 			}
 		} else {
-			if (REGEX_MATCH (ptr, pattern, TRUE)) {
+			if (wildmatpos(ptr, pattern, TRUE, srch_offsets, srch_offsets_size)) {
 				srch_lineno = i;
 				return i;
 			}
 		}
 	}
 
-	wait_message (0, _(txt_no_match));
+	info_message(_(txt_no_match));
 	return -1;
 }
 
@@ -490,20 +494,24 @@ search_article (
  * Return index in arts[] of article that matched or -1
  */
 int
-search_body (
-	int current_art)
+search_body(
+	int current_art,
+	t_bool repeat)
 {
 	char *buf;
 	int i;
+	t_bool forward_fake;
 
 	if (!(buf = get_search_pattern(
-			TRUE,
+			&forward_fake,				/* we pass a dummy var since body search has no `forward' */
+			repeat,
 			_(txt_search_body),
 			_(txt_search_body),
 			tinrc.default_search_art,
 			HIST_ART_SEARCH
 	))) return -1;
 
+	i_key_search_last = iKeySearchBody;	/* store last search type for repeated search */
 	total_cnt = curr_cnt = 0;			/* Reset global counter of articles done */
 
 	/*
@@ -511,9 +519,9 @@ search_body (
 	 */
 	if (CURR_GROUP.attribute->show_only_unread) {
 		for (i = 0; i < grpmenu.max; i++)
-			total_cnt += new_responses (i);
+			total_cnt += new_responses(i);
 	} else {
-		for (i = 0; i < top_art; i++) {
+		for_each_art(i) {
 			if (!IGNORE_ART(i))
 				total_cnt++;
 		}
@@ -522,11 +530,11 @@ search_body (
 	/*
 	 * Pre-compile if we're using full regex
 	 */
-	if (tinrc.wildcard && !(compile_regex (buf, &srch_regex, PCRE_EXTENDED | PCRE_CASELESS)))
+	if (tinrc.wildcard && !(compile_regex(buf, &srch_regex, PCRE_EXTENDED | PCRE_CASELESS)))
 		return -1;
 
 	srch_lineno = -1;
-	return search_group (1, current_art, buf, body_search);
+	return search_group(1, current_art, buf, body_search);
 }
 
 
