@@ -3,10 +3,10 @@
  *  Module    : signal.c
  *  Author    : I.Lea
  *  Created   : 1991-04-01
- *  Updated   : 2001-07-22
+ *  Updated   : 2002-06-21
  *  Notes     : signal handlers for different modes and window resizing
  *
- * Copyright (c) 1991-2002 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1991-2003 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -17,10 +17,7 @@
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *    This product includes software developed by Iain Lea.
- * 4. The name of the author may not be used to endorse or promote
+ * 3. The name of the author may not be used to endorse or promote
  *    products derived from this software without specific prior written
  *    permission.
  *
@@ -52,7 +49,7 @@
 #endif /* !VERSION_H */
 
 
-#if defined (VMS) && defined (SIGBUS)
+#if defined(VMS) && defined(SIGBUS)
 	/*
 	 * SIGBUS works on VMS, but many useful information will be hidden
 	 * if catched by signal handler.
@@ -63,7 +60,6 @@
 /*
  * Needed for resizing under an xterm
  */
-
 #ifdef HAVE_TERMIOS_H
 #	include <termios.h>
 #else
@@ -100,12 +96,12 @@
  */
 static const char *signal_name(int code);
 #ifdef SIGTSTP
-	static void handle_suspend (void);
+	static void handle_suspend(void);
 #endif /* SIGTSTP */
-static void _CDECL signal_handler (SIG_ARGS);
+static void _CDECL signal_handler(SIG_ARGS);
 
 #ifndef WEXITSTATUS
-#	define WEXITSTATUS(stat_val) ((unsigned)(stat_val) >> 8)
+#	define WEXITSTATUS(stat_val) ((unsigned) (stat_val) >> 8)
 #endif /* !WEXITSTATUS */
 #ifndef WIFEXITED
 #	define WIFEXITED(stat_val) (((stat_val) & 255) == 0)
@@ -117,64 +113,67 @@ static void _CDECL signal_handler (SIG_ARGS);
 
 int signal_context = cMain;
 int need_resize = cNo;
+/*
+ * # lines of non-static data available for display
+ */
+int NOTESLINES;
 
-int NOTESLINES;			/* # lines of useable area */
 
 #ifndef __LCLINT__ /* lclint doesn't like it */
 static const struct {
 	int code;
 	const char *name;
 } signal_list[] = {
-#ifdef SIGINT
+#	ifdef SIGINT
 	{ SIGINT,	"SIGINT" },	/* ctrl-C */
-#endif /* SIGINT */
-#ifdef SIGQUIT
+#	endif /* SIGINT */
+#	ifdef SIGQUIT
 	{ SIGQUIT,	"SIGQUIT " },	/* ctrl-\ */
-#endif /* SIGQUIT */
-#ifdef SIGILL
+#	endif /* SIGQUIT */
+#	ifdef SIGILL
 	{ SIGILL,	"SIGILL" },	/* illegal instruction */
-#endif /* SIGILL */
-#ifdef SIGFPE
+#	endif /* SIGILL */
+#	ifdef SIGFPE
 	{ SIGFPE,	"SIGFPE" },	/* floating point exception */
-#endif /* SIGFPE */
-#ifdef SIGBUS
+#	endif /* SIGFPE */
+#	ifdef SIGBUS
 	{ SIGBUS,	"SIGBUS" },	/* bus error */
-#endif /* SIGBUS */
-#ifdef SIGSEGV
+#	endif /* SIGBUS */
+#	ifdef SIGSEGV
 	{ SIGSEGV,	"SIGSEGV" },	/* segmentation violation */
-#endif /* SIGSEGV */
-#ifdef SIGPIPE
+#	endif /* SIGSEGV */
+#	ifdef SIGPIPE
 	{ SIGPIPE,	"SIGPIPE" },	/* broken pipe */
-#endif /* SIGPIPE */
-#ifdef SIGCHLD
+#	endif /* SIGPIPE */
+#	ifdef SIGCHLD
 	{ SIGCHLD,	"SIGCHLD" },	/* death of a child process */
-#endif /* SIGCHLD */
-#ifdef SIGPWR
+#	endif /* SIGCHLD */
+#	ifdef SIGPWR
 	{ SIGPWR,	"SIGPWR" },	/* powerfail */
-#endif /* SIGPWR */
-#ifdef SIGTSTP
+#	endif /* SIGPWR */
+#	ifdef SIGTSTP
 	{ SIGTSTP,	"SIGTSTP" },	/* terminal-stop */
-#endif /* SIGTSTP */
-#ifdef SIGHUP
+#	endif /* SIGTSTP */
+#	ifdef SIGHUP
 	{ SIGHUP,	"SIGHUP" },	/* hang up */
-#endif /* SIGHUP */
-#ifdef SIGUSR1
+#	endif /* SIGHUP */
+#	ifdef SIGUSR1
 	{ SIGUSR1,	"SIGUSR1" },	/* User-defined signal 1 */
-#endif /* SIGUSR1 */
-#ifdef SIGTERM
+#	endif /* SIGUSR1 */
+#	ifdef SIGTERM
 	{ SIGTERM,	"SIGTERM" },	/* termination */
-#endif /* SIGTERM */
-#ifdef SIGWINCH
+#	endif /* SIGTERM */
+#	ifdef SIGWINCH
 	{ SIGWINCH,	"SIGWINCH" },	/* window-size change */
-#endif /* SIGWINCH */
+#	endif /* SIGWINCH */
 };
 #endif /* !__LCLINT__ */
 
 
 #ifdef HAVE_NESTED_PARAMS
-RETSIGTYPE (*sigdisp(int signum, RETSIGTYPE (_CDECL *func)(SIG_ARGS)))(SIG_ARGS)
+	RETSIGTYPE (*sigdisp(int signum, RETSIGTYPE (_CDECL *func)(SIG_ARGS)))(SIG_ARGS)
 #else
-RETSIGTYPE (*sigdisp(signum, func))(SIG_ARGS)
+	RETSIGTYPE (*sigdisp(signum, func))(SIG_ARGS)
 	int signum;
 	RETSIGTYPE (_CDECL *func)(SIG_ARGS);
 #endif /* HAVE_NESTED_PARAMS */
@@ -184,33 +183,33 @@ RETSIGTYPE (*sigdisp(signum, func))(SIG_ARGS)
 	struct sigaction sa, osa;
 
 	sa.sa_handler = func;
-	sigemptyset (&sa.sa_mask);
+	sigemptyset(&sa.sa_mask);
 	sa.sa_flags = 0;
 #	ifdef SA_RESTART
 		sa.sa_flags |= SA_RESTART;
 #	endif /* SA_RESTART */
-	if (sigaction (signum, &sa, &osa) < 0)
+	if (sigaction(signum, &sa, &osa) < 0)
 		return SIG_ERR;
-
 	return (osa.sa_handler);
 #else
-#	define RESTORE_HANDLER(x, y)  signal (x, y)
-	return (signal (signum, func));
+#	define RESTORE_HANDLER(x, y)	signal(x, y)
+	return (signal(signum, func));
 #endif /* HAVE_POSIX_JC */
 }
 
+
 /*
  * Block/unblock SIGWINCH/SIGTSTP restarting syscalls
  */
 void
-allow_resize (
+allow_resize(
 	t_bool allow)
 {
 #ifdef HAVE_POSIX_JC
 	struct sigaction sa, osa;
 
 	sa.sa_handler = signal_handler;
-	sigemptyset (&sa.sa_mask);
+	sigemptyset(&sa.sa_mask);
 	sa.sa_flags = 0;
 #	ifdef SA_RESTART
 	if (!allow)
@@ -225,13 +224,15 @@ allow_resize (
 #endif /* HAVE_POSIX_JC */
 }
 
+
 static const char *
-signal_name (
+signal_name(
 	int code)
 {
 	size_t n;
 	const char *name = "unknown";
-	for (n = 0; n < SIZEOF(signal_list); n++) {
+
+	for (n = 0; n < ARRAY_SIZE(signal_list); n++) {
 		if (signal_list[n].code == code) {
 			name = signal_list[n].name;
 			break;
@@ -240,24 +241,25 @@ signal_name (
 	return name;
 }
 
+
 /*
  * Rescale the display buffer and redraw the contents according to
  * the current context
  * This should NOT be called from an interrupt context
  */
 void
-handle_resize (
+handle_resize(
 	t_bool repaint)
 {
 #if defined(SIGWINCH) || defined(SIGTSTP)
 #	ifdef SIGWINCH
-	repaint |= set_win_size (&cLINES, &cCOLS);
+	repaint |= set_win_size(&cLINES, &cCOLS);
 #	endif /* SIGWINCH */
 
 	if (cLINES < MIN_LINES_ON_TERMINAL || cCOLS < MIN_COLUMNS_ON_TERMINAL) {
-		ring_bell ();
+		ring_bell();
 		wait_message(3, _(txt_screen_too_small_exiting), tin_progname);
-		tin_done (EXIT_FAILURE);
+		tin_done(EXIT_FAILURE);
 	}
 
 	TRACE(("handle_resize(%d:%d)", signal_context, repaint));
@@ -276,27 +278,27 @@ handle_resize (
 
 	switch (signal_context) {
 		case cArt:
-			ClearScreen ();
-			show_art_msg (glob_group);
+			ClearScreen();
+			show_art_msg(glob_group);
 			break;
 		case cConfig:
-			refresh_config_page (-1);
+			refresh_config_page(-1);
 			break;
 		case cFilter:
-			refresh_filter_menu ();
+			refresh_filter_menu();
 			break;
 		case cInfopager:
-			display_info_page (0);
+			display_info_page(0);
 			break;
 		case cGroup:
 		case cSelect:
 		case cThread:
-			ClearScreen ();
+			ClearScreen();
 			currmenu->redraw();
 			break;
 		case cPage:
-			resize_article (TRUE, &pgart);
-			draw_page (glob_group, 0);
+			resize_article(TRUE, &pgart);
+			draw_page(glob_group, 0);
 			break;
 		case cMain:
 			break;
@@ -305,36 +307,38 @@ handle_resize (
 #endif /* SIGWINCH || SIGTSTP */
 }
 
+
 #ifdef SIGTSTP
 static void
-handle_suspend (
+handle_suspend(
 	void)
 {
 	TRACE(("handle_suspend(%d)", signal_context));
 
-	set_keypad_off ();
+	set_keypad_off();
 	if (!cmd_line)
-		set_xclick_off ();
+		set_xclick_off();
 
-	Raw (FALSE);
-	wait_message (0, _(txt_suspended_message), tin_progname);
+	Raw(FALSE);
+	wait_message(0, _(txt_suspended_message), tin_progname);
 
-	kill (0, SIGSTOP);				/* Put ourselves to sleep */
+	kill(0, SIGSTOP);				/* Put ourselves to sleep */
 
-	RESTORE_HANDLER (SIGTSTP, signal_handler);
+	RESTORE_HANDLER(SIGTSTP, signal_handler);
 
 	if (!batch_mode) {
-		Raw (TRUE);
+		Raw(TRUE);
 		need_resize = cRedraw;		/* Flag a redraw */
 	}
-	set_keypad_on ();
+	set_keypad_on();
 	if (!cmd_line)
-		set_xclick_on ();
+		set_xclick_on();
 }
 #endif /* SIGTSTP */
 
+
 static void _CDECL
-signal_handler (
+signal_handler(
 	int sig)
 {
 #ifdef SIGCHLD
@@ -351,7 +355,7 @@ signal_handler (
 		case SIGINT:
 #	if !defined(M_AMIGA) && !defined(__SASC)
 			if (!batch_mode) {
-				RESTORE_HANDLER (sig, signal_handler);
+				RESTORE_HANDLER(sig, signal_handler);
 				return;
 			}
 #	endif /* !M_AMIGA && !__SASC */
@@ -360,8 +364,8 @@ signal_handler (
 
 #ifdef SIGCHLD
 		case SIGCHLD:
-			wait (&wait_status);
-			RESTORE_HANDLER (sig, signal_handler);	/* death of a child */
+			wait(&wait_status);
+			RESTORE_HANDLER(sig, signal_handler);	/* death of a child */
 			system_status = WEXITSTATUS(wait_status);
 			return;
 #endif /* SIGCHLD */
@@ -382,55 +386,47 @@ signal_handler (
 #ifdef SIGWINCH
 		case SIGWINCH:
 			need_resize = cYes;
-			RESTORE_HANDLER (SIGWINCH, signal_handler);
+			RESTORE_HANDLER(SIGWINCH, signal_handler);
 			return;
 #endif /* SIGWINCH */
 
 		default:
 			break;
 	}
-	fprintf (stderr, "\n%s: signal handler caught %s signal (%d).\n", tin_progname, signal_name(sig), sig);
 
-#if defined(SIGHUP)
-	if (sig == SIGHUP) {
-		dangerous_signal_exit = TRUE;
-		tin_done (- SIGHUP);
-	}
+	fprintf(stderr, "\n%s: signal handler caught %s signal (%d).\n", tin_progname, signal_name(sig), sig);
+
+	switch (sig) {
+#ifdef SIGHUP
+		case SIGHUP:
 #endif /* SIGHUP */
-
-#if defined(SIGUSR1)
-	if (sig == SIGUSR1) {
-		dangerous_signal_exit = TRUE;
-		tin_done (- SIGUSR1);
-	}
+#ifdef SIGUSR1
+		case SIGUSR1:
 #endif /* SIGUSR1 */
-
-#if defined(SIGTERM)
-	if (sig == SIGTERM) {
-		dangerous_signal_exit = TRUE;
-		tin_done (- SIGTERM);
-	}
+#ifdef SIGTERM
+		case SIGTERM:
 #endif /* SIGTERM */
+			dangerous_signal_exit = TRUE;
+			tin_done(-sig);
+			/* NOTREACHED */
+			break;
 
-#if defined(SIGBUS) || defined(SIGSEGV)
-	if (
-#	ifdef SIGBUS
-		sig == SIGBUS
-#	endif /* SIGBUS */
-#	if defined(SIGBUS) && defined(SIGSEGV)
-		||
-#	endif /* SIGBUS && SIGSEGV */
-#	ifdef SIGSEGV
-		sig == SIGSEGV
-#	endif /* SIGSEGV */
-		) {
-		/* FIXME: -> lang.c */
-		my_fprintf (stderr, _("%s %s %s (\"%s\") [%s]: send a DETAILED bug report to %s\n"),
-		tin_progname, VERSION, RELEASEDATE, RELEASENAME, OSNAME, bug_addr);
-		my_fflush (stderr);
+#ifdef SIGBUS
+		case SIGBUS:
+#endif /*S IGBUS */
+#ifdef SIGSEGV
+		case SIGSEGV:
+#endif /* SIGSEGV */
+			/* FIXME: -> lang.c */
+			my_fprintf(stderr, _("%s %s %s (\"%s\") [%s]: send a DETAILED bug report to %s\n"), tin_progname, VERSION, RELEASEDATE, RELEASENAME, OSNAME, bug_addr);
+			my_fflush(stderr);
+			break;
+
+		default:
+			break;
 	}
-#endif /* SIGBUS || SIGSEGV */
-	cleanup_tmp_files ();
+
+	cleanup_tmp_files();
 
 #if 1
 /* #if defined(apollo) || defined(HAVE_COREFILE) */
@@ -447,22 +443,22 @@ signal_handler (
  * Otherwise revert to the default handler
  */
 void
-set_signal_catcher (
+set_signal_catcher(
 	int flag)
 {
 #ifdef SIGTSTP
 	if (do_sigtstp)
-		sigdisp (SIGTSTP, flag ? signal_handler : SIG_DFL);
+		sigdisp(SIGTSTP, flag ? signal_handler : SIG_DFL);
 #endif /* SIGTSTP */
 
 #ifdef SIGWINCH
-	sigdisp (SIGWINCH, flag ? signal_handler : SIG_DFL);
+	sigdisp(SIGWINCH, flag ? signal_handler : SIG_DFL);
 #endif /* SIGWINCH */
 }
 
 
 void
-set_signal_handlers (
+set_signal_handlers(
 	void)
 {
 	size_t n;
@@ -471,12 +467,12 @@ set_signal_handlers (
 	RETSIGTYPE (*ptr)(SIG_ARGS);
 #endif /* SIGTSTP */
 
-	for (n = 0; n < SIZEOF(signal_list); n++) {
+	for (n = 0; n < ARRAY_SIZE(signal_list); n++) {
 		switch ((code = signal_list[n].code)) {
 #ifdef SIGTSTP
 		case SIGTSTP:
-			ptr = sigdisp (code, SIG_DFL);
-			sigdisp (code, ptr);
+			ptr = sigdisp(code, SIG_DFL);
+			sigdisp(code, ptr);
 			if (ptr == SIG_IGN)
 				break;
 			/*
@@ -484,17 +480,21 @@ set_signal_handlers (
 			 * without job-control
 			 */
 			do_sigtstp = TRUE;
-			nobreak; /* FALLTHROUGH */
+			/* FALLTHROUGH */
 #endif /* SIGTSTP */
+
 		default:
-			sigdisp (code, signal_handler);
+			sigdisp(code, signal_handler);
 		}
 	}
 }
 
 
+/*
+ * Size the display at startup or rescale following a SIGWINCH etc.
+ */
 t_bool
-set_win_size (
+set_win_size(
 	int *num_lines,
 	int *num_cols)
 {
@@ -514,51 +514,52 @@ set_win_size (
 #ifdef HAVE_XCURSES
 	*num_lines = LINES - 1;		/* FIXME */
 	*num_cols = COLS;
-#else	/* curses/ncurses */
+#else /* curses/ncurses */
 
-#ifndef USE_CURSES
-	init_screen_array (FALSE);		/* deallocate screen array */
-#endif /* !USE_CURSES */
+#	ifndef USE_CURSES
+	init_screen_array(FALSE);		/* deallocate screen array */
+#	endif /* !USE_CURSES */
 
-#ifdef TIOCGSIZE
-	if (ioctl (0, TIOCGSIZE, &win) == 0) {
+#	ifdef TIOCGSIZE
+	if (ioctl(0, TIOCGSIZE, &win) == 0) {
 		if (win.ts_lines != 0)
 			*num_lines = win.ts_lines - 1;
 		if (win.ts_cols != 0)
 			*num_cols = win.ts_cols;
 	}
-#else
-#	ifdef TIOCGWINSZ
-	if (ioctl (0, TIOCGWINSZ, &win) == 0) {
+#	else
+#		ifdef TIOCGWINSZ
+	if (ioctl(0, TIOCGWINSZ, &win) == 0) {
 		if (win.ws_row != 0)
 			*num_lines = win.ws_row - 1;
 		if (win.ws_col != 0)
 			*num_cols = win.ws_col;
 	}
-#	else
-#		ifdef M_AMIGA
+#		else
+#			ifdef M_AMIGA
 	AmiGetWinSize(num_lines, num_cols);
 	(*num_lines)--;
-#		endif /* M_AMIGA */
-#	endif /* TIOCGWINSZ */
-#endif /* TIOCGSIZE */
+#			endif /* M_AMIGA */
+#		endif /* TIOCGWINSZ */
+#	endif /* TIOCGSIZE */
 
-#ifndef USE_CURSES
-	init_screen_array (TRUE);		/* allocate screen array for resize */
-#endif /* !USE_CURSES */
+#	ifndef USE_CURSES
+	init_screen_array(TRUE);		/* allocate screen array for resize */
+#	endif /* !USE_CURSES */
 
 #endif /* HAVE_XCURSES */
 
-	set_subj_from_size (*num_cols);
+	set_subj_from_size(*num_cols);
 
 	RIGHT_POS = *num_cols - 20;
-	MORE_POS  = *num_cols - 15;
-	set_noteslines (*num_lines);
+	MORE_POS = *num_cols - 15;
+	set_noteslines(*num_lines);
 	return (*num_lines != old_lines || *num_cols != old_cols);
 }
 
+
 void
-set_noteslines (
+set_noteslines(
 	int num_lines)
 {
 	NOTESLINES = num_lines - INDEX_TOP - (tinrc.beginner_level ? MINI_HELP_LINES : 1);
