@@ -3,10 +3,10 @@
  *  Module    : string.c
  *  Author    : Urs Janssen <urs@tin.org>
  *  Created   : 1997-01-20
- *  Updated   : 2001-04-24
+ *  Updated   : 2003-01-06
  *  Notes     :
  *
- * Copyright (c) 1997-2002 Urs Janssen <urs@tin.org>
+ * Copyright (c) 1997-2003 Urs Janssen <urs@tin.org>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -50,7 +50,7 @@
  * Kilo, Mega, Giga, Terra
  */
 char *
-tin_ltoa (
+tin_ltoa(
 	long value,
 	int digits)
 {
@@ -59,8 +59,8 @@ tin_ltoa (
 	int len;
 	int i = 0;
 
-	sprintf (buffer, "%ld", value);
-	len = (int) strlen (buffer);
+	sprintf(buffer, "%ld", value);
+	len = (int) strlen(buffer);
 
 	while (len > digits) {
 		len -= 3;
@@ -74,9 +74,9 @@ tin_ltoa (
 		buffer[digits - 1] = power[i];
 		buffer[digits] = '\0';
 	} else
-		sprintf (buffer, "%*ld", digits, value);
+		sprintf(buffer, "%*ld", digits, value);
 
-	return (buffer);
+	return buffer;
 }
 
 
@@ -86,25 +86,26 @@ tin_ltoa (
  *
  * also, strdup is not mandatory in ANSI-C
  */
-char *my_strdup (
+char *
+my_strdup(
 	const char *str)
 {
-	size_t len = strlen (str) + 1;
-	void *ptr = my_malloc (len);
+	size_t len = strlen(str) + 1;
+	void *ptr = my_malloc(len);
 
 	if (ptr == NULL)
 		return NULL;
 
-	memcpy (ptr, str, len);
+	memcpy(ptr, str, len);
 	return (char *) ptr;
 }
 
 
 /*
- *  strncpy that stops at a newline and null terminates
+ * strncpy that stops at a newline and null terminates
  */
 void
-my_strncpy (
+my_strncpy(
 	char *p,
 	const char *q,
 	size_t n)
@@ -118,11 +119,12 @@ my_strncpy (
 }
 
 
+#ifndef HAVE_STRCASESTR
 /*
  * case-insensitive version of strstr()
  */
 const char *
-strcasestr (
+strcasestr(
 	const char *haystack,
 	const char *needle)
 {
@@ -132,7 +134,7 @@ strcasestr (
 	h = haystack;
 	n = needle;
 	while (*haystack) {
-		if (tolower ((unsigned char)*h) == tolower ((unsigned char)*n)) {
+		if (tolower((unsigned char) *h) == tolower((unsigned char) *n)) {
 			h++;
 			n++;
 			if (!*n)
@@ -144,10 +146,11 @@ strcasestr (
 	}
 	return NULL;
 }
+#endif /* !HAVE_STRCASESTR */
 
 
 size_t
-mystrcat (
+mystrcat(
 	char **t,
 	const char *s)
 {
@@ -163,13 +166,13 @@ mystrcat (
 
 
 void
-str_lwr (
+str_lwr(
 	char *str)
 {
 	char *dst = str;
 
 	while (*str) {
-		*dst++ = (char)tolower((unsigned char)*str);
+		*dst++ = (char) tolower((unsigned char) *str);
 		str++;
 	}
 	*dst = '\0';
@@ -185,7 +188,7 @@ str_lwr (
  * find first occurrence of any char from str2 in str1
  */
 char *
-strpbrk (
+strpbrk(
 	char *str1,
 	char *str2)
 {
@@ -195,7 +198,7 @@ strpbrk (
 	for (ptr1 = str1; *ptr1 != '\0'; ptr1++) {
 		for (ptr2 = str2; *ptr2 != '\0'; ) {
 			if (*ptr1 == *ptr2++)
-				return (ptr1);
+				return ptr1;
 		}
 	}
 	return (char *) 0;
@@ -204,10 +207,10 @@ strpbrk (
 
 #ifndef HAVE_STRSTR
 /*
- * ANSI C strstr () - Uses Boyer-Moore algorithm.
+ * ANSI C strstr() - Uses Boyer-Moore algorithm.
  */
 char *
-strstr (
+strstr(
 	char *text,
 	char *pattern)
 {
@@ -218,16 +221,16 @@ strstr (
 	size_t textlen;
 	size_t patlen;
 
-	textlen = strlen (text);
-	patlen = strlen (pattern);
+	textlen = strlen(text);
+	patlen = strlen(pattern);
 
 	/* algorithm fails if pattern is empty */
 	if ((p1 = patlen) == 0)
-		return (text);
+		return text;
 
 	/* code below fails (whenever i is unsigned) if pattern too long */
 	if (p1 > textlen)
-		return (NULL);
+		return NULL;
 
 	/* set up deltas */
 	delta = deltaspace;
@@ -249,14 +252,14 @@ strstr (
 	i = textlen - patlen;
 	forever {
 		if (*p == *t && memcmp ((p - p1), (t - p1), p1) == 0)
-			return ((char *)t - p1);
+			return ((char *) t - p1);
 		j = delta[*t];
 		if (i < j)
 			break;
 		i -= j;
 		t += j;
 	}
-	return (NULL);
+	return NULL;
 }
 #endif /* !HAVE_STRSTR */
 
@@ -265,7 +268,7 @@ strstr (
  * handrolled atol
  */
 long
-atol (
+atol(
 	const char *s)
 {
 	long ret = 0;
@@ -287,10 +290,10 @@ atol (
 
 #ifndef HAVE_STRTOL
 /* fix me - put me in tin.h */
-#define DIGIT(x) (isdigit((unsigned char)x) ? ((x) - '0') : (10 + tolower((unsigned char)x) - 'a'))
-#define MBASE 36
+#	define DIGIT(x) (isdigit((unsigned char) x) ? ((x) - '0') : (10 + tolower((unsigned char) x) - 'a'))
+#	define MBASE 36
 long
-strtol (
+strtol(
 	const char *str,
 	char **ptr,
 	int use_base)
@@ -300,7 +303,7 @@ strtol (
 
 	if (use_base < 0 || use_base > MBASE)
 		goto OUT;
-	while (isspace ((unsigned char)*str))
+	while (isspace((unsigned char) *str))
 		++str;
 	if (*str == '-') {
 		++str;
@@ -324,32 +327,30 @@ strtol (
 	 * for any base > 10, the digits incrementally following
 	 * 9 are assumed to be "abc...z" or "ABC...Z"
 	 */
-	while (isalnum ((unsigned char)*str) && (xx = DIGIT(*str)) < use_base) {
+	while (isalnum((unsigned char) *str) && (xx = DIGIT(*str)) < use_base) {
 		/* accumulate neg avoids surprises near maxint */
 		val = use_base * val - xx;
 		++str;
 	}
 OUT:
-	if (ptr != (char **) 0)
+	if (ptr != NULL)
 		*ptr = str;
 
 	return (sign * (-val));
 }
-#undef DIGIT(x)
-#undef MBASE
+#	undef DIGIT(x)
+#	undef MBASE
 #endif /* !HAVE_STRTOL */
 
 
 /*
- *  strcmp that ignores case
+ * strcmp that ignores case
  */
-
-/*fix me - put me in tin.h */
-#define FOLD_TO_UPPER(a)	(toupper ((unsigned char)(a)))
-
 #ifndef HAVE_STRCASECMP
+/* fix me - put me in tin.h */
+#	define FOLD_TO_UPPER(a)	(toupper((unsigned char) (a)))
 int
-strcasecmp (
+strcasecmp(
 	const char *p,
 	const char *q)
 {
@@ -365,13 +366,13 @@ strcasecmp (
 
 #ifndef HAVE_STRNCASECMP
 int
-strncasecmp (
+strncasecmp(
 	const char *p,
 	const char *q,
 	size_t n)
 {
 	int r = 0;
-	for (; n && (r = (FOLD_TO_UPPER (*p) - FOLD_TO_UPPER (*q))) == 0; ++p, ++q, --n) {
+	for (; n && (r = (FOLD_TO_UPPER(*p) - FOLD_TO_UPPER(*q))) == 0; ++p, ++q, --n) {
 		if (*p == '\0')
 			return 0;
 	}
@@ -390,18 +391,18 @@ strncasecmp (
  * RETURN: trimmed string
  */
 char *
-str_trim (
+str_trim(
 	char *string)
 {
 	char *rp;		/* reading string pointer */
 	char *wp;		/* writing string pointer */
 	char *ls;		/* last space */
 
-	for (rp = wp = ls = string; isspace((int)*rp); rp++)		/* Skip leading space */
+	for (rp = wp = ls = string; isspace((int) *rp); rp++)		/* Skip leading space */
 		;
 
 	while (*rp) {
-		if (isspace((int)*rp)) {
+		if (isspace((int) *rp)) {
 			if (ls == NULL)		/* Remember last written space */
 				ls = wp;
 		} else
@@ -414,15 +415,15 @@ str_trim (
 	else
 		*wp = '\0';
 
-	return (string);
+	return string;
 }
 
 
 /*
- *  Return a pointer into s eliminating any TAB, CR and LF.
+ * Return a pointer into s eliminating any TAB, CR and LF.
  */
 char *
-eat_tab (
+eat_tab(
 	char *s)
 {
 	char *p1 = s;
@@ -447,7 +448,7 @@ eat_tab (
 
 /*
  * Format a shell command, escaping blanks and other awkward characters that
- * appear in the string arguments.  Replaces sprintf, except that we pass in
+ * appear in the string arguments. Replaces sprintf, except that we pass in
  * the buffer limit, so we can refrain from trashing memory on very long
  * pathnames.
  *
@@ -464,7 +465,7 @@ eat_tab (
 #define SH_META   "\\\'\"`$*%?()[]{}|<>^&;#~"
 
 int
-sh_format (
+sh_format(
 	char *dst,
 	size_t len,
 	const char *fmt,
@@ -497,13 +498,16 @@ sh_format (
 			case '%':
 				src = strcpy(temp, "%");
 				break;
+
 			case 's':
 				src = va_arg(ap, char *);
 				break;
+
 			case 'd':
 				sprintf(temp, "%d", va_arg(ap, int));
 				src = temp;
 				break;
+
 			default:
 				src = strcpy(temp, "");
 				break;
@@ -513,11 +517,9 @@ sh_format (
 				t_bool fix;
 
 				/*
-				 * This logic works for Unix.  Non-Unix systems
-				 * may require a different set of problem
-				 * chars, and may need quotes around the whole
-				 * string rather than escaping individual
-				 * chars.
+				 * This logic works for Unix. Non-Unix systems may require a
+				 * different set of problem chars, and may need quotes around
+				 * the whole string rather than escaping individual chars.
 				 */
 				if (quote == '"') {
 					fix = (strchr(SH_DOUBLE, *src) != 0);
@@ -548,9 +550,10 @@ sh_format (
 	else
 		*dst = '\0';
 
-	return (result);
+	return result;
 }
 
+
 #ifndef HAVE_STRERROR
 #	ifdef HAVE_SYS_ERRLIST
 #		ifdef M_AMIGA
@@ -565,17 +568,17 @@ sh_format (
 #		endif /* M_AMIGA */
 		extern int sys_nerr;
 #	endif /* HAVE_SYS_ERRLIST */
-
 char *
-my_strerror (
+my_strerror(
 	int n)
 {
 	static char temp[32];
+
 #	ifdef HAVE_SYS_ERRLIST
 	if (n >= 0 && n < sys_nerr)
 		return sys_errlist[n];
 #	endif /* HAVE_SYS_ERRLIST */
-	sprintf(temp, "Errno: %i", n);
+	sprintf(temp, "Errno: %d", n);
 	return temp;
 }
 #endif /* !HAVE_STRERROR */
@@ -584,26 +587,63 @@ my_strerror (
 /* strrstr() based on Lars Wirzenius' <lars.wirzenius@helsinki.fi> code */
 #ifndef HAVE_STRRSTR
 char *
-strrstr (
+strrstr(
 	const char *str,
 	const char *pat)
 {
-	size_t slen, plen;
 	const char *ptr;
+	size_t slen, plen;
 
-	if (str == (char *) 0 || pat == (char *) 0)
-		return (char *) 0;
+	if ((str != 0) && (pat != 0)) {
+		slen = strlen(str);
+		plen = strlen(pat);
 
-	slen = strlen(str);
-	plen = strlen(pat);
-
-	if (plen <= slen) {
-		for (ptr = str + (slen - plen); ptr > str; --ptr) {
-			if (*ptr == *pat && strncmp(ptr, pat, plen) == 0)
-				return (char *) ptr;
+		if ((plen != 0) && (plen <= slen)) {
+			for (ptr = str + (slen - plen); ptr > str; --ptr) {
+				if (*ptr == *pat && strncmp(ptr, pat, plen) == 0)
+					return (char *) ptr;
+			}
 		}
 	}
-
-	return (char *) 0;
+	return 0;
 }
 #endif /* HAVE_STRRSTR */
+
+
+#if defined(MULTIBYTE_ABLE) && !defined(NO_LOCALE)
+/*
+ * copy wide-chars from '*from' to '*to' until 'columns' columns are filled
+ * pad with spaces if necessary
+ */
+void
+wcspart(
+	wchar_t *to,
+	const wchar_t *from,
+	int columns,
+	int size_to)
+{
+	int n, i = 0;
+	wchar_t *ptr, *wbuf;
+
+	/* make sure all characters in from are printable */
+	wbuf = my_malloc(size_to * sizeof(wchar_t));
+	wcsncpy(wbuf, from, size_to);
+	wbuf[size_to - 1] = (wchar_t) '\0';
+	ptr = wconvert_to_printable(wbuf);
+
+	to[0] = (wint_t) '\0';
+	while (*ptr && i < size_to && wcswidth(to, size_to - 1) + wcwidth(*ptr) <= columns) {
+		to[i] = *ptr;
+		ptr++;
+		to[++i] = (wint_t) '\0';
+	}
+
+	/* pad with spaces */
+	n = columns - wcswidth(to, size_to - 1) + (int) wcslen(to);
+	for (; i < MIN(n, size_to - 1); i++)
+		to[i] = (wint_t) ' ';
+	to[i] = (wint_t) '\0';
+
+	free(wbuf);
+}
+#endif /* MULTIBYTE_ABLE && !NOLOCALE */
