@@ -3,10 +3,10 @@
  *  Module    : tags.c
  *  Author    : Jason Faultless <jason@radar.tele2.co.uk>
  *  Created   : 1999-12-06
- *  Updated   : 1999-12-06
+ *  Updated   : 2002-11-11
  *  Notes     : Split out from other modules
  *
- * Copyright (c) 1999-2002 Jason Faultless <jason@radar.tele2.co.uk>
+ * Copyright (c) 1999-2003 Jason Faultless <jason@radar.tele2.co.uk>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,11 +42,10 @@
 #endif /* !TIN_H */
 
 /* Local prototypes */
-static int get_multipart_info (int base_index, MultiPartInfo *setme);
-static int get_multiparts (int base_index, MultiPartInfo **malloc_and_setme_info);
-static int look_for_multipart_info (int base_index, MultiPartInfo* setme, char start, char stop, int *offset);
-static t_bool bParseRange (char *pcRange, int iNumMin, int iNumMax, int iNumCur, int *piRngMin, int *piRngMax);
-static void vDelRange (int iLevel, int iNumMax);
+static int get_multipart_info(int base_index, MultiPartInfo *setme);
+static int get_multiparts(int base_index, MultiPartInfo **malloc_and_setme_info);
+static int look_for_multipart_info(int base_index, MultiPartInfo *setme, char start, char stop, int *offset);
+static t_bool parse_range(char *range, int min, int max, int curr, int *range_start, int *range_end);
 
 int num_of_tagged_arts = 0;
 
@@ -58,7 +57,7 @@ int num_of_tagged_arts = 0;
  * @return nonzero on success
  */
 static int
-get_multipart_info (
+get_multipart_info(
 	int base_index,
 	MultiPartInfo *setme)
 {
@@ -81,7 +80,7 @@ get_multipart_info (
 
 
 static int
-look_for_multipart_info (
+look_for_multipart_info(
 	int base_index,
 	MultiPartInfo* setme,
 	char start,
@@ -95,29 +94,28 @@ look_for_multipart_info (
 	*offset = 0;
 
 	/* entry assertions */
-	assert (0 <= base_index && base_index < grpmenu.max && "invalid base_index");
-	assert (setme != NULL && "setme must not be NULL");
+	assert(0 <= base_index && base_index < grpmenu.max && "invalid base_index");
+	assert(setme != NULL && "setme must not be NULL");
 
 	/* parse the message */
 	subj = arts[base[base_index]].subject;
-	pch = strrchr (subj, start);
-	if (!pch)
+	if (!(pch = strrchr(subj, start)))
 		return 0;
-	if (!isdigit((int)pch[1]))
+	if (!isdigit((int) pch[1]))
 		return 0;
 	tmp.base_index = base_index;
 	tmp.subject_compare_len = pch - subj;
 	tmp.part_number = (int) strtol(pch + 1, &pch, 10);
 	if (*pch != '/' && *pch != '|')
 		return 0;
-	if (!isdigit((int)pch[1]))
+	if (!isdigit((int) pch[1]))
 		return 0;
-	tmp.total = (int) strtol (pch + 1, &pch, 10);
+	tmp.total = (int) strtol(pch + 1, &pch, 10);
 	if (*pch != stop)
 		return 0;
 	tmp.subject = subj;
 	*setme = tmp;
-	*offset = pch-subj;
+	*offset = pch - subj;
 	return 1;
 }
 
@@ -128,32 +126,33 @@ look_for_multipart_info (
  *
  * Weakness(?): only walks through the base messages.
  *
- * @return on success, the number of parts found.  On failure, zero if not a
+ * @return on success, the number of parts found. On failure, zero if not a
  * multipart or the negative value of the first missing part.
  * @param base_index index pointing to one of the messages in a multipart
  * message.
  * @param malloc_and_setme_info on success, set to a malloced array the
- * parts found.  Untouched on failure.
+ * parts found. Untouched on failure.
  */
 static int
-get_multiparts (
+get_multiparts(
 	int base_index,
 	MultiPartInfo **malloc_and_setme_info)
 {
 	MultiPartInfo tmp, tmp2;
 	MultiPartInfo *info = 0;
 	int i = 0;
+	int part_index;
 
 	/* entry assertions */
-	assert (0<=base_index && base_index<grpmenu.max && "Invalid base index");
-	assert (malloc_and_setme_info!=NULL && "malloc_and_setme_info must not be NULL");
+	assert(0 <= base_index && base_index < grpmenu.max && "Invalid base index");
+	assert(malloc_and_setme_info != NULL && "malloc_and_setme_info must not be NULL");
 
 	/* make sure this is a multipart message... */
 	if (!get_multipart_info(base_index, &tmp) || tmp.total < 1)
 		return 0;
 
 	/* make a temporary buffer to hold the multipart info... */
-	info = my_malloc (sizeof(MultiPartInfo) * tmp.total);
+	info = my_malloc(sizeof(MultiPartInfo) * tmp.total);
 
 	/* zero out part-number for the repost check below */
 	for (i = 0; i < tmp.total; ++i)
@@ -161,11 +160,9 @@ get_multiparts (
 
 	/* try to find all the multiparts... */
 	for (i = 0; i < grpmenu.max; ++i) {
-		int part_index = 0;
-
-		if (strncmp (arts[base[i]].subject, tmp.subject, tmp.subject_compare_len))
+		if (strncmp(arts[base[i]].subject, tmp.subject, tmp.subject_compare_len))
 			continue;
-		if (!get_multipart_info (i, &tmp2))
+		if (!get_multipart_info(i, &tmp2))
 			continue;
 
 		part_index = tmp2.part_number - 1;
@@ -180,7 +177,7 @@ get_multiparts (
 
 		/* repost check: do we already have this part? */
 		if (info[part_index].part_number != -1) {
-			assert (info[part_index].part_number == tmp2.part_number && "bookkeeping error");
+			assert(info[part_index].part_number == tmp2.part_number && "bookkeeping error");
 			continue;
 		}
 
@@ -191,7 +188,7 @@ get_multiparts (
 	/* see if we got them all. */
 	for (i = 0; i < tmp.total; ++i) {
 		if (info[i].part_number != i + 1) {
-			free (info);
+			free(info);
 			return -(i + 1); /* missing part #(i+1) */
 		}
 	}
@@ -210,7 +207,7 @@ get_multiparts (
  * @return number of messages tagged, or zero on failure
  */
 int
-tag_multipart (
+tag_multipart(
 	int base_index)
 {
 	MultiPartInfo *info = 0;
@@ -219,11 +216,11 @@ tag_multipart (
 
 	/* check for failure... */
 	if (qty == 0) {
-		info_message (_("Not a multi-part message")); /* FIXME: -> lang.c */
+		info_message(_("Not a multi-part message")); /* FIXME: -> lang.c */
 		return 0;
 	}
 	if (qty < 0) {
-		info_message(_("Missing part #%d"), -qty); /* FIXME: -> lang.c? */
+		info_message(_("Missing part #%d"), -qty); /* FIXME: -> lang.c */
 		return 0;
 	}
 
@@ -232,29 +229,30 @@ tag_multipart (
 	 * so num_of_tagged_arts doesn't get corrupted
 	 */
 	for (i = 0; i < qty; ++i) {
-		long art = base[info[i].base_index];
-		if (arts[art].tagged != 0)
-			remove_tag(art);
+		if (arts[base[info[i].base_index]].tagged != 0)
+			remove_tag(base[info[i].base_index]);
 	}
 
 	/*
 	 * get_multiparts() sorts info by part number,
 	 * so a simple for loop tags in the right order
 	 */
-	for (i = 0; i < qty; ++i) {
-		const int my_base_index = info[i].base_index;
-		arts[base[my_base_index]].tagged = ++num_of_tagged_arts;
-	}
+	for (i = 0; i < qty; ++i)
+		arts[base[info[i].base_index]].tagged = ++num_of_tagged_arts;
 
-	free (info);
+	free(info);
 
-	info_message (_("All parts tagged")); /* FIXME -> lang.c */
+	info_message(_("All parts tagged")); /* FIXME -> lang.c */
 	return qty;
 }
 
 
+/*
+ * Return the highest tag number of any article in thread
+ * rooted at base[n]
+ */
 int
-line_is_tagged (
+line_is_tagged(
 	int n)
 {
 	int code = 0;
@@ -277,16 +275,16 @@ line_is_tagged (
  * FALSE if we untagged it.
  */
 t_bool
-tag_article (
+tag_article(
 	int art)
 {
 	if (arts[art].tagged != 0) {
 		remove_tag(art);
-		info_message (_(txt_untagged_art));
+		info_message(_(txt_untagged_art));
 		return FALSE;
 	} else {
 		arts[art].tagged = ++num_of_tagged_arts;
-		info_message (_(txt_tagged_art));
+		info_message(_(txt_tagged_art));
 		return TRUE;
 	}
 }
@@ -298,15 +296,16 @@ tag_article (
  * greater than 'tag', fixup counters
  */
 void
-remove_tag (
+remove_tag(
 	long art)
 {
 	int i, j;
 
 	for (i = 0; i < grpmenu.max; ++i) {
-		for (j = (int) base[i]; j != -1; j = arts[j].thread)
+		for_each_art_in_thread(j, i) {
 			if (arts[j].tagged > arts[art].tagged)
 				--arts[j].tagged;
+		}
 	}
 	arts[art].tagged = 0;
 	--num_of_tagged_arts;
@@ -317,13 +316,13 @@ remove_tag (
  * Clear tag status of all articles. If articles were untagged, return TRUE
  */
 t_bool
-untag_all_articles (
+untag_all_articles(
 	void)
 {
 	int i;
 	t_bool untagged = FALSE;
 
-	for (i = 0; i < top_art; i++) {
+	for_each_art(i) {
 		if (arts[i].tagged != 0) {
 			arts[i].tagged = 0;
 			untagged = TRUE;
@@ -341,6 +340,9 @@ untag_all_articles (
 /*
  * Allows user to specify an group/article range that a followup
  * command will operate on (eg. catchup articles 1-56) # 1-56 K
+ * min/max/curr are the lowest/highest and current positions on the
+ * menu from which this was called; used as defaults if needed
+ * Return TRUE if a range was successfully read, parsed and set
  *
  * Allowed syntax is 0123456789-.$ (blanks are ignored):
  *   1-23    mark grp/art 1 thru 23
@@ -349,167 +351,160 @@ untag_all_articles (
  *   .-$     mark grp/art current thru last
  */
 t_bool
-bSetRange (
-	int iLevel,
-	int iNumMin,
-	int iNumMax,
-	int iNumCur)
+set_range(
+	int level,
+	int min,
+	int max,
+	int curr)
 {
-	char *pcPtr;
-	int iIndex;
-	int iNum;
-	int iRngMin;
-	int iRngMax;
-	t_bool bRetCode = FALSE;
+	char *range;
+	int artnum;
+	int i;
+	int depth;
+	int range_min;
+	int range_max;
 
-	switch (iLevel) {
+	switch (level) {
 		case SELECT_LEVEL:
-			pcPtr = tinrc.default_range_select;
+			range = tinrc.default_range_select;
 			break;
+
 		case GROUP_LEVEL:
-			pcPtr = tinrc.default_range_group;
+			range = tinrc.default_range_group;
 			break;
+
 		case THREAD_LEVEL:
-			pcPtr = tinrc.default_range_thread;
+			range = tinrc.default_range_thread;
 			break;
+
 		default:
-			return bRetCode;
+			return FALSE;
 	}
 
 #if 0
-	error_message ("Min=[%d] Max=[%d] Cur=[%d] DefRng=[%s]", iNumMin, iNumMax, iNumCur, pcPtr);
+	error_message("Min=[%d] Max=[%d] Cur=[%d] DefRng=[%s]", min, max, curr, range);
 #endif /* 0 */
-	sprintf (mesg, _(txt_enter_range), pcPtr);
+	sprintf(mesg, _(txt_enter_range), range);
 
-	if (!(prompt_string_default(mesg, pcPtr, _(txt_range_invalid), HIST_OTHER)))
-		return bRetCode;
+	if (!(prompt_string_default(mesg, range, _(txt_range_invalid), HIST_OTHER)))
+		return FALSE;
 
 	/*
 	 * Parse range string
 	 */
-	if (!bParseRange (pcPtr, iNumMin, iNumMax, iNumCur, &iRngMin, &iRngMax))
-		info_message (_(txt_range_invalid));
-	else {
-		bRetCode = TRUE;
-		switch (iLevel) {
-			case SELECT_LEVEL:
-				vDelRange (iLevel, iNumMax);
-				for (iIndex = iRngMin - 1; iIndex < iRngMax; iIndex++)
-					active[my_group[iIndex]].inrange = TRUE;
-				break;
-			case GROUP_LEVEL:
-				vDelRange (iLevel, iNumMax);
-				for (iIndex = iRngMin - 1; iIndex < iRngMax; iIndex++) {
-					for (iNum = (int) base[iIndex]; iNum != -1; iNum = arts[iNum].thread)
-						arts[iNum].inrange = TRUE;
-				}
-				break;
-			case THREAD_LEVEL:
-				vDelRange (iLevel, selmenu.max);
-				for (iNum = 0, iIndex = base[thread_basenote]; iIndex >= 0; iIndex = arts[iIndex].thread, iNum++) {
-					if (iNum >= iRngMin && iNum <= iRngMax)
-						arts[iIndex].inrange = TRUE;
-				}
-				break;
-			default:
-				bRetCode = FALSE;
-				break;
-		}
+	if (!parse_range(range, min, max, curr, &range_min, &range_max)) {
+		info_message(_(txt_range_invalid));
+		return FALSE;
 	}
-	return bRetCode;
+
+	switch (level) {
+		case SELECT_LEVEL:
+			for (i = 0; i < max; i++)			/* Clear existing range */
+				active[my_group[i]].inrange = FALSE;
+
+			for (i = range_min - 1; i < range_max; i++)
+				active[my_group[i]].inrange = TRUE;
+			break;
+
+		case GROUP_LEVEL:
+			for (i = 0; i < max; i++) {			/* Clear existing range */
+				for_each_art_in_thread(artnum, i)
+					arts[artnum].inrange = FALSE;
+			}
+
+			for (i = range_min - 1; i < range_max; i++) {
+				for_each_art_in_thread(artnum, i)
+					arts[artnum].inrange = TRUE;
+			}
+			break;
+
+		case THREAD_LEVEL:
+			/*
+			 * Debatably should clear all of arts[] depending on how you
+			 * interpret the (non)spec
+			 */
+			for (i = 0; i < grpmenu.max; i++) {			/* Clear existing range */
+				for_each_art_in_thread(artnum, i)
+					arts[artnum].inrange = FALSE;
+			}
+
+			depth = 1;
+			for_each_art_in_thread(artnum, thread_basenote) {
+				if (depth > range_max)
+					break;
+				if (depth >= range_min)
+					arts[artnum].inrange = TRUE;
+				depth++;
+			}
+			break;
+
+		default:
+			return FALSE;
+			break;
+	}
+	return TRUE;
 }
 
 
+/*
+ * Parse 'range', return the range start and end values in range_start and range_end
+ * min/max/curr are used to select defaults when n explicit start/end are given
+ */
 static t_bool
-bParseRange (
-	char *pcRange,
-	int iNumMin,
-	int iNumMax,
-	int iNumCur,
-	int *piRngMin,
-	int *piRngMax)
+parse_range(
+	char *range,
+	int min,
+	int max,
+	int curr,
+	int *range_start,
+	int *range_end)
 {
-	char *pcPtr;
-	t_bool bRetCode = FALSE;
-	t_bool bSetMax = FALSE;
-	t_bool bDone = FALSE;
+	char *ptr = range;
+	enum states { FINDMIN, FINDMAX, DONE };
+	int state = FINDMIN;
+	t_bool ret = FALSE;
 
-	pcPtr = pcRange;
-	*piRngMin = -1;
-	*piRngMax = -1;
+	*range_start = -1;
+	*range_end = -1;
 
-	while (*pcPtr && !bDone) {
-		if (*pcPtr >= '0' && *pcPtr <= '9') {
-			if (bSetMax) {
-				*piRngMax = atoi (pcPtr);
-				bDone = TRUE;
+	while (*ptr && state != DONE) {
+		if (isdigit(*ptr)) {
+			if (state == FINDMAX) {
+				*range_end = atoi (ptr);
+				state = DONE;
 			} else
-				*piRngMin = atoi (pcPtr);
-			while (*pcPtr >= '0' && *pcPtr <= '9')
-				pcPtr++;
+				*range_start = atoi (ptr);
+			while (isdigit(*ptr))
+				ptr++;
 		} else {
-			switch (*pcPtr) {
+			switch (*ptr) {
 				case '-':
-					bSetMax = TRUE;
+					state = FINDMAX;
 					break;
 				case '.':
-					if (bSetMax) {
-						*piRngMax = iNumCur;
-						bDone = TRUE;
+					if (state == FINDMAX) {
+						*range_end = curr;
+						state = DONE;
 					} else
-						*piRngMin = iNumCur;
+						*range_start = curr;
 					break;
 				case '$':
-					if (bSetMax) {
-						*piRngMax = iNumMax;
-						bDone = TRUE;
+					if (state == FINDMAX) {
+						*range_end = max;
+						state = DONE;
 					}
 					break;
 				default:
 					break;
 			}
-			pcPtr++;
+			ptr++;
 		}
 	}
 
-	if (*piRngMin >= iNumMin && *piRngMax > iNumMin && *piRngMax <= iNumMax)
-		bRetCode = TRUE;
+	if (*range_start >= min && *range_end >= *range_start && *range_end <= max)
+		ret = TRUE;
 
-	return bRetCode;
-}
-
-
-static void
-vDelRange (
-	int iLevel,
-	int iNumMax)
-{
-	int iIndex;
-
-	switch (iLevel) {
-		case SELECT_LEVEL:
-			for (iIndex = 0; iIndex < iNumMax - 1; iIndex++)
-				active[iIndex].inrange = FALSE;
-			break;
-		case GROUP_LEVEL:
-			for (iIndex = 0; iIndex < iNumMax - 1; iIndex++) {
-				int iNum;
-
-				for (iNum = (int) base[iIndex]; iNum != -1; iNum = arts[iNum].thread)
-					arts[iNum].inrange = FALSE;
-			}
-			break;
-		case THREAD_LEVEL:
-			for (iIndex = 0; iIndex < iNumMax - 1; iIndex++)
-				arts[iIndex].inrange = FALSE;
-#if 0
-			for (iIndex = base[thread_basenote]; iIndex >= 0; iIndex = arts[iIndex].thread)
-				arts[iIndex].inrange = FALSE;
-#endif /* 0 */
-			break;
-		default:
-			break;
-	}
+	return ret;
 }
 
 
@@ -517,59 +512,59 @@ vDelRange (
  * SELECTED CODE
  */
 void
-do_auto_select_arts (
+do_auto_select_arts(
 	void)
 {
 	int i;
 
-	for (i = 0; i < top_art; ++i) {
-		if (arts[i].status == ART_UNREAD && arts[i].selected != 1) {
+	for_each_art(i) {
+		if (arts[i].status == ART_UNREAD && !arts[i].selected) {
 #	ifdef DEBUG_NEWSRC
-			debug_print_comment ("group.c: X command");
+			debug_print_comment("group.c: X command");
 #	endif /* DEBUG_NEWSRC */
-			art_mark_read (&CURR_GROUP, &arts[i]);
+			art_mark_read(&CURR_GROUP, &arts[i]);
 			arts[i].zombie = TRUE;
 		}
 	}
 	if (CURR_GROUP.attribute->show_only_unread)
-		find_base (&CURR_GROUP);
+		find_base(&CURR_GROUP);
 
 	grpmenu.curr = 0;
-	show_group_page ();
+	show_group_page();
 }
 
 
 /* selection already happened in filter_articles() */
 void
-undo_auto_select_arts (
+undo_auto_select_arts(
 	void)
 {
 	int i;
 
-	for (i = 0; i < top_art; ++i) {
+	for_each_art(i) {
 		if (arts[i].status == ART_READ && arts[i].zombie) {
 #	ifdef DEBUG_NEWSRC
-			debug_print_comment ("group.c: + command");
+			debug_print_comment("group.c: + command");
 #	endif /* DEBUG_NEWSRC */
-			art_mark_unread (&CURR_GROUP, &arts[i]);
+			art_mark_unread(&CURR_GROUP, &arts[i]);
 			arts[i].zombie = FALSE;
 		}
 	}
 	if (CURR_GROUP.attribute->show_only_unread)
-		find_base (&CURR_GROUP);
+		find_base(&CURR_GROUP);
 
-	grpmenu.curr = 0;	/* do we want this ? */
-	show_group_page ();
+	grpmenu.curr = 0;	/* do we want this? */
+	show_group_page();
 }
 
 
 void
-undo_selections (
+undo_selections(
 	void)
 {
 	int i;
 
-	for (i = 0; i < top_art; i++) {
+	for_each_art(i) {
 		arts[i].selected = FALSE;
 		arts[i].zombie = FALSE;
 	}
